module mo_srf_emissions
  !---------------------------------------------------------------
  ! 	... surface emissions module
  !---------------------------------------------------------------

  use shr_kind_mod, only : r8 => shr_kind_r8, SHR_KIND_CL ! Guangxing Lin
  use chem_mods,    only : gas_pcnst
  use spmd_utils,   only : masterproc,iam
  use mo_tracname,  only : solsym
  use abortutils,   only : endrun
  use ioFileMod,    only : getfil
  use ppgrid,       only : pcols, begchunk, endchunk
  use cam_logfile,  only : iulog
  use tracer_data,  only : trfld,trfile
 
  use h_setkin_par, only : NACT !Guangxing Lin
  implicit none

  type :: emission
     integer           :: spc_ndx
     real(r8)          :: mw
     character(len=256):: filename
     character(len=16) :: species
     character(len=8)  :: units
     integer                   :: nsectors
     character(len=32),pointer :: sectors(:)
     type(trfld), pointer      :: fields(:)
     type(trfile)              :: file
  end type emission

  private

  public  :: srf_emissions_inti, set_srf_emissions, set_srf_emissions_time 

  save

  real(r8), parameter :: amufac = 1.65979e-23_r8         ! 1.e4* kg / amu
  integer, parameter :: gas_pcnst_um = NACT   !Guangxing Lin 
  logical :: has_emis(gas_pcnst)
  !added by Guangxing Lin
  logical :: has_emis_um(gas_pcnst_um)
  type(emission), allocatable :: emissions(:)
  type(emission), allocatable :: emissions_um(:) !Guangxing Lin
  integer                     :: n_emis_species 
  integer                     :: n_emis_species_um !Guangxing Lin 
  integer :: c10h16_ndx, isop_ndx

contains

  subroutine srf_emissions_inti( srf_emis_specifier, emis_type, emis_cycle_yr, emis_fixed_ymd, emis_fixed_tod )

    !-----------------------------------------------------------------------
    ! 	... initialize the surface emissions
    !-----------------------------------------------------------------------

    use chem_mods,        only : adv_mass
    use mo_constants,     only : d2r, pi, rearth
    use string_utils,     only : to_upper
    use mo_chem_utls,     only : get_spc_ndx 
    use tracer_data,      only : trcdata_init
    use cam_pio_utils,    only : cam_pio_openfile
    use pio,              only : pio_inquire, pio_nowrite, pio_closefile, pio_inq_varndims
    use pio,              only : pio_inq_varname, file_desc_t
    use um_aerosol_intr, only: um_get_spc_ndx ! added by Guangxing Lin 

    use h_setkin_mw, only : mw_data !Guangxing Lin
    use chem_surfvals, only : flbc_list


    implicit none

    !-----------------------------------------------------------------------
    ! 	... dummy arguments
    !-----------------------------------------------------------------------
    character(len=*), intent(in) :: srf_emis_specifier(:)
    character(len=*), intent(in) :: emis_type
    integer,          intent(in) :: emis_cycle_yr
    integer,          intent(in) :: emis_fixed_ymd
    integer,          intent(in) :: emis_fixed_tod

    !-----------------------------------------------------------------------
    ! 	... local variables
    !-----------------------------------------------------------------------
    integer  :: astat
    integer  :: j, l, m, n, i, nn                     ! Indices
    character(len=16)  :: spc_name
    character(len=256) :: filename

    character(len=16)  ::    emis_species(gas_pcnst)
    character(len=256) ::    emis_filenam(gas_pcnst)
    integer ::    emis_indexes(gas_pcnst)
    !added by Guangxing Lin
    character(len=16)  ::    emis_species_um(gas_pcnst_um)
    character(len=256) ::    emis_filenam_um(gas_pcnst_um)
    integer ::    emis_indexes_um(gas_pcnst_um)
    ! Guangxing Lin---temporaly set emissin file here, later should read in from
    ! name list
    character(len=256) :: um_srf_emis_specifier(gas_pcnst_um)
    
    integer :: vid, nvars, isec
    integer, allocatable :: vndims(:)
    type(file_desc_t) :: ncid
    character(len=32)  :: varname
    character(len=256) :: locfn
    integer :: ierr
    character(len=1), parameter :: filelist = ''
    character(len=1), parameter :: datapath = ''
    logical         , parameter :: rmv_file = .false.
    integer :: emis_cycle_yr_um

    has_emis(:) = .false.
    has_emis_um(:) = .false.  !added by Guangxing Lin
    emis_cycle_yr_um =2000 !Guangxing
   !added by Guangxing Lin
    um_srf_emis_specifier = ' '
   ! um_srf_emis_specifier(1)= "BENZENE  -> /glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/1992-2010/emissions.SOA_BENZENE.surface.1.9x2.5_c120313.nc"
    um_srf_emis_specifier(1)= "BENZENE  -> /glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/accmip_mz4_1850_2000/IPCC_emissions_BENZENE_surface_1850-2000_1.9x2.5_c120313.nc"
    um_srf_emis_specifier(2)= "BIGALK   -> /glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/1992-2010/emissions.BIGALK.surface.1.9x2.5_c110426.nc"       
    um_srf_emis_specifier(3)= 'C3H6     ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/1992-2010/emissions.C3H6_no_bg.surface.1.9x2.5_c121022.nc' 
    um_srf_emis_specifier(4)= 'C2H2     ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/1992-2010/emissions.C2H2.surface.1.9x2.5_c110426.nc'
    um_srf_emis_specifier(5)= 'C2H6     ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/1992-2010/emissions.C2H6_no_bg.surface.1.9x2.5_c121022.nc'
    um_srf_emis_specifier(6)= 'C3H8     ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/1992-2010/emissions.C3H8_no_bg.surface.1.9x2.5_c121022.nc'
    um_srf_emis_specifier(7)= 'HCHO     ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/1992-2010/emissions.CH2O.surface.1.9x2.5_c110426.nc'
    um_srf_emis_specifier(8)= 'CH3CHO   ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/1992-2010/emissions.CH3CHO.surface.1.9x2.5_c110426.nc'     
    um_srf_emis_specifier(9)= 'CH3COCH3 ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/1992-2010/emissions.CH3COCH3_no_bg.surface.1.9x2.5_c131205.nc'
    um_srf_emis_specifier(10)= 'CH3COOH  ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/1992-2010/emissions.CH3COOH.surface.1.9x2.5_c110426.nc' 
    um_srf_emis_specifier(11)= 'CH3OH    ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/1992-2010/emissions.CH3OH_no_bg.surface.1.9x2.5_c121022.nc'
    um_srf_emis_specifier(12)= 'CO       ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/1992-2010/emissions.CO_no_bg.surface.1.9x2.5_c121022.nc' 
    um_srf_emis_specifier(13)= 'HCOOH    ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/1992-2010/emissions.HCOOH.surface.1.9x2.5_c110426.nc'
    um_srf_emis_specifier(14)= 'MEK      ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/1992-2010/emissions.MEK.surface.1.9x2.5_c110426.nc'
    um_srf_emis_specifier(15)=  'NO       ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/1992-2010/emissions.NO.surface.1.9x2.5_c110426.nc' 
    um_srf_emis_specifier(16)= 'TOLUENE  ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/accmip_mz4_1850_2000/IPCC_emissions_SOA_TOLUENE_surface_1850-2000_1.9x2.5_c120313.nc'       
    um_srf_emis_specifier(17)= 'XYLENE   ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/accmip_mz4_1850_2000/IPCC_emissions_XYLENE_surface_1850-2000_1.9x2.5_c120313.nc'
    um_srf_emis_specifier(18)=  'DMS       ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/trop_mozart_aero/emis/aerocom_mam3_dms_surf_2000_c120315.nc'
    um_srf_emis_specifier(19)=   'SO2       ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/trop_mozart_aero/emis/ar5_mam3_so2_surf_2000_c120315.nc'
    um_srf_emis_specifier(20)=  'SOAG      ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/trop_mozart_aero/emis/ar5_mam3_soag_1.5_surf_2000_c120315.nc'
    um_srf_emis_specifier(21)=   'bc_a1     ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/trop_mozart_aero/emis/ar5_mam3_bc_surf_2000_c120315.nc'
    um_srf_emis_specifier(22)= 'num_a1    ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/trop_mozart_aero/emis/ar5_mam3_num_a1_surf_2000_c120315.nc'
    um_srf_emis_specifier(23)= 'num_a2    ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/trop_mozart_aero/emis/ar5_mam3_num_a2_surf_2000_c120315.nc'
    um_srf_emis_specifier(24)= 'pom_a1    ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/trop_mozart_aero/emis/ar5_mam3_oc_surf_2000_c120315.nc'
    um_srf_emis_specifier(25)=  'so4_a1    ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/trop_mozart_aero/emis/ar5_mam3_so4_a1_surf_2000_c120315.nc'
    um_srf_emis_specifier(26)=  'so4_a2->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/trop_mozart_aero/emis/ar5_mam3_so4_a2_surf_2000_c120315.nc'
    um_srf_emis_specifier(27)= 'BIGENE   ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/1992-2010/emissions.BIGENE.surface.1.9x2.5_c110426.nc'
    um_srf_emis_specifier(28)= 'C2H4     ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/emis/1992-2010/emissions.C2H4_no_bg.surface.1.9x2.5_c121022.nc' 
  !  write(iulog,*) 'gxlin-specificer', um_srf_emis_specifier(1:27)


     nn = 0
    count_emis: do n=1,gas_pcnst
       if ( len_trim(srf_emis_specifier(n) ) == 0 ) then
          exit count_emis
       endif

       i = scan(srf_emis_specifier(n),'->')
       spc_name = trim(adjustl(srf_emis_specifier(n)(:i-1)))
       filename = trim(adjustl(srf_emis_specifier(n)(i+2:)))

       m = get_spc_ndx(spc_name)

       if (m > 0) then
          has_emis(m) = .true.
          has_emis(m) = has_emis(m) .and. ( .not. any( flbc_list == spc_name ) )
       else 
          write(iulog,*) 'srf_emis_inti: spc_name ',spc_name,' is not included in the simulation'
          call endrun('srf_emis_inti: invalid surface emission specification')
       endif

       if ( has_emis(m) ) then
          nn = nn+1
          emis_species(nn) = spc_name
          emis_filenam(nn) = filename
          emis_indexes(nn) = m
       endif
    enddo count_emis

    n_emis_species = count(has_emis(:))
    
    !added by Guangxing Lin
     nn = 0
     !write(iulog,*) 'gxlin_pcnst: ', gas_pcnst_um
    count_emis_um: do n=1, gas_pcnst_um
       if ( len_trim(um_srf_emis_specifier(n) ) == 0 ) then
          exit count_emis_um
       endif

       i = scan(um_srf_emis_specifier(n),'->')
       spc_name = trim(adjustl(um_srf_emis_specifier(n)(:i-1)))
       filename = trim(adjustl(um_srf_emis_specifier(n)(i+2:)))
       !write(iulog,*) 'gxlin_spc_name ',spc_name 
       select case( spc_name )
         case( 'DMS' )
           m = um_get_spc_ndx('nDMS')
         case( 'SO2' )
           m = um_get_spc_ndx('fSO2')
         case( 'SOAG')
           m = um_get_spc_ndx('nOC')
         case( 'bc_a1'  )
           m = um_get_spc_ndx('fBC')
         case( 'num_a1'  )
           m = um_get_spc_ndx('SO4n3') !not used in IMPACT
         case( 'num_a2'  )
           m = um_get_spc_ndx('SO4n2') !not used in IMPACT
         case( 'pom_a1' )
           m = um_get_spc_ndx('fOC')
         case( 'so4_a1' ) !1 is accumulation in MAM3 -> m3 in IMPACT
           m = um_get_spc_ndx('SO4m3')
         case( 'so4_a2' )!2 is Aitken in MAM3 -> m2 in IMPACT, m1 is Nucleation mode in IMPACT
           m = um_get_spc_ndx('SO4m2')        
         case( 'CO' )
           m = um_get_spc_ndx('CO')
         case( 'NO' )
           m = um_get_spc_ndx('NO')
         case( 'MEK' )
           m = um_get_spc_ndx('MEK')
         case( 'BIGENE' )
           m = um_get_spc_ndx('BUTE')
         case( 'C2H6' )
           m = um_get_spc_ndx('C2H6')
         case( 'C3H6' )
           m = um_get_spc_ndx('PRPE')
         case( 'C3H8' )
           m = um_get_spc_ndx('C3H8')
         case( 'BIGALK' )
           m = um_get_spc_ndx('ALK4')
         case( 'CH3CHO' )
           m = um_get_spc_ndx('ALD2')
         case( 'HCHO' )
           m = um_get_spc_ndx('HCHO')
         case( 'CH3COCH3' )
           m = um_get_spc_ndx('ACET')
         case( 'C2H4' )
           m = um_get_spc_ndx('ETHE')
         case( 'BENZENE' )
           m = um_get_spc_ndx('BENZ')
         case( 'TOLUENE' )
           m = um_get_spc_ndx('TOLU')
         case( 'XYLENE' )
           m = um_get_spc_ndx('XYLE')
         case( 'CH3OH' )
           m = um_get_spc_ndx('MOH')
         case( 'C2H2' )
           m = um_get_spc_ndx('H2') ! Actually it belongs to MOH
         case( 'HCOOH' )
           m = um_get_spc_ndx('ACO2')
         case( 'CH3COOH' )
            m = um_get_spc_ndx('ACTA')
          end select

       if (m > 0) then
          has_emis_um(m) = .true.
          has_emis_um(m) = has_emis_um(m) .and. ( .not. any( flbc_list == spc_name ) )
       else 
          write(iulog,*) 'um_srf_emis_inti: spc_name ',spc_name,' is not included in the simulation'
          call endrun('um_srf_emis_inti: invalid surface emission specification')
       endif

       if ( has_emis_um(m) ) then
          nn = nn+1
          emis_species_um(nn) = spc_name
          emis_filenam_um(nn) = filename
          emis_indexes_um(nn) = m
         
       ! write(iulog,*) 'um_emisfilename_lin_1 = ', emis_filenam_um(nn) 
       ! write(iulog,*) 'um_emisfilename_lin_2 = ', filename 
       endif
    enddo count_emis_um

    n_emis_species_um =  count(has_emis_um(:))

    if (masterproc) write(iulog,*) 'srf_emis_inti: n_emis_species = ',n_emis_species

    allocate( emissions(n_emis_species), stat=astat )
    if( astat/= 0 ) then
       write(iulog,*) 'srf_emis_inti: failed to allocate emissions array; error = ',astat
       call endrun
    end if
    !added by Guangxing Lin
    if (masterproc) write(iulog,*) 'um_srf_emis_inti: n_emis_species = ',n_emis_species_um

    allocate( emissions_um(n_emis_species_um), stat=astat )
    if( astat/= 0 ) then
       write(iulog,*) 'um_srf_emis_inti: failed to allocate emissions array; error = ',astat
       call endrun
    end if

    !-----------------------------------------------------------------------
    ! 	... setup the emission type array
    !-----------------------------------------------------------------------
    do m=1,n_emis_species 
       emissions(m)%spc_ndx          = emis_indexes(m)
       emissions(m)%units            = 'Tg/y'
       emissions(m)%species          = emis_species(m)
       emissions(m)%mw               = adv_mass(emis_indexes(m))                     ! g / mole
       emissions(m)%filename         = emis_filenam(m)
    enddo
    ! Guangxing Lin
    do m=1,n_emis_species_um 
       emissions_um(m)%spc_ndx          = emis_indexes_um(m)
       emissions_um(m)%units            = 'Tg/y'
       emissions_um(m)%species          = emis_species_um(m)
       emissions_um(m)%mw               = mw_data(emis_indexes_um(m))                ! g / mole
       emissions_um(m)%filename         = emis_filenam_um(m)
    enddo

    !-----------------------------------------------------------------------
    ! read emis files to determine number of sectors
    !-----------------------------------------------------------------------
    spc_loop: do m = 1, n_emis_species

       emissions(m)%nsectors = 0
       
       call getfil (emissions(m)%filename, locfn, 0)
       call cam_pio_openfile ( ncid, trim(locfn), PIO_NOWRITE)
       ierr = pio_inquire (ncid, nvariables=nvars)

       allocate(vndims(nvars))

       do vid = 1,nvars

          ierr = pio_inq_varndims (ncid, vid, vndims(vid))

          if( vndims(vid) < 3 ) then
             cycle
          elseif( vndims(vid) > 3 ) then
             ierr = pio_inq_varname (ncid, vid, varname)
             write(iulog,*) 'srf_emis_inti: Skipping variable ', trim(varname),', ndims = ',vndims(vid), &
                  ' , species=',trim(emissions(m)%species)
             cycle
          end if

          emissions(m)%nsectors = emissions(m)%nsectors+1

       enddo

       allocate( emissions(m)%sectors(emissions(m)%nsectors), stat=astat )
       if( astat/= 0 ) then
         write(iulog,*) 'srf_emis_inti: failed to allocate emissions(m)%sectors array; error = ',astat
         call endrun
       end if

       isec = 1

       do vid = 1,nvars
          if( vndims(vid) == 3 ) then
             ierr = pio_inq_varname(ncid, vid, emissions(m)%sectors(isec))
             isec = isec+1
          endif

       enddo
       deallocate(vndims)
       call pio_closefile (ncid)

   ! from the difference between cesm1.1 and cesm1.2   
       allocate(emissions(m)%file%in_pbuf(size(emissions(m)%sectors)))
       emissions(m)%file%in_pbuf(:) = .false.

       call trcdata_init( emissions(m)%sectors, &
                          emissions(m)%filename, filelist, datapath, &
                          emissions(m)%fields,  &
                          emissions(m)%file, &
                          rmv_file, emis_cycle_yr, emis_fixed_ymd, emis_fixed_tod, emis_type )

    enddo spc_loop
    ! added by Guangxing Lin
    spc_loop_um: do m = 1, n_emis_species_um

       emissions_um(m)%nsectors = 0
       
       ! write(iulog,*) 'um_emisfilename_lin_0 = ', emissions_um(m)%filename
       call getfil (emissions_um(m)%filename, locfn, 0)
!       call getfil (emis_filenam_um(m), locfn, 0)
       call cam_pio_openfile ( ncid, trim(locfn), PIO_NOWRITE)
       ierr = pio_inquire (ncid, nvariables=nvars)

       allocate(vndims(nvars))

       do vid = 1,nvars

          ierr = pio_inq_varndims (ncid, vid, vndims(vid))

          if( vndims(vid) < 3 ) then
             cycle
          elseif( vndims(vid) > 3 ) then
             ierr = pio_inq_varname (ncid, vid, varname)
             write(iulog,*) 'um_srf_emis_inti: Skipping variable ', trim(varname),', ndims = ',vndims(vid), &
                  ' , species=',trim(emissions_um(m)%species)
             cycle
          end if

          emissions_um(m)%nsectors = emissions_um(m)%nsectors+1

       enddo

       allocate( emissions_um(m)%sectors(emissions_um(m)%nsectors), stat=astat )
       if( astat/= 0 ) then
         write(iulog,*) 'um_srf_emis_inti: failed to allocate emissions(m)%sectors array; error = ',astat
         call endrun
       end if

       isec = 1

       do vid = 1,nvars
          if( vndims(vid) == 3 ) then
             ierr = pio_inq_varname(ncid, vid, emissions_um(m)%sectors(isec))
             isec = isec+1
          endif

       enddo
       deallocate(vndims)
       call pio_closefile (ncid)

   ! from the difference between cesm1.1 and cesm1.2   
       allocate(emissions_um(m)%file%in_pbuf(size(emissions_um(m)%sectors)))
       emissions_um(m)%file%in_pbuf(:) = .false.
   
       call trcdata_init( emissions_um(m)%sectors, &
                          emissions_um(m)%filename, filelist, datapath, &
                          emissions_um(m)%fields,  &
                          emissions_um(m)%file, &
                          rmv_file, emis_cycle_yr_um, emis_fixed_ymd, emis_fixed_tod, emis_type )

    enddo spc_loop_um

    c10h16_ndx = get_spc_ndx('C10H16')
    isop_ndx = get_spc_ndx('ISOP')

  end subroutine srf_emissions_inti

  subroutine set_srf_emissions_time( pbuf2d, state )
    !-----------------------------------------------------------------------
    !       ... check serial case for time span
    !-----------------------------------------------------------------------

    use physics_types,only : physics_state
    use ppgrid,       only : begchunk, endchunk
    use tracer_data,  only : advance_trcdata
    use physics_buffer, only : physics_buffer_desc

    implicit none

    type(physics_state), intent(in):: state(begchunk:endchunk)                 
    type(physics_buffer_desc), pointer :: pbuf2d(:,:)

    !-----------------------------------------------------------------------
    !       ... local variables
    !-----------------------------------------------------------------------
    integer :: m

        ! write(iulog,*) 'set_srf_emissions_time_gxlin_0 '
    do m = 1,n_emis_species
       call advance_trcdata( emissions(m)%fields, emissions(m)%file, state, pbuf2d  )
    end do
        ! write(iulog,*) 'set_srf_emissions_time_gxlin_1 '
    ! added by Guangxing Lin
    do m = 1,n_emis_species_um
       call advance_trcdata( emissions_um(m)%fields, emissions_um(m)%file, state, pbuf2d  )
    end do
        ! write(iulog,*) 'set_srf_emissions_time_gxlin_2 '

  end subroutine set_srf_emissions_time

  ! adds surf flux specified in file to sflx
  subroutine set_srf_emissions( rlats, rlons, lchnk, sflx, ncol )
    !--------------------------------------------------------
    !	... form the surface fluxes for this latitude slice
    !--------------------------------------------------------

    use mo_constants, only : pi
    use time_manager, only : get_curr_calday
    use string_utils, only : to_lower, GLC
    use um_aerosol_intr, only: um_get_spc_ndx, um_emis_srf, um_emis_srf_has ! added by Cheng
     
    implicit none

    !--------------------------------------------------------
    !	... Dummy arguments
    !--------------------------------------------------------
    integer,  intent(in)    :: ncol                  ! columns in chunk
    integer,  intent(in)    :: lchnk                 ! chunk index
    real(r8), intent(in)    :: rlons(ncol)           ! chunk longitudes (radians)
    real(r8), intent(in)    :: rlats(ncol)           ! chunk latitudes (radians)
    real(r8), intent(inout) :: sflx(pcols,gas_pcnst) ! surface emissions ( kg/m^2/s )

    !--------------------------------------------------------
    !	... local variables
    !--------------------------------------------------------
    integer  ::  i, m, n
    real(r8) ::  factor
    real(r8) ::  dayfrac            ! fration of day in light
    real(r8) ::  iso_off            ! time iso flux turns off
    real(r8) ::  iso_on             ! time iso flux turns on

    logical  :: polar_day,polar_night
    real(r8) :: doy_loc
    real(r8) :: sunon,sunoff
    real(r8) :: loc_angle
    real(r8) :: latitude
    real(r8) :: declination
    real(r8) :: tod
    real(r8) :: calday

    real(r8), parameter :: dayspy = 365._r8
    real(r8), parameter :: twopi = 2.0_r8 * pi
    real(r8), parameter :: pid2  = 0.5_r8 * pi
    real(r8), parameter :: dec_max = 23.45_r8 * pi/180._r8
    real(r8), parameter :: om_to_oc  = 1.4_r8

    real(r8) :: flux(ncol)
    real(r8) :: mfactor
    integer  :: isec, ispec !Cheng added ispec

    character(len=12),parameter :: mks_units(4) = (/ "kg/m2/s     ", &
                                                     "kg/m2/sec   ", &
                                                     "kg/m^2/s    ", &
                                                     "kg/m^2/sec  " /)
    character(len=12) :: units
    real(r8) :: in_sflx(pcols,gas_pcnst) ! surface emissions ( kg/m^2/s )

    in_sflx(:,:) = 0._r8

    !--------------------------------------------------------
    !	... set non-zero emissions
    !--------------------------------------------------------
    emis_loop : do m = 1,n_emis_species

       n = emissions(m)%spc_ndx

       flux(:) = 0._r8
       do isec = 1,emissions(m)%nsectors
          flux(:ncol) = flux(:ncol) + emissions(m)%fields(isec)%data(:ncol,1,lchnk)
       enddo

#if (defined MODAL_AERO)
       select case( emissions(m)%species )
       case( 'SOAG' )
          flux(:ncol) = flux(:ncol) * om_to_oc
       case( 'pom_a1', 'pom_a3' ) ! for both 3-mode and 7-mode ? -- should be done offline!
          flux(:ncol) = flux(:ncol) * om_to_oc
       end select
#endif

       units = to_lower(trim(emissions(m)%fields(1)%units(:GLC(emissions(m)%fields(1)%units))))

       if ( any( mks_units(:) == units ) ) then
          if (  emissions(m)%species == 'ISOP' .or. emissions(m)%species == 'C10H16' ) then
             in_sflx(:ncol,n) = flux(:ncol)
          else
             sflx(:ncol,n) = sflx(:ncol,n) + flux(:ncol)
          endif
       else
          mfactor = amufac * emissions(m)%mw
          if (  emissions(m)%species == 'ISOP' .or. emissions(m)%species == 'C10H16' ) then
             in_sflx(:ncol,n) = flux(:ncol) * mfactor
          else
             sflx(:ncol,n) = sflx(:ncol,n) + flux(:ncol) * mfactor
          endif
       endif

   
    end do emis_loop
    ! added by Guangxing Lin
    emis_loop_um : do m = 1,n_emis_species_um

       n = emissions_um(m)%spc_ndx

       flux(:) = 0._r8
       do isec = 1,emissions_um(m)%nsectors
          flux(:ncol) = flux(:ncol) + emissions_um(m)%fields(isec)%data(:ncol,1,lchnk)
       enddo

#if (defined MODAL_AERO)
       select case( emissions_um(m)%species )
       case( 'SOAG' )
          flux(:ncol) = flux(:ncol) * om_to_oc
       case( 'pom_a1', 'pom_a3' ) ! for both 3-mode and 7-mode ? -- should be done offline!
          flux(:ncol) = flux(:ncol) * om_to_oc
       end select
#endif



   !-------------
   ! added by Cheng
   !--------------
!       if (masterproc) then
!       	  write(*,'(A,3I3,A,A,A,f12.8)'),'mo_srf:', lchnk,m,n,' ',emissions(m)%species,units,emissions(m)%mw
!       endif
!   FC5 output     
! 1  4 DMS             molecules/cm   62.1324
! 2  3 SO2             molecules/cm   64.0648
! 3  5 SOAG            molecules/cm   12.0110
! 4  9 bc_a1           molecules/cm   12.0110
! 5 12 num_a1          (particles/c    1.0074
! 6 16 num_a2          (particles/c    1.0074
! 7  7 pom_a1          molecules/cm   12.0110
! 8  6 so4_a1          molecules/cm  115.10734
! 9 13 so4_a2          molecules/cm  115.10734
       
    !-------------------------------------
    !following is added by Cheng to store srf flux in um_aerosol_intr,oc has been converted to OM above
    !-------------------------------------
       ispec = -1
       mfactor = amufac !* emissions(m)%mw    !use mfactor = amufac, from #/cm2/s to 1e3*mole/m2/s, simply times mw (g) to get kg/m2/s
       !mfactor = 1.0_r8 !keep #/cm2/s
       select case( trim(emissions_um(m)%species) )  ! convert to kg/m2/s from #/cm2/s
         case( 'DMS' )
           ispec = um_get_spc_ndx('nDMS')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) !* mfactor ! #/cm2/s, 
         case( 'SO2' )
           ispec = um_get_spc_ndx('fSO2')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) !* mfactor ! to #/cm2/s
         case( 'SOAG')
           ispec = um_get_spc_ndx('nOC')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) * mfactor * 12.0110_r8! to kg/m2/s, this is already OM, see the om_to_oc(1.4)applied above
         case( 'bc_a1'  )
           ispec = um_get_spc_ndx('fBC')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) * mfactor * 12.0110_r8 ! to kg/m2/s
         case( 'pom_a1' )
           ispec = um_get_spc_ndx('fOC')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) * mfactor * 12.0110_r8 ! to kg/m2/s, it is alrady OM, same as SOAG
         case( 'so4_a1' ) !1 is accumulation in MAM3 -> m3 in IMPACT
           ispec = um_get_spc_ndx('SO4m3')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) !* mfactor ! #/cm2/s
         case( 'so4_a2' )!2 is Aitken in MAM3 -> m2 in IMPACT, m1 is Nucleation mode in IMPACT
           ispec = um_get_spc_ndx('SO4m2')        
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) !* mfactor ! #/cm2/s
         ! added by Guangxing Lin
         case( 'CO' )
           ispec = um_get_spc_ndx('CO')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) ! #/cm2/s, 
         case( 'NO' )
           ispec = um_get_spc_ndx('NO')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) ! #/cm2/s, 
         case( 'MEK' )
           ispec = um_get_spc_ndx('MEK')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) ! #/cm2/s, 
         case( 'BIGENE' )
           ispec = um_get_spc_ndx('BUTE')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) ! #/cm2/s, 
         !write(iulog,*) 'gxlin2_ispec ', ispec 
         !write(iulog,*) 'gxlin2_srf ', um_emis_srf(lchnk,:ncol,ispec)
         case( 'C2H6' )
           ispec = um_get_spc_ndx('C2H6')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) ! #/cm2/s, 
         case( 'C3H6' )
           ispec = um_get_spc_ndx('PRPE')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) ! #/cm2/s, 
         case( 'C3H8' )
           ispec = um_get_spc_ndx('C3H8')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) ! #/cm2/s, 
         case( 'BIGALK' )
           ispec = um_get_spc_ndx('ALK4')
           um_emis_srf(lchnk,:ncol,ispec) =15.3d0/26.6d0* flux(:ncol) ! #/cm2/s, 
         !write(iulog,*) 'gxlin2_ispec ', ispec 
         !write(iulog,*) 'gxlin2_srf ', um_emis_srf(lchnk,:ncol,ispec)
           ispec = um_get_spc_ndx('ALK7')
           um_emis_srf(lchnk,:ncol,ispec) =11.3d0/26.6d0* flux(:ncol) ! #/cm2/s, 
         !write(iulog,*) 'gxlin2_ispec ', ispec 
         !write(iulog,*) 'gxlin2_srf ', um_emis_srf(lchnk,:ncol,ispec)
         case( 'CH3CHO' )
           ispec = um_get_spc_ndx('ALD2')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) ! #/cm2/s, 
         case( 'HCHO' )
           ispec = um_get_spc_ndx('HCHO')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) ! #/cm2/s, 
         case( 'CH3COCH3' )
           ispec = um_get_spc_ndx('ACET')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) ! #/cm2/s, 
         case( 'C2H4' )
           ispec = um_get_spc_ndx('ETHE')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) ! #/cm2/s, 
         case( 'BENZENE' )
           ispec = um_get_spc_ndx('BENZ')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) ! #/cm2/s, 
         case( 'TOLUENE' )
           ispec = um_get_spc_ndx('TOLU')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) ! #/cm2/s, 
         case( 'XYLENE' )
           ispec = um_get_spc_ndx('XYLE')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) ! #/cm2/s, 
         case( 'CH3OH' )
           ispec = um_get_spc_ndx('MOH')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) ! #/cm2/s, 
         case( 'C2H2' )
           ispec = um_get_spc_ndx('H2') ! actually it belongs to MOH
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) ! #/cm2/s, 
         case( 'HCOOH' )
           ispec = um_get_spc_ndx('ACO2')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) ! #/cm2/s, 
         case( 'CH3COOH' )
           ispec = um_get_spc_ndx('ACTA')
           um_emis_srf(lchnk,:ncol,ispec) = flux(:ncol) ! #/cm2/s, 
       end select
         
           	
        if (ispec .ge. 1) then ! has emission
          um_emis_srf_has(ispec) = .true. 
        end if
          
          um_emis_srf_has(um_get_spc_ndx('ALK4')) = .true. 
          um_emis_srf_has(um_get_spc_ndx('ALK7')) = .true. 
   !---------end output to um_aerosol arrays ---------------------------------
   
    end do emis_loop_um

    calday = get_curr_calday()
    doy_loc     = aint( calday )
    declination = dec_max * cos((doy_loc - 172._r8)*twopi/dayspy)
    tod = (calday - doy_loc) + .5_r8

    do i = 1,ncol
       !
       polar_day   = .false.
       polar_night = .false.
       !
       loc_angle = tod * twopi + rlons(i)
       loc_angle = mod( loc_angle,twopi )
       latitude =  rlats(i)
       !
       !------------------------------------------------------------------
       !        determine if in polar day or night
       !        if not in polar day or night then
       !        calculate terminator longitudes
       !------------------------------------------------------------------
       if( abs(latitude) >= (pid2 - abs(declination)) ) then
          if( sign(1._r8,declination) == sign(1._r8,latitude) ) then
             polar_day = .true.
             sunoff = 2._r8*twopi
             sunon  = -twopi
          else
             polar_night = .true.
          end if
       else
          sunoff = acos( -tan(declination)*tan(latitude) )
          sunon  = twopi - sunoff
       end if

       !--------------------------------------------------------
       !	... adjust alpha-pinene for diurnal variation
       !--------------------------------------------------------
       if( c10h16_ndx > 0 ) then
          if( has_emis(c10h16_ndx) ) then
             if( .not. polar_night .and. .not. polar_day ) then
                dayfrac = sunoff / pi
                in_sflx(i,c10h16_ndx) = in_sflx(i,c10h16_ndx) / (.7_r8 + .3_r8*dayfrac)
                if( loc_angle >= sunoff .and. loc_angle <= sunon ) then
                   in_sflx(i,c10h16_ndx) = in_sflx(i,c10h16_ndx) * .7_r8
                endif
             end if
             sflx(i,c10h16_ndx) = sflx(i,c10h16_ndx) + in_sflx(i,c10h16_ndx)
          end if
       end if

       !--------------------------------------------------------
       !	... adjust isoprene for diurnal variation
       !--------------------------------------------------------
       if( isop_ndx > 0 ) then
          if( has_emis(isop_ndx) ) then
             if( .not. polar_night ) then
                if( polar_day ) then
                   iso_off = .8_r8 * pi
                   iso_on  = 1.2_r8 * pi
                else
                   iso_off = .8_r8 * sunoff
                   iso_on  = 2._r8 * pi - iso_off
                end if
                if( loc_angle >= iso_off .and. loc_angle <= iso_on ) then
                   in_sflx(i,isop_ndx) = 0._r8
                else
                   factor = loc_angle - iso_on
                   if( factor <= 0._r8 ) then
                      factor = factor + 2._r8*pi
                   end if
                   factor = factor / (2._r8*iso_off + 1.e-6_r8)
                   in_sflx(i,isop_ndx) =in_sflx(i,isop_ndx) * 2._r8 / iso_off * pi * (sin(pi*factor))**2
                end if
             else
                in_sflx(i,isop_ndx) = 0._r8
             end if
             sflx(i,isop_ndx) = sflx(i,isop_ndx) + in_sflx(i,isop_ndx)
          end if
       end if

    end do


  end subroutine set_srf_emissions

end module mo_srf_emissions
