
         module  h_rbchemvars
          use h_rbchemmech 
         implicit none
         public
c=============================================================================
c
c $Id: rbchemvars.h,v 1.4 2002/05/02 15:50:22 akinorii Exp $
c
c CODE DEVELOPER
c   Sandy Sillman, U of M. (IMPACT version, 2007)
c
c FILE
c   rbchemvars.h
c
c DESCRIPTION:  variable declarations and common blocks
c   for I/O variables that are passed from the main program
c   to the rb (quad) chem. solver as the solver is invoked.
c   (Does not include allocated arrays)
c
c This file is included in both the entry program 
c  (quadchem_solver.F in IMPACT) 
c  and subsequent subroutines.
c
c VERSION: 92307:  COMMON BLOCKS REMOVED, real*8
c COMMON BLOCKS:  Only for variables that are constant over all grids.
c                 Others have been removed.
c
c ==========================================================================

c ichemvars.h   from  chemvars.EXT    April, 2007
c   INCLUDE and COMMON file 
c     for RADICAL BALANCE-BACK EULER solver for chemistry (quadchem)
c      (chemmain.f cheminit.f chemrates chemsolve.f, linslv.f, jval2.f)
c
c chemvars.EXT must go with chemmech.EXT 
c       with chemmech.EXT first - it contains indices.
c
c  This file includes declarations and common blocks
c   for INPUT-OUTPUT VARIABLES
c   which are passed to and  from the solver for photochemistry 
c   at each time step.
c
c  IMPACT version:  LARGE I-O ARRAYS HAVE BEEN MOVED to ichemarrays.h.
c   They will be ALLOCATED and DEALLOCATED in ichemmain1.f
c
c  Divided into ESSENTIAL input/output 
c    and OPTIONAL output which is rarely used.
c
c  Other INCLUDE files: 
c   chemmech.EXT =  variables for the chem. mechanism
c     that are set once and do not change as the mechanism is called.
c   chemlocal.EXT = Variables that are shared among the subroutines
c     in the chemistry solver, but not used elsewhere and not saved.
c
c Program written by Sanford Sillman
c History:
c  12/06. Program written by Sandy Sillman based on commq7.EXT
c  7/07  IMPACT version (.h), arrays separated into ichemarrays.h
c -------------------------------------------------------------
c      implicit none  -- IMPLICIT  is in boxvars.EXT, chemlocal.EXT
c      implicit real*8 (a-h,o-z)

c ESSENTIAL index parameters:   c_kvec, c_icdim, c_rdim
c   moved to CHEMMECH

c READ/WRITE AND VECTOR INDICES - moved to chemmech.EXT.
c
c BASIC INPUT ARRAYS:  CONCENTRATIONS, DEPOSITION. 
c   c_xcin, c_xcout, c_xcav, c-xcemit, c_xcwdep
c   moved to chemarrays.h
c
c  - Note:  Allocate/deallocate these in chemmain1; declare and pass through.
c 
c  CHEMISTRY INPUT PARAMETERS (CPARAMS):  
c         Parameters used for as part of chemistry calculation
c
c  Note:  Allocate?  No.  Just in common.

c c_time:              time step (sec)
c c_h2oliq             Liquid Water Content (LWC) grams cm-3 (acqua)
c c_rainfr             Rainout fraction for time step
c c_droplet            Droplet radius for gas-aq calc, cm (.001)
c                        (not yet vectorized - hard set at .001)
c c_lexpo(kvec)        Flag for modified backward Euler solution
c                        with exponential decay
c 
c FUTURE, NOT CURRENTLY USED:  
c c_rgasaq(kvec,nr)     Gas-aq transfer rate, s-1 (optional, <0 cancels)
c c_lgasaq(kvec)        Flag to calculate (T) gas-aqueous transfer rate
c c_lstsaq              Flag for steady state gas/aq partitioning
c                        (T for steady state)
c 
      real*8 ::  c_time             ! time step (sec)
      real*8 ::  c_h2oliq(c_kvec)   ! LWC grams cm-3
      real*8 ::  c_rainfr(c_kvec)   ! rainout fraction
      real*8 ::  c_h2ogas(c_kvec)   ! H2O abs humid molec/cm3

      real*8 ::  c_h2oaer(c_kvec)   ! aerosol water grams cm-3
      real*8 ::  c_raer(c_kvec)     ! wet aerosol radius cm 
      
      real*8 ::  c_rgasaq(c_kvec,c_rdim)  !  gas->aq rate
      real*8 ::  c_droplet(c_kvec)        ! droplet radius, cm
      logical :: c_lgasaq(c_kvec)       ! Flag to calc. gas-aq rate
      logical :: c_lstsaq(c_kvec)       ! Flag for steady state gas-aq
      logical :: c_lexpo(c_kvec)        ! Flag for expo decay solution

c      common/CHEMPARAMS/  c_time
c      common/CHEMPARAML/          c_lgasaq, c_lstsaq

c Deleted from common:  species that vary among grid boxes
c      c_h2oliq ,   c_rainfr,  c_h2ogas
c    * ,    c_rgasaq, c_droplet, c_lexpo

c  RATE INPUT PARAMETERS
c         Parameters used only  to calculate rate constants 
c           (note:  rate calc. also uses h2oliq, h2ogas) 
c
c c_dens               DENSITY molec cm-3
c c_temp               TEMPERATURE K
c c_saersa             SULFATE AEROSOL SURFACE AREA cm2 cm-3

      real*8 ::  c_temp(c_kvec)    ! temperature K
      real*8 ::  c_dens(c_kvec)    ! density molec cm-3
      real*8 ::  c_saersa(c_kvec)  ! s aero surf area cm2/cm3

c     common/RATEPARAMS/   c_temp, c_dens, c_saersa

c  PHOTOLYSIS INPUT PARAMETERS (HVPARAMS):  
c      Parameters used in calculating HV RATES.
c      Note: see also DATA FOR HV PARAMETERIZATION (chemmech)
c            and RETURNED J-VALUE ARRAY (jval, local in hvrates.f)
c
c c_jparam(   22)      Input parameters for j-value calculation
c                      (vectorize in future?)
c    c_jparam( 1)=zenith angle, degrees (calculated in program)
c    c_jparam( 2)=altitude (KM)  (OPTION - kPa)
c    c_jparam( 3)=ozone column (DU)
c    c_jparam( 4)=SO2   column (DU)  (0.1=1 ppb, SO2 0-1km)
c    c_jparam( 5)=NO2   column (DU)  (0.1=1 ppb, NO2 0-1km)
c 
c    c_jparam( 6)=aerosol optical depth
c                        (0.36 = Elterman 1968, normal, 0.76=polluted)
c    c_jparam( 7)=surface albedo   (0.1)
c    c_jparam( 8)= aerosol single scattering albedo
c                             (.75-.99, .75 Logan, .99 Madronich)
c    c_jparam( 9)= cloud optical depth for cloud ABOVE current alt.
c    c_jparam(10)= cloud optical depth for cloud BELOW current alt.
c
c    c_jparam(11)= opt-depth-weighted altitude for clouds above (KM)
c    c_jparam(12)= opt-depth-weighted altitude for clouds below (KM)
c    c_jparam(13)= temperature (K)
c 
c    c_jparam(20)= c_idate yymmddd (2000=100) or  day number (1-366)
c
c    c_lat (kk)     Latitude, degrees
c    c_lon (kk)     Longitude, degrees
c    c_hour         Hour at end of simulated time interval (decimal hrs)
c    (c_time also used - time interval - above.)
c    c_idate        Date YYMMDD (YY=>100) 
c                   Note, ZEN1 has alt.for YYYYDDD
c        (lat, lon, hour and date used in SOLAR ZENITH ANGLE  calc.)
c
      real*8 ::   c_lat(c_kvec)  ! Latitude, degrees
      real*8 ::   c_lon(c_kvec)  ! Longitude, degrees
      real*8 ::   c_hour         ! Hour at end , EST (hrs)
      integer :: c_idate                   ! Date YYMMDD (YY=100 for 2000)
      real*8 ::   c_jparam(22)   !J-value input parameters

c     common/HVPARAMS/ c_lat, c_lon, c_hour, c_jparam
c     common/HVPARAMS2/ c_idate

c OPTIONAL CHEM OUTPUT ARRAYS  - moved to chemarrays.h
c   c_rp, c_rl, c_rr
c
c  May be used for subsequent analysis but not necessary.
c  OPTION: Move this from chemvars to chemlocal
c
c    Note:  Allocate.
c
c OPTIONAL OUTPUTS RELATING TO NUMERICS: 
c   Currently in chemlocal:  
c    xohtest, xnotest, fohtest, final iter, history, geomavg

      real*8 ::  c_ohtest      ! test: dOH/OH or dOH/HO2
      real*8 ::  c_notest      ! test: dNO2/NO2
      integer :: c_iter                  ! chem. number of iterations

c     common/CHEMTEST/ c_ohtest, c_notest, c_iter

c
c ----------------------------------------
c END OF chemvars.EXT
c ----------------------------------------
c ================================================================
c
         end module  h_rbchemvars
