c linslv.f  
c    from bnrchemv5.f
c
c This contains the subroutines LINSLV and RESOLV
c   for inverting a matrix.  
c   (Dates from Prather, 1988, and Numerical Recipes)
c
c It solves for X in the matrix equation AX=B.  
c
c     call LINSLV(A, B, X, N) for AX=B, used dimension N
c          A is updated to reduced form  by LINSLV.
c          NOTE: declared dimension of A, B, and X are set at 100 (not N)
c
c     call RESOLV(A, B, X, N) for AX=B, with A reduced from LINSLV.
c          (if error in last solution is small)
c
c THE ORIGINAL CALL:  
c       err=abs(xoo(i)/xr( 1 ,ic))
c      if(errxo.lt.ermat) lnumat=.false.
c      if(lnumat) call LINSLV(fxo,xoo,nchem)
c      if(.not.lnumat) call RESOLV(fxo,xoo,nchem)
c
c MODIFICATIONS FROM bnrchemv5.f:  
c   include 'commq5.f' and 'commb5.f' are removed.
c   matrix A is parameter of subroutine.
c   
c
c ------------------------------------------------------

      SUBROUTINE LINSLV(A,B,X,N)
C  *** SUB -LINSLV- SOLVES THE FOLLOWING MATRIX EQUATION--
C  ***     A(N,N)*X(N) = B(N)  BY REDUCING THE A-MATRIX IN PLACE.
C  *** THE ENTRY -RESOLV- ASSUMES THAT THE A-MATRIX HAS BEEN PROPERLY
C  ***     REDUCED AND JUST SOLVES FOR X(N).  THIS OPTION SAVES TIME
C  ***     WHEN THE SYSTEM IS TO BE RESOLVED WITH A NEW B-VECTOR..  (Prather)

      IMPLICIT NONE

c
      DIMENSION B(100),X(100), A(100,100)
      DIMENSION S(100)
      DIMENSION IPA(100)

c     DOUBLE PRECISION    or   real*8
      real*8           A, B, X
      real*8           S, SMAX,DIV
      INTEGER N, KR, K, KRM1, KRMAX, KRP1, J, JP, JP1, I, IPA

      DO 10 KR=1,N
        DO    K=1,N
          S(K) = A(K,KR)
        ENDDO                !DO    K=1,N
        IF(KR.EQ.1) GO TO 14
        KRM1 = KR - 1
        DO    J=1,KRM1
          JP = IPA(J)
          A(J,KR) = S(JP)
          S(JP) = S(J)
          JP1 = J + 1
          DO    I=JP1,N
            S(I) = S(I) - A(I,J)*A(J,KR)
          ENDDO                         !DO    I=JP1,N
        ENDDO                        !DO    J=1,KRM1
14      KRMAX = KR
        SMAX =  ABS(S(KR))
        DO 15 I=KR,N
          IF( ABS(S(I)).Le.SMAX) GO TO 15
          KRMAX = I
          SMAX =  ABS(S(I))
15      CONTINUE                  !DO 15 I=KR,N
        IPA(KR) = KRMAX
        A(KR,KR) = S(KRMAX)
        DIV = 1.0E0/S(KRMAX)
        S(KRMAX) = S(KR)
        IF(KR.EQ.N) GO TO 10
        KRP1 = KR + 1
        DO    I=KRP1,N
          A(I,KR) = S(I)*DIV
        ENDDO                   !DO    I=KRP1,N
10    CONTINUE                !DO 10 KR=1,N
      CALL RESOLV(A,B,X,IPA,N)
      RETURN
      END
c    -------------------------------------------

      SUBROUTINE RESOLV(A,B,X, IPA, N)
C ENTRY FOR BACK SOLUTION WITH DIFFERENT B-VALUE
c

      IMPLICIT NONE

      DIMENSION B(100),X(100), A(100,100)
      DIMENSION S(100)
      DIMENSION IPA(100)

      real*8           A, B, X
      real*8           S, SUMM
      INTEGER I, IP, IP1, II, IIP1, J, IPA, N


C xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
c      CALL CPUTIM(-11)
C xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
      DO    I=1,N
        S(I) = B(I)
      ENDDO                   !DO    I=1,N
      DO    I=1,N
        IP = IPA(I)
        X(I) = S(IP)
        S(IP) = S(I)
        IF(I.EQ.N) GO TO 23
        IP1 = I + 1
        DO    J=IP1,N
          S(J) = S(J) - A(J,I)*X(I)
        ENDDO                   !DO    J=IP1,N
      ENDDO                   !DO    I=1,N
23    CONTINUE

      DO    I=1,N
        II = N + 1 - I
        SUMM = X(II)
c NOTE POSSIBLE ERROR IN THIS NEXT CHANGED LINE.
        IF(II.LT.N) THEN
          IIP1 = II + 1
          DO    J=IIP1,N
            SUMM = SUMM - A(II,J)*X(J)
          ENDDO                     !DO    J=IIP1,N
        ENDIF                      !IF(II.LT.N) THEN
        X(II) = SUMM/A(II,II)
      ENDDO                   !DO    I=1,N

      RETURN
      END
  

c ==============================================
c END

