c=============================================================================
c
c $Id: quadchem.F,v 1.3 2002/07/14 00:19:51 akinorii Exp $
c
c CODE DEVELOPER
c   Original code from Sandy Sillman, U of M.
c   IMPACT quadchem modifications:  Akinori Ito.
c   2007 IMPACT version:  Sandy Sillman
c
c FILE
c   quadchem.F
c
c ORIGINAL FILE
c   chemsolve1.f
c
c=============================================================================
c
c quadchem110109:
c  adding separate OH-HO2 vs RO2 solution for nighttime chemistry
c    (also CBMZ ro2-ro2 lumping option)
c    (also midlump correction)
c
c quadchem022609: 
c     LUMPED SPECIES change calculation: 2008 LUMP OPTION
c quadchem042808:
c    2008 corrections found through Luis Olcese aerosol version added,
c      should not change results.
c quadchem012508:
c  (NO3-N2O5 error corrected 012208)
c  (expo decay error corrected 101607)
c  Allocate/deallocate moved to quadchem_solver (101307)
c  Version with separate ONESOLVE (=chemsolve1)
c  AX, BX, CX allocated in quadchem and passed.
c  Common blocks eliminated, except rbchemmech.h (92307)
c
c FTiming statements included.
c
c ROUTINE
c   quadchem
c
c   with subroutines:  
c chemsolve, brreac, brpro, excorr, noxsolve, ohsolve, setgeom,
c  presolve, prelump, midlump, postlump, ohwrite.  and aquasolve
c
c STANDARD ARGUMENTS:  
c I/O
c   c_xcin   input concentrations (molec cm-3)
c   c_xcout  output concentrations (molec cm-3)
c   c_xcav   average concentration over time interval(molec cm-3)
c   c_xcemit emissions during time pd (molec cm-3, alreddy included in c_xcin)
c  (c_xcwdep  removal through wet deposition, molec cm-3)
c
c REACTION RATES (output)
c   c_rp    Photochemical production rates molec/cm3
c   c_rl    Photochemical loss       rates molec/cm3
c   c_rr    Photochemical reaction   rates molec/cm3
c
c RATE CONSTANTS (passed from rate calculation)
c   ratek   Rate constants (molec cm-3 s-1 or s-1)
c   rateh   Henry;s law coefficients (mole l-1 atm-1)
c   rateq   Aqueous equilibrium constants (mole lit-1)
c   rateqq  Special equilibrium constants (mole lit-1)
c   rhdif   Henry's law coeff. modified by droplet diffusion
c   egasaq  Gas-to-aqueous transfer coefficient, s-1
c
c OTHER CRITICAL I/O  (passed through common block rbchemvars.h):
c   See quadchem_solver.F.
c      c_h2ogas    H2O molec/cm3
c      c_h2oliq    LWC (gram/cm3)
c      c_lexpo     Expo decay flag, .TRUE. for remote and free troposphere.

c ADDED ARGUMENTS from Akinori Ito version:  
c    itloop = c_kvec
c    num_species = c_nchem2
c    num_active = c_nchem1
c
c    All these should have been entered through chem. mechanism read
c    and passed to this subroutine through rbchemmech.h
c
c FUTURE CHANGES/NEEDS: 
c    Bring in c_h2ogas, c_h2oliq, c_lexpo from quadchem_solver.F
c    For aqueous version - add wet deposition to quadchem_solver.F
c    Here, add flag to skip aqueous if LWC=0.
c
c
c  GOT HERE:  Next, find and add all these as arguments
c  NEW ARGUMENTS, FORMERLY IN COMMON BLOCKS:    9/21/07.
c
c In quadchem_solver: 
c  initiate h2oliq, rainfr droplet , lexpo
c     c_ohtest, c_notest, c_iter returned for written output.
c
c DELETED COMMON BLOCKS:   rbchemvars and rbchemlocal
c
c  rbchemvars: 
c    &   , c_temp,  c_h2oliq ,   c_rainfr,  c_droplet, c_lexpo
c    &   , c_ohtest, c_notest, c_iter
c  rbchemlocal
c      senshx, senhcat, 
c      oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2, xfohtest
c       ,sourcnx, sinknx
c
c  Not used in quadchem :  c_dens, c_saersa,  jparam, etc
c 
c
c
c ----------------------------------------------------------------
c  FILE:  
c  quadchem.F  (chemsolve.f)    April, 2007   
c 
c  Programs to solve for species concentrations 
c  using the RADICAL BALANCE-BACK EULER solution for photochemistry.
c
c  Full notes on development are at the end of this file  
c  and in older boxchemv7.f
c 
c  Compile for box model:  
c   boxmain.f boxpro.f chemmainbox.f cheminit.f chemrates.f quadchem.f 
c    (aquasolve.f) linslv.f jval2.f
c  Include:  chemvars.EXT, chemmech.EXT, chemlocal.EXT, boxvars.EXT
c
c  Input files: 
c     REACTION.DAT (REACTION7_GMI_AQHG06)  :  mechanism
c     TUVGRID2                             :  hv data, see jval2.f
c     fort.87  (fort.87hg_gmi_rpa)         :  concentrations+emissions
c     fort.41                              :  Run setup
c       (also REACTION7_GMI_LKS06hv with fort.87gmifull-global - update)
c           (fort.87gmi-global - GMI w/telomers. No REACTION7 yet.)
c
c     Tests:  fort.43test407.
c       Includes 12-hr expo decay (2a vs 2b), + full chem P-L cases.
c     Time test:  See quadchv7.f for comparison between versions.
c
c     Future option: nonsteady  state aqueous
c     Future option: exponential decay  solution
c     Long-term option: integrate with aerosol solution
c
c  Subroutines:
c     quadchem:  driver program for radical balance solver 
c     chemsolve: solution for individual species, pairs or groups.
c     brreac:    calculates rate for specified reaction
c     brpro:     calculates rate, assigns production and loss for reaction
c     excorr:    corrects OH production sums for 'exchange' reactions
c     noxsolve:  solution for NO, NO2 and O3
c     ohsolve:   solution for OH and HO2 (also modifies CO3-)
c     setgeom:   Sets geometric averaging based on past iterations
c     presolve:  Preliminary approximate solution for O3, NOx, OH, NO3.
c     prelump:   Sets initial concentrations for 'lumped' species
c                  and sets initial aqueous=0.
c     midlump:    Re-sets 'prior' partitioning for lumped species
c                  based on interim chemical production rates.
c     postlump:   Sums lumped species and aqueous species into gas master
c                  after calculation is complete.
c     ohwrite:    Writes details of odd hydrogen balance.
c 
c  Program  written by Sanford Sillman
c  ---------------------------------------------------------
c  Program history:
c    12/06 Initial program by Sandy Sillman based on boxchemv7.f 
c  ---------------------------------------------------------
c
c Notes:  
c Features of this solution:
c   This part of the program uses sums of gas and aqueous species
c     that are linked through Henry's law and aqueous equilibria.
c   It solves for them as a single "species", 
c     based on chemical production/loss, 
c     and leaving the gas/aq partitioning intact.
c   A separate program (aquasolve) solves for gas/aqueous partitioning.
c     (the chemistry  and gas-aqueous are solved in iterative sequence)
c
c   This solver uses the 'radical balance' approach.
c    It solves for OH and HO2 based on production/loss of radicals
c    Each radical source/sink is also assigned a sensitivity to OH.
c    OH is solved from a linear interpolation:  
c      radical sources-sinks = A + B*OH
c      (This is a fragment of the full back-Euler equation
c         with a sparse matrix)
c
c   Other species are solved individually 
c                                 in reactant-to-product sequence.
c
c      (species with mutual interactions are solved individually
c        if they interact slowly - e.g. RO2 and ROOH)
c
c   This version includes solutions for 
c    (1) interacting pairs of species (e.g. MCO3-PAN)
c    (2) "chains" of paired species (e.g. Hg0<->HgOH<->HgCl etc)
c         (format:  A<->B, A<->C, C<->C2, etc.)
c    (3) full back-Euler sub-solution for 3 or more interacting species.
c
c   Other features:  
c
c    Geometric averaging of present and prior solutions for OH:
c      the geom. averaging parameter is set based on iterative history.
c      (range 0.1-1.0 - it decreases when the iteration oscillates).
c      OPTION:  set minimum value for geometric parameter (0.1 or 0.5)
c              (0.1 for slow, sure convergence.  Max is always 1.)
c
c   Most recent corrections:
c     MULTISOLVE option for aqueous Cl-, Br-
c     Conservation-of-mass adjustment for pair groups.
c     H2O2 still has special geometric solution.
c     CO3 is linked to the radical solution, adjusted based on Hx.
c
c   Corrected problems: 
c     OH+Cl=>ClOH caused nonconvergence 
c       - fixed with multisolve and geometric averaging.
c     Hg oscillated with aqueous Hg=0:  
c       - the problem was a too-high zero protect (if xr<1e-20).
c     EXCORR can cause errors if it links two NON-PAIRED species.
c       - the excorr species must always be solved as pairs.
c
c  Full notes (including TIME TESTS and IMPROVEMENTS)
c     are in quadchv7.f
c 
c --------------------END PROGRAM INFORMATION---------------------
      subroutine quadchem
     &   (c_xcin, c_xcout, c_xcav, c_xcemit,
c    &                                        c_xcwdep,
     &    c_rp, c_rl, c_rr, 
     &   xcfinr,
     &    ratek, rateh, rateq, rateqq, rhdif, egasaq
     &   , c_temp,  c_h2oliq ,   c_rainfr,  c_droplet, c_lexpo
     &   , c_ohtest, c_notest, c_iter

     &    ,xc, xclastq
     &    ,cpm, cpro, history, geomavg, rlm, rlmulti, rloss, rloss1
     &    ,rlpair, rpm, rpmulti, rppair, rpro, rpro1, rrp, rrl
     &    ,xrm, xrp, xrppair, xrr, xrrm, xrrpair, AX, BX, XX
     &   )
c
c
c This is the driver program 
c   for the radical-balance back-Euler solution for photochemistry.
c
c It calls a preliminary partitioning (prelump), 
c Establishes a preliminary solution (presolve)
c And then sets up a standard iteration.
c
c The iteration includes:
c    aquasolve: program to solve for gas-aquoues partitioning 
c    chemsolve:solution for individual species or for species pairs
c      called in reactant-to-product order.
c    ohsolve:  radical balance solution for OH and HO2.
c
c Input and output to the program operate through the common blocks
c  (chemvars.EXT and chemmech.EXT)
c
c Input and output are processed by chemmain (file chemmainbox)
c
c Called by:    chemmain, now quadchem_solver - Do_Quad_Solver
c Calls to: 
c           chemrates   - set rate constants
c           hvrates     - set hv rate constants
c           prelump     - initial partition of lumped and gs-aqueous sp.
c           presolve    - initial solution for OH
c           midlump     - interim partition of lumped species.
c           aquasolve   - solution for gas-aqueous partitioning
c           chemsolve   - solution for individual species and groups
c           ohsolve     - solution for OH/HO2
c           postlump    - sum lumped species into lump sum, 
c                          and aqueous into gas-master.
c 
c ---
c
c  Program  written by Sanford Sillman
c  ---------------------------------------------------------
c  Program history:
c    12/06 Initial program by Sandy Sillman based on boxchemv7.f 
c ---------------------------------------------------------------
c
c      use Ftiming_Dao
      
      use h_imp_dims
      use h_imp_control
      use h_rbchemmech
c      use h_rbchemvars
c      use h_rbchemarrays
      use h_rbchemlocal, only: avogadrl
c      use h_rbchemlocarr
      IMPLICIT NONE

c#     include "imp_dims.h"
c#     include "imp_control.h"
c#     include "rbchemmech.h"
c#     include "rbchemvars.h"  
c#     include "rbchemarrays.h"  
c#     include "rbchemlocal.h"

c#     include "rbchemlocarr.h"
c
c  LOCAL VARIABLES
c
c  ncsol(c_cdim)      Species list passed in call to chemsolve

      integer :: ncsol(c_cdim)

c     integer :: iter            ! c_iter in rbchemmech.h, iter in locarr

c     real*8  :: xcfinr(c_kvec, c_cdim)          ! ratio FINAL/AVG cncn
c --below are originally from rbchemarrys---
       real*8 ::  c_xcin( c_kvec,c_cdim) ! concentr molec/cm3
       real*8 ::  c_xcout( c_kvec,c_cdim) ! concentr, mol/cm3
       real*8 ::  c_xcav( c_kvec,c_cdim)  ! concentr, mol/cm3
       real*8 ::  c_xcemit( c_kvec,c_cdim) ! emis, molec/cm3
      real*8 ::  c_rp(c_kvec,c_cdim)  ! Production, molec/cm3
      real*8 ::  c_rl(c_kvec,c_cdim)  ! Loss, molec/cm3
      real*8 ::  c_rr(c_kvec,c_rdim)  ! Reaction rate m/cm3
      real*8 :: xcfinr( c_kvec,c_cdim) ! ratio FINAL/AVG cncn
      real*8 :: ratek(c_kvec,c_rdim)  ! Rate constants
      real*8 :: rateh(c_kvec,c_hdim)  ! Henry's law constants
      real*8 :: rateq(c_kvec,c_aqdim)  ! Aqueous eq. constants
      real*8 :: rateqq(c_kvec,c_aqqdim) ! Special eq. constants
      real*8 :: rhdif(c_kvec,c_hdim) ! H. mod by diffusion
      real*8 :: egasaq(c_kvec,c_hdim) ! Gas-to-aq transfer s-1
      real*8 ::  c_temp(c_kvec)    ! temperature K
      real*8 ::  c_h2oliq(c_kvec)   ! LWC grams cm-3
      real*8 ::  c_rainfr(c_kvec)   ! rainout fraction
      real*8 ::  c_droplet(c_kvec)        ! droplet radius, cm
      logical :: c_lexpo(c_kvec)        ! Flag for expo decay solution
      real*8 ::  c_ohtest      ! test: dOH/OH or dOH/HO2
      real*8 ::  c_notest      ! test: dNO2/NO2
      integer :: c_iter                  ! chem. number of iterations
      real*8 :: xc( c_kvec,c_cdim) ! concentration molec/cm3
      real*8 :: xclastq( c_kvec,c_cdim) ! conc molec/cm3

      real*8 :: cpm(c_kvec, c_cdim, c_cdim) !Cross-prod for multi
      real*8 :: cpro(c_kvec,c_cdim, c_cdim)   ! Cross-production 
c 
      real*8 :: history(c_kvec,c_cdim,400)  ! Cncn for each iter
      real*8 :: geomavg(c_kvec,c_cdim)   ! Geom. avg factor
c
      real*8 :: rlm(c_kvec,c_cdim)     ! Loss for multi-spec 
      real*8 :: rlmulti(c_kvec       ) ! Loss group sum for multi
      real*8 :: rloss(c_kvec,c_cdim)   ! Solver loss rate
      real*8 :: rloss1(c_kvec,c_cdim)  ! Interim rloss w/out cpro
      real*8 :: rlpair(c_kvec,c_cdim)  ! Loss rate for pair grp
      real*8 :: rpm(c_kvec,c_cdim)     ! Production rate for multi
      real*8 :: rpmulti(c_kvec       ) !Product'n multi group sum
      real*8 :: rppair(c_kvec,c_cdim)  ! Production for pair grp
      real*8 :: rpro(c_kvec,c_cdim)    ! Solver production rate
      real*8 :: rpro1(c_kvec,c_cdim)   ! Interim rpro w/out cpro 
      real*8 :: rrp(c_kvec,c_cdim)     ! Interim production rate 
      real*8 :: rrl(c_kvec,c_cdim)     ! Interim loss rate 
      real*8 :: xrm(c_kvec,c_cdim)     !  Initial conc. for multi
      real*8 :: xrp(c_kvec,c_cdim)     !Solver prior concentration
      real*8 :: xrppair(c_kvec)        ! Solver prior pair group cn.
      real*8 :: xrr(c_kvec,c_cdim)     ! Solver concentration
      real*8 :: xrrm(c_kvec,c_cdim)    !Initial solution for multi
      real*8 :: xrrpair(c_kvec)        ! Solution for pair group sum
      real*8 :: BX(100), XX(100), AX(100,100) ! For back_Eulersolution AX=B

!---------------below are local variables----------------------
      real*8 ::        oddhdel(c_kvec,2)   ! Summed Hx sens to Oh,
c   mol/cm3
      real*8 ::        oddhloh(c_kvec,2)   ! Hx loss from OH,  mol/cm3
      real*8 ::        oddhlho2(c_kvec,2)   ! Hx loss from HO2,  mol/cm3
      real*8 ::        oddhsrc(c_kvec,2)   ! Summed P Hx, mol/cm3
      real*8 ::        oddhsum(c_kvec,2)   ! Summed net P/L Hx, mol/cm3
      real*8 ::        senshx(c_kvec,2)   ! Hx sens: d ln(pHx)/d
c       ln([OH])
      real*8 ::        senhcat( c_kvec,99) ! Hx sens. by species categ.

      real*8 :: sourcnx(c_kvec)   ! Summed NOx source, mol/cm3
      real*8 :: sinknx(c_kvec)    ! Summed NOx sink, mol/cm3

      real*8 :: xfohtest          ! OH/HO2 convergence test
c-------below are local vars  

c ---------------------------------------------------------------
c DECLARE AND  ALLOCATE LOCAL ARRAYS.
c   These will be passed to all subroutines
c    and are included in 'chemlocarr.h'

c     ,xc, xclastq
c     ,cpm, cpro, history, geomavg, rlm, rlmulti, rloss, rloss1
c     ,rlpair, rpm, rpmulti, rppair, rpro, rpro1, rrp, rrl
c     ,xrm, xrp, xrppair, xrr, zrrm, xrrpair, AX, BX, XX

c MOVED TO quadchem_solver

c ---------------------------------------------------------------
          real*8 :: stoicx                         ! stoichiometry sum
      integer :: ic,ic1,ic2,ic3,icc,ics,ics2,icc1,icc2,icc3 !Chem index
      integer :: is,iss,iscs,isc2            !Chem local
      integer :: ich,icq,icx,icx1,icx2,icy1,icy2,icp,icp1,icp2 !Chem
      integer :: icr1,icr2,icr,isr1,isr2,icpair  !chem index - pair and
      integer :: ionsum                                  ! aquasolve ion
      integer :: ica1,ica2,icb1,icb2,nra1,nra2,nrb1,nrb2 !Aquasolve chem
      integer :: neq                                     !Aqueous
      integer :: nc,nc1,nc2,ncc,ncf,nn,nne               !Chem species
      integer :: nr,nr1,nr2,nrh,nrq,nrqq,nrx,np          ! Reaction
      integer :: icat1,icat2,icatp,icatp2  ! indices used for species
      integer :: kk, kw                            ! Vectorization
      integer :: i, j, k, ii, ij, iii, n           ! General counters
       integer :: nsol           ! Number of pair groups being solved
      integer :: nsolv          ! Number of species being solved for
          real*8 :: alpha(c_kvec)   ! General vector variable
      real*8 ::  beta(c_kvec)   ! General vector variable
      real*8 :: gamma(c_kvec)   ! General vector variable
      real*8 :: prior(c_kvec)   ! Prior species conc (molec/cm3)

      if (pr_diag) then
        Write (6,*) 'Quadchem called by ', loc_proc
        Write (43,*) 'Quadchem called by ', loc_proc
      end if

      kk = 1

c PRELIMINARY CALLS:  BEFORE SOLUTION
c   Note, must also call the initial read + process at the start.
c     chemread, hvread and cheminit

c OPTION:  CALL HERE TO SET REACTION RATE AND HV CONSTANTS.
c   (can be here or in chemmain1.f)
c
c  SET REACTION RATE CONSTANTS
c                          call chemrates
c        &   (ratek, rateh, rateq, rateqq)
c
c SET PHOTOLYSIS RATE CONSTANTS
c                          call hvrates   (ratek)
c
c (end OPTION)

c
c SET H2O CONCENTRATION IN CHEMISTRY ARRAY FROM INPUT VARIABLE
c   CUT - DONE IN quadchem_solver.F
c     if(c_nh2o.gt.0) then
c vec                      do kk=1,c_kvec
c      xc( kk, c_nh2o) = c_h2ogas(kk)
c vec                      enddo ! do kk=1,c_kvec
c     endif                  !if(c_nh2o.gt.0) then
c
c
c PRELUMP:  SETS 'PRIOR' CONCENTRATIONS (XXO) AND LUMPED SPECIES

c      if (do_ftiming) call Ftiming_On  ('prelump')

      call prelump
     &   (c_xcin, c_xcout, c_xcav, c_xcemit,
     &    c_rp, c_rl, c_rr ,
     &    xc,xcfinr,  xclastq ,
     &    rrp, rrl, rppair, rlpair, geomavg, c_rainfr) 
c      if (do_ftiming) call Ftiming_Off  ('prelump')


c PRELIMINARY SOLUTION FOR OH, HO2, O3, NOx.  : 
c Solution uses the following reactions:  
c       #1:  NO2+hv->NO+O3          #2: NO+O3->NO2
c       #9:  O3+hv->2OH             #12: NO2+OH->HNO3
c       #16: OH+CO->HO2             #17: OH+O3->HO2
c       #18: HO2+NO->OH+NO2         #22: HO2+HO2->H2O2
c       #46: OH+CH4->HO2
c 
c      if (do_ftiming) call Ftiming_On  ('presolve')

       call presolve
     &    (c_xcin, xc, ratek)
c      if (do_ftiming) call Ftiming_Off  ('presolve')

c ---------------------------------------------------------------
c  BEGIN ITERATIVE RUNS
c ---------------------------------------------------------------
      do 1700 c_iter = 1,c_numitr


      if(c_kkw.gt.0) then
         write(c_out,1901)    c_iter
1901     format(//,70('-'),/,'BEGIN ITERATION =',3i3)
         write(c_out,1902)
1902     format(70('-'),/)
       endif

c RE-SET PARTITIONING OF LUMPED SPECIES BASED ON CHEM. PRODUCTION.
      if(c_iter.ge.2) then

c     if (do_ftiming) call Ftiming_On  ('midlump')

          call midlump (c_xcin, xc, c_rp, c_rl,c_iter)
c     if (do_ftiming) call Ftiming_Off  ('midlump')

       endif

c CALL AQEOUS CHEMISTRY SOLVER FROM WITHIN ITERATION
c          (NON VECTOR VERSION - call only if LWC nonzero)
c     call aquasolve
      if(c_h2oliq( 1).ge.1.0e-20) then
c       if (do_ftiming) call Ftiming_On  ('aquasolve')
       call aquasolve

     &   (c_xcin, c_xcout, c_xcav, c_xcemit,
     &    c_rp, c_rl, c_rr ,
     &    xc,xcfinr,  xclastq, 
     &    ratek, rateh, rateq, rateqq, rhdif, egasaq, 

     &     xrp, xrr, rpro, rloss, rpro1, rloss1, 
     &       rrp, rrl, cpro,
     &       xrm, xrrm, rpm, rlm,cpm, rpmulti, rlmulti,
     &       rppair, rlpair, xrppair, xrrpair, 
     &       history, geomavg 
     &    , c_temp,  c_h2oliq ,  c_droplet, c_iter)
c 
c       if (do_ftiming) call Ftiming_Off  ('aquasolve')
      end if

c SET SENHCAT FOR FIRST ITERATION. (sensitivity to change in OH).
c INITIAL SENHCAT = 1 FOR ODD HYDROGEN; 0 FOR OTHER SPECIES

      if(c_iter.eq.1) then
        do ii=1,30
         is = 0
c 2009 modification - add category 8
         if((ii.eq.8.or.ii.eq.9.or.ii.eq.10).or.ii.eq.3) is=1

c           if(c_kkw.gt.0) write(c_out,*) ii, is

c                              do  kk=1,c_kmax ! kk vector loop
          senhcat(kk,ii) =   float(is)
c                              enddo ! kk vector loop
        enddo   ! ii
       if(c_kkw.gt.0) write(c_out,1611) (senhcat(c_kkw,ii),ii=1,20)
     *     ,(senhcat(c_kkw,ii),ii=31,40)
      endif

c  WRITE SENHCAT
       if(c_kkw.gt.0) write(c_out,1611) (senhcat(c_kkw,ii),ii=1,20)
     *     ,(senhcat(c_kkw,ii),ii=31,40)
1611  format(/,'TEST SENHCAT 1-20,31-40:',/,(10f6.3))

c ZERO ACCUMULATED SUMS AT START OF EACH CASCADE LOOP: 
c   oddhsum = summed net source/sink for odd hydrogen radicals
c   oddhdel = sum of each net source/sink of oddh 
c                multiplied by sensitivity to OH concentration.
c   oddhsrc = summed source only
c   oddhloh = summed loss of OH
c   oddhlho2 = summed loss of HO2 (removal)
c   sourcnx = summed NOx source
c   sinknx = summed NOx sink
c
c   rrp, rrl = running sum of production, loss of each species
c              not set to zero here - zero when spec. is solved.
c
c   c_rr  reaction rates.  
c              
c ALSO,  PRESERVE PRIOR CONCENTRATION HERE 
c xclastq =PRIOR BUT AFTER AQUASOL)

      do ic=1,c_nchem2
c                                  do  kk=1,c_kmax ! kk vector loop
c        rrp( kk ,ic) = 0.
c        rrl( kk ,ic) = 0.
         xclastq(kk,ic) = xc(kk,ic)
c                                  enddo ! kk vector loop
      enddo           !do ic=1,c_nchem2

c                                  do  kk=1,c_kmax ! kk vector loop
c 2009 modification of arrays
        do i=1,2
       oddhsum(kk,i) = 0.
       oddhsrc(kk,i) = 0.
       oddhdel(kk,i) = 0.
       oddhloh(kk,i) = 0.
       oddhlho2(kk,i) = 0.
        enddo
       sourcnx(kk) = 0.
       sinknx(kk) = 0.
c                                  enddo ! kk vector loop

c     do 1651 nr=1,c_nreac
c                                  do  kk=1,c_kmax ! kk vector loop
c       c_rr(kk,nr)=0.
c                                  enddo ! kk vector loop
c 1651  continue

c -------------------------------------------
c CASCADE SOLUTION for individual species or species groups:
c  SPECIES CALL IS SET BY ORDERED ARRAY c_cascade ->ncsol
c -------------------------------------------
c
c 
c REGULAR DEBUG WRITE - OPTION
c        if(c_kkw.gt.0.and.c_iter.eq.1) write(c_out,1663)
c c        if(c_kkw.gt.0              ) write(c_out,1663)
c 1663     format(/,' CASCADE:  CHEMICAL INDICES:')


c LOOP TO CALL SPECIES SOLUTION
c      if (do_ftiming) call Ftiming_On   ('chemsolve-1loop')

       do 1660 i=1,c_nchem2
       if(c_cascade(i,1).eq.0) go to 1661
       do ic=1,nsdim
         ncsol(ic) = c_cascade(i,ic)
       enddo

c  SECOND DUPLICATE RPRO LOOP  to test loop time
c  WITH DUMMY BRPRO2 - WAS HERE, CUT (see TRY8c)

c REGULAR DEBUG WRITE - OPTION
c        if(c_kkw.gt.0.and.c_iter.eq.1)  then
c             write(c_out,*) (ncsol(ic),ic=1,3), c_tchem(ncsol(1))
c        endif



       if  (ncsol(2).eq.0.and.ncsol(3).eq.0.and.
     &       c_nppair(ncsol(1),3) .eq. 0)                then

       call chemsolve1(ncsol, 
     &    c_xcin, c_xcout, c_xcav, c_xcemit,
     &    c_rp, c_rl, c_rr ,
     &    xc,xcfinr,  xclastq, 
     &    ratek, rateh, rateq, rateqq, rhdif, egasaq, 

     &     xrp, xrr, rpro, rloss, rpro1, rloss1, 
     &       rrp, rrl, cpro,
     &       xrm, xrrm, rpm, rlm,cpm, rpmulti, rlmulti,
     &       rppair, rlpair, xrppair, xrrpair, 
     &       history, geomavg 

     &        , senshx, senhcat
     &        , oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2, xfohtest
     &        ,sourcnx, sinknx, c_notest

     &     ,  c_h2oliq ,   c_rainfr,  c_lexpo, c_iter
     &        )


             else

       call chemsolve(ncsol, 
     &    c_xcin, c_xcout, c_xcav, c_xcemit,
     &    c_rp, c_rl, c_rr ,
     &    xc,xcfinr,  xclastq, 
     &    ratek, rateh, rateq, rateqq, rhdif, egasaq, 

     &     xrp, xrr, rpro, rloss, rpro1, rloss1, 
     &       rrp, rrl, cpro,
     &       xrm, xrrm, rpm, rlm,cpm, rpmulti, rlmulti, AX, BX, XX,
     &       rppair, rlpair, xrppair, xrrpair, 
     &       history, geomavg 

     &        , senshx, senhcat
     &        , oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2, xfohtest
     &        ,sourcnx, sinknx, c_notest

     &     ,  c_h2oliq ,   c_rainfr,  c_lexpo, c_iter
     &        )

               endif
c
c 

1660   continue
1661   continue
c END CASCADE LOOP 

c      if (do_ftiming) call Ftiming_Off  ('chemsolve-1loop')

c -------------------------------------------
c
c -----------------------------
c SPECIAL TREATMENT FOR NO3-N2O5 - HNO3
c -----------------------------
c
c NO3-N2O5 is solved normally, as paired species
c HNO3 is solved first, then NO3-N2O5.
c  (pair declaration is hard-wired in chemread)
c
c IT IS 'SPECIAL' ONLY BECAUSE IT REQUIRES A SPECIAL CATEGORY
C NO3 REACTS WITH MOST SPECIES; SO SPECIES MUST NOT BE ASSIGNED 
c TO NO3.
c

      ic1=c_nno3
      ic2=c_nn2o5
      ic3=c_nhno3

      do ic=2,nsdim
        ncsol(ic) = 0
      enddo
      ncsol(1) = ic3

c      if (do_ftiming) call Ftiming_On  ('chemsolve-2')
      call chemsolve1(ncsol, 
     &    c_xcin, c_xcout, c_xcav, c_xcemit,
     &    c_rp, c_rl, c_rr ,
     &    xc,xcfinr,  xclastq, 
     &    ratek, rateh, rateq, rateqq, rhdif, egasaq, 

     &     xrp, xrr, rpro, rloss, rpro1, rloss1, 
     &       rrp, rrl, cpro,
     &       xrm, xrrm, rpm, rlm,cpm, rpmulti, rlmulti,
     &       rppair, rlpair, xrppair, xrrpair, 
     &       history, geomavg 

     &        , senshx, senhcat
     &        , oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2, xfohtest
     &        ,sourcnx, sinknx, c_notest

     &     ,  c_h2oliq ,   c_rainfr,  c_lexpo, c_iter
     &        )


c
c      if (do_ftiming) call Ftiming_Off  ('chemsolve-2')
c 

      ncsol(1) = ic1
c      if (do_ftiming) call Ftiming_On  ('chemsolve-3')
      call chemsolve(ncsol, 
     &    c_xcin, c_xcout, c_xcav, c_xcemit,
     &    c_rp, c_rl, c_rr ,
     &    xc,xcfinr,  xclastq, 
     &    ratek, rateh, rateq, rateqq, rhdif, egasaq, 

     &     xrp, xrr, rpro, rloss, rpro1, rloss1, 
     &       rrp, rrl, cpro,
     &       xrm, xrrm, rpm, rlm,cpm, rpmulti, rlmulti, AX, BX, XX,
     &       rppair, rlpair, xrppair, xrrpair, 
     &       history, geomavg 

     &        , senshx, senhcat
     &        , oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2, xfohtest
     &        ,sourcnx, sinknx, c_notest

     &     ,  c_h2oliq ,   c_rainfr,  c_lexpo, c_iter
     &        )


c 
c      if (do_ftiming) call Ftiming_Off  ('chemsolve-3')
c -------------------------------------------
c SPECIAL TREATMENT FOR NOX-OX AND ODD HYDROGEN FAMILIES
c -------------------------------------------
c O3-NOx SPECIAL SOLUTION:
c SIMULTANEOUS FOR THREE SPECIES, USING OX AND NOX RELATIONSHIPS.
c THIS IS INCLUDED IN SUBROUTINE 'NOXSOLVE', INVOKED BY CHEMSOLVE.
c
c TO USE, INVOKE CHEMSOLVE(IC1,IC2,IC3) IN PRECISE ORDER: O3, NO2, NO.
c IDENTIFIED BY SPECIAL SPECIES INDEX
c THE KEY REACTIONS O3+NO->NO2, NO2+hv->NO+O3 ARE SUMMED IN CPRO.
c
c WRITE
c      if(c_kkw.gt.0) write(c_out,221) xc(c_kkw,1),xc(c_kkw,2),xc(c_kkw,3)
c 221    format('BEFORE-NOXSOLVE: O3 NO2 NO=',3(1pe10.3))

      do ic=2,nsdim
        ncsol(ic) = 0
      enddo
 
      ncsol(1)=c_no3
      ncsol(2)=c_nno2

      ncsol(3)=c_nno 
c      if (do_ftiming) call Ftiming_On  ('chemsolve-4')
      call chemsolve(ncsol, 
     &    c_xcin, c_xcout, c_xcav, c_xcemit,
     &    c_rp, c_rl, c_rr ,
     &    xc,xcfinr,  xclastq, 
     &    ratek, rateh, rateq, rateqq, rhdif, egasaq, 

     &     xrp, xrr, rpro, rloss, rpro1, rloss1, 
     &       rrp, rrl, cpro,
     &       xrm, xrrm, rpm, rlm,cpm, rpmulti, rlmulti, AX, BX, XX,
     &       rppair, rlpair, xrppair, xrrpair, 
     &       history, geomavg 

     &        , senshx, senhcat
     &        , oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2, xfohtest
     &        ,sourcnx, sinknx, c_notest

     &     ,  c_h2oliq ,   c_rainfr,  c_lexpo, c_iter
     &        )

c
c 

c      if (do_ftiming) call Ftiming_Off  ('chemsolve-4')
c ---------------------
c ODD HYDROGEN SOLUTION: 
c ---------------------
c   CALL SUBROUTINE OHSOLVE(OH,HO2,H2O2) TO SOLVE FOR OH, HO2.
c   SPECIES IDENTIFIED BY SPECIAL  INDEX

c      if (do_ftiming) call Ftiming_On  ('ohsolve')
      call ohsolve(c_noh, c_nho2, c_nh2o2,
     &    c_xcin, c_xcout, c_xcav, c_xcemit,
     &    c_rp, c_rl, c_rr ,
     &    xc,xcfinr,  xclastq, 
     &    ratek, rateh, rateq, rateqq, rhdif, egasaq, 
     &     xrp, xrr, rpro, rloss, rpro1, rloss1, 
     &       rrp, rrl, cpro,
     &       xrm, xrrm, rpm, rlm,cpm, rpmulti, rlmulti, AX, BX, XX,
     &       rppair, rlpair, xrppair, xrrpair, 
     &       history, geomavg   
     &        , senshx, senhcat, c_ohtest
     &        , oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2, xfohtest
     &        ,sourcnx, sinknx, c_notest
     &     ,  c_h2oliq ,   c_rainfr,  c_lexpo, c_iter
     &        )

c 
c      if (do_ftiming) call Ftiming_Off  ('ohsolve')
c  (computer alternative)
c     ic9=c_noh
c     ic10=c_nho2
c     ic11=c_nh2o2
c     call ohsolve(ic9,ic10,ic11)


c --------------------------------------
c END OF ITERATION.  WRITE, TEST AND EXIT.
C --------------------------------------

c -------------------------------------------------------
c  SUMMARY ITERATIVE WRITE:  THIS SECTION (ALONG WITH OHWRITE)
c  CAN BE COPIED AND USED OUTSIDE QUADCHEM. (see CHEMWRIT)
c -------------------------------------------------------
c IF LPRT OPTION (KKW>0) WRITE FULL SUMMARY FOR ITERATION.
c (WRITES FOR NOX AND HX ARE IN NOXSOLVE AND OHSOLVE)
c 
c QUADCHEM ITERATIVE WRITE:  
c NOTE:  WRITE GAS+AQUEOUS SUM HERE,  AND LIST OF REACTIONS.
c        GAS-AQUEOUS PARTITIONING ITER SUMMARY is in AQUASOLVE.

      if(c_kkw.gt.0) then
        write(c_out,1801) c_iter, c_kkw
1801    format(//,' SUMMARY WRITE:  ITER =',i3, '    VECTOR KKW=',i3)
c       write(c_out,1804) c_IDATE, c_hour,  c_lat(1), c_lon(1),c_temp(1)
1804   format('IDATE xhour lat lon temp=',i8,4f8.2)
c       write(c_out,1805) (c_jparam(i),i=1,13),c_jparam(20)
1805   format('JPARAMS: zenith altmid dobson so2 no2 aaerx aerssa',
     *  ' albedo',/,'cld-below cld-above claltB claltA temp date',
     *   /,(8f10.3))
        write(c_out,1802)
1802    format(/,' #  IC     XCOUT     XCIN  XCF/AV       RL',
     *              '        RP        dXC      dR')
1803     format(i4,a8,2(1pe10.3),0pf7.3,1x,(4(1pe10.3)))

c OPTION:  ORIGINAL WRITE (all species)
c       do 1810 j=1,c_nchem2
c        write(c_out,1803) j, c_tchem(j),xc(c_kkw,j),  c_xcin(c_kkw,j),
c    *       c_rl(c_kkw,j), c_rp(c_kkw,j)
c 1810    continue

c OPTION:  WRITE GAS-AQUEOUS SUMS
c   MAKE XR,  RP, RL=sum ; write gas-masters only


        do j=1,c_nchem2
          if(c_npequil(j).eq.j) then
            xrr(c_kkw,1) = 0.
            rpro(c_kkw,1) = 0.
            rloss(c_kkw,1) = 0.
            do  neq=1,(c_nequil(  j)+1)
              ic=j
              if(neq.gt.1) ic = c_ncequil(  j,(neq-1))
              alpha( c_kkw) = 1.
              if(neq.gt.1) alpha( 1)= c_h2oliq( 1)*avogadrl
              rloss(c_kkw,1) = rloss(c_kkw,1)+c_rl(c_kkw,ic)
              rpro(c_kkw,1)  = rpro(c_kkw,1) +c_rp(c_kkw,ic)
              xrr(c_kkw,1)   = xrr(c_kkw,1)  +xc(c_kkw,ic)*alpha(c_kkw)
            enddo             !do  neq=1,(c_nequil(  j)+1)
            beta(1)  = xrr(c_kkw,1) -   c_xcin(c_kkw,j)
            gamma(1) = rpro(c_kkw,1) - rloss(c_kkw,1)
            write(c_out,1803) j, c_tchem(j)
     *       ,xrr(c_kkw,1),  c_xcin(c_kkw,j), xcfinr(c_kkw,j)
     *       ,rloss(c_kkw,1), rpro(c_kkw,1)
     *        ,beta(1), gamma(1)
c    *       ,geomavg(c_kkw,j)
          endif             !if(c_npequil(j).eq.j) then
        enddo            !do j=1,c_nchem2


c OPTION - INCLUDE REACTIONS
        write(c_out,1806) c_iter,c_kkw
1806    format(//,' REACTION RATES:  ITER =',i3, '    VECTOR KKW=',i3)
         do      nr=1,c_nreac
          write(c_out,1807) nr, (c_treac(j,nr),j=1,5), c_rr(c_kkw ,nr),
     *    ratek(c_kkw ,nr)
1807      format(i4,2x,a8,'+',a8,'=>',a8,'+',a8,'+',a8,2(1pe10.3))
         enddo


      endif
c END SUMMARY WRITE
c --------------------------------------------------------

c IF KMAX=1, USE CONTROLS TO TEST FOR CONVERGENCE AND EXIT.
c IF VECTORIZED (KMAX>1), NO CONTROLLED EXIT
c   NOTE NIGHTTIME ADDED EXIT ABOVE: XOHTEST,FOHTEST*0.001.

      if(c_kkw.gt.0) write(c_out,1701) c_iter,c_ohtest,c_notest
1701  format(/' TEST FOR OH,NO CONVERGENCE =',i3,2(1pe10.3))

        if(c_kmax.eq.1) then
          if(ratek(1,  1 ).ge.1.00E-04.and.c_ohtest.lt.xfohtest) 
     *     c_ohtest=xfohtest

           if(c_iter.gt.3.and.(c_ohtest.lt.c_converge.and.
     *               c_notest.lt.c_converge))
     *          go to 1710

c ADDED EXIT: IF NOX CONVERGES AND OH IS LOW:
c REPLACED BY LOWERING XOHTEST, FOHTEST.
c  NOTE, THIS REQUIRES HARD-WIRED RATEK(1), XR(OH).
c          if(c_iter.gt.3.and.ratek(1,  1).lt.0.001.and.
c    *       (xc( 1 , 9).lt.0.100E+06 .and.c_notest.lt.c_converge))
c    *          go to 1710

        endif

1700   continue
1710   continue
c -----------------------
c END ITERATIVE LOOP
c -----------------------


c POSTLUMP:  SUM LUMPED SPECIES INTO LUMP-SUM AND CONVERT CONCENTRATIONS
C TO LUMP-PARTITION FRACTIONS.  (Also sum Aq-Equil species?)

c      if (do_ftiming) call Ftiming_On  ('postlump')
      call postlump
     &   (c_xcin, c_xcout, c_xcav, c_xcemit,
c    &                                        c_xcwdep,
     &    c_rp, c_rl, c_rr ,
     &    xc,xcfinr,  c_h2oliq, c_rainfr )
c 
c      if (do_ftiming) call Ftiming_Off  ('postlump')

c DEALLOCATE AT END OF QUADCHEM
c   moved to quadchem-solver


      if (pr_diag) then
        Write (6,*) 'Finished Quadchem called by ', loc_proc
      end if

c END QUADCHEM
2000  return
      end
c -----------------------------------------------------------------
c
c ----------------------------------------------------
c SOLVER SUBROUTINES - SOLVE FOR INDIVIDUAL SPECIES CONCENTRATIONS 
c ----------------------------------------------------
c
c
c NEW ADDED CHEMSOLVE1  - ONESOLVE ONLY  
          subroutine chemsolve1(ncsol, 
     &    c_xcin, c_xcout, c_xcav, c_xcemit,
     &    c_rp, c_rl, c_rr ,
     &    xc,xcfinr,  xclastq, 
     &    ratek, rateh, rateq, rateqq, rhdif, egasaq, 

     &     xrp, xrr, rpro, rloss, rpro1, rloss1, 
     &       rrp, rrl, cpro,
     &       xrm, xrrm, rpm, rlm,cpm, rpmulti, rlmulti,
     &       rppair, rlpair, xrppair, xrrpair, 
     &       history, geomavg 

     &        , senshx, senhcat
     &        , oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2, xfohtest
     &        ,sourcnx, sinknx, c_notest

     &     ,  c_h2oliq ,   c_rainfr,  c_lexpo, c_iter
     &        )

c SHORTER ARGUMENT LIST
c    &    c_xcin, c_xcout, c_xcav, c_xcemit,
c    &    c_rp, c_rl, c_rr ,
c    &    xc,xcfinr,  xclastq, 
c    &    ratek, rateh, rateq, rateqq, rhdif, egasaq, 

c    &     xrp, xrr, rpro, rloss, rpro1, rloss1, 
c    &       rrp, rrl,      
c    &       rppair, rlpair, 
c    &       history, geomavg 

c    &        , senshx, senhcat
c    &        , oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2, xfohtest
c    &        ,sourcnx, sinknx, c_notest

c    &     ,  c_h2oliq ,   c_rainfr,  c_lexpo, c_iter
c    &        )

c 

c Special solution for concentrations of O3, NO2 and NO.

c     FUTURE DO:  back-Euler solution for multi-species
c
c This is the main solver for individual species concentrations.
c It includes:
c (1) A solution for an individual species - 
c      (regarded as a sum of gas + aqueous linked through equilibria)
c 
c (2a) A solution for a pair of rapidly interacting species
c
c (2b) A solution for a 'chain' of linked pairs
c                                       (A<->B, A<->C1, C1<->C2, etc.)
c
c (3) A reverse-Euler solution for a group of 3+  interacting species.
c      ('MULTISOLVE')
c
c (4) Call to special solution for O3-NO2-NO ('NOXSOLVE')
c
c It also establishes rates for reactions associated with the species;
c And calculates summed production, loss of reaction products,
c  and odd hydrogen radical sums.
c 
c  FOR A SINGLE SPECIES OR PAIR, MAKE IC2<=0. 
c
c THE SOLUTION SEQUENCE:  
c  (a) Calculate losses/rates of reactions tht affect the species
c      (including linked gas+aqueous equilibrium species)
c  (b) Solve for species concentrations
c        with separate solutions for:
c         SINGLE SPECIES
c         PAIR AND PAIR CHAIN
c         NOXSOLVE
c         MULTISOLVE
c  (c) Add production of product species and odd-h radicals 
c       to the production sums.
c
c  Input: Prior species concentrations, reactions and rate constants.
c  Output:  Updated species concentrations (xc).
c
c Called by:  quadchem.
c Calls to:  
c     brpro - sums production and loss from reaction
c     brreac - calculates reaction rates.
c     setgeom - establish geometric mean parameter for iteration
c     noxsolve - solution for O3-NO-NO2
c     LINSLV - matrix iteration for MULTISOLVE solution.
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c
c -------------------------------------------------------------------
c ----------------------------------------------------------------
c
c  PREVIOUS:  FOR TWO SPECIES THE 'EXCHANGE' SPECIES MUST COME FIRST.
c  NO LONGER.  EXCHANGE IS NOT DEPENDENT ON ORDER-OF-SPECIES.
c  (ONLY NOXSOLVE USES RPRO, RLOSS, CPRO FROM CHEMSOLVE 
c    IN A SPECIFIC ORDER.  EXCORR IS NOT CALLED FOR NOX.)  
c ----------------------------------------------------------------
c
      use h_rbchemmech
c      use h_rbchemvars
c      use h_rbchemarrays
      use h_rbchemlocal, only: avogadrl
c      use h_rbchemlocarr

       IMPLICIT NONE


c#     include "rbchemmech.h"
c#     include "rbchemvars.h"
c#     include "rbchemarrays.h"
c#     include "rbchemlocal.h"
c#     include "rbchemlocarr.h"
c 
c SHORTER LOCAL LIST:  
c  LOCAL VARIABLES - INPUT/OUTPUT
      integer :: ncsol(c_cdim)       ! Species list passed to chemsolve

c     logical :: llexpo(c_kvec)       ! Flag for exponential decay solution
c     logical :: doneexpo(c_kvec)     ! Flag for completion of expo decay.

c LOCAL VARIABLES:  INTERNAL

c     logical :: lself               ! flag for self-reaction
c     real*8 :: rself(c_kvec,20)     ! Rate of self-reaction
c END SHORT LIST 

c FULL LIST 
c  LOCAL VARIABLES - INPUT/OUTPUT
c      integer :: ncsol(c_cdim)       ! Species list passed to chemsolve
c --below are originally from rbchemarrys---
       real*8 ::  c_xcin( c_kvec,c_cdim) ! concentr molec/cm3
       real*8 ::  c_xcout( c_kvec,c_cdim) ! concentr, mol/cm3
       real*8 ::  c_xcav( c_kvec,c_cdim)  ! concentr, mol/cm3
       real*8 ::  c_xcemit( c_kvec,c_cdim) ! emis, molec/cm3
      real*8 ::  c_rp(c_kvec,c_cdim)  ! Production, molec/cm3
      real*8 ::  c_rl(c_kvec,c_cdim)  ! Loss, molec/cm3
      real*8 ::  c_rr(c_kvec,c_rdim)  ! Reaction rate m/cm3
        real*8 :: xc( c_kvec,c_cdim) ! concentration molec/cm3
       real*8 :: xcfinr( c_kvec,c_cdim) ! ratio FINAL/AVG cncn
       real*8 :: xclastq( c_kvec,c_cdim) ! conc molec/cm3
      real*8 :: ratek(c_kvec,c_rdim)  ! Rate constants
      real*8 :: rateh(c_kvec,c_hdim)  ! Henry's law constants
      real*8 :: rateq(c_kvec,c_aqdim)  ! Aqueous eq. constants
      real*8 :: rateqq(c_kvec,c_aqqdim) ! Special eq. constants
      real*8 :: rhdif(c_kvec,c_hdim) ! H. mod by diffusion
      real*8 :: egasaq(c_kvec,c_hdim) ! Gas-to-aq transfer s-1
      real*8 :: xrp(c_kvec,c_cdim)     !Solver prior concentration
      real*8 :: xrr(c_kvec,c_cdim)     ! Solver concentration
      real*8 :: rpro(c_kvec,c_cdim)    ! Solver production rate
      real*8 :: rloss(c_kvec,c_cdim)   ! Solver loss rate
      real*8 :: rpro1(c_kvec,c_cdim)   ! Interim rpro w/out cpro 
      real*8 :: rloss1(c_kvec,c_cdim)  ! Interim rloss w/out cpro
      real*8 :: rrp(c_kvec,c_cdim)     ! Interim production rate 
      real*8 :: rrl(c_kvec,c_cdim)     ! Interim loss rate 
      real*8 :: cpro(c_kvec,c_cdim, c_cdim)   ! Cross-production 
      real*8 :: xrm(c_kvec,c_cdim)     !  Initial conc. for multi
      real*8 :: xrrm(c_kvec,c_cdim)    !Initial solution for multi
      real*8 :: rpm(c_kvec,c_cdim)     ! Production rate for multi
      real*8 :: rlm(c_kvec,c_cdim)     ! Loss for multi-spec 
      real*8 :: cpm(c_kvec, c_cdim, c_cdim) !Cross-prod for multi
      real*8 :: rpmulti(c_kvec       ) !Product'n multi group sum
      real*8 :: rlmulti(c_kvec       ) ! Loss group sum for multi
      real*8 :: rppair(c_kvec,c_cdim)  ! Production for pair grp
      real*8 :: rlpair(c_kvec,c_cdim)  ! Loss rate for pair grp
      real*8 :: xrppair(c_kvec)        ! Solver prior pair group cn.
      real*8 :: xrrpair(c_kvec)        ! Solution for pair group sum
      real*8 :: history(c_kvec,c_cdim,400)  ! Cncn for each iter
      real*8 :: geomavg(c_kvec,c_cdim)   ! Geom. avg factor
      real*8 ::        senshx(c_kvec,2)   ! Hx sens: d ln(pHx)/d
      real*8 ::        senhcat( c_kvec,99) ! Hx sens. by species categ.
      real*8 ::        oddhsum(c_kvec,2)   ! Summed net P/L Hx, mol/cm3
      real*8 ::        oddhdel(c_kvec,2)   ! Summed Hx sens to Oh,
      real*8 ::        oddhsrc(c_kvec,2)   ! Summed P Hx, mol/cm3
      real*8 ::        oddhloh(c_kvec,2)   ! Hx loss from OH,  mol/cm3
      real*8 ::        oddhlho2(c_kvec,2)   ! Hx loss from HO2,  mol/cm3
      real*8 :: xfohtest          ! OH/HO2 convergence test 
      real*8 :: sourcnx(c_kvec)   ! Summed NOx source, mol/cm3
      real*8 :: sinknx(c_kvec)    ! Summed NOx sink, mol/cm3
      real*8 ::  c_notest      ! test: dNO2/NO2
      real*8 ::  c_h2oliq(c_kvec)   ! LWC grams cm-3
      real*8 ::  c_rainfr(c_kvec)   ! rainout fraction
      logical :: c_lexpo(c_kvec)        ! Flag for expo decay solution
      integer :: c_iter                  ! chem. number of iterations
      
!----Guangxing Lin-----below are local variables 
      real*8 ::  c_droplet(c_kvec)        ! droplet radius, cm
      real*8 ::  c_ohtest      ! test: dOH/OH or dOH/HO2
c below are from rbchemlcoal

c   mol/cm3
c       ln([OH])



      logical :: llexpo(c_kvec)       ! Flag for exponential decay solution
      logical :: doneexpo(c_kvec)     ! Flag for completion of expo decay.

c     DOUBLE PRECISION :: BX(100), XX(100), AX(100,100) ! For back_Eulersolution AX=B
c 3b CHANGE HERE 
c     real*8, allocatable ::  BX(:)     ! For back_Eulersolution AX=B
c     real*8, allocatable ::  XX(:)     ! For back_Eulersolution AX=B
c     real*8, allocatable ::  AX(:,:)   ! For back_Eulersolution AX=B
       real*8 :: stoicx                         ! stoichiometry sum
      integer :: ic,ic1,ic2,ic3,icc,ics,ics2,icc1,icc2,icc3 !Chem index
      integer :: is,iss,iscs,isc2                    !Chem local index
          integer :: ich,icq,icx,icx1,icx2,icy1,icy2,icp,icp1,icp2 !Chem
      integer :: icr1,icr2,icr,isr1,isr2,icpair  !chem index - pair and
      integer :: ionsum                                  ! aquasolve ion
      integer :: ica1,ica2,icb1,icb2,nra1,nra2,nrb1,nrb2 !Aquasolve chem
      integer :: neq                                     !Aqueous
      integer :: nc,nc1,nc2,ncc,ncf,nn,nne               !Chem species
      integer :: nr,nr1,nr2,nrh,nrq,nrqq,nrx,np          ! Reaction
      integer :: icat1,icat2,icatp,icatp2  ! indices used for species
      integer :: kk, kw                            ! Vectorization
      integer :: i, j, k, ii, ij, iii, n           ! General counters
       integer :: nsol           ! Number of pair groups being solved
      integer :: nsolv          ! Number of species being solved for
          real*8 :: alpha(c_kvec)   ! General vector variable
      real*8 ::  beta(c_kvec)   ! General vector variable
      real*8 :: gamma(c_kvec)   ! General vector variable
      real*8 :: prior(c_kvec)   ! Prior species conc (molec/cm3)
      integer :: ncsolv(c_cdim) ! Species number (ic) for species in
      integer :: nssolv(c_cdim) ! Pair group number (is) for spec. in

c LOCAL VARIABLES:  INTERNAL
      real*8 :: xpair       ! Summed factor for pair group 
c                        net pair group  production w/ cons. mass.
      real*8 :: xmulti      ! Summed factor for multi group 
c                        net multi group  production w/ cons. mass.
c
      real*8 ::  rself(c_kvec, 20)        ! Rate of self-reaction
c
      logical :: lself               ! flag for self-reaction


c END FULL LIST. NOTE, DEALLOCATE AT END

c
c ----------------------------------------------------------------

       kk=1


c PRELIMINARY:  SELF-REACTION FLAG
       lself = .FALSE.

c PRELIMINARY:  make ic point to GAS-MASTER and PRIMARY OF PAIRED SPECIES
c   and count nsol = number of active species groups solved for.
c
       nc = 1
         ic=ncsol(nc)
         if(ic.gt.0) then
           ncsol(nc) = c_nppair(c_npequil(ic), 2) 
           nsol = nc
         else
            return
         endif                !if(ic.gt.0) then
c

c ---------------------------------------------
c SUMMATION OF RLOSS - CHEMICAL LOSSES AND CROSS-PRODUCTS
c ---------------------------------------------
c
c  -----------------------
c  PRELIMINARY:  ESTABLISH LIST OF LINKED SPECIES
c  -----------------------
c  nsol = number of pair groups to be solved for (set above)
c  ncsol(n) = ic of head of each pair group to be solved for (input above)
c  nsolv = number of species to be solved for 
c          (includes pairs and multi-solve)
c
c ncsolv(i) = ic of each species in order (ncsol(1) and paired, ncsol(2), etc)
c
c nssolv(i) = is of each species = multi-species group for ncsolv
c
c Note:  loop over nsolv automatically solves for multiple-species groups 
c         as well as single pair grouping



c PRELIMINARY:  FLAG FOR EXPONENTIAL DECAY SOLUTION
c   Use expo decay only for single species solution (unpaired, no aq.) 
c   (FUTURE DO.  Not included yet.)


c       do  kk=1,c_kmax           ! kk vector loop
          llexpo(kk)=.FALSE.
          if(c_lexpo(kk)) then
c          if( (nsolv.eq.1).and.(c_nequil(ncsol(1)).eq.0)
          if(                  (c_nequil(ncsol(1)).eq.0)
     *          .and. (c_tchem(ncsol(1)).ne.'    H2O2')
     *          .and. (c_icat(ncsol(1)).eq.1.or.c_icat(ncsol(1)).eq.2)
     *          .and.(.not.c_lsts(ncsol(1)))      )  llexpo(kk)=.TRUE.
          endif                      !if(c_lexpo) then
c       enddo                   ! kk vector loop
c

c ===============================
c ONESOLVE - PART OF ONESOLVE/FULSOLVE LOOP - FOR SINGLE NONPAIRED SPECIES
c ===============================

               nc = 1
               nsolv = 1
               ncsolv(1) = ncsol(1) 

c PRELIMINARY: CALCULATE REACTION RATES,RP and RL  FOR PRODUCT REACTIONS
c ASSOCIATED WITH SPECIES (i.e. that are sources, not sinks)
c
c      do nc=1, nsolv
         ics = ncsolv(nc)

         if(c_nnrchp(ics).gt.0) then
           do     i=1,c_nnrchp(ics)
             nr = c_nrchmp(ics,i)
             call brpro(nr, ratek,
     &         c_rr, xc, rrl, rrp, rppair, rlpair, c_h2oliq
     &        , senshx, senhcat
     &        , oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2, xfohtest
     &        ,sourcnx, sinknx
     &        )


c CHEMSOLVE DEBUG WRITE
c             if(c_kkw.gt.0.   and.nsol.gt.1) then
c c           if(c_kkw.gt.0.   and.c_tchem(ics).eq.'    HCHO') then
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) then
c               write(c_out,1001) nr,(c_treac(j,nr),j=1,5),ics
c    *         , c_tchem(ics), c_oddhx(nr,1), c_pronox(nr)
c             endif

           enddo                 !do     i=1,c_nnrchp(ics)
         endif                !if(c_nnrchp(ics).gt.0) then
c      enddo               !do nc=1, nsolv
c

c
c ZERO RPRO, RLOSS, XRP, CPRO.  (XRM, XRRM for multisolve) (
c      (RPPAIR, RLPAIR, XRPPAIR, XRRPAIR for pair group sums)
c
c          do i=1,nsolv 
           i=1
c           do  kk=1,c_kmax           ! kk vector loop
             rpro(kk,i) =  0.
             rloss(kk,i) = 0.
             xrp(kk,i) = 0.
             rself(kk,i) = 0.
c           enddo                   ! kk vector loop
c          enddo



c  -----------------
c  LOOP TO CALCULATE CHEM. LOSS RATES AND CROSS-PRODUCTS
c  -----------------

c      do nc=1, nsolv 
c        ics = ncsolv(nc)
c
c ESTABLISH RPRO, RLOSS AND XRP FOR THE 'BASE' SPECIES
c        do kk=1,c_kmax      ! kk vector loop
          rpro(kk,nc) = rpro(kk,nc)+  rrp(kk, ics) +   c_xcin(kk,ics)
          rloss(kk,nc) =rloss(kk,nc)+  rrl(kk, ics) 
          xrp(kk,nc) =  xrp(kk,nc) +  xc(kk,ics)
c        enddo               ! kk vector loop

c TEMPORARY CHEMSOLVE DEBUG WRITE
c        if(c_kkw.gt.0   .and.nsol.gt.1)
c c      if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3)
c    *      write(c_out, 91) c_tchem(ics), rrp(c_kkw,ics),
c    *        c_xcin(c_kkw,ics), rpro(c_kkw, nc)
c    *      ,rrl(c_kkw,ics), rloss(c_kkw, nc)
c 91       format(' START CHEMSOLV: IC RRP XXO RPRO RRL RLOSS= ',a8,/,
c    *      5(1pe10.3))

c ADD ALL AQUEOUS-EQUILIBRIUM SPECIES INTO SUMMED RPRO, RLOSS AND XRP.
c CONVERTING AQUEOUS INTO GAS UNITS (AVOGADRL)
c Also include RAINOUT as loss.

         if(c_nequil(ics).gt.0)       then

           do  neq=1,c_nequil(ics)
             ic = c_ncequil(ics,neq)
c            do kk=1,c_kmax    ! kk vector loop
               xrp(kk,nc) =   xrp(kk,nc) 
     *            + xc(kk,ic)*c_h2oliq(kk)*avogadrl
               rpro(kk,nc) = rpro(kk,nc) + rrp(kk,ic)
               rloss(kk,nc) = rloss(kk,nc)+rrl(kk,ic)
     *            +c_rainfr(kk) * xc(kk,ic)*c_h2oliq(kk)*avogadrl
c            enddo           ! kk vector loop

c TEMPORARY CHEMSOLVE DEBUG WRITE
c            if(c_kkw.gt.0.and.nsol.gt.1) 
c c           if(c_kkw.gt.0.   and.c_tchem(ics).eq.'    HCHO') 
c c          if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) 
c    *          write(c_out, 91) c_tchem( ic), rrp(c_kkw, ic),
c    *            c_xcin(c_kkw, ic), rpro(c_kkw, nc)
c    *      ,rrl(c_kkw,ic), rloss(c_kkw, nc)

           enddo         ! do neq=1,c_nequil(ics)
         endif
c END EQUILIBRIUM SUMMATION.  

c CHEMSOLVE DEBUG WRITE
c        if(c_kkw.gt.0  .and.nsol.gt.1) 
c c      if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) 
c    *      write(c_out,105) c_tchem(ics), 
c    *      xrp(c_kkw,nc), rpro(c_kkw,nc), rloss(c_kkw,nc)
c 105      format(' START CHEMSOLVE: SPECIES, XRP, RPRO, RLOSS=',
c    *            /,a8,2x,3(1pe10.3))
c
c ADD INITIAL SPECIES XRP, RPRO, RLOSS TO XRPPAIR, RPPAIR, RLPAIR
c  IF PAIR GROUP HAS >3 MEMBERS AND NO MULTISOLVE (FOR NORMALIZATION)
c   CUT 


c ADD INITIAL SPECIES XRP, RPRO, RLOSS TO XRM, RPM, RLM: for MULTISOLVE
c  (note:  no PAIRFAC, different from PAIR NORMALIZATION)
c CUT 

c ADD LOSS REACTIONS AND CROSS-PRODUCT REACTIONS TO RLOSS,RPRO, CPRO.
c
c ALSO SET SELF-REACTION FLAG (lself)
c
c ALSO SET FLAG FOR SECOND REACTANT ALSO INCLUDED IN SPECIES LIST (nc2)
c     (pair or multisolve.  Important for NO+O3->NO2.)


         if(             c_nnrchem(ics).gt.0) then

c SET INDICES
c FEBRUARY 2005 CHANGE - icr1=npequil is before if(icr2.gt.0) BUG FIX.
           do  i=1,c_nnrchem(ics)
             nr = c_nrchem(ics,i)
             icr1 = c_reactant(nr,1)
             icr2 = c_reactant(nr,2)
             icr1 = c_npequil(icr1)
             nc2  = 0            ! index to count loss from 2nd reactant
             if(icr2.gt.0) then
               icr2 = c_npequil(icr2)
               if(icr1.eq.ics.and.icr2.eq.ics) lself=.TRUE.
c  FEB 2005OPTION:  
c     IF REACTANTS ARE IN SAME PAIR GROUP, SET SELF-REACTION FLAG
c
c  FUTURE DO, FIX SELF-REACTION.
c     (  In SELF-REACTION, do not set 2nd reactant - it messes up CPRO
c          unless CPRO algorithm changed. )
c
               if(c_nppair(icr1,2).eq.c_nppair(icr2,2)) lself =.TRUE.
               do ncc=1,nsolv

                 if(ncc.ne.nc) then
                   if(ncsolv(ncc).eq.icr1.or.ncsolv(ncc).eq.icr2) then
                               nc2=ncc
                               ics2 = ncsolv(ncc)
                   endif
                 endif                 !if(ncc.ne.nc) then
               enddo                !do ncc=1,nsolv
             endif                !if(icr2.gt.0) then

c CHEMSOLVE DEBUG WRITE  
c c            if(c_kkw.gt.0   .and.nsol.gt.1) then
c            if(c_kkw.gt.0                 ) then
c c          if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) then
c              write(c_out,1001) nr,(c_treac(j,nr),j=1,5)
c    *         , ics,c_tchem(ics) , c_oddhx(nr,1), c_pronox(nr)
c 1001    format('REACTION #',i4,': ',a8,'+',a8,'=>',a8,',',a8,',',a8,/,
c    *     ' KEY SPECIES =',i4,2x,a8,'  ODD-H=',f8.2,'  PRONOX=',f8.2)
c            endif


c CALL BRREAC - to establish reaction rate
             call brreac(nr,
     &         c_rr, xc, ratek, c_h2oliq )

c ADD LOSS FOR FIRST REACTANT
c            do kk=1,c_kmax      ! kk vector loop
c              rloss(kk,nc) = rloss(kk,nc) + c_rr(kk,nr)*c_stoiloss(nr)
               rloss(kk,nc) = rloss(kk,nc) + c_rr(kk,nr)
c            enddo            ! kk vector loop

c  stoiloss OPTION:  
c IF SELF-REACTION: ADD AGAIN.  (2nd reactant index is zero)

             if(icr1.eq.icr2) then
c              do kk=1,c_kmax      ! kk vector loop
                 rloss(kk,nc) = rloss(kk,nc) + c_rr(kk,nr)
                 rself(kk,nc) = rself(kk,nc) + 2.*c_rr(kk,nr)
c              enddo            ! kk vector loop
             endif              ! if(icr1.eq.icr2) then

c CHEMSOLVE DEBUG WRITE
c c            if(c_kkw.gt.0  .and.nsol.gt.1) 
c            if(c_kkw.gt.0                ) 
c c          if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) 
c    *        write(c_out,108) c_tchem(ics), 
c    *          rloss(c_kkw,nc), c_rr(c_kkw,nr)
c 108          format(' CHEMSOLVE LOSS FOR MAIN SPECIES:  IC = ',
c    *           a8,/,
c    *           '    RLOSS , RR = ',2(1pe10.3))
 

c SECOND SPECIES OPTION:
c ADD LOSS OF 2ND REACTANT FOR DIFFERENT SPECIES. (for NO+O3->NO2).
c ADD TO RLOSS1 ALSO (which may  have been written before 
c    - RLOSS1 only excludes CPRO and PSEUDO-SOLUTION.
c
c NOTE: omit stoiloss, which adjusts only for main species 
c       appearing in reactant and product. 

c
             if(nc2.gt.0) then
c              do kk=1,c_kmax      ! kk vector loop
                 rloss(kk,nc2) = rloss(kk,nc2) + c_rr(kk,nr)
                 rloss1(kk,nc2)=rloss1(kk,nc2) + c_rr(kk,nr)
c              enddo            ! kk vector loop

c  CHEMSOLVE DEBUG WRITE
c c              if(c_kkw.gt.0  .and.nsol.gt.1) 
c              if(c_kkw.gt.0                ) 
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) 
c    *          write(c_out, 109) c_tchem(ics),
c    c           c_tchem(icr2), rloss(c_kkw,nc2), c_rr(c_kkw,nr)
c 109            format(' CHEMSOLVE LOSS FOR SECOND SPECIES: IC, IC2 =',
c    *           a8,2x,a8,/,
c    *           '    RLOSS2, RR = ',2(1pe10.3))

             endif            !if(nc2.gt.0) then

c ADD PRODUCTS TO CPRO or RPRO.
c CPRO MATRIX  (kk,reactant,product).  (note prodarr=gas-master product)
c

c            do ncc=1,nsolv 
             ncc=1
               if( c_prodarr(nr,ncsolv(ncc)).ne.0.) then
                icc = ncsolv(ncc)

c Adjust LOSS if PRODUCT EQUALS REACTANT, skip PRO.
                 stoicx = c_prodarr(nr,ncsolv(ncc))
                 if(stoicx.gt.1.) stoicx=1.
                 rloss(kk,ncc)=rloss(kk,ncc) - c_rr(kk,nr)*stoicx
                 if(ncc.eq.nc2) 
     *            rloss1(kk,ncc)=rloss1(kk,ncc) - c_rr(kk,nr)*stoicx


               endif          !if( c_prodarr(nr,ncsolv(ncc)).ne.0.) then
 


           enddo  ! do  i=1,c_nnrchem(ics)
         endif    !  c_nnrchem(ics).gt.0
c END - ADD LOSS REACTIONS AND CROSS-PRODUCTS

c

c NONSTEADY STATE ADJUSTMENT FOR MULTISOLVE SUM
c   (ahead of STEADY STATE ADJUSTMENT to use RLOSS=0)
c



c NONSTEADY STATE ADJUSTMENT:  IF NONSTEADY STATE OR ZERO RLOSS, ADD XRP.
c  (EQUIVALENT TO RL/XR + 1).
c  (NOTE:  STEADY-STATE OPTION STILL INCLUDES XXO IN RPRO.
c   FOR STEADY STATE XXO SHOULD EQUAL EITHER EMISSIONS OR ZERO.)
c    (XXO SET IN PRELUMP)

c WITH ZERO PROTECT FOR RLOSS


c        do kk=1,c_kmax      ! kk vector loop
          if(.not.c_lsts(ics).or.rloss(kk,nc).eq.0) 
     *          rloss(kk,nc) = rloss(kk,nc) + xrp(kk,nc)
          if(rloss(kk,nc).le.0.) rloss(kk,nc) = 1.0e-08
c        enddo                       ! kk vector loop

c  (OPTION:  FOR A SELF-REACTION (stoiloss>1, e.g. HO2+HO2, stoiloss=2.)
c   AN EXACT LINEARIZED NR SOLUTION WOULD REQUIRE 
c   RLOSS=+2*RR*STOILOSS, RPRO=+RR*STOILOSS. (d/dR = twice loss rate.)
c   TO IMPLEMENT THIS, IF STOILOSS>1, ADD ADDITIONAL RR*STOILOSS
c   TO BOTH RLOSS AND RPRO, WITHIN LOOP 125 (and 225 for twosolve).

c   This is cut because IT MAY SCREW HO2+HO2 REACTIONS.  
c  )

c  SAVE RLOSS1, RPRO1 
c     =RPRO without CPRO; RLOSS w/o internal solution adjustments.

c        do kk=1,c_kmax      ! kk vector loop
           rloss1(kk,nc) = rloss(kk,nc)
           rpro1(kk,nc) = rpro(kk,nc)
c        enddo                    ! kk vector loop

c      enddo       !  do nc=1, nsolv 

c  ---------------------
c  END LOOP TO CALCULATE CHEM. LOSS RATES AND CROSS-PRODUCTS
c  ---------------------

c SET SELF-REACTION FLAG:  false if self-reaction is insignificant
c       (ADDED 408 APRIL 2008)
c       (note future option:  save history and use to prevent oscill.)
c       (note - this prevents NO3-N2O5 error, but hard-wired NO3 retained)
c  LSELF OPTION: 2009 CORRECTION - skip for iter=1)
c
c     if(lself) then
c     if(lself.and.c_iter.gt.2) then
      if(lself.and.c_iter.gt.1) then
        lself=.FALSE.
        do nc=1,nsolv
         if(rself(kk,nc).gt.0.33*rloss1(kk,nc)) lself=.TRUE.

c TEMPORARY DEBUG WRITE 408
c        ics = ncsolv(nc)
c       if(c_kkw.gt.0) write(c_out,*)
c    &    'RSELF: tchem, ics, nc, rloss, rself, lself',
c    &      c_tchem(ncsolv(nc)), ncsolv(nc),nc,rloss1(kk,nc),
c    &      rself  (kk,nc), lself

        enddo             !do nc=1,nsolv
      endif             !if(lself) then
c END-  SET SELF-REACTION FLAG
c


c ---------------------------------------------
c END - SUMMATION OF RLOSS - CHEMICAL LOSSES AND CROSS-PRODUCTS
c ---------------------------------------------

c ---------------------------------------------
c CALCULATE SPECIES CONCENTRATIONS:  INTERNAL PAIRS
c ---------------------------------------------
c
c  THE ALGORITHM:  
c   For two interacting species:  A<->B
c   Back-Euler solution:  xr = xrprior * (Pi + CjiPj/Lj)/(Li-CijCji/Lj)
c                            = xrprior * (   Pi'       )/(  Li'      )
c     where Li = prior absol. loss rate + xrprior (non-steady state)
c           Pi = prior production + c_xcin
c           Cji = cross production of i from j
c           Pi', Li' = pseudo-P, L with cross prod'n from paired subspecies.
c
c     (neg. not possible since Li>Cij, Lj>Cji)
c
c   For a chain of interacting species:  A<->B, A<->C, C<->C2, etc.
c     in order from primary to subspecies:  Cij=0 for j<i
c     CjiPj, etc. is implicit sum over all paired subspecies j
c     Solve from sub-species to primary species Pi'/Li'    
c        where Pi', Li' includes Cji, j>i
c     Then solve from primary Xi to subspecies Xj 
c      with  Cij from primary Xi updated and added to Pj
c
c   This effectively uses a prior secondary partitioning (C<->C2)
c    to generate primary partitioning (A<->B, A<->C) and primary XR
c    It uses primary (A) to solve for secondary (B, C, etc.)
c      while preserving C<->C2 partitioning (implicit in Pi'/Li')
c
c   FOR MULTISOLVE/NOXSOLVE (STANDARD OPTION):
c    PAIR SOLUTION is normalized to keep original within-pair sum unchanged.
c    NOXSOLVE or MULTISOLVE changes pair sums (preserving internal partition)
c     based on interaction between pair groups.

c   OPTION for NOXSOLVE/MULTSOLVE:  
c    should the PAIR SOLUTION include cross-production from other groups?
c    LOSS includes CP to other group.
c    Either subtract prior Cij from Li, or add prior Cij to Lj
c     (ABOVE)
c ------------------------------------

c ---------
c LOOP TO  ADJUST RP, RL FOR SUB-SPECIES CROSS PRODUCTION (PSEUDO-RP, RL)
c  CUT 
c ---------
c  LOOP TO SOLVE SPECIES CONCENTRATIONS- INTERNAL PAIRS
c ---------
c   Order:  primary-to-subspecies chain among paired species
c   After calculation, add modified CPRO to CP-down-species.
c

c      do nc=1, nsolv
       nc=1
         ics = ncsolv(nc)
         is=nssolv(nc)

c   BACKWARD EULER SOLUTION FOR SINGLE SPECIES 
c    The true equation is =RP/(RL/XR); modified to avoid second divide.
c     (skip from here - move lower)

c c                      do  kk=1,c_kmax      ! kk vector loop
c        if(.not.llexpo(kk).or.c_iter.le.2) then
c          xrr(kk, nc)= xrp(kk,nc)*rpro(kk,nc)/rloss(kk,nc)
c        endif       !if(.not.llexpo) then
c c                      enddo            ! kk vector loop
c
c  EXPONENTIAL DECAY OPTION:  SINGLE SPECIES ONLY.

c  EQUATION:  dc/dt= r-kc, c=Co at t=0. (r=rp/time +emission rate)
c  Ct = r/k + (Co - r/k) exp(-kt)
c  Cavg = r/k + ((Co-r/k)/kt)*(1- exp(-kt))    =  (Co-Cf)/kt + r/k
c       Cf= Co + rt - kt*Cav    --checks
c
c Here, Co = xcin - xcemit
c       rt = rpro-xcin+xcemit
c       kt = (rloss-xrp)/xrp
c
c  Result saved as:
c       xc = Cavg (used for reaction rate calculations)
c        (xrr = Cavg, then turned into Cavg/Cprior 'NEW/OLD' below,,
c                 and used to get xc gas,aq -as in standard back-Euler.)
c       xcfinr = Cf/Cavg  (used in postlump to get xcout)
c       NOTE: xcfinr=1 in BACK-EULER solution.

c ERROR - Original used ic, not ics in xcfinr(ic) with xcin, xcemit.
c         Corrected
c
c ERROR OCCURS when xrp = 1e-1, xcin =1e-4 after re-set lumped species.
c
c CORRECTIONS:  llexpo only for iter>2.  llexpo only for rates >1.
c                correction - to skip if xfinr is wild
c  


c                      do  kk=1,c_kmax      ! kk vector loop
         doneexpo(kk) = .FALSE. 
         if(llexpo(kk)                ) then

           if(xrp(kk,nc).gt.0
     *        .and. ( rloss(kk,nc)-xrp(kk,nc) .gt.0. )
     *        .and. ( rloss(kk,nc)+rpro(kk,nc).gt.1. )
     *                                       ) then

c    alpha = kt =(rloss-xrp)/xrp
           alpha(kk) = rloss(kk,nc)/xrp(kk,nc) - 1.
c
c    beta = rt/kt = (rpro-xcin+xcemit)/alpha 
           beta(kk) = (rpro(kk,nc)-c_xcin(kk,ics)+c_xcemit(kk,ics)) 
     *               /alpha(kk)


c    Cf =   r/k + (    Co     - r/k) exp(-kt)
c    Cf =  beta + (xcin-xcemit-beta)*exp(-alpha)
c    NOTE: This must be greater than (xcin-xcemit)
c
c ORIGINAL ERROR - USED ic NOT ics HERE AND BELOW: 
c    xcfinr=, xrr =, xcfinr =, xcfinr =

c ORIGINAL ALGORITHM - large number numerical error
c            xcfinr(kk,ics) = beta(kk) 
c    *          +(c_xcin(kk,ics) - c_xcemit(kk,ics) - beta(kk))
c    *           *exp(0.-alpha(kk))

c FIXED ALGORITHM
             xcfinr(kk,ics) = beta(kk) * (1.- exp(0.-alpha(kk)) )
     *          +(c_xcin(kk,ics) - c_xcemit(kk,ics)           )
     *           *exp(0.-alpha(kk))

c    Cav = (    Co     -Cf)/kt    + r/k
c    Cav = (xcin-xcemit-Cf)/alpha + beta
c
c
             xrr(kk,nc) = beta(kk)
     *          +( c_xcin(kk,ics) - c_xcemit(kk,ics) - xcfinr(kk,ics))
     *          /alpha(kk)
c
c Cf/Cav  - with ZERO PROTECT
             if(xrr(kk,nc).gt.0) then
                xcfinr(kk,ics) = xcfinr(kk,ics)/xrr(kk,nc)

c Done-expo flag. (within ZERO PROTECT loop)
c  OPTION:  protect against wild values
                doneexpo(kk) = .TRUE. 
c               if(xcfinr(kk,ics).lt.10.and.xcfinr(kk,ics).gt.0.1) 
c    *             doneexpo(kk) = .TRUE. 

             endif     !if(xrr(kk,nc).gt.0) then

c SPECIAL CHEMSOLVE DEBUG WRITE - LLEXPO SOLVER
c          if(c_kkw.gt.0. and. 
c    *         (c_tchem(ic).eq.'    R3N1'.or.
c    *          c_tchem(ics).eq.'    R3N1')        ) then
c            write(c_out,251) rloss(c_kkw, nc), xrp(c_kkw,nc), 
c    *                                                alpha(c_kkw)
c 251          format(/,'LLEXPO DEBUG: rloss, xrp, alpha=rl/xrp-1:',/,
c    *         (8(1pe10.3))   )
c            write(c_out,254) rpro (c_kkw, nc), c_xcin(c_kkw,ics), 
c    *            c_xcemit(c_kkw,ics),               beta(c_kkw)
c            write(c_out,*) ic, ics, c_tchem(ic), c_tchem(ics)
c            write(c_out,*) c_iter
c 254          format(/,'LLEXPO DEBUG: rpro,xcin, xcemit, beta=rt/kt',/,
c    *         (8(1pe10.3))   )
c            write(c_out,256)  xcfinr(c_kkw,ic) 
c 256          format(/,'XCFINR = ', 1pe10.3)
c          endif    !if(c_kkw.gt.0.)
c END DEBUG WRITE

c          else            !if(alpha(kk).gt.0) then

           endif           !if(xrp(kk,nc).gt.0) then

         endif       !if(.not.llexpo) then

c   BACKWARD EULER SOLUTION FOR SINGLE SPECIES 
c    The true equation is =RP/(RL/XR); modified to avoid second divide.

c  Use Back-Euler solution if option is selected, or as fallback if expo fails.
c  Note:  ZERO PROTECT above for rloss.  

         if(.not.doneexpo(kk)) then 
            xrr(kk, nc)= xrp(kk,nc)*rpro(kk,nc)/rloss(kk,nc)
            xcfinr(kk,ics) = 1.

c TEMPORARY DEBUG WRITE                          
c          if(c_kkw.gt.0. and.c_tchem(ics).eq.'    R3N1')  then
c             write(c_out,262) 
c             write(c_out,*)c_iter, c_tchem(ics), doneexpo(c_kkw) 
c 262           format(/, 'LLEXPO DEBUG: BACK EULER SOLUTION FOR: ')
c          endif

         endif                 !if(.not.doneexpo(kk) then 

c                      enddo            ! kk vector loop



c    H2O2 SPECIAL OPTION (for AQUEOUS CO3 NONCONVERGENCE)
c      MAXIMUM H2O2 = PRIOR H2O2 + ODDHSRC
c      TO USE THIS, NEED TO SUM ODDHSRC (sources only) ALONG WITH ODDHSUM. 
c 2009 modification
c 
         if(c_tchem(ics).eq.'    H2O2') then
c          do kk=1,c_kmax      ! kk vector loop
             if(xrr(kk,  nc).gt.oddhsrc(kk,1)+  c_xcin(kk,ics)) then
                xrr(kk, nc)  =oddhsrc(kk,1)+  c_xcin(kk,ics)

c ZERO PROTECT CORRECTION 52004 - cut, should never be zero
c  --> if it  goes below 1e-30 can be zero!!!
c           if(xrr(kk, 1).lt.1.0e-30) xrr(kk,1)=1.0e-30
             endif
c (THIS LINE REPLACED WITH SECTION IMMEDIATELY BELOW.)
c            xrr(kk, nc) =(xrr(kk,nc) **0.70) * (xrp(kk,nc)) **0.30
c          enddo       ! kk vector loop
         endif       !if(c_tchem(ics).eq.'    H2O2')

c   ***DIFFICULT CONVERGENCE OPTION***   
c      GEOMETRIC MEAN (0.7 or 0.5) IF FLAGGED.

c   CONVERGENCE OPTION (June 2009): geom avg for first 1-3 iterations
c                                  (but not for  NO3)   ==>NOT USED.
c             (test with fort.87cohc3, cohc_u, rb_hg_rpai)
c             (note: lself correction above is necessary)

c   FLAG FOR:  H2O2   and SELF-REACTION
c   (Self-reaction flag for SO5- +SO5- as well as H2O2).
c
c   If CO3, H2O2 here - slow convergence. Else, CO3-H2O2 small oscillation
c     (changed:  CO3 linked to Hx. H2O2 remains here)

c SPECIAL FIX FOR NO3, N2O5(1/08):  set lself=F;  these must not be solved
c    with geometric avg - messes up NOx
c

           if(c_tchem(ics).eq.'     NO3'
     &        .or.c_tchem(ics).eq.'    N2O5' ) lself = .FALSE.
           if(ics.eq.4.or.ics.eq.5) lself = .FALSE.

c temporary debug write
         if(c_kkw.gt.0.and.(ics.eq.4.or.ics.eq.5))
     &      write(6    ,*) "lcs, lself= ", ics, c_tchem(ics), lself

         if(lself
     *      .or. c_tchem(ics).eq.'    H2O2'    ! always include
c
c FAILED OPTIONS
c    *      .or. c_tchem(ics).eq.'     CLG'    !  fails
c    *      .or. c_tchem(ics).eq.'    HCLG'
c    *      .or. c_tchem(ics).eq.'    HCL2'
c    *      .or. c_tchem(ics).eq.'     CLO'
c    *      .or. c_tchem(ics).eq.'     NO3'    ! fails
c    *      .or. c_tchem(ics).eq.'    N2O5'    ! fails
c    *      .or. c_tchem(ics).eq.'     CO3'
c    *      .or. c_tchem(ncsol(1)).eq.'    HBRG'
     *                                            ) then
c          do kk=1,c_kmax      ! kk vector loop
c             xrr(kk, nc) =(xrr(kk,nc) **0.85) * (xrp(kk,nc)) **0.15
c             xrr(kk, nc) =(xrr(kk,nc) **0.70) * (xrp(kk,nc)) **0.30
              xrr(kk, nc) =(xrr(kk,nc) **0.50) * (xrp(kk,nc)) **0.50
c          enddo        ! kk vector loop
         endif       ! lself

c DIFFICULT CONVERGENCE OPTION:  SETGEO IF FLAGGED
c   FLAG FOR H2O2 - EITHER ABOVE (AUTOMATIC) OR HERE
c   FAILS

c        if(c_tchem(ics).eq.'    xxxx' 
c    *                                            ) then
c          if(c_iter.ge.4) then
c            call setgeom(ics,  geomavg, history, c_iter)
c          endif               !if(c_iter.ge.4) then
c          do kk=1, c_kmax          ! kk vector loop
c             xrr(kk, nc) =(xrr(kk,nc) **geomavg(kk,ics)) 
c    *                     * (xrp(kk,nc)) **(1.-geomavg(kk,ics))
c             history(kk,ics,c_iter) = xrr(kk,nc)
c          enddo                  ! kk vector loop
c        endif              !if(c_tchem(ics).eq.'    H2O2' 

c DIFFICULT CONVERGENCE OPTION:   (ADD 30905)
c  PROTECT AGAINST WILD OSCILLATION OF HNO3 FOR FIRST FEW ITERS
c  (For NO3->HNO3->NOx error with high BR)
c  (HNO3 icat 16;  may also use for all slow species icat=1)

         if(c_iter.le.3.and.c_icat(ics).eq.16) then
c        if(c_iter.le.3.and.(c_icat(ics.eq.1.or.c_icat(ics).eq.16)) then
c          do kk=1, c_kmax          ! kk vector loop
              if(xrr(kk, nc).gt.5.*xrp(kk,nc))
     *             xrr(kk, nc) =  5.*xrp(kk,nc)
              if(xrr(kk, nc).lt.0.2*xrp(kk,nc))
     *             xrr(kk, nc) = 0.2*xrp(kk,nc)
c          enddo                  ! kk vector loop
         endif             !if(c_iter.le.3.and.c_icat(ics).eq.16) then

c   REGULAR DEBUG WRITE 
c        if(c_kkw.gt.0                 ) 
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) 
c    *        write(c_out,101) c_tchem(ics), ics,xrr(c_kkw, nc) ,
c    *   xrp(c_kkw,nc) ,rpro(c_kkw,nc), rloss(c_kkw,nc)
c 101    format(' CHEMSOLVE (PAIRED SPECIES):  IC  XR  XRP  RPRO  RLOSS'
c    *      ,/,a8,2x,i5,(8(1pe10.3)))
 

c
c    RESET XRR (TOTAL SPECIES SOLUTION) EQUAL TO NEW/OLD RATIO
c        do kk=1,c_kmax      ! kk vector loop
           xrr(kk,nc) = xrr(kk,nc)  /xrp(kk, nc)
c        enddo           ! kk vector loop
c
c   PARTITION THE TOTAL SPECIES BETWEEN GAS-MASTER AND AQUEOUS
c    USING THE SAME GAS-AQUEOUS RATIOS AS PRIOR.

c    ALL CONCENTRATIONS (GAS AND AQUEOUS) ARE UPDATED BY THE RATIO XR/XRP.  
c     (PRIOR TIME STEP, XRIT, PRESERVED HERE)

         do  neq=1,(c_nequil(ics)+1)
           ic=ics
           if(neq.gt.1) ic = c_ncequil(ics,(neq-1))
c  TEST WRITE - GAS/AQUEOUS  
c c          if(c_kkw.gt.0.and.c_tchem(ics).eq.'     CO3') 
c          if(c_kkw.gt.0                         ) 
c    *        write(c_out,  131) c_tchem(ic), xc(1,ic),xrp(1,nc)
c 131        format(' CO3 AQUEOUS TEST.  PRIOR/POST XR XRP=',2x,a8,
c    *             2(1pe10.3))

c          do kk=1,c_kmax      ! kk vector loop
              xc(kk,ic) = xc(kk,ic) *xrr(kk,nc)  
              c_xcout(kk,ic) = c_xcout(kk,ic)
c          enddo        ! kk vector loop

c  TEST WRITE - GAS/AQUEOUS
c c        if(c_kkw.gt.0.and.c_tchem(ncsol(1)).eq.'     CO3') 
c          if(c_kkw.gt.0                         ) 
c    *       write(c_out,  131) c_tchem(ic), xc(1,ic),xc(1,ncsol(1))

         enddo  ! do  neq=1,(c_nequil(ics)+1)

c   END - PARTITION BETWEEN GAS AND AQUEOUS 

c   UPDATE RLOSS, CPRO, AND XRP BASED ON NEW SPECIES CONCENTRATION
c    AND ADD UPDATED CPRO TO RPRO FOR PRODUCTION TO PAIRED SUBSPECIES  
c
c    (updated CPRO only used here for sub-species; 
c       since primary species formula assumes original CPRO, XRP
c       but update done for all CPRO, to be consistent with updated XRP.
c     Updated RLOSS, CPRO, XRP will be used in MULTISOLVE.)


c  UPDATE RLOSS, XRP
c        do kk=1,c_kmax      ! kk vector loop
           rloss(kk,nc) = rloss(kk,nc) * xrr(kk,nc)
           xrp(kk,nc) = xrp(kk,nc) * xrr(kk,nc)
c        enddo          !kk vector loop
c
c 
c UPDATE RPRO - CUT, only if nsolv>nc 

c


c      enddo         !nc=1,nssolv

c ---------
c  END LOOP TO SOLVE SPECIES CONCENTRATIONS- INTERNAL PAIRS
c ---------
c ---------------------------------------------
c END CALCULATE SPECIES CONCENTRATIONS:  INTERNAL PAIRS
c ---------------------------------------------

c
c ---------------------------------------------
c CALCULATE SPECIES CONCENTRATIONS: MULTISOLVE AND NOXSOLVE
c ---------------------------------------------
c CUT
c
c ---------------------------------------------
c CALCULATE CHEMICAL PRODUCTION AND LOSSES FOR DOWN-CASCADE SPECIES.
c ---------------------------------------------
c 
c  *** END IF-LOOP -1 TO SKIP LOSS AND XR CALC FOR SLOW-SPECIES 'PRE'.
c          (END OF ONESOLVE  'POST' LOOP)
c  CUT JANUARY 2005
c             endif               ! if(ic2.ne.-1.and.ic2.ne.-3) then

c  *** IF-LOOP -2 TO SKIP DOWN-CASCADE CALC. FOR SLOW-SPECIES 'POST'.
c         (LOOP EXECUTED FOR full ONESOLVE AND FOR 'PRE')
c  CUT JANUARY 2005
c             if(ic2.ne.-2) then
c  ***
c

c  LOOP TO CALCULATE DOWN-CASCADE PRODUCTION 
c  LOOP TO  RECORD RRP, RRL AND ZERO RP, RL.

c   Note:  brpro adds losses, cross-production to RP, RL (for record).
c   Subsequent down-cascade production will be include in next iteration.


c      do nc=1, nsolv      
       nc=1
         ics = ncsolv(nc)

c TEMPORARY TEST WRITE (why is loop so long?)
c        if(c_kkw.gt.0) write(6,*) 'chemsolve brpro loop: ics='
c    &        ,ics, c_tchem(ics)

         if(c_nnrchem(ics).gt.0) then
           do i=1,c_nnrchem(ics)
             nr =  c_nrchem(ics,i)      

c TEMPORARY TEST WRITE (why is loop so long?)
c        if(c_kkw.gt.0) write(6,*) 'reaction: ',
c    &          nr, c_treac(1,nr), c_treac(2,nr)

             call brpro(nr, ratek,
     &         c_rr, xc, rrl, rrp, rppair, rlpair, c_h2oliq
     &        , senshx, senhcat
     &        , oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2, xfohtest
     &        ,sourcnx, sinknx
     &        )
           enddo        !do i=1,c_nnrchem(ics)
         endif        !if(c_nnrchem(ics).gt.0)
c      enddo        !do nc=1,nsolv


c LOOP TO CALL EXCORR
c  CUT, ONLY FOR PAIRS


c  LOOP TO  RECORD RP, RL 
c  NOTE:  RRP, RRL represent running sum through cascade.
c         RP, RL represent final sum through cascade.
c  ALSO- these are used in AQUASOLVE.
c
c OPTION:  SUM GAS+AQUEOUS RP, RL INTO GAS-ONLY RP, RL?
c   no, this may mess up AQUASOLVE.
c
c OPTION:  THIS CAN BE MOVED ABOVE EXCORR 
c    so that it preserves full RP, RL for exchanged species
c    (but may not record back-forth for  down-cascade)

c      do nc=1, nsolv      
       nc=1
         ics = ncsolv(nc)
         do  neq=1,(c_nequil(ics)+1)
           ic=ics
           if(neq.gt.1) then
             ic = c_ncequil(ics,neq-1)
c 
c OPTION: RECORD SUM OF GAS+AQUEOUS PRODUCTION IN GAS-MASTER
c            c_rp(kk,ics) = c_rp(kk,ics) + rrp(kk,ic)
c            c_rl(kk,ics) = c_rl(kk,ics) + rrl(kk,ic)
c
           endif            !if(neq.gt.1) then

c          do kk=1,c_kmax          ! kk vector loop
             c_rp(kk, ic) = rrp(kk,ic)
             c_rl(kk,ic) = rrl(kk,ic)
c          enddo                 ! kk vector loop

         enddo         !do  neq=1,(c_nequil(ics)+1)
c      enddo        !do nc=1,nsolv

c  LOOP TO ZERO RRP, RRL
c   ZERO RRP HERE so that running sum for next iteration includes 
c           down-cascade production.


c      do nc=1, nsolv      
       nc=1
         ics = ncsolv(nc)
         do  neq=1,(c_nequil(ics)+1)
           ic=ics
           if(neq.gt.1) then
             ic = c_ncequil(ics,neq-1)
           endif

           rrp(kk,ic) = 0.
           rrl(kk,ic) = 0.

         enddo         !do  neq=1,(c_nequil(ics)+1)
c      enddo        !do nc=1,nsolv


c FULL LIST - DEALLOCATE

c ----------------------------------------------------------------
c END CHEMSOLVE1
c ----------------------------------------------------------------

c END CHEMSOLVE1
2000  return
         end
c -------------------------------------------

c
c ORIGINAL CHEMSOLVE - FOR MULTISOLVE
          subroutine chemsolve(ncsol, 
     &    c_xcin, c_xcout, c_xcav, c_xcemit,
     &    c_rp, c_rl, c_rr ,
     &    xc,xcfinr,  xclastq, 
     &    ratek, rateh, rateq, rateqq, rhdif, egasaq, 

     &     xrp, xrr, rpro, rloss, rpro1, rloss1, 
     &       rrp, rrl, cpro,
     &       xrm, xrrm, rpm, rlm,cpm, rpmulti, rlmulti, AX, BX, XX,
     &       rppair, rlpair, xrppair, xrrpair, 
     &       history, geomavg 

     &        , senshx, senhcat
     &        , oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2, xfohtest
     &        ,sourcnx, sinknx, c_notest

     &     ,  c_h2oliq ,   c_rainfr,  c_lexpo, c_iter
     &        )



c 

c Special solution for concentrations of O3, NO2 and NO.

c     FUTURE DO:  back-Euler solution for multi-species
c
c This is the main solver for individual species concentrations.
c It includes:
c (1) A solution for an individual species - 
c      (regarded as a sum of gas + aqueous linked through equilibria)
c 
c (2a) A solution for a pair of rapidly interacting species
c
c (2b) A solution for a 'chain' of linked pairs
c                                       (A<->B, A<->C1, C1<->C2, etc.)
c
c (3) A reverse-Euler solution for a group of 3+  interacting species.
c      ('MULTISOLVE')
c
c (4) Call to special solution for O3-NO2-NO ('NOXSOLVE')
c
c It also establishes rates for reactions associated with the species;
c And calculates summed production, loss of reaction products,
c  and odd hydrogen radical sums.
c 
c  FOR A SINGLE SPECIES OR PAIR, MAKE IC2<=0. 
c
c THE SOLUTION SEQUENCE:  
c  (a) Calculate losses/rates of reactions tht affect the species
c      (including linked gas+aqueous equilibrium species)
c  (b) Solve for species concentrations
c        with separate solutions for:
c         SINGLE SPECIES
c         PAIR AND PAIR CHAIN
c         NOXSOLVE
c         MULTISOLVE
c  (c) Add production of product species and odd-h radicals 
c       to the production sums.
c
c  Input: Prior species concentrations, reactions and rate constants.
c  Output:  Updated species concentrations (xc).
c
c Called by:  quadchem.
c Calls to:  
c     brpro - sums production and loss from reaction
c     brreac - calculates reaction rates.
c     setgeom - establish geometric mean parameter for iteration
c     noxsolve - solution for O3-NO-NO2
c     LINSLV - matrix iteration for MULTISOLVE solution.
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c
c -------------------------------------------------------------------
c ----------------------------------------------------------------
c
c  PREVIOUS:  FOR TWO SPECIES THE 'EXCHANGE' SPECIES MUST COME FIRST.
c  NO LONGER.  EXCHANGE IS NOT DEPENDENT ON ORDER-OF-SPECIES.
c  (ONLY NOXSOLVE USES RPRO, RLOSS, CPRO FROM CHEMSOLVE 
c    IN A SPECIFIC ORDER.  EXCORR IS NOT CALLED FOR NOX.)  
c ----------------------------------------------------------------
c
      use h_rbchemmech
c      use h_rbchemvars
c      use h_rbchemarrays
      use h_rbchemlocal, only: avogadrl
c      use h_rbchemlocarr
      IMPLICIT NONE


c#     include "rbchemmech.h"
c#     include "rbchemvars.h"
c#     include "rbchemarrays.h"
c#     include "rbchemlocal.h"
c#     include "rbchemlocarr.h"
c 
c 
       integer :: ncsol(c_cdim)       ! Species list passed to chemsolve
c --below are originally from rbchemarrys---
       real*8 ::  c_xcin( c_kvec,c_cdim) ! concentr molec/cm3
       real*8 ::  c_xcout( c_kvec,c_cdim) ! concentr, mol/cm3
       real*8 ::  c_xcav( c_kvec,c_cdim)  ! concentr, mol/cm3
       real*8 ::  c_xcemit( c_kvec,c_cdim) ! emis, molec/cm3
      real*8 ::  c_rp(c_kvec,c_cdim)  ! Production, molec/cm3
      real*8 ::  c_rl(c_kvec,c_cdim)  ! Loss, molec/cm3
      real*8 ::  c_rr(c_kvec,c_rdim)  ! Reaction rate m/cm3
        real*8 :: xc( c_kvec,c_cdim) ! concentration molec/cm3
       real*8 :: xcfinr( c_kvec,c_cdim) ! ratio FINAL/AVG cncn
       real*8 :: xclastq( c_kvec,c_cdim) ! conc molec/cm3
          real*8 :: ratek(c_kvec,c_rdim)  ! Rate constants
      real*8 :: rateh(c_kvec,c_hdim)  ! Henry's law constants
      real*8 :: rateq(c_kvec,c_aqdim)  ! Aqueous eq. constants
      real*8 :: rateqq(c_kvec,c_aqqdim) ! Special eq. constants
      real*8 :: rhdif(c_kvec,c_hdim) ! H. mod by diffusion
      real*8 :: egasaq(c_kvec,c_hdim) ! Gas-to-aq transfer s-1
      real*8 :: xrp(c_kvec,c_cdim)     !Solver prior concentration
      real*8 :: xrr(c_kvec,c_cdim)     ! Solver concentration
      real*8 :: rpro(c_kvec,c_cdim)    ! Solver production rate
      real*8 :: rloss(c_kvec,c_cdim)   ! Solver loss rate
      real*8 :: rpro1(c_kvec,c_cdim)   ! Interim rpro w/out cpro 
      real*8 :: rloss1(c_kvec,c_cdim)  ! Interim rloss w/out cpro
      real*8 :: rrp(c_kvec,c_cdim)     ! Interim production rate 
      real*8 :: rrl(c_kvec,c_cdim)     ! Interim loss rate 
      real*8 :: cpro(c_kvec,c_cdim, c_cdim)   ! Cross-production 
      real*8 :: xrm(c_kvec,c_cdim)     !  Initial conc. for multi
      real*8 :: xrrm(c_kvec,c_cdim)    !Initial solution for multi
      real*8 :: rpm(c_kvec,c_cdim)     ! Production rate for multi
      real*8 :: rlm(c_kvec,c_cdim)     ! Loss for multi-spec 
      real*8 :: cpm(c_kvec, c_cdim, c_cdim) !Cross-prod for multi
      real*8 :: rpmulti(c_kvec       ) !Product'n multi group sum
      real*8 :: rlmulti(c_kvec       ) ! Loss group sum for multi
      real*8 :: BX(100), XX(100), AX(100,100) ! For back_Eulersolution AX=B
      real*8 :: rppair(c_kvec,c_cdim)  ! Production for pair grp
      real*8 :: rlpair(c_kvec,c_cdim)  ! Loss rate for pair grp
      real*8 :: xrppair(c_kvec)        ! Solver prior pair group cn.
      real*8 :: xrrpair(c_kvec)        ! Solution for pair group sum
      real*8 :: history(c_kvec,c_cdim,400)  ! Cncn for each iter
      real*8 :: geomavg(c_kvec,c_cdim)   ! Geom. avg factor
      real*8 ::        senshx(c_kvec,2)   ! Hx sens: d ln(pHx)/d
      real*8 ::        senhcat( c_kvec,99) ! Hx sens. by species categ.
      real*8 ::        oddhsum(c_kvec,2)   ! Summed net P/L Hx, mol/cm3
      real*8 ::        oddhdel(c_kvec,2)   ! Summed Hx sens to Oh,
      real*8 ::        oddhsrc(c_kvec,2)   ! Summed P Hx, mol/cm3
      real*8 ::        oddhloh(c_kvec,2)   ! Hx loss from OH,  mol/cm3
      real*8 ::        oddhlho2(c_kvec,2)   ! Hx loss from HO2,  mol/cm3
      real*8 :: xfohtest          ! OH/HO2 convergence test
      real*8 :: sourcnx(c_kvec)   ! Summed NOx source, mol/cm3
      real*8 :: sinknx(c_kvec)    ! Summed NOx sink, mol/cm3
      real*8 ::  c_notest      ! test: dNO2/NO2
      real*8 ::  c_h2oliq(c_kvec)   ! LWC grams cm-3
      real*8 ::  c_rainfr(c_kvec)   ! rainout fraction
      logical :: c_lexpo(c_kvec)        ! Flag for expo decay solution
      integer :: c_iter                  ! chem. number of iterations
c----below are local variables----Guangxing LIn 
c 
c
       real*8 ::  c_droplet(c_kvec)        ! droplet radius, cm
      real*8 ::  c_ohtest      ! test: dOH/OH or dOH/HO2
c below are from rbchemlcoal

c   mol/cm3
c       ln([OH])


      logical :: llexpo(c_kvec)       ! Flag for exponential decay solution
      logical :: doneexpo(c_kvec)     ! Flag for completion of expo decay.

c Allocatable variable, moved to quadchem.
       real*8 :: stoicx                         ! stoichiometry sum
      integer :: ic,ic1,ic2,ic3,icc,ics,ics2,icc1,icc2,icc3 !Chem index
      integer :: is,iss,iscs,isc2                    !Chem local index
          integer :: ich,icq,icx,icx1,icx2,icy1,icy2,icp,icp1,icp2 !Chem
      integer :: icr1,icr2,icr,isr1,isr2,icpair  !chem index - pair and
      integer :: ionsum                                  ! aquasolve ion
      integer :: ica1,ica2,icb1,icb2,nra1,nra2,nrb1,nrb2 !Aquasolve chem
      integer :: neq                                     !Aqueous
      integer :: nc,nc1,nc2,ncc,ncf,nn,nne               !Chem species
      integer :: nr,nr1,nr2,nrh,nrq,nrqq,nrx,np          ! Reaction
      integer :: icat1,icat2,icatp,icatp2  ! indices used for species
      integer :: kk, kw                            ! Vectorization
      integer :: i, j, k, ii, ij, iii, n           ! General counters
       integer :: nsol           ! Number of pair groups being solved
      integer :: nsolv          ! Number of species being solved for
          real*8 :: alpha(c_kvec)   ! General vector variable
      real*8 ::  beta(c_kvec)   ! General vector variable
      real*8 :: gamma(c_kvec)   ! General vector variable
      real*8 :: prior(c_kvec)   ! Prior species conc (molec/cm3)
      integer :: ncsolv(c_cdim) ! Species number (ic) for species in
      integer :: nssolv(c_cdim) ! Pair group number (is) for spec. in

c LOCAL VARIABLES:  INTERNAL
      real*8 :: xpair       ! Summed factor for pair group 
c                        net pair group  production w/ cons. mass.
      real*8 :: xmulti      ! Summed factor for multi group 
c                        net multi group  production w/ cons. mass.

      real*8 ::  rself(c_kvec, 20)        ! Rate of self-reaction
c
      logical :: lself               ! flag for self-reaction

c
c ALLOCATE AX, BX, XX - moved to multisolve
c ----------------------------------------------------------------

       kk=1


c PRELIMINARY:  SELF-REACTION FLAG
       lself = .FALSE.

c PRELIMINARY:  make ic point to GAS-MASTER and PRIMARY OF PAIRED SPECIES
c   and count nsol = number of active species groups solved for.
c
       nsol = 0
       do nc=1,nsdim
         ic=ncsol(nc)
         if(ic.gt.0) then
           ncsol(nc) = c_nppair(c_npequil(ic), 2) 
           nsol = nc
         else                 !if(ic.gt.0) then
          go to  501
         endif                !if(ic.gt.0) then
       enddo            !do ic=1,nsdim
501    continue
c

c ---------------------------------------------
c SUMMATION OF RLOSS - CHEMICAL LOSSES AND CROSS-PRODUCTS
c ---------------------------------------------
c
c  -----------------------
c  PRELIMINARY:  ESTABLISH LIST OF LINKED SPECIES
c  -----------------------
c  nsol = number of pair groups to be solved for (set above)
c  ncsol(n) = ic of head of each pair group to be solved for (input above)
c  nsolv = number of species to be solved for 
c          (includes pairs and multi-solve)
c
c ncsolv(i) = ic of each species in order (ncsol(1) and paired, ncsol(2), etc)
c
c nssolv(i) = is of each species = multi-species group for ncsolv
c
c Note:  loop over nsolv automatically solves for multiple-species groups 
c         as well as single pair grouping

         nsolv = 0
         do is=1,nsol
           nsolv     = nsolv   + 1 + c_nppair(ncsol(is), 3 )
         enddo

         nc=0
         do is=1,nsol
           ic=ncsol(is)
           if(ic.gt.0) then
             do i=1,(c_nppair(ic,3)+1)
               icc = ic
               if(i.gt.1) then
                 icc=c_nppair(ic,i+2)
               endif
               nc=nc+1
               ncsolv(nc) = icc
               nssolv(nc) = is

c TEMPORARY CHEMSOLVE DEBUG WRITE
c      if(c_kkw.gt.0.and.c_iter.eq.1)  then
c       write(c_out, *)' is, ncsol(is) = ', is, ncsol(is) 
c    &      ,c_tchem(ncsol(is))  
c       write(c_out,*)' nc, ncsolv(nc) =', 
c    &                  nc, ncsolv(nc), c_tchem(ncsolv(nc))
c      endif

             enddo
           endif
         enddo


c TEMPORARY CHEMSOLVE DEBUG WRITE
c      if(c_kkw.gt.0.and.c_iter.eq.1) 
c    &  write(c_out, *)' START CHEMSOLVE: ic, tchem, nsol nsolv =',
c    &       ncsol(nc), c_tchem(ncsol(nc)), nsol, nsolv

c PRELIMINARY:  FLAG FOR EXPONENTIAL DECAY SOLUTION
c   Use expo decay only for single species solution (unpaired, no aq.) 
c   (FUTURE DO.  Not included yet.)


c       do  kk=1,c_kmax           ! kk vector loop
          llexpo(kk)=.FALSE.
          if(c_lexpo(kk)) then
           if( (nsolv.eq.1).and.(c_nequil(ncsol(1)).eq.0)
     *          .and. (c_tchem(ncsol(1)).ne.'    H2O2')
c     *          .and. (c_icat(ncsol(1)).eq.1.or.c_icat(ncsol(1)).eq.2)
     *          .and.(.not.c_lsts(ncsol(1)))      )  llexpo(kk)=.TRUE.
          endif                      !if(c_lexpo) then
c       enddo                   ! kk vector loop
c

c PRELIMINARY: CALCULATE REACTION RATES,RP and RL  FOR PRODUCT REACTIONS
c ASSOCIATED WITH SPECIES (i.e. that are sources, not sinks)
c
       do nc=1, nsolv
         ics = ncsolv(nc)
         icpair = c_nppair(ics,2)

         if(c_nnrchp(ics).gt.0) then
           do     i=1,c_nnrchp(ics)
             nr = c_nrchmp(ics,i)
             call brpro(nr, ratek,
     &         c_rr, xc, rrl, rrp, rppair, rlpair, c_h2oliq
     &        , senshx, senhcat
     &        , oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2, xfohtest
     &        ,sourcnx, sinknx
     &        )


c CHEMSOLVE DEBUG WRITE
c             if(c_kkw.gt.0.   and.nsol.gt.1) then
c c           if(c_kkw.gt.0.   and.c_tchem(ics).eq.'    HCHO') then
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) then
c               write(c_out,1001) nr,(c_treac(j,nr),j=1,5),ics
c    *         , c_tchem(ics), c_oddhx(nr,1), c_pronox(nr)
c             endif

           enddo                 !do     i=1,c_nnrchp(ics)
         endif                !if(c_nnrchp(ics).gt.0) then
       enddo               !do nc=1, nsolv
c

c
c ZERO RPRO, RLOSS, XRP, CPRO.  (XRM, XRRM for multisolve) (
c      (RPPAIR, RLPAIR, XRPPAIR, XRRPAIR for pair group sums)
c
           do i=1,nsolv 
c           do  kk=1,c_kmax           ! kk vector loop
             rpro(kk,i) =  0.
             rloss(kk,i) = 0.
             xrp(kk,i) = 0.
             xrm(kk,i) = 0.
             xrrm(kk,i) = 0.
             rpm(kk,i) = 0.
             rlm(kk,i) = 0.
             rself(kk,i) = 0.
c           enddo                   ! kk vector loop


             do ii=1,nsolv
c             do  kk=1,c_kmax           ! kk vector loop
               cpro(kk,i,ii)=0.
                cpm(kk,i,ii)=0.
c             enddo                   ! kk vector loop
             enddo
           enddo
c          do  kk=1,c_kmax           ! kk vector loop
             xrppair(kk  ) = 0.
             xrrpair(kk  ) = 0.
             rpmulti(kk  ) = 0.
             rlmulti(kk  ) = 0.
c          enddo                   ! kk vector loop


c  -----------------
c  LOOP TO CALCULATE CHEM. LOSS RATES AND CROSS-PRODUCTS
c  -----------------

       do nc=1, nsolv 
         ics = ncsolv(nc)
         iscs=nssolv(nc)
         icpair = c_nppair(ics,2)
c
c ESTABLISH RPRO, RLOSS AND XRP FOR THE 'BASE' SPECIES
c        do kk=1,c_kmax      ! kk vector loop
          rpro(kk,nc) = rpro(kk,nc)+  rrp(kk, ics) +   c_xcin(kk,ics)
          rloss(kk,nc) =rloss(kk,nc)+  rrl(kk, ics) 
          xrp(kk,nc) =  xrp(kk,nc) +  xc(kk,ics)
c        enddo               ! kk vector loop

c TEMPORARY CHEMSOLVE DEBUG WRITE
c        if(c_kkw.gt.0   .and.nsol.gt.1)
c c      if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3)
c    *      write(c_out, 91) c_tchem(ics), rrp(c_kkw,ics),
c    *        c_xcin(c_kkw,ics), rpro(c_kkw, nc)
c    *      ,rrl(c_kkw,ics), rloss(c_kkw, nc)
c 91       format(' START CHEMSOLV: IC RRP XXO RPRO RRL RLOSS= ',a8,/,
c    *      5(1pe10.3))

c ADD ALL AQUEOUS-EQUILIBRIUM SPECIES INTO SUMMED RPRO, RLOSS AND XRP.
c CONVERTING AQUEOUS INTO GAS UNITS (AVOGADRL)
c Also include RAINOUT as loss.

         if(c_nequil(ics).gt.0)       then

           do  neq=1,c_nequil(ics)
             ic = c_ncequil(ics,neq)
c            do kk=1,c_kmax    ! kk vector loop
               xrp(kk,nc) =   xrp(kk,nc) 
     *            + xc(kk,ic)*c_h2oliq(kk)*avogadrl
               rpro(kk,nc) = rpro(kk,nc) + rrp(kk,ic)
               rloss(kk,nc) = rloss(kk,nc)+rrl(kk,ic)
     *            +c_rainfr(kk) * xc(kk,ic)*c_h2oliq(kk)*avogadrl
c            enddo           ! kk vector loop

c TEMPORARY CHEMSOLVE DEBUG WRITE
c            if(c_kkw.gt.0.and.nsol.gt.1) 
c c           if(c_kkw.gt.0.   and.c_tchem(ics).eq.'    HCHO') 
c c          if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) 
c    *          write(c_out, 91) c_tchem( ic), rrp(c_kkw, ic),
c    *            c_xcin(c_kkw, ic), rpro(c_kkw, nc)
c    *      ,rrl(c_kkw,ic), rloss(c_kkw, nc)

           enddo         ! do neq=1,c_nequil(ics)
         endif
c END EQUILIBRIUM SUMMATION.  

c CHEMSOLVE DEBUG WRITE
c        if(c_kkw.gt.0  .and.nsol.gt.1) 
c c      if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) 
c    *      write(c_out,105) c_tchem(ics), 
c    *      xrp(c_kkw,nc), rpro(c_kkw,nc), rloss(c_kkw,nc)
c 105      format(' START CHEMSOLVE: SPECIES, XRP, RPRO, RLOSS=',
c    *            /,a8,2x,3(1pe10.3))
c
c ADD INITIAL SPECIES XRP, RPRO, RLOSS TO XRPPAIR, RPPAIR, RLPAIR
c  IF PAIR GROUP HAS >3 MEMBERS AND NO MULTISOLVE (FOR NORMALIZATION)

c        if(nsolv.ge.3.and.nssolv(nsolv).eq.1) then
         if(nsolv.ge.3.and.nsol.eq.1         ) then
c          do kk=1,c_kmax    ! kk vector loop
             xrppair(kk   ) = xrppair(kk   ) +  xrp(kk,nc) 
     *                                      * c_pairfac(ics)
             rppair(kk,icpair) =  rppair(kk,icpair)  +  rpro(kk,nc) 
     *                                      * c_pairfac(ics)
             rlpair(kk,icpair) =  rlpair(kk,icpair)  +  rloss(kk,nc)
     *                                      * c_pairfac(ics)
c            enddo           ! kk vector loop

c CHEMSOLVE DEBUG WRITE
c            if(c_kkw.gt.0) 
c c         if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3)
c    *         write(c_out,104) c_pairfac(ics),
c    *          xrppair(kk), rppair(kk,icpair), rlpair(kk,icpair)
c 104             format(' START PAIR SUM:  SPECIES PAIRFAC; ',
c    *                ' XRPAIR, RP, RL =',/,4(1pe10.3))

         endif              !if(nsolv.ge.3.and.nsol.eq.1         ) then

c ADD INITIAL SPECIES XRP, RPRO, RLOSS TO XRM, RPM, RLM: for MULTISOLVE
c  (note:  no PAIRFAC, different from PAIR NORMALIZATION)

         if(nsol.gt.1                        ) then
c          do kk=1,c_kmax    ! kk vector loop
             xrm(kk,iscs)    = xrm(kk,iscs)   +  xrp(kk,nc) 
             rpm(kk,iscs)      =  rpm(kk,iscs)       +  rpro(kk,nc) 
             rlm(kk,iscs)      =  rlm(kk,iscs)       +  rloss(kk,nc)
             rpmulti(kk)       =  rpmulti(kk)        +  rpro(kk,nc) 
     *                                      * c_multfac(ics)
             rlmulti(kk)       =  rlmulti(kk)        +  rloss(kk,nc)
     *                                      * c_multfac(ics)
c            enddo           ! kk vector loop

c CHEMSOLVE DEBUG WRITE
c            if(c_kkw.gt.0) write(c_out,204) 
c c         if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3)
c    *         write(c_out,204) 
c    *          xrm(c_kkw,iscs) , rpm(c_kkw,iscs)     , rlm(c_kkw,iscs)
c    *         ,rpmulti(c_kkw), rlmulti(c_kkw)
c 204             format(' START PAIR SUM:  ',
c    *                ' XRPAIR, RP, RL =',/,5(1pe10.3))

         endif              !if(nsol.gt.1                        ) then

c ADD LOSS REACTIONS AND CROSS-PRODUCT REACTIONS TO RLOSS,RPRO, CPRO.
c
c ALSO SET SELF-REACTION FLAG (lself)
c
c ALSO SET FLAG FOR SECOND REACTANT ALSO INCLUDED IN SPECIES LIST (nc2)
c     (pair or multisolve.  Important for NO+O3->NO2.)


         if(             c_nnrchem(ics).gt.0) then

c SET INDICES
c FEBRUARY 2005 CHANGE - icr1=npequil is before if(icr2.gt.0) BUG FIX.
           do  i=1,c_nnrchem(ics)
             nr = c_nrchem(ics,i)
             icr1 = c_reactant(nr,1)
             icr2 = c_reactant(nr,2)
             icr1 = c_npequil(icr1)
             nc2  = 0            ! index to count loss from 2nd reactant
             if(icr2.gt.0) then
               icr2 = c_npequil(icr2)
               if(icr1.eq.ics.and.icr2.eq.ics) lself=.TRUE.
c  FEB 2005OPTION:  
c     IF REACTANTS ARE IN SAME PAIR GROUP, SET SELF-REACTION FLAG
c
c  FUTURE DO, FIX SELF-REACTION.
c     (  In SELF-REACTION, do not set 2nd reactant - it messes up CPRO
c          unless CPRO algorithm changed. )
c
               if(c_nppair(icr1,2).eq.c_nppair(icr2,2)) lself =.TRUE.
               do ncc=1,nsolv

                 if(ncc.ne.nc) then
                   if(ncsolv(ncc).eq.icr1.or.ncsolv(ncc).eq.icr2) then
                               nc2=ncc
                               ics2 = ncsolv(ncc)
                   endif
                 endif                 !if(ncc.ne.nc) then
               enddo                !do ncc=1,nsolv
             endif                !if(icr2.gt.0) then

c CHEMSOLVE DEBUG WRITE  
c c            if(c_kkw.gt.0   .and.nsol.gt.1) then
c            if(c_kkw.gt.0                 ) then
c c          if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) then
c              write(c_out,1001) nr,(c_treac(j,nr),j=1,5)
c    *         , ics,c_tchem(ics) , c_oddhx(nr,1), c_pronox(nr)
c 1001    format('REACTION #',i4,': ',a8,'+',a8,'=>',a8,',',a8,',',a8,/,
c    *     ' KEY SPECIES =',i4,2x,a8,'  ODD-H=',f8.2,'  PRONOX=',f8.2)
c            endif


c CALL BRREAC - to establish reaction rate
             call brreac(nr,
     &         c_rr, xc, ratek, c_h2oliq )

c ADD LOSS FOR FIRST REACTANT
c            do kk=1,c_kmax      ! kk vector loop
c              rloss(kk,nc) = rloss(kk,nc) + c_rr(kk,nr)*c_stoiloss(nr)
               rloss(kk,nc) = rloss(kk,nc) + c_rr(kk,nr)
c            enddo            ! kk vector loop

c  stoiloss OPTION:  
c IF SELF-REACTION: ADD AGAIN.  (2nd reactant index is zero)

             if(icr1.eq.icr2) then
c              do kk=1,c_kmax      ! kk vector loop
                 rloss(kk,nc) = rloss(kk,nc) + c_rr(kk,nr)
                 rself(kk,nc) = rself(kk,nc) + 2.*c_rr(kk,nr)
c              enddo            ! kk vector loop
             endif              ! if(icr1.eq.icr2) then

c CHEMSOLVE DEBUG WRITE
c c            if(c_kkw.gt.0  .and.nsol.gt.1) 
c            if(c_kkw.gt.0                ) 
c c          if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) 
c    *        write(c_out,108) c_tchem(ics), 
c    *          rloss(c_kkw,nc), c_rr(c_kkw,nr)
c 108          format(' CHEMSOLVE LOSS FOR MAIN SPECIES:  IC = ',
c    *           a8,/,
c    *           '    RLOSS , RR = ',2(1pe10.3))
 

c SECOND SPECIES OPTION:
c ADD LOSS OF 2ND REACTANT FOR DIFFERENT SPECIES. (for NO+O3->NO2).
c ADD TO RLOSS1 ALSO (which may  have been written before 
c    - RLOSS1 only excludes CPRO and PSEUDO-SOLUTION.
c
c NOTE: omit stoiloss, which adjusts only for main species 
c       appearing in reactant and product. 

c
             if(nc2.gt.0) then
c              do kk=1,c_kmax      ! kk vector loop
                 rloss(kk,nc2) = rloss(kk,nc2) + c_rr(kk,nr)
                 rloss1(kk,nc2)=rloss1(kk,nc2) + c_rr(kk,nr)
c              enddo            ! kk vector loop

c  CHEMSOLVE DEBUG WRITE
c c              if(c_kkw.gt.0  .and.nsol.gt.1) 
c              if(c_kkw.gt.0                ) 
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) 
c    *          write(c_out, 109) c_tchem(ics),
c    c           c_tchem(icr2), rloss(c_kkw,nc2), c_rr(c_kkw,nr)
c 109            format(' CHEMSOLVE LOSS FOR SECOND SPECIES: IC, IC2 =',
c    *           a8,2x,a8,/,
c    *           '    RLOSS2, RR = ',2(1pe10.3))

             endif            !if(nc2.gt.0) then

c ADD PRODUCTS TO CPRO or RPRO.
c CPRO MATRIX  (kk,reactant,product).  (note prodarr=gas-master product)
c

             do ncc=1,nsolv 
               if( c_prodarr(nr,ncsolv(ncc)).ne.0.) then
                icc = ncsolv(ncc)

c Adjust LOSS if PRODUCT EQUALS REACTANT, skip PRO.
                if(ncc.eq.nc.or.ncc.eq.nc2) then
                 stoicx = c_prodarr(nr,ncsolv(ncc))
                 if(stoicx.gt.1.) stoicx=1.
                 rloss(kk,ncc)=rloss(kk,ncc) - c_rr(kk,nr)*stoicx
                 if(ncc.eq.nc2) 
     *            rloss1(kk,ncc)=rloss1(kk,ncc) - c_rr(kk,nr)*stoicx

                else            !if(ncc.eq.nc.or.ncc.eq.nc2) then

c ADD TO CPRO - ONLY IF SPECIES ARE DIRECT PAIRS
c   ALSO IF MULTI.
c   (MULTI CPRO is added to RPRO for pair solution below,t not here - 
c     to avoid doublecounting in RPM.) 


                 if( (c_nppair(ics,1).eq.icc 
     *                 .or.c_nppair(icc,1 ).eq.ics)
     *              .or.(nssolv(nc).ne.nssolv(ncc))   
     *                      )then
c                  do kk=1,c_kmax      ! kk vector loop
                     cpro(kk,nc,ncc) = cpro(kk,nc,ncc) 
     *                 + c_rr(kk,nr)*c_prodarr(nr,ncsolv(ncc))
c                  enddo               ! kk vector loop

c TEMPORARY TEST DEBUG WRITE
c                 if(c_kkw.gt.0  .and.nsol.gt.1) 
c c               if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) 
c    *              write(c_out,1103) 
c    *                nc,ncc,cpro(c_kkw,nc,ncc)
c 1103                format('NC, NCC, CPRO=', 2i5,8(1pe10.3))

                 else                 !if( (c_nppair(ics,1).eq.icc 
C ADD TO RPRO IF NOT DIRECT PAIR
c                  do kk=1,c_kmax      ! kk vector loop
                     rpro(kk,   ncc) = rpro(kk,   ncc) 
     *                 + c_rr(kk,nr)*c_prodarr(nr,ncsolv(ncc))
c                  enddo               ! kk vector loop

c TEMPORARY TEST DEBUG WRITE
c                  if(c_kkw.gt.0  .and.nsol.gt.1) 
c c                if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) 
c    *              write(c_out,1104)   nc,ncc,rpro(c_kkw,ncc)
c 1104                format('NC, NCC, RPRO=', 2i5,8(1pe10.3))

                 endif                !if( (c_nppair(ics,1).eq....then

c  TEMPORARY TEST DEBUG WRITE
c                   if(c_kkw.gt.0  .and.nsol.gt.1) 
c                   if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) 
c    *              write(c_out,1101)
c    *              nc, ncc, ics, ncsolv(ncc), nr, c_rr(c_kkw,nr),
c    *              c_prodarr(nr,ncsolv(ncc))
c 1101                format(' CPRO FORMATION: NC NCC IC(nc) IC(ncc)',
c    *                ' NR RR PRODARR=',/, 5i5, 2(1Pe10.3))

               endif               !if(ncc.ne.nc) then

c   CPRO FROM SECOND REACTANT
c   FEBRUARY 2005 CHANGE:  
c     IF DIRECT PAIR, ADD TO CPRO AND SUBTRACT FROM RPRO
c       (it should have just been added to RPRO for 1st reactant)
c     OTHERWISE, SKIP.  
c      (Note:  for MULTI, CPRO is recorded and added to RPRO for PAIRSOLVE.
c       This must be done just once, for 1st reactant, to avoid double count.)
c 

c CHANGE
               if(nc2.gt.0)                       then

                 if( (c_nppair(ics2,1).eq.icc 
     *                 .or.c_nppair(icc,1 ).eq.ics2)
     *                      )then
c DIRECT PAIR:   ADD 2ND REACTANT CPRO, ADJUST RPRO. (NCPRO CUT)

c                  do kk=1,c_kmax      ! kk vector loop
                     cpro(kk,nc2,ncc) = cpro(kk,nc2,ncc) 
     *                 + c_rr(kk,nr)*c_prodarr(nr,ncsolv(ncc))
                     rpro(kk,    ncc) = rpro(kk,    ncc) 
     *                 - c_rr(kk,nr)*c_prodarr(nr,ncsolv(ncc))
c                  enddo               ! kk vector loop

c TEMPORARY TEST DEBUG WRITE
c                 if(c_kkw.gt.0  .and.nsol.gt.1) 
c c               if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) 
c    *              write(c_out,1105) 
c    *              nc,nc2,ncc,cpro(c_kkw,nc2,ncc), rpro(c_kkw,ncc)
c 1105                format('NC,NC2, NCC, CPRO,RPRO=', 3i5,8(1pe10.3))


c  INTERNAL BUT NOT DIRECT:  DO NOTHING
c                else             !if( (c_nppair(ics2,1).eq.icc then
c                  if   (nssolv(nc).eq.nssolv(ncc)) then
c  MULTI:  DO NOTHING
c                  else          !if   (nssolv(nc).eq.nssolv(ncc)) then

c END CPRO FROM 2ND REACTANT
                 endif            !if( (c_nppair(ics2,1).eq.icc then

c  TEMPORARY TEST DEBUG WRITE
c                   if(c_kkw.gt.0  .and.nsol.gt.1) 
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) 
c    *           write(c_out,1101)
c    *           nc2, ncc, ncsolv(nc2), ncsolv(ncc), 
c    *           nr, c_rr(c_kkw,nr),
c    *           c_prodarr(nr,ncsolv(ncc))

                endif              !if(ncc.eq.nc.or.ncc.eq.nc2) then
               endif          !if( c_prodarr(nr,ncsolv(ncc)).ne.0.) then
 
             enddo               !  do ncc=1,nsolv 

c END LOOP - CPRO MATRIX

c CALCULATE NET PRODUCTION AND LOSS FOR PAIR GROUP
c  (FOR NORMALIZATION OF GROUP w>3 MEMBERS)
             if(nsolv.ge.3.and.nsol.eq.1         ) then
               xpair = 0.
               if(c_nppair(icr1,2).eq.icpair) 
     *              xpair = xpair - c_pairfac(icr1)
               if(icr2.gt.0) then
                 if(c_nppair(icr2,2).eq.icpair) 
     *            xpair =  xpair - c_pairfac(icr2)
               endif
               do ncc=1,nsolv
                  xpair = xpair + c_prodarr(nr,ncsolv(ncc))
     *                                   *c_pairfac(ncsolv(ncc))
               enddo             !do ncc=1,nsolv


               if(xpair.lt.0.) then
c                do kk=1,c_kmax        ! kk vector loop
                   rlpair(kk,icpair)  = rlpair(kk,icpair)
     *                                     -xpair*c_rr(kk,nr)
c                enddo               ! kk vector loop
               endif                 !if(xpair.lt.0.) then

               if(xpair.gt.0.) then
c                do kk=1,c_kmax        ! kk vector loop
                   rppair(kk,icpair) =rppair(kk,icpair)
     *                       +xpair*c_rr(kk,nr)
c                enddo               ! kk vector loop
               endif                 !if(xpair.lt.0.) then

c CHEMSOLVE DEBUG WRITE
c              if(c_kkw.gt.0) 
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) 
c    *             write (c_out,110) xpair, c_rr(kk,nr),
c    *             rlpair(c_kkw,icpair), rppair(c_kkw,icpair)
c 110            format(' CHEM REACTION ADDED TO PAIR GROUP SUM:',/,
c    *              ' FACTOR, RR, RLPAIR, RPPAIR =', 4(1pe10.3))

             endif              !if(nsolv.ge.3.and.nsol.eq.1 ) then

c CALCULATE NET PRODUCTION AND LOSS FOR PAIR GROUP
c  FOR MULTISOLVE (AND NORMALIZATION OF GROUP w>3 MEMBERS)
c
             if(nsol.gt.1                        ) then
               isr1 = nssolv(nc)
               isr2 = 0
               if(nc2.gt.0) then
                 isr2=nssolv(nc2)
               endif            !if(nc.gt.0) then

c MULTISOLVE DEBUG WRITE
c            if(c_kkw.gt.0) then
c c          if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) then
c              write(c_out,1201) nr,(c_treac(j,nr),j=1,5),ics
c    *            , c_tchem(ics)
c 1201    format('REACTION #',i4,': ',a8,'+',a8,'=>',a8,',',a8,',',a8,/,
c    *     ' KEY SPECIES =',i4,2x,a8)
c               write(c_out,*) nc, nc2, isr1, isr2
c            endif

c LOOP THROUGH PAIR GROUPS

               xmulti = 0.
               do is=1,nsol
                 ic=ncsol(is)

c   ESTABLISH NET PRO/LOSS FOR PAIR GROUP
                 xpair = 0.
                 if(is.eq.isr1)  xpair = xpair - 1.
                 if(is.eq.isr2)  xpair = xpair - 1.

                 if(is.eq.isr1)  xmulti= xmulti- c_multfac(ic)
                 if(is.eq.isr2)  xmulti= xmulti- c_multfac(ic)

c ERROR HERE  - THIS WAS c_nppair(icpair,.) THROUGHOUT LOOP
c             -  SHOULD BE c_nppair(ic,..)

                 do iss=1,(c_nppair(ic    ,3)+1)
                  icc=ic
                  if(iss.gt.1) icc=c_nppair(ic    ,iss+2)
                  xpair = xpair + c_prodarr(nr,      (icc))
                  xmulti= xmulti+ c_prodarr(nr,      (icc))
     *                           *c_multfac(icc)

                 enddo             !do ncc=1,nsolv

c MULTISOLVE DEBUG WRITE
c                if(c_kkw.gt.0) 
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3)
c    *            write(c_out,1211) is,c_tchem(ic), xpair
c 1211             format(' MULTISOLVE SETUP: IS TCHEM XPAIR XMULTI =',
c    *                i3,2x,a8,2(1pe10.3))


c   ADD TO RLM LOSS
                 if(xpair.lt.0.) then
c                  do kk=1,c_kmax        ! kk vector loop
                     rlm(kk,is)      = rlm(kk,is)     -xpair*c_rr(kk,nr)
c                  enddo               ! kk vector loop

c MULTISOLVE DEBUG WRITE
c                  if(c_kkw.gt.0) 
c c               if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3)
c    *                 write(c_out,1212)rlm(c_kkw,is)  ,c_rr(c_kkw,nr)
c 1212               format(' ADD -XPAIR*RR TO RLM: RLM, RR='
c    *                              ,2(1pe10.3))

c SUBTRACT CROSS-LOSS FROM CPM.  (add negative XPAIR)
c   (NOTE:  Just one cross-loss;  
c     2nd cross-loss will be counted when loop reaches the other reactant)
c
                   if(isr1.gt.0.and.isr1.ne.is) then
c                    do kk=1,c_kmax        ! kk vector loop
                       cpm(kk,isr1,is) = cpm(kk,isr1,is) 
     *                                     + xpair*c_rr(kk,nr)
c                      cpm(kk,is,isr1) = cpm(kk,is,isr1) 
c    *                                    + xpair*c_rr(kk,nr)
c                    enddo               ! kk vector loop

c MULTISOLVE DEBUG WRITE
c                  if(c_kkw.gt.0) 
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3)
c    *               write(c_out, 1213) isr1,
c    *                cpm(kk,isr1,is), cpm(kk,is, isr1)
c 1213               format(' SUBTRACT XPAIR*RR FROM CPM: ISR1,CP,CP=',
c    *                     2x,i4,2(1pe10.3))


                   endif               !if(isr1.gt.0) then
                   if(isr2.gt.0.and.isr2.ne.is) then
c                    do kk=1,c_kmax        ! kk vector loop
                       cpm(kk,isr2,is) = cpm(kk,isr2,is) 
     *                                       + xpair*c_rr(kk,nr)
c                      cpm(kk,is,isr2) = cpm(kk,is,isr2) 
c    *                                       + xpair*c_rr(kk,nr)
c                    enddo               ! kk vector loop

c MULTISOLVE DEBUG WRITE
c                  if(c_kkw.gt.0) 
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3)
c    *               write(c_out, 1214) isr2,
c    *                cpm(kk,isr2,is), cpm(kk,is, isr2)
c 1214               format(' SUBTRACT XPAIR*RR FROM CPM: ISR2,CP,CP=',
c    *                     2x,i4,2(1pe10.3))

                   endif               !if(isr2.gt.0) then

c   ADJUSTMENT FOR SELF-REACTION (includes 2 reactants in one pair group)
c     (Back-Euler: dR/dx for rate kx^2 is 2kx;  loss per reaction=2.
c      So total loss is 2R;  sensitivity is 4 R.
c      Add 2R to RPM, 4R to RLM.)
c     HERE:  XPAIR*R already added to RLM.  Add again to RLM, RPM.
c
                   if(isr1.eq.is.and.isr2.eq.is) then
c                    do kk=1,c_kmax        ! kk vector loop
                       rlm(kk,is)        = rlm(kk,is)    
     *                                         - xpair*c_rr(kk,nr)
                       rpm(kk,is)        = rpm(kk,is)    
     *                                         - xpair*c_rr(kk,nr)
c                    enddo               ! kk vector loop
                   endif              !if(isr1.eq.is.and.isr2.eq.is) 

                 endif                 !if(xpair.lt.0.) then

c ADD TO RPM
                 if(xpair.gt.0.) then
c                  do kk=1,c_kmax        ! kk vector loop
                     rpm(kk,is)      = rpm(kk,is)   + xpair*c_rr(kk,nr)
c                  enddo               ! kk vector loop

c MULTISOLVE DEBUG WRITE
c                  if(c_kkw.gt.0) 
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) 
c    *                 write(c_out,1222)rpm(c_kkw,is)  , c_rr(c_kkw,nr)
c 1222               format(' ADD +XPAIR*RR TO RPM: RPM, RR='
c    *                                 ,2(1pe10.3))


c ADD TO CPM  CROSS-PRO.
                   if(isr1.gt.0.and.isr1.ne.is) then
c                    do kk=1,c_kmax        ! kk vector loop
                       cpm(kk,isr1,is)    = cpm(kk,isr1,is)   
     *                                      + xpair* c_rr(kk,nr)
c                    enddo               ! kk vector loop

c MULTISOLVE DEBUG WRITE
c                  if(c_kkw.gt.0) 
c c                if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) 
c    *               write(c_out, 1223) isr1,  cpm(kk,isr1,is)
c 1223               format(' ADD      XPAIR*RR TO   CPM: ISR1,CP,CP=',
c    *                     2x,i4,2(1pe10.3))

                   endif               !if(isr1.gt.0) then
                   if(isr2.gt.0.and.isr2.ne.is) then
c                    do kk=1,c_kmax        ! kk vector loop
                       cpm(kk,isr2,is)    = cpm(kk,isr2,is)   
     *                                      + xpair* c_rr(kk,nr)
c                    enddo               ! kk vector loop

c MULTISOLVE DEBUG WRITE
c                  if(c_kkw.gt.0) 
c             if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3)
c    *                write(c_out, 1224) isr2,   cpm(kk,isr2,is)
c 1224               format(' ADD      XPAIR*RR TO   CPM: ISR2,CP   =',
c    *                     2x,i4,2(1pe10.3))

                   endif               !if(isr2.gt.0) then

                 endif              !if(xpair.gt.0.) then


c CHEMSOLVE DEBUG WRITE
c                if(c_kkw.gt.0) 
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3)
c    *               write (c_out,210) xpair, c_rr(kk,nr),
c    *                 rlm(c_kkw,is)       , rpm(c_kkw,is)        
c 210              format(' CHEM REACTION ADDED TO PAIR GROUP SUM:',/,
c    *             ' FACTOR, RR, RLM   , RPM    =', 4(1pe10.3))

               enddo        !do is=1,nsol
c END LOOP THROUGH PAIR GROUPS
c
c MULTI GROUP SUM

               if(xmulti.lt.0.) then
c                do kk=1,c_kmax        ! kk vector loop
                   rlmulti(kk)  = rlmulti(kk)
     *                                     -xmulti*c_rr(kk,nr)
c                enddo               ! kk vector loop
               endif                 !if(xmulti.lt.0.) then

               if(xmulti.gt.0.) then
c                do kk=1,c_kmax        ! kk vector loop
                   rpmulti(kk) =rpmulti(kk)+xmulti*c_rr(kk,nr)
c                enddo               ! kk vector loop
               endif                 !if(xmulti.gt.0.) then

c CHEMSOLVE DEBUG WRITE
c                if(c_kkw.gt.0) 
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3)
c    *              write (c_out,211) xmulti,c_rr(kk,nr),
c    *               rlmulti(c_kkw)      , rpmulti(c_kkw)       
c 211              format(' CHEM REACTION ADDED TO MULTI GROUP SUM:',/,
c    *             ' FACTOR, RR, RLM   , RPM    =', 4(1pe10.3))

             endif     !if(nsol.gt.1                        ) then
c END CALCULATE NET PRODUCTION AND LOSS FOR PAIR GROUP

           enddo  ! do  i=1,c_nnrchem(ics)
         endif    !  c_nnrchem(ics).gt.0
c END - ADD LOSS REACTIONS AND CROSS-PRODUCTS

c NONSTEADY STATE ADJUSTMENT FOR PAIR GROUP SUM
c   (ahead of STEADY STATE ADJUSTMENT to use RLOSS=0)
c   (note RPPAIR already includes XXO)
c
         if(nsolv.ge.3.and.nsol.eq.1         ) then
c          do kk=1,c_kmax      ! kk vector loop
            if(.not.c_lsts(ics).or.rloss(kk,nc).eq.0) 
     *          rlpair(kk,icpair)   = rlpair(kk,icpair)   
     *                   + xrp(kk,nc)*c_pairfac(ics)
            if(rloss(kk,nc).le.0.) 
     *          rlpair(kk,icpair)   = rlpair(kk,icpair)   
     *                   + 1.0e-08   *c_pairfac(ics)
c          enddo                       ! kk vector loop
         endif            !if(nsolv.ge.3.and.nsol.eq.1         ) then

c CHEMSOLVE DEBUG WRITE
c        if(c_kkw.gt.0) 
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3)
c    *         write(c_out, 107) c_pairfac(ics), xrp(c_kkw,nc),
c    *                  rlpair(c_kkw,icpair)
c 107        format(' PRIOR CONC.   ADDED TO PAIR GROUP SUM:',/,
c    *            ' FACTOR, XRP, RLPAIR =', 4(1pe10.3))

c NONSTEADY STATE ADJUSTMENT FOR MULTISOLVE SUM
c   (ahead of STEADY STATE ADJUSTMENT to use RLOSS=0)
c
         if(nsol.gt.1                        ) then
c          do kk=1,c_kmax      ! kk vector loop
            if(.not.c_lsts(ics).or.rloss(kk,nc).eq.0) then
                rlm(kk,iscs)          = rlm(kk,iscs)          
     *                   + xrp(kk,nc)
                rlmulti(kk)           = rlmulti(kk)
     *                   + xrp(kk,nc)*c_multfac(ics)
            endif        !if(.not.c_lsts(ics).or.rloss(kk,nc).eq.0) then
            if(rloss(kk,nc).le.0.)  then
                rlm(kk,iscs)          = rlm(kk,iscs)          
     *                   + 1.0e-08   
                rlmulti(kk)           = rlmulti(kk)
     *                   + 1.0e-08   *c_multfac(ics)
            endif          !if(rloss(kk,nc).le.0.)  then
c          enddo                       ! kk vector loop
         endif            !if(nsol.gt.1                        ) then

c CHEMSOLVE DEBUG WRITE
c        if(c_kkw.gt.0) 
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3)
c    *           write(c_out, 207) c_pairfac(ics), xrp(c_kkw,nc),
c    *                  rlm(c_kkw,iscs)        
c        if(c_kkw.gt.0) write(c_out,*) ics,nc,iscs
c 207        format(' PRIOR CONC.   ADDED TO PAIR GROUP SUM:',/,
c    *            ' FACTOR, XRP, RLPAIR =', 4(1pe10.3))


c NONSTEADY STATE ADJUSTMENT:  IF NONSTEADY STATE OR ZERO RLOSS, ADD XRP.
c  (EQUIVALENT TO RL/XR + 1).
c  (NOTE:  STEADY-STATE OPTION STILL INCLUDES XXO IN RPRO.
c   FOR STEADY STATE XXO SHOULD EQUAL EITHER EMISSIONS OR ZERO.)
c    (XXO SET IN PRELUMP)

c WITH ZERO PROTECT FOR RLOSS


c        do kk=1,c_kmax      ! kk vector loop
          if(.not.c_lsts(ics).or.rloss(kk,nc).eq.0) 
     *          rloss(kk,nc) = rloss(kk,nc) + xrp(kk,nc)
          if(rloss(kk,nc).le.0.) rloss(kk,nc) = 1.0e-08
c        enddo                       ! kk vector loop

c  (OPTION:  FOR A SELF-REACTION (stoiloss>1, e.g. HO2+HO2, stoiloss=2.)
c   AN EXACT LINEARIZED NR SOLUTION WOULD REQUIRE 
c   RLOSS=+2*RR*STOILOSS, RPRO=+RR*STOILOSS. (d/dR = twice loss rate.)
c   TO IMPLEMENT THIS, IF STOILOSS>1, ADD ADDITIONAL RR*STOILOSS
c   TO BOTH RLOSS AND RPRO, WITHIN LOOP 125 (and 225 for twosolve).

c   This is cut because IT MAY SCREW HO2+HO2 REACTIONS.  
c  )

c  SAVE RLOSS1, RPRO1 
c     =RPRO without CPRO; RLOSS w/o internal solution adjustments.

c        do kk=1,c_kmax      ! kk vector loop
           rloss1(kk,nc) = rloss(kk,nc)
           rpro1(kk,nc) = rpro(kk,nc)
c        enddo                    ! kk vector loop

       enddo       !  do nc=1, nsolv 

c  ---------------------
c  END LOOP TO CALCULATE CHEM. LOSS RATES AND CROSS-PRODUCTS
c  ---------------------


c SET SELF-REACTION FLAG:  false if self-reaction is insignificant
c       (ADDED 408 APRIL 2008)
c       (note future option:  save history and use to prevent oscill.)
c       (note - this prevents NO3-N2O5 error, but hard-wired NO3 retained)
c
      if(lself) then
        lself=.FALSE.
        do nc=1,nsolv
         if(rself(kk,nc).gt.0.33*rloss1(kk,nc)) lself=.TRUE.

c TEMPORARY DEBUG WRITE 408
c        ics = ncsolv(nc)
c       if(c_kkw.gt.0) write(c_out,*)
c    &    'RSELF: tchem, ics, nc, rloss, rself, lself',
c    &      c_tchem(ncsolv(nc)), ncsolv(nc),nc,rloss1(kk,nc),
c    &      rself  (kk,nc), lself

        enddo             !do nc=1,nsolv
      endif             !if(lself) then
c END-  SET SELF-REACTION FLAG


c  OPTION:  Add external CPRO to RPRO for multi-species.
c  CURRENT OPTION:  
c   Internal XRR will be solved with RPRO from prior values of multi-species.
c   RLOSS already includes loss to other multi-species groups.
c  ALTERNATIVE: 
c   subtract CP to multi-species from RLOSS


       if(nsol.gt.1) then
         do nc=1, nsolv 
           ics = ncsolv(nc)
           do ncc=1,nsolv
             if(nssolv(ncc).ne.nssolv(nc)) then
c              do kk=1,c_kmax      ! kk vector loop
                 rpro(kk,nc) = rpro(kk,nc) + cpro(kk,ncc,nc)
c              enddo              ! kk vector loop
             endif             !(nssolv(ncc).ne.nssolv(nc))

           enddo     ! ncc=1, nsolv

c   CHEMSOLVE DEBUG WRITE - RPRO, RLOSS, CPRO, BEFORE CPRO PAIR ADJUSTMENT
c     (RPRO1= prior without CP;RPRO includes CP from multi-spec. groups)
 
c          if(c_kkw.gt.0.and.nsolv.gt.1  .and.nsol.gt.1) then
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) then
c             write(c_out,111) c_tchem(ics), 
c    *       rpro1(c_kkw,nc),rpro(c_kkw,nc), rloss(c_kkw,nc)   
c 111      format('CHEMSOLVE INITIAL VALUES: RPRO1, RPRO+multi,RLOSS= ' 
c    *        ,/,a8,2x,(8(1pe10.3)))
c           write(c_out,112) (cpro(c_kkw,nc,ncc),ncc=1,nsolv)
c 112       format(' CPRO(nc,ncc),ncc=1,nsolv):',/,(8(1pe10.3)))
c          endif         !if(c_kkw.gt.0.and.nsolv.gt.1) 

         enddo       !  do nc=1, nsolv 
       endif       !if(nsol.gt.1) then

c ---------------------------------------------
c END - SUMMATION OF RLOSS - CHEMICAL LOSSES AND CROSS-PRODUCTS
c ---------------------------------------------

c ---------------------------------------------
c CALCULATE SPECIES CONCENTRATIONS:  INTERNAL PAIRS
c ---------------------------------------------
c
c  THE ALGORITHM:  
c   For two interacting species:  A<->B
c   Back-Euler solution:  xr = xrprior * (Pi + CjiPj/Lj)/(Li-CijCji/Lj)
c                            = xrprior * (   Pi'       )/(  Li'      )
c     where Li = prior absol. loss rate + xrprior (non-steady state)
c           Pi = prior production + c_xcin
c           Cji = cross production of i from j
c           Pi', Li' = pseudo-P, L with cross prod'n from paired subspecies.
c
c     (neg. not possible since Li>Cij, Lj>Cji)
c
c   For a chain of interacting species:  A<->B, A<->C, C<->C2, etc.
c     in order from primary to subspecies:  Cij=0 for j<i
c     CjiPj, etc. is implicit sum over all paired subspecies j
c     Solve from sub-species to primary species Pi'/Li'    
c        where Pi', Li' includes Cji, j>i
c     Then solve from primary Xi to subspecies Xj 
c      with  Cij from primary Xi updated and added to Pj
c
c   This effectively uses a prior secondary partitioning (C<->C2)
c    to generate primary partitioning (A<->B, A<->C) and primary XR
c    It uses primary (A) to solve for secondary (B, C, etc.)
c      while preserving C<->C2 partitioning (implicit in Pi'/Li')
c
c   FOR MULTISOLVE/NOXSOLVE (STANDARD OPTION):
c    PAIR SOLUTION is normalized to keep original within-pair sum unchanged.
c    NOXSOLVE or MULTISOLVE changes pair sums (preserving internal partition)
c     based on interaction between pair groups.

c   OPTION for NOXSOLVE/MULTSOLVE:  
c    should the PAIR SOLUTION include cross-production from other groups?
c    LOSS includes CP to other group.
c    Either subtract prior Cij from Li, or add prior Cij to Lj
c     (ABOVE)
c ------------------------------------

c ---------
c LOOP TO  ADJUST RP, RL FOR SUB-SPECIES CROSS PRODUCTION (PSEUDO-RP, RL)
c ---------
c  RPRO, RLOSS ADJUSTED FOR INTERNAL CP PRODUCTION FROM SUB-SPECIES ONLY.
c  (This loop automatically includes each multi-species grouping.)
c  (Loop order:  from sub-species to primary species;  
c     so that primary species includes subspecies RP with sub-CP adjustment.)

c (RPRO1, RLOSS1 preserves original RPRO, RLOSS w/o adjustments)


       if(nsolv.gt.1) then
         do nc=(nsolv-1), 1, -1
           ics = ncsolv(nc)
           do ncc=(nc+1), nsolv
             if(nssolv(nc).eq.nssolv(ncc)) then
               icc = ncsolv(ncc)
c              do kk=1,c_kmax      ! kk vector loop
                 rpro(kk,nc) = rpro(kk,nc)
     *            + cpro(kk,ncc,nc) * (rpro(kk,ncc)/rloss(kk,ncc))
                 alpha(kk) = 
     *              cpro(kk,ncc,nc)*(cpro(kk,nc,ncc)/rloss(kk,ncc))
                 rloss(kk,nc) = rloss(kk,nc) - alpha(kk)
                 if(rloss(kk,nc).le.0.) then
c                   write(c_out,121 ) c_tchem(ncsol(1)), ncsol(1), 
c     *              nc, ncc, rloss(kk,nc), alpha(kk), cpro(kk,ncc,nc),
c     *               cpro(kk,nc,ncc), rloss(kk,ncc)
c121                format(/,'MAJOR ERROR IN CHEMSOLVE: ',
c     *                      'RLOSS = 0 FROM CPRO ADJUSTMENT.',/,
c     *                      ' ic1, nc, ncc=  ', a8,3i4,/,
c     *                      ' rloss, alpha, cpro-nc, cpro-ncc, rl-ncc='
c     *                      ,/, (5(1pe10.3))    )
                   rloss(kk,nc) = rloss(kk,nc) + alpha(kk)
                 endif
c              enddo          !kk vector loop

c   CHEMSOLVE DEBUG WRITE - RPRO, RLOSS  AFTER CPRO PAIR ADJUSTMENT
c     (RP, RL adjusted as pseudo-RP, RL with CP from subspecies to primary.
c       FINAL RP below will also include CP from primary to subspecies.) 

c              if(c_kkw.gt.0.  and.nsol.gt.1) then
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) then
c                write(c_out,113) c_tchem(ics), c_tchem(icc),
c    *             rpro1(c_kkw,nc),rpro(c_kkw,nc),rloss1(c_kkw,nc)
c    *            , rloss(c_kkw,nc)   
c 113               format(' CHEMSOLVE CP PSEUDO-RP,RL: ',
c    *                   ' FOR   ', a8,'    FROM   ',a8,/,
c    *             'RPRO1, RPRO, RLOSS1, RLOSS = '  
c    *              ,/,(8(1pe10.3)))
c              endif

             endif      ! nssolv(nc)
           enddo    !ncc=(nc+1), nsolv
         enddo      !nc=nsolv, 1, -1
       endif      ! nsolv.gt.1


c ---------
c  END LOOP TO ADJUST RP, RL FOR CP
c ---------
c
c ---------
c  LOOP TO SOLVE SPECIES CONCENTRATIONS- INTERNAL PAIRS
c ---------
c   Order:  primary-to-subspecies chain among paired species
c   After calculation, add modified CPRO to CP-down-species.
c

       do nc=1, nsolv
         ics = ncsolv(nc)
         is=nssolv(nc)
         icpair = c_nppair(ics,2)

c   BACKWARD EULER SOLUTION FOR SINGLE SPECIES 
c    The true equation is =RP/(RL/XR); modified to avoid second divide.
c     (skip from here - move lower)

c c                      do  kk=1,c_kmax      ! kk vector loop
c        if(.not.llexpo(kk).or.c_iter.le.2) then
c          xrr(kk, nc)= xrp(kk,nc)*rpro(kk,nc)/rloss(kk,nc)
c        endif       !if(.not.llexpo) then
c c                      enddo            ! kk vector loop
c
c  EXPONENTIAL DECAY OPTION:  SINGLE SPECIES ONLY.

c  EQUATION:  dc/dt= r-kc, c=Co at t=0. (r=rp/time +emission rate)
c  Ct = r/k + (Co - r/k) exp(-kt)
c  Cavg = r/k + ((Co-r/k)/kt)*(1- exp(-kt))    =  (Co-Cf)/kt + r/k
c       Cf= Co + rt - kt*Cav    --checks
c
c Here, Co = xcin - xcemit
c       rt = rpro-xcin+xcemit
c       kt = (rloss-xrp)/xrp
c
c  Result saved as:
c       xc = Cavg (used for reaction rate calculations)
c        (xrr = Cavg, then turned into Cavg/Cprior 'NEW/OLD' below,,
c                 and used to get xc gas,aq -as in standard back-Euler.)
c       xcfinr = Cf/Cavg  (used in postlump to get xcout)
c       NOTE: xcfinr=1 in BACK-EULER solution.

c ERROR - Original used ic, not ics in xcfinr(ic) with xcin, xcemit.
c         Corrected
c
c ERROR OCCURS when xrp = 1e-1, xcin =1e-4 after re-set lumped species.
c
c CORRECTIONS:  llexpo only for iter>2.  llexpo only for rates >1.
c                correction - to skip if xfinr is wild
c  


c                      do  kk=1,c_kmax      ! kk vector loop
         doneexpo(kk) = .FALSE. 
         if(llexpo(kk)                ) then

           if(xrp(kk,nc).gt.0
     *        .and. ( rloss(kk,nc)-xrp(kk,nc) .gt.0. )
     *        .and. ( rloss(kk,nc)+rpro(kk,nc).gt.1. )
     *                                       ) then

c    alpha = kt =(rloss-xrp)/xrp
           alpha(kk) = rloss(kk,nc)/xrp(kk,nc) - 1.
c
c    beta = rt/kt = (rpro-xcin+xcemit)/alpha 
           beta(kk) = (rpro(kk,nc)-c_xcin(kk,ics)+c_xcemit(kk,ics)) 
     *               /alpha(kk)


c    Cf =   r/k + (    Co     - r/k) exp(-kt)
c    Cf =  beta + (xcin-xcemit-beta)*exp(-alpha)
c    NOTE: This must be greater than (xcin-xcemit)
c
c ORIGINAL ERROR - USED ic NOT ics HERE AND BELOW: 
c    xcfinr=, xrr =, xcfinr =, xcfinr =

c ORIGINAL ALGORITHM - large number numerical error
c            xcfinr(kk,ics) = beta(kk) 
c    *          +(c_xcin(kk,ics) - c_xcemit(kk,ics) - beta(kk))
c    *           *exp(0.-alpha(kk))

c FIXED ALGORITHM
             xcfinr(kk,ics) = beta(kk) * (1.- exp(0.-alpha(kk)) )
     *          +(c_xcin(kk,ics) - c_xcemit(kk,ics)           )
     *           *exp(0.-alpha(kk))

c    Cav = (    Co     -Cf)/kt    + r/k
c    Cav = (xcin-xcemit-Cf)/alpha + beta
c
c
             xrr(kk,nc) = beta(kk)
     *          +( c_xcin(kk,ics) - c_xcemit(kk,ics) - xcfinr(kk,ics))
     *          /alpha(kk)
c
c Cf/Cav  - with ZERO PROTECT
             if(xrr(kk,nc).gt.0) then
                xcfinr(kk,ics) = xcfinr(kk,ics)/xrr(kk,nc)

c Done-expo flag. (within ZERO PROTECT loop)
c  OPTION:  protect against wild values
                doneexpo(kk) = .TRUE. 
c               if(xcfinr(kk,ics).lt.10.and.xcfinr(kk,ics).gt.0.1) 
c    *             doneexpo(kk) = .TRUE. 

             endif     !if(xrr(kk,nc).gt.0) then

c SPECIAL CHEMSOLVE DEBUG WRITE - LLEXPO SOLVER
c          if(c_kkw.gt.0. and. 
c    *         (c_tchem(ic).eq.'    R3N1'.or.
c    *          c_tchem(ics).eq.'    R3N1')        ) then
c            write(c_out,251) rloss(c_kkw, nc), xrp(c_kkw,nc), 
c    *                                                alpha(c_kkw)
c 251          format(/,'LLEXPO DEBUG: rloss, xrp, alpha=rl/xrp-1:',/,
c    *         (8(1pe10.3))   )
c            write(c_out,254) rpro (c_kkw, nc), c_xcin(c_kkw,ics), 
c    *            c_xcemit(c_kkw,ics),               beta(c_kkw)
c            write(c_out,*) ic, ics, c_tchem(ic), c_tchem(ics)
c            write(c_out,*) c_iter
c 254          format(/,'LLEXPO DEBUG: rpro,xcin, xcemit, beta=rt/kt',/,
c    *         (8(1pe10.3))   )
c            write(c_out,256)  xcfinr(c_kkw,ic) 
c 256          format(/,'XCFINR = ', 1pe10.3)
c          endif    !if(c_kkw.gt.0.)
c END DEBUG WRITE

c          else            !if(alpha(kk).gt.0) then

           endif           !if(xrp(kk,nc).gt.0) then

         endif       !if(.not.llexpo) then

c   BACKWARD EULER SOLUTION FOR SINGLE SPECIES 
c    The true equation is =RP/(RL/XR); modified to avoid second divide.

c  Use Back-Euler solution if option is selected, or as fallback if expo fails.
c  Note:  ZERO PROTECT above for rloss.  

         if(.not.doneexpo(kk)) then 
            xrr(kk, nc)= xrp(kk,nc)*rpro(kk,nc)/rloss(kk,nc)
            xcfinr(kk,ics) = 1.

c TEMPORARY DEBUG WRITE                          
c          if(c_kkw.gt.0. and.c_tchem(ics).eq.'    R3N1')  then
c             write(c_out,262) 
c             write(c_out,*)c_iter, c_tchem(ics), doneexpo(c_kkw) 
c 262           format(/, 'LLEXPO DEBUG: BACK EULER SOLUTION FOR: ')
c          endif

         endif                 !if(.not.doneexpo(kk) then 

c                      enddo            ! kk vector loop



c    H2O2 SPECIAL OPTION (for AQUEOUS CO3 NONCONVERGENCE)
c      MAXIMUM H2O2 = PRIOR H2O2 + ODDHSRC
c      TO USE THIS, NEED TO SUM ODDHSRC (sources only) ALONG WITH ODDHSUM. 
c 2009 modification
c 
         if(c_tchem(ics).eq.'    H2O2') then
c          do kk=1,c_kmax      ! kk vector loop
             if(xrr(kk,  nc).gt.oddhsrc(kk,1)+  c_xcin(kk,ics)) then
                xrr(kk, nc)  =oddhsrc(kk,1)+  c_xcin(kk,ics)

c ZERO PROTECT CORRECTION 52004 - cut, should never be zero
c  --> if it  goes below 1e-30 can be zero!!!
c           if(xrr(kk, 1).lt.1.0e-30) xrr(kk,1)=1.0e-30
             endif
c (THIS LINE REPLACED WITH SECTION IMMEDIATELY BELOW.)
c            xrr(kk, nc) =(xrr(kk,nc) **0.70) * (xrp(kk,nc)) **0.30
c          enddo       ! kk vector loop
         endif       !if(c_tchem(ics).eq.'    H2O2')

c   ***DIFFICULT CONVERGENCE OPTION***   
c      GEOMETRIC MEAN (0.7 or 0.5) IF FLAGGED.
c   FLAG FOR:  H2O2   and SELF-REACTION
c   (Self-reaction flag for SO5- +SO5- as well as H2O2).
c
c   If CO3, H2O2 here - slow convergence. Else, CO3-H2O2 small oscillation
c     (changed:  CO3 linked to Hx. H2O2 remains here)


c SPECIAL FIX FOR NO3, N2O5(1/08):  set lself=F;  these must not be solved
c    with geometric avg - messes up NOx
c

           if(c_tchem(ics).eq.'     NO3'
     &        .or.c_tchem(ics).eq.'    N2O5' ) lself = .FALSE.
           if(ics.eq.4.or.ics.eq.5) lself = .FALSE.



         if(lself
     *      .or. c_tchem(ics).eq.'    H2O2'
c    *      .or. c_tchem(ics).eq.'     CLG'    !  fails
c    *      .or. c_tchem(ics).eq.'    HCLG'
c    *      .or. c_tchem(ics).eq.'    HCL2'
c    *      .or. c_tchem(ics).eq.'     CLO'
c    *      .or. c_tchem(ics).eq.'     NO3'    ! fails
c    *      .or. c_tchem(ics).eq.'    N2O5'    ! fails
c    *      .or. c_tchem(ics).eq.'     CO3'
c    *      .or. c_tchem(ncsol(1)).eq.'    HBRG'
     *                                            ) then
c          do kk=1,c_kmax      ! kk vector loop
c             xrr(kk, nc) =(xrr(kk,nc) **0.85) * (xrp(kk,nc)) **0.15
c             xrr(kk, nc) =(xrr(kk,nc) **0.70) * (xrp(kk,nc)) **0.30
              xrr(kk, nc) =(xrr(kk,nc) **0.50) * (xrp(kk,nc)) **0.50
c          enddo        ! kk vector loop
         endif       ! lself

c DIFFICULT CONVERGENCE OPTION:  SETGEO IF FLAGGED
c   FLAG FOR H2O2 - EITHER ABOVE (AUTOMATIC) OR HERE
c   FAILS

c        if(c_tchem(ics).eq.'    xxxx' 
c    *                                            ) then
c          if(c_iter.ge.4) then
c            call setgeom(ics,  geomavg, history, c_iter)
c          endif               !if(c_iter.ge.4) then
c          do kk=1, c_kmax          ! kk vector loop
c             xrr(kk, nc) =(xrr(kk,nc) **geomavg(kk,ics)) 
c    *                     * (xrp(kk,nc)) **(1.-geomavg(kk,ics))
c             history(kk,ics,c_iter) = xrr(kk,nc)
c          enddo                  ! kk vector loop
c        endif              !if(c_tchem(ics).eq.'    H2O2' 

c DIFFICULT CONVERGENCE OPTION:   (ADD 30905)
c  PROTECT AGAINST WILD OSCILLATION OF HNO3 FOR FIRST FEW ITERS
c  (For NO3->HNO3->NOx error with high BR)
c  (HNO3 icat 16;  may also use for all slow species icat=1)

         if(c_iter.le.3.and.c_icat(ics).eq.16) then
c        if(c_iter.le.3.and.(c_icat(ics.eq.1.or.c_icat(ics).eq.16)) then
c          do kk=1, c_kmax          ! kk vector loop
              if(xrr(kk, nc).gt.5.*xrp(kk,nc))
     *             xrr(kk, nc) =  5.*xrp(kk,nc)
              if(xrr(kk, nc).lt.0.2*xrp(kk,nc))
     *             xrr(kk, nc) = 0.2*xrp(kk,nc)
c          enddo                  ! kk vector loop
         endif             !if(c_iter.le.3.and.c_icat(ics).eq.16) then

c   REGULAR DEBUG WRITE 
        if(c_kkw.gt.0                 ) 
c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) 
     *        write(c_out,101) c_tchem(ics), ics,xrr(c_kkw, nc) ,
     *   xrp(c_kkw,nc) ,rpro(c_kkw,nc), rloss(c_kkw,nc)
 101    format(' CHEMSOLVE (PAIRED SPECIES):  IC  XR  XRP  RPRO  RLOSS'
     *      ,/,a8,2x,i5,(8(1pe10.3)))
 
c   ADD SOLUTION TO PAIR SUM FOR PAIR GROUP
         if(nsolv.ge.3.and.nssolv(nsolv).eq.1) then
c          do kk=1,c_kmax          ! kk vector loop
             xrrpair(kk) = xrrpair(kk) + xrr(kk,nc)
     *                                      * c_pairfac(ics)
c          enddo                 ! kk vector loop

         endif          !if(nsolv.ge.3.and.nssolv(nsolv).eq.1) then

c   SUM XRRM FOR NORMALIZATION FOR MULTISOLVE
c
         if(nsol.gt.1                       ) then
c          do kk=1,c_kmax      ! kk vector loop
c            xrm(kk,is) =  xrm(kk,is) + xrp(kk,nc)*c_pairfac(ics) ! OLD ERR
             xrrm(kk,is) = xrrm(kk,is) + xrr(kk,nc)
c          enddo           ! kk vector loop
         endif          !if(nsol.gt.1                       ) then
c
c    RESET XRR (TOTAL SPECIES SOLUTION) EQUAL TO NEW/OLD RATIO
c        do kk=1,c_kmax      ! kk vector loop
c          xrm(kk,is) = xrm(kk,is) + xrp(kk,nc)        ! original
c          xrrm(kk,is) = xrrm(kk,is) + xrr(kk,nc)        ! original
           xrr(kk,nc) = xrr(kk,nc)  /xrp(kk, nc)
c        enddo           ! kk vector loop
c
c   PARTITION THE TOTAL SPECIES BETWEEN GAS-MASTER AND AQUEOUS
c    USING THE SAME GAS-AQUEOUS RATIOS AS PRIOR.

c    ALL CONCENTRATIONS (GAS AND AQUEOUS) ARE UPDATED BY THE RATIO XR/XRP.  
c     (PRIOR TIME STEP, XRIT, PRESERVED HERE)

         do  neq=1,(c_nequil(ics)+1)
           ic=ics
           if(neq.gt.1) ic = c_ncequil(ics,(neq-1))
c  TEST WRITE - GAS/AQUEOUS  
c c          if(c_kkw.gt.0.and.c_tchem(ics).eq.'     CO3') 
c          if(c_kkw.gt.0                         ) 
c    *        write(c_out,  131) c_tchem(ic), xc(1,ic),xrp(1,nc)
c 131        format(' CO3 AQUEOUS TEST.  PRIOR/POST XR XRP=',2x,a8,
c    *             2(1pe10.3))

c          do kk=1,c_kmax      ! kk vector loop
              xc(kk,ic) = xc(kk,ic) *xrr(kk,nc)  
              c_xcout(kk,ic) = c_xcout(kk,ic)
c          enddo        ! kk vector loop

c  TEST WRITE - GAS/AQUEOUS
c c        if(c_kkw.gt.0.and.c_tchem(ncsol(1)).eq.'     CO3') 
c          if(c_kkw.gt.0                         ) 
c    *       write(c_out,  131) c_tchem(ic), xc(1,ic),xc(1,ncsol(1))

         enddo  ! do  neq=1,(c_nequil(ics)+1)

c   END - PARTITION BETWEEN GAS AND AQUEOUS 

c   UPDATE RLOSS, CPRO, AND XRP BASED ON NEW SPECIES CONCENTRATION
c    AND ADD UPDATED CPRO TO RPRO FOR PRODUCTION TO PAIRED SUBSPECIES  
c
c    (updated CPRO only used here for sub-species; 
c       since primary species formula assumes original CPRO, XRP
c       but update done for all CPRO, to be consistent with updated XRP.
c     Updated RLOSS, CPRO, XRP will be used in MULTISOLVE.)


c  UPDATE RLOSS, XRP
c        do kk=1,c_kmax      ! kk vector loop
           rloss(kk,nc) = rloss(kk,nc) * xrr(kk,nc)
           xrp(kk,nc) = xrp(kk,nc) * xrr(kk,nc)
c        enddo          !kk vector loop
c
c UPDATE CPRO
         do ncc=1,nsolv
c          do kk=1,c_kmax      ! kk vector loop
              cpro(kk,nc,ncc) = cpro(kk,nc,ncc) * xrr(kk,nc)
c          enddo          !kk vector loop
         enddo            !do ncc=1,nsolv
c 
c UPDATE RPRO
         if(nsolv.gt.nc) then
           do ncc=(nc+1), nsolv
             if(nssolv(nc).eq.nssolv(ncc)) then
c              do kk=1,c_kmax      ! kk vector loop
                 rpro(kk,ncc) = rpro(kk,ncc) + cpro(kk,nc,ncc)
c              enddo               ! kk vector loop
             endif              !if(nssolv(nc).eq.nssolv(ncc))
           enddo              !do ncc=(nc+1), nsolv
         endif           !if(nsolv.gt.nc) 

c
c  FUTURE CHANGE - use this instead of UPDATE CPRO, RPRO just above.
c UPDATE CPRO AND RPRO for sub-species only
c        if(nsolv.gt.nc) then
c          do ncc=(nc+1), nsolv
c            if(nssolv(nc).eq.nssolv(ncc)) then
c c              do kk=1,c_kmax      ! kk vector loop
c                cpro(kk,nc,ncc) = cpro(kk,nc,ncc) * xrr(kk,nc)
c                rpro(kk,ncc) = rpro(kk,ncc) + cpro(kk,nc,ncc)
c c              enddo               ! kk vector loop
c            endif              !if(nssolv(nc).eq.nssolv(ncc))
c          enddo              !do ncc=(nc+1), nsolv
c        endif           !if(nsolv.gt.nc)


       enddo         !nc=1,nssolv

c ---------
c  END LOOP TO SOLVE SPECIES CONCENTRATIONS- INTERNAL PAIRS
c ---------
c ---------
c  NORMALIZATION:  UPDATE SPECIES FOR PAIR GROUP CONSERVATION OF MASS
c                  OR FOR MULTISOLVE TO PRESERVE ORIGINAL PAIR GROUP SUM.
c  OPTION:  UPDATE SPECIES CONCENTRATIONS FOR PAIR GROUP CONSERVATION OF MASS
c ---------
c   PAIR GROUP SUM IS USED ONLY IF PAIR GROUP HAS >3 MEMBERS
c     AND NO MULTISOLVE (since multisolve normalizes from prior )

c FROM THIS VERSION - FUTURE CHANGE (remove normalization in multisolve)
         if(nsolv.ge.3.or.nsol.gt.1         ) then

c  DO FOR ALL PAIR GROUPS
c   Indices:
c     nc=chemsolve species counter number
c     is = chemsolve group number
c     icpair = pair group lead species
c     ics = this species of pair group.
c      (This loop is equivalent to:
c           nc = 1, nsolv;   ics = ncsolv(nc) )

           do is=1,nsol
c            icc=ncsol(is)
c            icpair=ncsol(is)
             icpair = c_nppair(ncsol(is),2)

c     MULTISOLVE NORMALIZATION FACTOR:  XRM/XRRM 
c       (note, XRM different from XRPAIR:  XRPAIR includes PAIRFAC)
c
             if(nsol.gt.1) then
c              do kk=1,c_kmax      ! kk vector loop
                 alpha(kk) = 1.

                 if(xrrm(kk,is).gt.0) then
                   alpha(kk) = xrm(kk,is)/xrrm(kk,is)
                 endif            !if(xrrm(kk,is).gt.0) then

c              enddo             ! kk vector loop
 
c     PAIR GROUP NORMALIZATION FACTOR:
c        Group sum = RP/RL.  Adj*XRR = XRP*RP/RL, Adj=(XRP/XRR)*(RP/RL)
c
             else              !if(nsol.gt.1) then
c              do kk=1,c_kmax      ! kk vector loop
                 alpha(kk) = 1.
                 if(rlpair(kk,icpair).gt.0.and.xrrpair(kk).gt.0) then
                    alpha(kk) =   (xrppair(kk)/xrrpair(kk))
     *               * (rppair(kk,icpair)/rlpair(kk,icpair))
                 endif        !if(rlpair(kk).gt.0.and.xrrpair(kk).) then

c c            enddo             ! kk vector loop
             endif             !if(nsol.gt.1) then

c DO FOR ALL SPECIES IN PAIR GROUP


             nc=0
             do iss=1,(c_nppair(icpair,3)+1)
               nc=nc+1
               ics = icpair
c              ics = c_nppair(icpair,2)   ! no difference
               if(iss.gt.1) then
                 ics = c_nppair(icpair,iss+2)
               endif
c              icpair = c_nppair(ics,2)    ! should be equal to setting above.
c
c
c PAIR GROUP NORMALIZATION.

             do  neq=1,(c_nequil(ics)+1)
               ic=ics
               if(neq.gt.1) ic = c_ncequil(ics,(neq-1))
c                do kk=1,c_kmax      ! kk vector loop
                  xc(kk,ic) = xc(kk,ic) *alpha(kk)
c                enddo        ! kk vector loop
             enddo         ! do  neq=1,(c_nequil(ics)+1)

             enddo              !do i=1,(c_nppair(ic,3)+1)
c END LOOP - DO FOR ALL SPECIES IN PAIR GROUP

c CHEMSOLVE REGULAR DEBUG WRITE
          if(c_kkw.gt.0) 
c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) 
     *         write(c_out,102) xrrpair(c_kkw)
     *        , xrppair(c_kkw), rppair(c_kkw,icpair)
     *         , rlpair(c_kkw,icpair),  alpha(c_kkw)
     *           ,gamma(c_kkw)
 102        format(' CHEMSOLVE PAIR ADJUSTMENT: xrrpair xrppair ',
     *            'rppair  rlpair ADJ. FACTOR:',/,(5(1pe10.3)))

           enddo             !do is=1,nsol
c END LOOP - DO FOR ALL PAIR GROUPS.

         endif          !if(nsolv.ge.3.and.nssolv(nsolv).eq.1) then

c ---------
c  END UPDATE SPECIES CONCENTRATIONS FOR PAIR GROUP SUM CONSERVATION OF MASS
c ---------
c ---------------------------------------------
c END CALCULATE SPECIES CONCENTRATIONS:  INTERNAL PAIRS
c ---------------------------------------------

c
c ---------------------------------------------
c CALCULATE SPECIES CONCENTRATIONS: MULTISOLVE AND NOXSOLVE
c ---------------------------------------------
c
c   The algorithms:  
c    MULTISOLVE does a full back-euler matrix inversion
c     using LINSLV/RESOLV to solve for >2 rapidly interacting species
c    It solves for interacting species groups 
c     where each group may consist of a pair or pair chain.
c    The groups are identified by input to CHEMSOLV (currently ic1, ic2, ic3)
c    MULTISOLVE would preserve the previously calculated ratio 
c     among paired species or species pair chains
c     and aqueous equilibria.
c
c    NOXSOLVE is a special solution for O3-NO-NO2 
c      based on the rapid exchange O3+NO<->NO2
c      it uses preliminary calculations in CHEMSOLV and then calls NOXSOLVE.
c      
c   FOR MULTISOLVE/NOX-SOLVE (STANDARD OPTION):
c    PAIR SOLUTION is normalized to keep original within-pair sum unchanged.
c    NOXSOLVE or MULTISOLVE changes pair sums (preserving internal partition)
c     based on interaction between pair groups.
c    (implemented above in INTERNAL PAIRS )
c
c NOTE:  pair group sums were set above in INTERNAL PAIRS
c   xrm = prior pair group sum 
c   xrrm = updated pair group sum (not done yet)
c   xrp = updated individual (gas+aq) species sum from PAIR SOLUTION.
c 
c  ---------
c  MULTISOLVE CONTROL:  DO ONLY IF MULTISOLVE SPECIES NONZERO
c  ---------
c      if(nssolv(nsolv).gt.1) then
       if(nsol.gt.1         ) then
c
c ---------
c LOOP TO NORMALIZE INTERNAL PAIR SOLUTION FOR NOXSOLVE/MULTISOLVE
c ---------
c  CUT - DONE ABOVE

c  NOTE:  CHEMISTRY PRODUCTION, LOSS SET ABOVE
c   NOT UPDATED AFTER PAIR SOLUTION.

c ---------
c  CALL NOXSOLVE 
c ---------
c NOXSOLVE solves for (O3, NO2, NO) in that order
c    using O3+NO<->NO2 - Ox and NOx, and counted sourcnx, sourcox, etc.
c    O3, NO2, etc. can also be part of a pair (for O3<->O1D)
c Solution is returned as XRRM(KK,IS)
c

c  (NOXSOLVE OPTION - EITHER CALL NOXSOLVE OR USE MULTISOLVE HERE.
c    IF MULTISOLVE, CONVERGENCE IS SLOWER AND MORE DIFFICULT)
c

         if (ncsol(1).eq.c_no3      .and.ncsol(2).eq.c_nno2) then 

c        if(c_iter.eq.-1) then    ! ALT TO ELIMINATE

             call noxsolve(ncsol(1),ncsol(2),ncsol(3) ,

     &    c_xcin, c_xcout, c_xcav, c_xcemit,
     &    c_rp, c_rl, c_rr ,
     &    xc,xcfinr,  xclastq, 
     &    ratek, rateh, rateq, rateqq, rhdif, egasaq, 

     &     xrp, xrr, rpro, rloss, rpro1, rloss1, 
     &       rrp, rrl, cpro,
     &       xrm, xrrm, rpm, rlm,cpm, rpmulti, rlmulti,
     &       rppair, rlpair, xrppair, xrrpair, 
     &       history, geomavg 
     &       ,c_notest, c_iter, senhcat ,sourcnx, sinknx)
c 

c CONVERT SOLUTION INTO NEW/OLD RATIO (XRRM/XRM)
           do is=1,nsol
c            do kk=1,c_kmax      ! kk vector loop
               xrrm(kk,is) = xrrm(kk,is)  / xrm(kk, is)
c            enddo             ! kk vector loop
           enddo             !do is=1,nsol

         else             !if(c_tchem(ncsol(1)).eq.    '  O3'.and
c 
c  ---------
c  SET UP AND CALL MULTISOLVE
c  ---------
c   FUTURE DO:       
c      Control to replace MULTISOLVE with simple solution if CPM=0.
C      Possible:  call RESOLV for easy convergence -
c        must then save and replace reduced AX, IPA.
c
c   ENTER VECTOR AND MATRIX FOR AX=B BACK-EULER SOLUTION
c       Equation:  Xi = Xio + Pi(X)-Li(X) = Xio +Pi(Xp)-Li(Xp) +dRi/dxj (Xj-Xjp)
c                  Xi-Xip= (Xio + Pi(Xp)) - (Xip + Li(Xp)) + d(P-L)/dxj (Xj-Xjp)
c                 (I-dR/dxj)(Xj-Xjp) = ((Xio + Pi(Xp)) - (Xip + Li(Xp))
c               [Xjp - (dR/dxj)*Xjp][Xj/Xjp-1] = (Xio + Pi(Xp)) - (Xip + Li(Xp)
c                      AX     *    XX          =   BX
c       XX = xrrm - 1
c       BX = rpm - rlm
c       AA(i,i) = rlm
c       AA(i,j) = -cpm(j,i) for j.ne.i

c ALLOCATE AX, BX, CX. Allocate here - moved to quadchem

C  VECTOR LOOP - solution is not vectorized
c        do kk=1,c_kmax                 ! kk vector loop

c  SET ARRAY

           do is=1,nsol
            BX(is) = rpm(kk,is) - rlm(kk,is)
            do iss=1,nsol
              AX(is,iss) = 0. - cpm(kk,iss,is)
            enddo                 !do iss=1,nsol
            AX(is,is) = rlm(kk,is)
           enddo                !do is=1,nsol

c TEMPORARY MULTISOLVE DEBUG WRITE  ncsol(is)
c         if(c_kkw.gt.0) then
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) then
c           do is=1,nsol
c            write(c_out,221) is, c_tchem(ncsol(is))
c 221          format(/'MULTISOLVE: IS, SPECIES = ',i4,2x,a8)
c            write(c_out,222) rpm(c_kkw,is), rlm(c_kkw,is), BX(is)
c 222          format('   RPM RLM BX(=rpm-rlm) = ',5(1pe10.3))
c            write(c_out,223) (cpm(c_kkw,iss,is),iss=1,nsol)
c            write(c_out,223) (AX(is,iss),iss=1,nsol)
c 223          format('  CPM/AX = ', 4(1pe10.3))
c           enddo          !do is=1,nsol
c         endif

c CALL MULTISOLVE
           call LINSLV(AX, BX, XX, nsol)

c ENTER RESULT INTO XRRM (ratio adjustment)
c OPTION:  Limit size of change, also protect against zero.
           do is=1,nsol
             xrrm(kk,is) = XX(is) + 1.
             if(xrrm(kk,is).lt.0.001) xrrm(kk,is) = 0.001
           enddo                !do is=1,nsol

c        enddo                        ! kk vector loop
C END  VECTOR LOOP - solution is not vectorized
c

c  MULTISOLVE NORMALIZATION OPTION
c         do kk=1,c_kmax            ! kk vector loop
            gamma(kk) = 0.
            beta(kk) = 0.
c         enddo                   ! kk vector loop
          do is=1,nsol
c           ic =ncsol(is)
            ic = c_nppair(ncsol(is),2)
c           do kk=1,c_kmax            ! kk vector loop
              gamma(kk) = gamma(kk) + xrm(kk,is)
     *                  *c_multfac(ic)
               beta(kk) =  beta(kk) + xrm(kk,is)*xrrm(kk,is)
     *                  *c_multfac(ic)
c           enddo                   ! kk vector loop
          enddo                   !do is=1,nsol

c         do kk=1,c_kmax      ! kk vector loop
           alpha(kk) = 1.
           if(rlmulti(kk).gt.0.and. beta(kk)  .gt.0) then
              alpha(kk) =   (gamma(kk)  /beta(kk)   )
     *               * (rpmulti(kk)      /rlmulti(kk)      )
           endif          !if(rlmulti(kk).gt.0.and. beta(kk)  .gt.0) then
          do is=1,nsol
c           do kk=1,c_kmax            ! kk vector loop
c OPTION HERE
              xrrm(kk,is) = xrrm(kk,is) * alpha(kk)
c           enddo                   ! kk vector loop
          enddo                   !do is=1,nsol

c TEMPORARY MULTISOLVE DEBUG WRITE - MULTISOLVE NORMALIZATION
         if(c_kkw.gt.0) then
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) then
           write(c_out,226 ) beta(c_kkw), gamma(c_kkw),
     *       rpmulti(c_kkw), rlmulti(c_kkw), alpha(c_kkw)
 226        format(/,'MULTISOLVE NORMALIZE: XRR XRP RP RL NORMFAC=',
     *              /,5(1pe10.3))
         endif               !if(c_kkw.gt.0) then

c c            enddo             ! kk vector loop

c DEALLOCATE AX, BX, XX for multisolve - Deallocate here, moved to quadchem.


         endif            !if(c_tchem(ncsol(1)).eq.'      O3'.and
c  ---------
c  END:  CALL NOXSOLVE/MULTISOLVE
c  ---------

c   REGULAR DEBUG WRITE
        if(c_kkw.gt.0)  then
c c           if(c_kkw.gt.0.   and.ncsol(1).eq.c_no3) then
          do nc=1, nsolv
            ics = ncsolv(nc)
            if(c_nppair(ics,2).eq.ics) then
              is=nssolv(nc)
              write(c_out,106) c_tchem(ics), ics,xrrm(c_kkw, is) ,
     *    xrm(c_kkw,is) , rpm(c_kkw,is),   rlm(c_kkw,is)
 106    format(' CHEMSOLVE (MULTISOLVE/NOX):  IC  XRR XRP  RPRO  RLOSS'
     *      ,/,a8,2x,i5,(8(1pe10.3)))
            endif       !if(c_nppair(ic,2).eq.ic) 
          enddo          !do nc=1, nsolv
        endif         !if(c_kkw.gt.0) 
c
c  -------
c  ENTER NOXSOLVE/MULTISOLVE VALUES FOR PAIRS AND GAS/AQUEOUS
c  -------
c  Enter for all paired species; partition among gas/aqueous
         do nc = 1, nsolv
           is=nssolv(nc)
           ics = ncsolv(nc)
           
c   UPDATE EACH PAIR SPECIES AND GAS/AQUEOUS SPECIES BY THE RATIO XRM/XRMP
c    (This preserves prior partitioning among pair species and gas/aqueous.)

           do  neq=1,(c_nequil(ics)+1)
             ic=ics
             if(neq.gt.1) ic = c_ncequil(ics,(neq-1))

c            do kk=1,c_kmax      ! kk vector loop
               xc(kk,ic) = xc(kk,ic) *xrrm(kk,is)
c            enddo        ! kk vector loop

           enddo  ! do  neq=1,(c_nequil(ics)+1)

c   END - PARTITION BETWEEN GAS AND AQUEOUS

         enddo      !do nc = 1, nsolv
c  -------
c  END - ENTER NOXSOLVE/MULTISOLVE VALUES INTO PAIRS AND GAS/AQUEOUS
c  -------

       endif             !if(nsol.gt.1)  (nssolv(nsolv).gt.1) 
c  ---------
c  END MULTISOLVE CONTROL
c  ---------
c
c ---------------------------------------------
c END - CALCULATE SPECIES CONCENTRATIONS: MULTISOLVE AND NOXSOLVE
c ---------------------------------------------

c
c ---------------------------------------------
c CALCULATE CHEMICAL PRODUCTION AND LOSSES FOR DOWN-CASCADE SPECIES.
c ---------------------------------------------
c 
c  *** END IF-LOOP -1 TO SKIP LOSS AND XR CALC FOR SLOW-SPECIES 'PRE'.
c          (END OF ONESOLVE  'POST' LOOP)
c  CUT JANUARY 2005
c             endif               ! if(ic2.ne.-1.and.ic2.ne.-3) then

c  *** IF-LOOP -2 TO SKIP DOWN-CASCADE CALC. FOR SLOW-SPECIES 'POST'.
c         (LOOP EXECUTED FOR full ONESOLVE AND FOR 'PRE')
c  CUT JANUARY 2005
c             if(ic2.ne.-2) then
c  ***
c

c  LOOP TO CALCULATE DOWN-CASCADE PRODUCTION 
c  LOOP TO  RECORD RRP, RRL AND ZERO RP, RL.

c   Note:  brpro adds losses, cross-production to RP, RL (for record).
c   Subsequent down-cascade production will be include in next iteration.


       do nc=1, nsolv      
         ics = ncsolv(nc)

c TEMPORARY TEST WRITE (why is loop so long?)
         if(c_kkw.gt.0) write(6,*) 'chemsolve brpro loop: ics='
     &        ,ics, c_tchem(ics)

         if(c_nnrchem(ics).gt.0) then
           do i=1,c_nnrchem(ics)
             nr =  c_nrchem(ics,i)      

c TEMPORARY TEST WRITE (why is loop so long?)
         if(c_kkw.gt.0) write(6,*) 'reaction: ',
     &          nr, c_treac(1,nr), c_treac(2,nr)

             call brpro(nr, ratek,
     &         c_rr, xc, rrl, rrp, rppair, rlpair, c_h2oliq
     &        , senshx, senhcat
     &        , oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2, xfohtest
     &        ,sourcnx, sinknx
     &        )
           enddo        !do i=1,c_nnrchem(ics)
         endif        !if(c_nnrchem(ics).gt.0)
       enddo        !do nc=1,nsolv


c DUPLICATE RPRO LOOP  to test loop time 
c  WITH DUMMY BRPRO2 
c  CUT FROM HERE, see TRY8c




c LOOP TO CALL EXCORR
c  EXCORR adjusts down-cascade RP, RL, PRONOX and  ODD-H SENSITIVITY
c    for back-forth exchange reactions.
c
c  This avoids bad solution when down-cascade summed RP, RL are dominated
c    by  a rapid back-forth reaction (MCO3<->PAN, HNO4<->NO2+HO2)
c
c  The exchange reaction impact would be exaggerated down-cascade
c    by a straight solution, the down-cascade solution would not 
c    account for rapid back-forth exchange 
c      (= negative feedback for changed rates)
c
c  To avoid this, down-cascade RP, RL, PRONOX and ODD-H SENSITIVITY
c   are reduced by an amount that reflects back-forth  response to 
c   change in rates.  Effective RP, RL is  reduced by linked back-reaction.
c
c

       do nc=1, nsolv      
         ics = ncsolv(nc)
         if(c_exspec(ics,1).ne.0) call excorr(ics, 
     &    c_xcin, c_xcout, c_xcav, c_xcemit,
     &    c_rp, c_rl, c_rr ,
     &    xc,xcfinr,  xclastq, 
     &    ratek, rateh, rateq, rateqq, rhdif, egasaq, 
     &     xrp, xrr, rpro, rloss, rpro1, rloss1, 
     &       rrp, rrl, cpro,
     &       xrm, xrrm, rpm, rlm,cpm, rpmulti, rlmulti,
     &       rppair, rlpair, xrppair, xrrpair, 
     &       history, geomavg 
     &    , c_h2oliq, senshx, senhcat
     &    , oddhsum, oddhdel,  sourcnx, sinknx
     &     )
c 

       enddo        !do nc=1,nsolv
c

c  LOOP TO  RECORD RP, RL 
c  NOTE:  RRP, RRL represent running sum through cascade.
c         RP, RL represent final sum through cascade.
c  ALSO- these are used in AQUASOLVE.
c
c OPTION:  SUM GAS+AQUEOUS RP, RL INTO GAS-ONLY RP, RL?
c   no, this may mess up AQUASOLVE.
c
c OPTION:  THIS CAN BE MOVED ABOVE EXCORR 
c    so that it preserves full RP, RL for exchanged species
c    (but may not record back-forth for  down-cascade)

       do nc=1, nsolv      
         ics = ncsolv(nc)
         do  neq=1,(c_nequil(ics)+1)
           ic=ics
           if(neq.gt.1) then
             ic = c_ncequil(ics,neq-1)
c 
c OPTION: RECORD SUM OF GAS+AQUEOUS PRODUCTION IN GAS-MASTER
c            c_rp(kk,ics) = c_rp(kk,ics) + rrp(kk,ic)
c            c_rl(kk,ics) = c_rl(kk,ics) + rrl(kk,ic)
c
           endif            !if(neq.gt.1) then

c          do kk=1,c_kmax          ! kk vector loop
             c_rp(kk, ic) = rrp(kk,ic)
             c_rl(kk,ic) = rrl(kk,ic)
c          enddo                 ! kk vector loop

         enddo         !do  neq=1,(c_nequil(ics)+1)
       enddo        !do nc=1,nsolv

c  LOOP TO ZERO RRP, RRL
c   ZERO RRP HERE so that running sum for next iteration includes 
c           down-cascade production.


       do nc=1, nsolv      
         ics = ncsolv(nc)
         icpair = c_nppair(ics,2)
         rppair(kk,icpair) = 0.
         rlpair(kk,icpair) = 0.
         do  neq=1,(c_nequil(ics)+1)
           ic=ics
           if(neq.gt.1) then
             ic = c_ncequil(ics,neq-1)
           endif

           rrp(kk,ic) = 0.
           rrl(kk,ic) = 0.

         enddo         !do  neq=1,(c_nequil(ics)+1)
       enddo        !do nc=1,nsolv
c
c  *** END IF-LOOP -2 TO SKIP DOWN-CASCADE CALC FOR SLOW SPECIES 'POST'.
c        (END OF LOOP FOR full ONESOLVE AND 'PRE')
c  CUT JAUNARY 2005
c             endif                  ! if(ic2.ne.-2) then
c  ***
c
c

c 
c ----------------------------------------------------------------
c END CHEMSOLVE.
c ----------------------------------------------------------------

c END CHEMSOLVE
2000  return
         end
c -------------------------------------------

c CHEMSOLVE ENDS HERE
c 

c
c


      subroutine brreac(nr, 
     &         c_rr, xc, ratek , c_h2oliq)


c BRREAC CALCULATES REACTION RATE FOR THE SPECIFIED REACTION 
c AND STORES IT IN RR(). 
c FOR AQUEOUS REACTIONS, IT CALCULATES RR IN GAS UNITS.
c CALLED BY CHEMSOLVE, NOXSOLVE, EXCORR, etc.
c
c Input:  nr  (reaction number)
c Output:  c_rr (rate of reaction, molec/cm3/step)
c 
c Called by:   
c     chemsolve
c     noxsolve
c     excorr
c
c Calls to:  none.
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c
c -------------------------------------------------------------------

c ----------------------------------------
       use h_rbchemmech
       use h_rbchemvars, only: c_time
       use h_rbchemlocal, only: avogadrl
      IMPLICIT NONE

c#     include "rbchemmech.h"
c#     include "rbchemvars.h"
c#     include "rbchemlocal.h"

c VARIABLES FROM rbchemarrays.h, rbchemlocarr.h
      integer :: nr
      real*8 ::  c_rr(c_kvec,c_rdim)  ! Reaction rate m/cm3

      real*8 :: xc( c_kvec,c_cdim) ! concentration molec/cm3
      real*8 :: ratek(c_kvec,c_rdim)  ! Rate constants
       real*8 ::  c_h2oliq(c_kvec)   ! LWC grams cm-3

!=========below are local variables ===Guangxing LIn
      integer ::  kk,icr1, icr2
       kk=1

c NOTE - ASSUME THAT ICR1>0 THROUGHOUT.
      icr1 = c_reactant(nr,1)
      icr2 = c_reactant(nr,2)
c     if(icr1.le.0) return

c  CALCULATE REACTION RATE.
c     do 15211 kk=1,c_kmax
        c_rr(kk,nr) = c_time*ratek(kk,nr)*xc(kk,icr1)
15211 continue

      if(icr2.gt.0) then
c       do 15214 kk=1,c_kmax
         c_rr(kk,nr) = c_rr(kk,nr) * xc(kk,icr2)
15214   continue
      endif

c  CONVERT AQUEOUS REACTIONS TO GAS UNITS
c    AQUEOUS REACTION IS IDENTIFIED BY REACTANT NUMBER
c       DIFFERENT FROM ITS GAS-MASTER SPECIES.
      if(icr1.ne.c_npequil(icr1)) then
c       do 15216 kk=1,c_kmax
         c_rr(kk,nr) = c_rr(kk,nr)*c_h2oliq(kk)*avogadrl
15216   continue
      endif


c END BRREAC

2000  return
      end
c ----------------------------------------


      subroutine brpro(nr, ratek,
     &         c_rr, xc, rrl, rrp, rppair, rlpair, c_h2oliq
     &        , senshx, senhcat
     &        , oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2, xfohtest
     &        ,sourcnx, sinknx
     &        )


c (compare to prelump:  xc, c_rp, c_rl, c_rr, rrp, rrl, rppair, geomav,)
c    but not: nr, ratek, c_h2oliq, senshx....)


c
c This calculates reaction rates (RR), 
c  and adds to species production (RP) and loss (RL)
c  for a given reaction (nr).
c
c It also sums species loss rates (rloss) and radical sums
c   (oddhsum, etc) for the NOx and HOx solution.
c
c It also adjusts pair sums (RPPAIR, RLPAIR)
c
c This is called from chemsolve (solution for individual species)
c  before the calculation of species concentrations, 
c  for reactions associated with species production
c
c And is called after the calculation of the species concentration
c  to get the complete sum of species losses and production
c  for all reactants and products of reactions linked to the species.
c
c  (The resulting species production/loss is used in the solution
c    for species concentrations. 
c   Note, RP and RL are set to zero when the species concentration
c    is calculated, so that species RP and RL are sums for 
c    all reactions over a full cycle of present+past iterations.)
c
c Inputs:  reaction number (nr)
c
c Outputs:  Reaction rate (RR) based on species concentrations; 
c           Species production and loss (RP, RL)
c           Chemistry production sums (oddhsum, etc.)
c
c Called by:    chemsolve.
c
c Calls to:  brreac
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c
c -------------------------------------------------------------------
c ----------------------------------------------------------
      use h_rbchemmech
c      use h_rbchemvars
c      use h_rbchemlocal, only: kk,icpair, ic1, ic2, ic, n, stoicx
      IMPLICIT NONE


c#     include "rbchemmech.h"
c#     include "rbchemvars.h"
c#     include "rbchemlocal.h"

c VARIABLES FROM rbchemarrays.h, rbchemlocarr.h
      integer :: nr
      real*8 :: ratek   (c_kvec, c_rdim)
      real*8 ::  c_rr(c_kvec,c_rdim)  ! Reaction rate m/cm3

      real*8 :: xc( c_kvec,c_cdim) ! concentration molec/cm3
      real*8 :: rrl(c_kvec,c_cdim)    ! Interim loss rate
      real*8 :: rrp(c_kvec,c_cdim)    ! Interim production rate
      real*8 :: rppair(c_kvec,c_cdim) ! Production for pair grp
      real*8 :: rlpair(c_kvec,c_cdim) ! Loss rate for pair grp
      real*8 ::  c_h2oliq(c_kvec)   ! LWC grams cm-3
      real*8 ::        senshx(c_kvec,2)   ! Hx sens: d ln(pHx)/d
      real*8 ::        senhcat( c_kvec,99) ! Hx sens. by species categ.
      real*8 ::        oddhsum(c_kvec,2)   ! Summed net P/L Hx, mol/cm3
      real*8 ::        oddhdel(c_kvec,2)   ! Summed Hx sens to Oh,
      real*8 ::        oddhsrc(c_kvec,2)   ! Summed P Hx, mol/cm3
      real*8 ::        oddhloh(c_kvec,2)   ! Hx loss from OH,  mol/cm3
      real*8 ::        oddhlho2(c_kvec,2)   ! Hx loss from HO2,  mol/cm3
      real*8 :: xfohtest          ! OH/HO2 convergence test 
      real*8 :: sourcnx(c_kvec)   ! Summed NOx source, mol/cm3
      real*8 :: sinknx(c_kvec)    ! Summed NOx sink, mol/cm3

c below are local variables ====Guangxing Lin 



         real*8 :: stoicx                         ! stoichiometry
      integer :: ic,ic1,ic2,ic3,icc,ics,ics2,icc1,icc2,icc3 !Chem index
      integer :: is,iss,iscs,isc2                    !Chem local index
          integer :: ich,icq,icx,icx1,icx2,icy1,icy2,icp,icp1,icp2 !Chem
      integer :: icr1,icr2,icr,isr1,isr2,icpair  !chem index - pair and
      integer :: ionsum                                  ! aquasolve ion
      integer :: ica1,ica2,icb1,icb2,nra1,nra2,nrb1,nrb2 !Aquasolve chem
      integer :: neq                                     !Aqueous
      integer :: nc,nc1,nc2,ncc,ncf,nn,nne               !Chem species
      integer :: nr1,nr2,nrh,nrq,nrqq,nrx,np          ! Reaction
      integer :: icat1,icat2,icatp,icatp2  ! indices used for species
      integer :: kk, kw                            ! Vectorization
      integer :: i, j, k, ii, ij, iii, n           ! General counters
       integer :: nsol           ! Number of pair groups being solved
      integer :: nsolv          ! Number of species being solved for
          real*8 :: alpha(c_kvec)   ! General vector variable
      real*8 ::  beta(c_kvec)   ! General vector variable
      real*8 :: gamma(c_kvec)   ! General vector variable
      real*8 :: prior(c_kvec)   ! Prior species conc (molec/cm3)
      integer :: ncsolv(c_cdim) ! Species number (ic) for species in
      integer :: nssolv(c_cdim) ! Pair group number (is) for spec. in
 

      kk=1



c PRELIM:  CALCULATE REACTION RATE  (alt: cut and invoke elsewhere?)
      call brreac(nr,
     &         c_rr, xc, ratek , c_h2oliq)

c
c PAIR INDEX:  IDENTIFY PAIR GROUP OF NON-KEY-SPECIES REACTANT ONLY.
c    REQUIRES TWO REACTANTS, WHERE 1ST REACTANT IS KEY SPECIES (nicreac)
        icpair = 0
          ic1 = c_npequil(c_reactant(nr,1))
          ic2=0
          if(c_reactant(nr,2).gt.0) then
            ic2 = c_npequil(c_reactant(nr,2))
            if(c_nicreac(nr).eq.ic1) then  
              if(c_nppair(ic2,2).ne.c_nppair(ic1,2) )
     *                     icpair = c_nppair(ic2,2)
            endif                !if(c_nicreac(nr).eq.ic1) then  

c IF 2ND REACTANT IS KEY SPECIES - can't handle, would make adj. below messy.
c           if(c_nicreac(nr).eq.ic2) then  
c             if(c_nppair(ic2,2).ne.c_nppair(ic1,2) )
c    *             i       icpair = c_nppair(ic1,2)
c           endif                !if(c_nicreac(nr).eq.ic1) then  

          endif                !if(c_reactant(nr,2).gt.0) then

c ENTER RATE AS LOSS (RL) FOR REACTANTS:

        ic=c_reactant(nr,1)
c       do 15304 kk=1,c_kmax
         rrl(kk,ic) = rrl(kk,ic)+c_rr(kk,nr)
15304   continue

c TEMPORARY BRPRO DEBUG WRITE 
c       if(c_kkw.gt.0) write(c_out, 511) c_tchem(ic),  xc(c_kkw,ic),
c    *        rrl(c_kkw,ic), c_rr(c_kkw,nr)
c 511     format('BRPRO: IC XR RRL RR=  ',a8,2x,3(1pe10.3))

        ic=c_reactant(nr,2)
        if(ic.gt.0) then
c         do 15307 kk=1,c_kmax
           rrl(kk,ic) = rrl(kk,ic)+c_rr(kk,nr)
15307     continue

c TEMPORARY BRPRO DEBUG WRITE 
c       if(c_kkw.gt.0) write(c_out, 511) c_tchem(ic), xc(c_kkw,ic), 
c    *     rrl(c_kkw,ic), c_rr(c_kkw,nr)

        endif


c ENTER RATE AS PRODUCT (RP) FOR PRODUCTS.
c NOTE:  ASSUME IC>0 FOR PRODUCTS IDENTIFIED BY NNPRO.

      if(c_nnpro(nr).gt.0) then
        do 120 n=1,c_nnpro(nr)
         ic = c_product(nr,n)
c          do 15317 kk=1,c_kmax
            rrp(kk,ic) = rrp(kk,ic) + c_rr(kk,nr)*c_stoich(nr,n)
15317      continue

c TEMPORARY BRPRO DEBUG WRITE 
c       if(c_kkw.gt.0) write(c_out, 512) c_tchem(ic), 
c    *                           rrp(c_kkw,ic), c_rr(kk,nr)
c 512     format('BRPRO: IC RRP RR=  ',a8,2x,3(1pe10.3))

c OPTION - IF PRODUCT EQUALS REACTANT, SUBTRACT FROM RP AND RL
c (This is important for OHL and CLOH in new representation
c  of reaction CL-+OHL=>CLOH (+OHL)
c 
c ( Prior:  This only matters for R1O2+R2O2->R1O2+products.  
c Reaction is entered twice, once for R1O2 products, once for R2O2.
c So RR is entered as R1O2 loss and product - possible error.
c  For all other reactions, STOILOSS would correct.
c  Since it converges anyway, no problem.
c
           if(ic.eq.c_reactant(nr,1).or.ic.eq.c_reactant(nr,2)) then
             stoicx = c_stoich(nr,n)
             if(stoicx.gt.1) stoicx=1.
c            do 15318 kk=1,c_kmax
              rrl(kk,ic) = rrl(kk,ic) - c_rr(kk,nr)*stoicx
              rrp(kk,ic) = rrp(kk,ic) - c_rr(kk,nr)*stoicx
c 15318        continue

c TEMPORARY DEBUG WRITE 
c       if(c_kkw.gt.0) write(c_out, 513) c_tchem(ic), 
c    *      rrl(c_kkw,ic),rrp(c_kkw,ic), c_rr(kk,nr)
c 513     format('BRPRO: IC RRL RRP RR=  ',a8,2x,4(1pe10.3))


           else   !if(ic.eq.c_reactant(nr,1).or.ic.eq.c_reactant(nr,2)) then
c END  PRODUCT=REACTANT OPTION.
c   2008 CORRECTION:  - PAIR ADJUSTMENT JUST BELOW IS SKIPPED
c                           IF PRODUCT=REACTANT OPTION IS DONE.
c                    since PRODUCT=REACTANT adjustment does the same thing.


c PAIR ADDITION:  FOR DOWN-CASCADE 2nd REACTANT ONLY
c         IF REACTANT AND PRODUCT ARE IN SAME PAIR GROUP,
c            THEN SUBTRACT REDUNDANT PRO/LOSS  FROM RPPAIR, RLPAIR
c            (for same pair group as key species, calc in CHEMSOLVE)
c
c  PAIR INDICES (ic2, icpair) IDENTIFIED ABOVE
c
c    KEY SPECIES IS c_nicreac(nr).  
c     PAIR SPECIES IS icpair = c_nppair(c_npequil(ic1), 2)

            if(icpair.gt.0) then
              if(c_nppair(c_npequil(ic),2).eq.icpair) then
               stoicx = c_pairfac(c_npequil(ic)) * c_stoich(nr,n)
               if(stoicx.gt.c_pairfac(ic2)) stoicx = c_pairfac(ic2)
c              do kk=1,c_kmax       ! kk vector loop
                 rppair(kk,icpair) = rppair(kk,icpair)
     *               - c_rr(kk,nr)*stoicx
                 rlpair(kk,icpair) = rlpair(kk,icpair)
     *               - c_rr(kk,nr)*stoicx
c              enddo              ! kk vector loop

c TEMPORARY DEBUG WRITE
c              if(c_kkw.gt.0) then
c                 write(c_out, 514) c_tchem(icpair), 
c    *           c_tchem(ic), stoicx,
c    *           rppair(c_kkw,icpair), rlpair(c_kkw,icpair),
c    *           c_rr(kk,nr)
c 514     format('BRPRO: SUBTRACT DOWN-CASCADE PAIR PRO/LOSS.',
c    *            ' IC, ICPAIR=',
c    *           2x,a8,2x,a8,/,
c    *           '  STOICX RPPAIR RLPAIR RR=',(4(1pe10.3)))
c              endif

              endif        !if(c_nppair(c_npequil(ic),2).eq.icpair) then
            endif              !if(icpair.gt.0) then
c END PAIR ADDITION FOR PRODUCT=REACTANT PAIRS

           endif  !if(ic.eq.c_reactant(nr,1).or.ic.eq.c_reactant(nr,2)) then
c END PRODUCT=REACTION OPTION CONTROL,
c                  which includes PAIR ADJUSTMENT (2008 CORRECTION)

c

c END LOOP TO ENTER RATE AS PRODUCT (RP)
120     continue
      endif


c SUM ODD-H NET AND ODD NITROGEN PRODUCTION (ONLY) 
c INCLUDES IDENTIFICATION AND SUM OF dHX/DOH FOR THE REACTION.

c (SENSHX CALC MUST BE IN ITERATIVE LOOP - SENHCAT CHANGES.)

c 2009 modifications - senshx
c      do 15321 kk=1,c_kmax
        senshx(kk,1) = 0.
        senshx(kk,2) = 0.
15321  continue

       do 125 n=1,2
        ic = c_reactant(nr,n)
        if(ic.gt.0) then
          ic=c_npequil(ic)
          if(c_icat(ic).gt.0) then
c kvec     do kk=1, kmax   ! kk vector loop
            senshx(kk,1) = senshx(kk,1) + senhcat(kk, c_icat(ic))
c kvec     enddo           ! kk vector loop
           if(c_icat(ic).ne.3) then
c kvec       do kk=1, kmax   ! kk vector loop
              senshx(kk,2) = senshx(kk,2) + senhcat(kk, c_icat(ic))
c kvec       enddo           ! kk vector loop
           endif
          endif
        endif
125    continue


c OPTION: double sensitivity for PARAMETERIZED RO2-RO2 reactions
       if(c_nrk(nr).eq.-13) then
c kvec     do kk=1, kmax   ! kk vector loop
            senshx(kk,1) = 2.*senshx(kk,1)
c kvec     enddo           ! kk vector loop
      endif

c 2009 modifications 
       do i=1,2

c kvec  do kk=1,c_kmax   ! kk vector loop
          oddhsum(kk,i) = oddhsum(kk,i) + c_rr(kk,nr)*c_oddhx(nr,i)
          oddhdel(kk,i) = oddhdel(kk,i) + c_rr(kk,nr)*c_oddhx(nr,i)
     *     *senshx(kk,i)
c kvec  enddo             ! kk vector loop

         if(c_oddhx(nr,i).gt.0) then
c kvec    do kk=1,c_kmax   ! kk vector loop
            oddhsrc(kk,i) = oddhsrc(kk,i) + c_rr(kk,nr)*c_oddhx(nr,i)
c kvec    enddo             ! kk vector loop
         endif

         if(ic1.eq.c_noh      ) then
c kvec    do kk=1,c_kmax   ! kk vector loop
             oddhloh(kk,i) = oddhloh(kk,i) - c_rr(kk,nr)*c_oddhx(nr,i)
c kvec    enddo            ! kk vector loop 
c    &       nr,i,c_oddhx(nr,i), (c_treac(nrx,nr),nrx=1,5)
         endif                      !if(c_tchem(ic1).eq.'      OH') then


         if(ic1.eq.c_nho2     ) then
c kvec    do kk=1,c_kmax   ! kk vector loop
           oddhlho2(kk,i) = oddhlho2(kk,i) - c_rr(kk,nr)*c_oddhx(nr,i)
c kvec    enddo             ! kk vector loop
         endif                      !if(c_tchem(ic1).eq.'     HO2') then

         if(ic2.gt.0) then
           if(ic2.eq.c_noh      ) then
c kvec       do kk=1,c_kmax   ! kk vector loop
               oddhloh(kk,i) = oddhloh(kk,i) - c_rr(kk,nr)*c_oddhx(nr,i)
c kvec       enddo             ! kk vector loop
           endif                      !if(c_tchem(ic2).eq.'      OH') then

           if(ic2.eq.c_nho2     ) then
c kvec      do kk=1,c_kmax   ! kk vector loop
             oddhlho2(kk,i) = oddhlho2(kk,i) - c_rr(kk,nr)*c_oddhx(nr,i)
c kvec      enddo             ! kk vector loop
           endif                      !if(c_tchem(ic2).eq.'     HO2') then
         endif                     !if(ic2.gt.0) then

       enddo    !do i=1,2
c

       if(c_pronox(nr).gt.0) then
c        do 15324 kk=1,c_kmax
          sourcnx(kk) = sourcnx(kk) + c_rr(kk,nr)*c_pronox(nr)
15324    continue
       endif
       if(c_pronox(nr).lt.0) then
c        do 15325 kk=1,c_kmax
           sinknx(kk) =  sinknx(kk) + c_rr(kk,nr)*c_pronox(nr)
15325    continue
       endif

c REGULAR DEBUG WRITE
c  (2004: 1pE10.3, f8.2 causes bug in written output.  Fixed.)
 
c      if(c_kkw.gt.0) write(c_out,15331) nr, (c_treac(nrx,nr),nrx=1,5),
c    *                          c_rr(c_kkw,nr)
c      if(c_kkw.gt.0) write(c_out,15332) 
c    *   c_oddhx(nr,1), senshx(c_kkw,1),c_pronox(nr), oddhsum(c_kkw,1),
c    *   oddhdel(c_kkw,1), sourcnx(c_kkw), sinknx(c_kkw)
c    *   ,rrp(c_kkw,9),rrl(c_kkw,9)
c      if(c_kkw.gt.0) write(c_out,15332)
c    *   c_oddhx(nr,2), senshx(c_kkw,2),c_pronox(nr), oddhsum(c_kkw,2),
c    *   oddhdel(c_kkw,2)
c 15331  format(/,' BRPRO:  NR   RR  ',i5,2x,a8,'+',a8,
c    *     '->',a8,'+',a8,'+',a8,2x,(1pe10.3) )
c 15332  format(
c    *  '  ODDHXFAC SENSHX PRONOX   ODDHSUM  ODDHDEL SOURCNX SINKNX',
c    *  ' RP9 RL9,10,11',/, 2f8.2,5(1pe10.3),/(8(1pe10.3)))



c END BRPRO
2000  return
      end
c ----------------------------------------






       subroutine excorr(ic1, 
     &    c_xcin, c_xcout, c_xcav, c_xcemit,
     &    c_rp, c_rl, c_rr ,
     &    xc,xcfinr,  xclastq, 
     &    ratek, rateh, rateq, rateqq, rhdif, egasaq, 

     &     xrp, xrr, rpro, rloss, rpro1, rloss1, 
     &       rrp, rrl, cpro,
     &       xrm, xrrm, rpm, rlm,cpm, rpmulti, rlmulti,
     &       rppair, rlpair, xrppair, xrrpair, 
     &       history, geomavg 

     &    , c_h2oliq, senshx, senhcat
     &    , oddhsum, oddhdel,  sourcnx, sinknx
     &     )
c 


c EXCORR IS A CORRECTION FOR "EXCHANGE" (BACK-FORTH) REACTIONS 
c (e.g. PAN<->MCO3, HNO4<->HO2, etc.)
c It goes through the back-forth reactions (expro, exloss)
c associated with the exchange species and "undoes" part of them.
c
c For a SINGLE exchange species (eg HNO4<->NOx, Hx)
c it undoes all except NET FORWARD or BACKWARD.
c 1998 CHANGE:  it undoes based on species lifetime.

c For TWO LINKED SPECIES (PAN-MCO3) it calculates allowable 
c  back-forth from PRIOR PAN->MCO3->PAN, and prior MCO3->PAN->MCO3
c  and undoes the rest.
c
c NOTE: SOLUTION FOR TWO LINKED SPECIES is possible
c  only if species are solved simultaneously in same reaction pair
c  or (FUTURE DO) in MULTISOLVE.
c  The 2nd species is identified by EXSPEC - set in QUADINIT.
c
c The UNDO part undoes all RP, RL for down-cascade species,
c including ODDNSUM and ODDHDEL.  
c  (But not ODDHSUM, that is unaffected by back-forth exchange.)
c
c  UPDATED FOR AQUEOUS TWOSOLVE.  WATCH OUT FOR PAN, CLOH.
c
c  2005 CHANGE (OPTION):  updates loss (rrl) for exchanged species also.
c   normally, excorr comes after species is solved for, so no matter
c   but it screws up when down-cascade species
c    (e.g. HG3A <=>HG# + SO3- special equilibrium)
c
c Inputs:  species number (ic1) for either individual species
c            or 1st of species pair.
c          Rate of reactions (RR), species production and loss (RP, RL)
c
c Outputs:  Updated RP, RL for subsequent products 
c             from back-forth exchange reactions;  
c             also updated sums (ODDHDEL)
c
c Called by:    chemsolve (after species solution)
c
c Calls to:  None.
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c
c -------------------------------------------------------------------
c
c --------------------------
      use h_rbchemmech
      use h_rbchemlocal, only: avogadrl 
c      use h_rbchemvars
c      use h_rbchemlocal, only: kk,icx, prior, ii, nr,is,ics, neq,
c     & ic,alpha, avogadrl,beta,i, n, np
c      use h_rbchemlocarr
c      use h_rbchemarrays
      IMPLICIT NONE

c#     include "rbchemmech.h"
c#     include "rbchemvars.h"
c#     include "rbchemlocal.h"
c#     include "rbchemlocarr.h"
c#     include "rbchemarrays.h"

c LOCAL VARIABLES
c
c  ratex(kk)  Rate of back-forth exchange reactions (molec/cm3/timestep)
c              Summed for all exchange reactions affecting species.
c              Equal to minimum of forward and backward rates
c              (more complex for paired species)
c             Preliminary value, used to set 'undo'.
c
c undo (kk)  Rate of back-forth exchange reaction to be 'undone'
c              Adjusted for each  indiv. reactc_ion(molec/cm3/timestep)
c
      integer :: ic1
       real*8 ::  c_xcin( c_kvec,c_cdim) ! concentr molec/cm3
       real*8 ::  c_xcout( c_kvec,c_cdim) ! concentr, mol/cm3
       real*8 ::  c_xcav( c_kvec,c_cdim)  ! concentr, mol/cm3
       real*8 ::  c_xcemit( c_kvec,c_cdim) ! emis, molec/cm3
      real*8 ::  c_rp(c_kvec,c_cdim)  ! Production, molec/cm3
      real*8 ::  c_rl(c_kvec,c_cdim)  ! Loss, molec/cm3
      real*8 ::  c_rr(c_kvec,c_rdim)  ! Reaction rate m/cm3
        real*8 :: xc( c_kvec,c_cdim) ! concentration molec/cm3
       real*8 :: xcfinr( c_kvec,c_cdim) ! ratio FINAL/AVG cncn
       real*8 :: xclastq( c_kvec,c_cdim) ! conc molec/cm3
        real*8 :: ratek(c_kvec,c_rdim)  ! Rate constants
      real*8 :: rateh(c_kvec,c_hdim)  ! Henry's law constants
      real*8 :: rateq(c_kvec,c_aqdim)  ! Aqueous eq. constants
      real*8 :: rateqq(c_kvec,c_aqqdim) ! Special eq. constants
      real*8 :: rhdif(c_kvec,c_hdim) ! H. mod by diffusion
      real*8 :: egasaq(c_kvec,c_hdim) ! Gas-to-aq transfer s-1
      real*8 :: xrp(c_kvec,c_cdim)     !Solver prior concentration
      real*8 :: xrr(c_kvec,c_cdim)     ! Solver concentration
      real*8 :: rpro(c_kvec,c_cdim)    ! Solver production rate
      real*8 :: rloss(c_kvec,c_cdim)   ! Solver loss rate
      real*8 :: rpro1(c_kvec,c_cdim)   ! Interim rpro w/out cpro 
      real*8 :: rloss1(c_kvec,c_cdim)  ! Interim rloss w/out cpro
      real*8 :: rrp(c_kvec,c_cdim)     ! Interim production rate 
      real*8 :: rrl(c_kvec,c_cdim)     ! Interim loss rate 
      real*8 :: cpro(c_kvec,c_cdim, c_cdim)   ! Cross-production 
      real*8 :: xrm(c_kvec,c_cdim)     !  Initial conc. for multi
      real*8 :: xrrm(c_kvec,c_cdim)    !Initial solution for multi
      real*8 :: rpm(c_kvec,c_cdim)     ! Production rate for multi
      real*8 :: rlm(c_kvec,c_cdim)     ! Loss for multi-spec 
      real*8 :: cpm(c_kvec, c_cdim, c_cdim) !Cross-prod for multi
      real*8 :: rpmulti(c_kvec       ) !Product'n multi group sum
      real*8 :: rlmulti(c_kvec       ) ! Loss group sum for multi
      real*8 :: rppair(c_kvec,c_cdim)  ! Production for pair grp
      real*8 :: rlpair(c_kvec,c_cdim)  ! Loss rate for pair grp
      real*8 :: xrppair(c_kvec)        ! Solver prior pair group cn.
      real*8 :: xrrpair(c_kvec)        ! Solution for pair group sum
      real*8 :: history(c_kvec,c_cdim,400)  ! Cncn for each iter
      real*8 :: geomavg(c_kvec,c_cdim)   ! Geom. avg factor
       real*8 ::  c_h2oliq(c_kvec)   ! LWC grams cm-3
      real*8 ::        senshx(c_kvec,2)   ! Hx sens: d ln(pHx)/d
      real*8 ::        senhcat( c_kvec,99) ! Hx sens. by species categ.
c       ln([OH])
      real*8 ::        oddhsum(c_kvec,2)   ! Summed net P/L Hx, mol/cm3
      real*8 ::        oddhdel(c_kvec,2)   ! Summed Hx sens to Oh,
      real*8 :: sourcnx(c_kvec)   ! Summed NOx source, mol/cm3
      real*8 :: sinknx(c_kvec)    ! Summed NOx sink, mol/cm3


!---------Below are local varibles========Guangxing Lin      
      real*8 :: ratex(c_kvec)  ! Exchange reaction sum  mol/cm3
      real*8 :: undo(c_kvec)   ! Exchange undo rate mol/cm3
      integer :: nex                     ! Index for number of ex vars
c      c --below are originally from rbchemarrys---
c----below are from rbchemlocarr

c
c  
      real*8 ::  c_rainfr(c_kvec)   ! rainout fraction
       real*8 ::  c_droplet(c_kvec)        ! droplet radius, cm
      logical :: c_lexpo(c_kvec)        ! Flag for expo decay solution
      real*8 ::  c_ohtest      ! test: dOH/OH or dOH/HO2
      real*8 ::  c_notest      ! test: dNO2/NO2
      integer :: c_iter                  ! chem. number of iterations
c below are from rbchemlcoal

c   mol/cm3
      real*8 ::        oddhloh(c_kvec,2)   ! Hx loss from OH,  mol/cm3
      real*8 ::        oddhlho2(c_kvec,2)   ! Hx loss from HO2,  mol/cm3
      real*8 ::        oddhsrc(c_kvec,2)   ! Summed P Hx, mol/cm3


      real*8 :: xfohtest          ! OH/HO2 convergence test 
c     real*8 :: c_xcwdep(c_kvec, c_cdim)
          real*8 :: stoicx                         ! stoichiometry sum
      integer :: ic,ic2,ic3,icc,ics,ics2,icc1,icc2,icc3 !Chem index
      integer :: is,iss,iscs,isc2                    !Chem local index
          integer :: ich,icq,icx,icx1,icx2,icy1,icy2,icp,icp1,icp2 !Chem
      integer :: icr1,icr2,icr,isr1,isr2,icpair  !chem index - pair and
      integer :: ionsum                                  ! aquasolve ion
      integer :: ica1,ica2,icb1,icb2,nra1,nra2,nrb1,nrb2 !Aquasolve chem
      integer :: neq                                     !Aqueous
      integer :: nc,nc1,nc2,ncc,ncf,nn,nne               !Chem species
      integer :: nr,nr1,nr2,nrh,nrq,nrqq,nrx,np          ! Reaction
      integer :: icat1,icat2,icatp,icatp2  ! indices used for species
      integer :: kk, kw                            ! Vectorization
      integer :: i, j, k, ii, ij, iii, n           ! General counters
       integer :: nsol           ! Number of pair groups being solved
      integer :: nsolv          ! Number of species being solved for
          real*8 :: alpha(c_kvec)   ! General vector variable
      real*8 ::  beta(c_kvec)   ! General vector variable
      real*8 :: gamma(c_kvec)   ! General vector variable
      real*8 :: prior(c_kvec)   ! Prior species conc (molec/cm3)
      integer :: ncsolv(c_cdim) ! Species number (ic) for species in
      integer :: nssolv(c_cdim) ! Pair group number (is) for spec. in

c
c --------------------------

      kk=1
c
      if(ic1.eq.0) return

c     if(c_expro(ic1,1).le.0.or.c_exloss(ic1,1).le.0) return
      if(c_exspec(ic1,1).eq.0) return


c MAIN LOOP FOR MULTIPLE EXCHANGED SPECIES
c   Exchange reactions are possible with different species partners.
c   Sum up all exchange reactions with the same species 
c          and solve  simultaneously

      do nex=1,20      ! MAIN LOOP 1000
        if(c_exspec(ic1,nex).eq.0) return
        icx = c_exspec(ic1,nex)

c ZERO
c       do kk=1,c_kmax            ! kk vector loop
          cpro(kk,1,2) = 0.
          cpro(kk,2,1) = 0.
          ratex(kk)  = 0.
          prior(kk)    = 0.
c       enddo                   ! kk vector loop
c
c EXCORR DEBUG WRITE
c       if(c_kkw.gt.0) write (c_out,611) c_tchem(ic1)
c 611     format (/,' BEGIN EXCORR:  IC1 (ICX) = ', a8,2x,a8)
c       if(c_kkw.gt.0.and.icx.gt.0) then
c          write(c_out,611) c_tchem(ic1), c_tchem(icx)
c       endif

c SUM EXCHANGE REACTIONS  AS CPRO
c    (multiple reactions are allowed for single exchange)

        do  ii=1,5
          nr = c_exloss(ic1,nex,ii)
          if(nr.gt.0) then
c           do kk=1,c_kmax           ! kk vector loop
              cpro(kk,1,2) = cpro(kk,1,2) +  c_rr(kk,nr)
c           enddo                  ! kk vector loop

c EXCORR DEBUG WRITE
c           if(c_kkw.gt.0) write(c_out,614) nr, (c_treac(j,nr),j=1,5), 
c    *         c_rr(c_kkw,nr)
c 614         format(' REACTION: ',i4,2x,a8,'+',a8,'=>',a8,'+',a8,'+'
c    *        , a8, ' RR=', 1pe10.3)
c           if(c_kkw.gt.0) write(c_out,615) cpro(kk,1,2), cpro(kk,2,1)
c 615         format ('CROSS PROD 12, 21 = ',2(1pe10.3)  )

          endif                ! if(nr.gt.0)
          np = c_expro(ic1,nex,ii)
          if(np.gt.0) then
c           do kk=1,c_kmax           ! kk vector loop
              cpro(kk,2,1) = cpro(kk,2,1) + c_rr(kk,np)
c           enddo                  ! kk vector loop

c EXCORR DEBUG WRITE
c           if(c_kkw.gt.0) write(c_out,614) np, (c_treac(j,np),j=1,5),
c    *         c_rr(c_kkw,np)
c           if(c_kkw.gt.0) write(c_out,615) cpro(kk,1,2), cpro(kk,2,1)

          endif               ! if(np.gt.0)
        enddo            ! do ii=1,5

c ESTABLISH RATEX FOR SINGLE-REACTION CASE (e.g. HNO4<->Nx,Hx)
c EQUAL TO MIN(RPRO,RLOSS)
c 1998 CHANGE:  MULTIPLIED BY FACTOR FOR EXCHANGE LIFETIME
c 2000 CHANGE:  IF BETA=0, DO ALSO FOR CASE WHERE IC2 NONZERO

        if(icx.le.0) then
c         do kk=1,c_kmax             ! kk vector loop
            ratex(kk) = cpro(kk,1,2)
            if(ratex(kk).gt.cpro(kk,2,1)) ratex(kk)=cpro(kk,2,1)
            if(cpro(kk,1,2).gt.0) then
              ratex(kk) = ratex(kk)*
     *         (cpro(kk,1,2)/(cpro(kk,1,2)+xc(kk,ic1)) )
            endif                     !if(cpro(kk,1,2).gt.0) then
c         enddo                    ! kk vector loop
c EXCORR DEBUG WRITE
c           if(c_kkw.gt.0) write(c_out,621) ratex(c_kkw)
c 621         format(' EXCORR:  SINGLE SPECIES RATEX =', 1pe10.3)

        endif                !if(icx.le.0) 

c IF IC2>0 AND GAS, TWO LINKED SPECIES. 
c CALCULATE THE SIZE OF THE REDUNDENT EXCHANGE (RATEX):
c
c  (REDUNDENT EXCHANGE DERIVED FROM TWOSOLVE SOLUTION ABOVE.
c   REDUNDENT EX = CP(2->1) FROM SOLUTION WITH ZERO XR2 SOURCE (RP2=0)
c   PLUS CP(1->2) FROM MATRIX SOLUTION WITH ZERO INITIAL XR(1) (RP1=0).
c   FROM TWOSOLVE SOLUTION, ABOVE, 
c       PARTIAL X2=XP2*(CP2*RP1)/(RL1*RL2 - CP1*CP2) 
c       REDUNDANT 2->1EX = CP1'*XP2*CP2*RP1/denom.
c                        =CP1*CP2*RP1/denom.  SAME FOR 1->2EX.  
c    )
c        
c MODIFIED TO ALLOW AQUEOUS, SPECIAL EQUILIBRIUM SPECIES (2000)

        if(icx.gt.0) then

c ZERO AND SUM RPRO, RLOSS, XRP FOR TWO EXCHANGE SPECIES
          do  is=1,2
c           do kk=1,c_kmax         ! kk vector loop
              rloss(kk,is) = 0.
              rpro(kk,is)  = 0.
              xrp(kk,is)   = 0.
c             enddo                ! kk vector loop
            ics = c_npequil(ic1)
            if( is.eq.2) ics = c_npequil(icx)
            do neq=1,(c_nequil(ics)+1)
              ic=ics
              if(neq.gt.1)ic=c_ncequil(ics,(neq-1))
c             do  kk=1,c_kmax        ! kk vector loop
                alpha(kk) = 1.
                if(neq.gt.1) alpha(kk)= c_h2oliq(kk)*avogadrl
                rloss(kk,is) = rloss(kk,is)+rrl(kk,ic)
                rpro(kk,is)  = rpro(kk,is) +rrp(kk,ic)
                xrp(kk,is)   = xrp(kk,is)  +xc(kk,ic)*alpha(kk)
                prior(kk) = prior(kk) +   c_xcin(kk,ic)*alpha(kk)
c             enddo                ! kk vector loop
            enddo            ! neq-1,(c_nequil(ics)+1)
          enddo            ! is=1,2

c POSSIBLE ERROR, MUST SUBTRACT CPRO FROM RPRO - ELSE ITS INCLUDED!

c SOLUTION FOR RATEX. 
c   (If denominator is zero, use single-species solution instead)

c         do kk=1,c_kmax        ! kk vector loop
            beta(kk) = (rloss(kk,1)+xrp(kk,1))*(rloss(kk,2)+xrp(kk,2)) 
     *                - cpro(kk,1,2)*cpro(kk,2,1)
            if(beta(kk).gt.0) then
              ratex(kk) = cpro(kk,1,2)*(cpro(kk,2,1)
     *             *(rpro(kk,1)+rpro(kk,2) + prior(kk)               )
     *                 /beta(kk) )
c EXCORR DEBUG WRITE
c             if(c_kkw.gt.0) write(c_out,624) ratex(c_kkw)
c 624           format(' EXCORR:  TWO SPECIES FULL RATEX =', 1pe10.3)
c TEMPORARY EXCORR DEBUG WRITE - DETAILS
c             if(c_kkw.gt.0) write(c_out,625) 
c    *          rloss(c_kkw,1), xrp(c_kkw,1), rloss(c_kkw,2)
c    *         , xrp(c_kkw,2), cpro(c_kkw,1,2), cpro(c_kkw,2,1)
c 625          format(
c    *           '  (EXCORR: rloss xrp1 rloss2 xrp2 cpro12 cpro21=)',
c    *               /, (8(1pe10.3)) )
c             if(c_kkw.gt.0) write(c_out,626) 
c    *          rpro(c_kkw,1), rpro(c_kkw,2), prior(c_kkw), beta(c_kkw)
c 626           format('  (EXCORR: rpro1 rpro2 prior beta =)',
c    *        /, (8(1pe10.3)) )

            else
              ratex(kk) = cpro(kk,1,2)
              if(ratex(kk).gt.cpro(kk,2,1)) ratex(kk)=cpro(kk,2,1)
              if(cpro(kk,1,2).gt.0.) then
                ratex(kk) = ratex(kk)*
     *           (cpro(kk,1,2)/(cpro(kk,1,2)+xc(kk,ic1)) )
              endif                    !if(cpro(kk,1,2).gt.0.) then

c EXCORR DEBUG WRITE
c             if(c_kkw.gt.0) write(c_out,627) ratex(c_kkw)
c 627           format(' EXCORR:  TWO SPECIES SIMPLE RATEX =', 1pe10.3)

            endif            !  if(beta(kk).gt.0) 
c         enddo              ! kk vector loop

      endif          !if(icx.gt.0) 

c PROTECT AGAINST RATEX>EXCHANGE, RPRO FOR TWO-REACTION CASE
c        do kk=1,c_kmax        ! kk vector loop
          if(ratex(kk).gt.cpro(kk,1,2)) ratex(kk)=cpro(kk,1,2)
          if(ratex(kk).gt.cpro(kk,2,1)) ratex(kk)=cpro(kk,2,1)
c        enddo               ! kk vector loop

c UNDO LOOP FOR INDIVIDUAL REACTIONS;  
c UNDO EXCHANGE PRODUCTION AND LOSS REACTIONS IN PROPORTION TO SIZES
c SO THAT TOTAL UNDO IN EACH DIRECTION EQUALS RATEX.

        do  ii=1,5       ! 120
         do i=1,2        ! 125
          if(i.eq.1) nr=c_exloss(ic1,nex,ii)
          if(i.eq.2) nr=c_expro(ic1,nex,ii)

c SET UNDO FOR INDIVIDUAL REACTION.
c NOTE DIVIDE-BY-ZERO DANGER.  RPRO, RLOSS SHOULD NEVER BE ZERO.
          if(nr.gt.0) then

            if(i.eq.1) then
c             do kk=1,c_kmax       ! kk vector loop
               undo(kk) = 0.
               if(cpro(kk,1,2).gt.0)
     *         undo(kk) = ratex(kk)*c_rr(kk,nr)/cpro(kk,1,2)
c             enddo               ! kk vector loop
            else
c             do kk=1,c_kmax      ! kk vector loop
               undo(kk) = 0.
               if(cpro(kk,2,1).gt.0)
     *         undo(kk) = ratex(kk)*c_rr(kk,nr)/cpro(kk,2,1)
c             enddo              ! kk vector loop
            endif          ! if(i.eq.1)

c UNDO LOSS FROM REACTANTS
c   2000 MODIFICATION:  UNDO FOR REAL REACTANTS, NOT GAS EQUIVALENTS
c    (npequil comment out)

           do  n=1,2
            ic = c_reactant(nr,n)
            if(ic.gt.0) then
c             ic=c_npequil(ic)
c 2005 CHANGE OPTION:  DON'T SKIP FOR MAIN SPECIES. 
c   CAUSES ERROR IF MAIN SPECIES IS OFF CASCADE.
              if(ic.ne.ic1.and.ic.ne.icx) then
c             if(ic.ne.0 ) then
c
c               do kk=1,c_kmax     ! kk vector loop
                 rrl(kk,ic) = rrl(kk,ic) -undo(kk)
c               enddo            ! kk vector loop

c EXCORR REGULAR DEBUG WRITE (REGULAR w/BRPRO)
c       if(c_kkw.gt.0) write(c_out, 513) c_tchem(ic),  
c    *        rrl(c_kkw,ic), c_rr(c_kkw,nr), undo(c_kkw)
c 513     format('EXCORR IC RRL RR UNDO=  ',a8,2x,3(1pe10.3))

              endif
           endif
           enddo       !do  n=1,2 

c  UNDO PRODUCTION.  
           if(c_nnpro(nr).gt.0) then
            do  n=1,c_nnpro(nr)
              ic = c_product(nr,n)
              if(ic.gt.0) then
c               ic=c_npequil(ic)
c 2005 CHANGE OPTION:  DON'T SKIP FOR MAIN SPECIES. 
c   CAUSES ERROR IF MAIN SPECIES IS OFF CASCADE.
                if(ic.ne.ic1.and.ic.ne.icx) then
c               if(ic.ne.0                ) then
c
c                 do kk=1,c_kmax      ! kk vector loop
                   rrp(kk,ic) = rrp(kk,ic)  - c_stoich(nr,n)*undo(kk)
c                 enddo             ! kk vector loop

c EXCORR REGULAR DEBUG WRITE (REGULAR w/BRPRO)
c       if(c_kkw.gt.0) write(c_out, 514) c_tchem(ic),  
c    *        rrp(c_kkw,ic), c_rr(c_kkw,nr), undo(c_kkw)
c 514     format('EXCORR IC RRP RR UNDO=  ',a8,2x,3(1pe10.3))

                endif
              endif
            enddo         !   do n=1,c_nnpro(nr)
           endif       !if(c_nnpro(nr).gt.0) 

c UNDO ODD NITROGEN PRODUCTION AND ODD-H SENSITIVITY
c 2009 modifications
c
c            do kk=1,c_kmax         ! kk vector loop
              senshx(kk,1) = 0.
              senshx(kk,2) = 0.
c            enddo                ! kk vector loop

             do n=1,2
              ic = c_reactant(nr,n)
              if(ic.gt.0) then
                ic = c_npequil(ic)
                if(c_icat(ic).gt.0) then

c                 do kk=1,c_kmax     ! kk vector loop
                   senshx(kk,1) = senshx(kk,1)
     &                               + senhcat(kk,c_icat(ic))
c                 enddo            ! kk vector loop
                  if(c_icat(ic).ne.3) then
c kvec             do kk=1, kmax   ! kk vector loop
                    senshx(kk,2) = senshx(kk,2)
     &                               + senhcat(kk, c_icat(ic))
c kvec             enddo           ! kk vector loop
                  endif

                endif           !if(c_icat(ic).gt.0) 
              endif          ! if(ic.gt.0)
325          continue
             enddo             !do n=1,2

c 2009 modification
             do n=1,2
c              do  kk=1,c_kmax          ! kk vector loop
                oddhdel(kk,n) = oddhdel(kk,n) - c_oddhx(nr,n)*undo(kk)
     *           *senshx(kk,n)
c              enddo                  ! kk vector loop
             enddo       !do n=1,2


             if(c_pronox(nr).gt.0) then
c              do  kk=1,c_kmax          ! kk vector loop
                sourcnx(kk) = sourcnx(kk)-c_pronox(nr)*undo(kk)
c              enddo                  ! kk vector loop
             endif            ! if(c_pronox(nr).gt.0)
             if(c_pronox(nr).lt.0) then
c              do  kk=1,c_kmax          ! kk vector loop
                 sinknx(kk) =  sinknx(kk)-c_pronox(nr)*undo(kk)
c              enddo                  ! kk vector loop
             endif            ! if(c_pronox(nr).lt.0)


c REGULAR DEBUG WRITE
c          if(c_kkw.gt.0) write(c_out,15592)  (c_treac(nrx,nr),nrx=1,5)
c 15592      format(/,'EXCORR:    ',a8,'+',a8,'=>',a8,'+',a8,'+',a8)
c          if(c_kkw.gt.0) write(c_out,15591) nr
c    *       , c_rr(c_kkw,nr),undo(c_kkw),
c    *     c_oddhx(nr,1), senshx(c_kkw,1),c_pronox(nr), 
c    *      oddhsum(c_kkw,1),
c    *       oddhdel(c_kkw,1), sourcnx(c_kkw), sinknx(c_kkw)
c    *       ,rrp(c_kkw,9),rrl(c_kkw,9)
c 15591      format('EXCORR:  NR   RR  UNDO ',i5,2(e10.3),/,
c    *     '  ODDHXFAC SENSHX PRONOX   ODDHSUM  ODDHDEL SOURCNX SINKNX',
c    *     ' RP9 RL9',/,2f5.2,(8(e10.3)))



          endif     !if(nr.gt.0) 
         enddo            !do i=1,2         125
        enddo           !do  ii=1,5        120
c END UNDO LOOP FOR INDIVIDUAL REACTIONS


      enddo     !do nex=1,20  MAIN LOOP 1000
c END MAIN LOOP  FOR  MULTIPLE EXCHANGED SPECIES


c END EXCORR
2000    return
        end
c --------------------------------
c
c

      subroutine noxsolve(ic1,ic2,ic3,
c                         O3  NO2  NO
     &    c_xcin, c_xcout, c_xcav, c_xcemit,
     &    c_rp, c_rl, c_rr ,
     &    xc,xcfinr,  xclastq, 
     &    ratek, rateh, rateq, rateqq, rhdif, egasaq, 

     &     xrp, xrr, rpro, rloss, rpro1, rloss1, 
     &       rrp, rrl, cpro,
     &       xrm, xrrm, rpm, rlm,cpm, rpmulti, rlmulti,
     &       rppair, rlpair, xrppair, xrrpair, 
     &       history, geomavg 
     &       ,c_notest, c_iter, senhcat ,sourcnx, sinknx)
c 

c Special solution for concentrations of O3, NO2 and NO.
c
c This uses reaction rates, sums of RLOSS, gas-aqueous sums
c   and cross-production (CPRO)  from CHEMSOLVE. 
c
c FUTURE OPTION:  Flag to keep NOx at pre-set value,
c                  while adjusting NO and NO2
c                 (fails so far.  Set SOURCNX = SINKNX?)
c
c NOTE:  THIS SUBROUTINE MAY NEED TO USE R12: OH+NO2->HNO3 SPECIAL.
c
c
c Inputs:    Species numbers (ic) for O3, NO2, NO.
c            Also uses:  
c            Species sums for production, loss and cross-production
c            (xrpm, rlm, rpm, cpm) from MULTISOLVE part of CHEMSOLVE.
c            NOx sums (sourcnx, sinknx) from full program.
c
c Outputs:  xrrm:  concentrations for O3, NO, NO2.
c
c           (used in MULTISOLVE part of CHEMSOLVE 
c             to create updated species solution array
c             with gas/aqueous partitioning)
c
c Called by:    chemsolve (as alternative species solution)
c
c Calls to:  None.
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c
c -------------------------------------------------------------------
c
c 
c ---------------------------------------------
      use h_rbchemmech
c      use h_rbchemvars
c      use h_rbchemarrays
c      use h_rbchemlocal, only: kk, alpha, gamma,beta, ic
c      use h_rbchemlocarr
      IMPLICIT NONE


c#     include "rbchemmech.h"
c#     include "rbchemvars.h"
c#     include "rbchemarrays.h"
c#     include "rbchemlocal.h"
c#     include "rbchemlocarr.h"


      integer :: ic1,ic2,ic3
       real*8 ::  c_xcin( c_kvec,c_cdim) ! concentr molec/cm3
       real*8 ::  c_xcout( c_kvec,c_cdim) ! concentr, mol/cm3
       real*8 ::  c_xcav( c_kvec,c_cdim)  ! concentr, mol/cm3
       real*8 ::  c_xcemit( c_kvec,c_cdim) ! emis, molec/cm3
      real*8 ::  c_rp(c_kvec,c_cdim)  ! Production, molec/cm3
      real*8 ::  c_rl(c_kvec,c_cdim)  ! Loss, molec/cm3
      real*8 ::  c_rr(c_kvec,c_rdim)  ! Reaction rate m/cm3
        real*8 :: xc( c_kvec,c_cdim) ! concentration molec/cm3
       real*8 :: xcfinr( c_kvec,c_cdim) ! ratio FINAL/AVG cncn
       real*8 :: xclastq( c_kvec,c_cdim) ! conc molec/cm3
        real*8 :: ratek(c_kvec,c_rdim)  ! Rate constants
      real*8 :: rateh(c_kvec,c_hdim)  ! Henry's law constants
      real*8 :: rateq(c_kvec,c_aqdim)  ! Aqueous eq. constants
      real*8 :: rateqq(c_kvec,c_aqqdim) ! Special eq. constants
      real*8 :: rhdif(c_kvec,c_hdim) ! H. mod by diffusion
      real*8 :: egasaq(c_kvec,c_hdim) ! Gas-to-aq transfer s-1
      real*8 :: xrp(c_kvec,c_cdim)     !Solver prior concentration
      real*8 :: xrr(c_kvec,c_cdim)     ! Solver concentration
      real*8 :: rpro(c_kvec,c_cdim)    ! Solver production rate
      real*8 :: rloss(c_kvec,c_cdim)   ! Solver loss rate
      real*8 :: rpro1(c_kvec,c_cdim)   ! Interim rpro w/out cpro 
      real*8 :: rloss1(c_kvec,c_cdim)  ! Interim rloss w/out cpro
      real*8 :: rrp(c_kvec,c_cdim)     ! Interim production rate 
      real*8 :: rrl(c_kvec,c_cdim)     ! Interim loss rate 
      real*8 :: cpro(c_kvec,c_cdim, c_cdim)   ! Cross-production 
      real*8 :: xrm(c_kvec,c_cdim)     !  Initial conc. for multi
      real*8 :: xrrm(c_kvec,c_cdim)    !Initial solution for multi
      real*8 :: rpm(c_kvec,c_cdim)     ! Production rate for multi
       real*8 :: rlm(c_kvec,c_cdim)     ! Loss for multi-spec 
      real*8 :: cpm(c_kvec, c_cdim, c_cdim) !Cross-prod for multi
      real*8 :: rpmulti(c_kvec       ) !Product'n multi group sum
      real*8 :: rlmulti(c_kvec       ) ! Loss group sum for multi
      real*8 :: rppair(c_kvec,c_cdim)  ! Production for pair grp
      real*8 :: rlpair(c_kvec,c_cdim)  ! Loss rate for pair grp
      real*8 :: xrppair(c_kvec)        ! Solver prior pair group cn.
      real*8 :: xrrpair(c_kvec)        ! Solution for pair group sum
      real*8 :: history(c_kvec,c_cdim,400)  ! Cncn for each iter
      real*8 :: geomavg(c_kvec,c_cdim)   ! Geom. avg factor
      real*8 ::  c_notest      ! test: dNO2/NO2
      integer :: c_iter                  ! chem. number of iterations
      real*8 ::        senhcat( c_kvec,99) ! Hx sens. by species categ.
      real*8 :: sourcnx(c_kvec)   ! Summed NOx source, mol/cm3
      real*8 :: sinknx(c_kvec)    ! Summed NOx sink, mol/cm3


c----below are local variables ===Guangxing Lin 
c 

      real*8 ::  c_h2oliq(c_kvec)   ! LWC grams cm-3
      real*8 ::  c_rainfr(c_kvec)   ! rainout fraction
       real*8 ::  c_droplet(c_kvec)        ! droplet radius, cm
      logical :: c_lexpo(c_kvec)        ! Flag for expo decay solution
      real*8 ::  c_ohtest      ! test: dOH/OH or dOH/HO2
c      below are from rbchemlcoal

      real*8 ::        oddhdel(c_kvec,2)   ! Summed Hx sens to Oh,
c   mol/cm3
      real*8 ::        oddhloh(c_kvec,2)   ! Hx loss from OH,  mol/cm3
      real*8 ::        oddhlho2(c_kvec,2)   ! Hx loss from HO2,  mol/cm3
      real*8 ::        oddhsrc(c_kvec,2)   ! Summed P Hx, mol/cm3
      real*8 ::        oddhsum(c_kvec,2)   ! Summed net P/L Hx, mol/cm3
      real*8 ::        senshx(c_kvec,2)   ! Hx sens: d ln(pHx)/d
c       ln([OH])
      real*8 :: xfohtest          ! OH/HO2 convergence test 

           real*8 :: stoicx                         ! stoichiometry sum
      integer :: ic,icc,ics,ics2,icc1,icc2,icc3 !Chem index
      integer :: is,iss,iscs,isc2                    !Chem local index
          integer :: ich,icq,icx,icx1,icx2,icy1,icy2,icp,icp1,icp2 !Chem
      integer :: icr1,icr2,icr,isr1,isr2,icpair  !chem index - pair and
      integer :: ionsum                                  ! aquasolve ion
      integer :: ica1,ica2,icb1,icb2,nra1,nra2,nrb1,nrb2 !Aquasolve chem
      integer :: neq                                     !Aqueous
      integer :: nc,nc1,nc2,ncc,ncf,nn,nne               !Chem species
      integer :: nr,nr1,nr2,nrh,nrq,nrqq,nrx,np          ! Reaction
      integer :: icat1,icat2,icatp,icatp2  ! indices used for species
      integer :: kk, kw                            ! Vectorization
      integer :: i, j, k, ii, ij, iii, n           ! General counters
       integer :: nsol           ! Number of pair groups being solved
      integer :: nsolv          ! Number of species being solved for
          real*8 :: alpha(c_kvec)   ! General vector variable
      real*8 ::  beta(c_kvec)   ! General vector variable
      real*8 :: gamma(c_kvec)   ! General vector variable
      real*8 :: prior(c_kvec)   ! Prior species conc (molec/cm3)
      integer :: ncsolv(c_cdim) ! Species number (ic) for species in
      integer :: nssolv(c_cdim) ! Pair group number (is) for spec. in

c LOCAL VARIABLES

c xnox     NOx concentration (NO+NO2) molec/cm3
c xox      Ox concentration  (O3+NO2) molec/cm3

      real*8 :: xnox(c_kvec)     ! NOx, molec/cm3
      real*8 :: xox(c_kvec)      !  Ox, molec/cm3

      real*8 :: oxox       ! Prior Ox for write
      real*8 :: sourcox    ! Ox source for write
      real*8 :: sinkox     ! Ox sink  for write

       real*8 :: xk2       ! Quadratic parameter for write
       real*8 :: xk2n      ! Quadratic parameter for write
       real*8 :: xrln      ! Quadratic parameter for write
       real*8 :: xjn       ! Quadratic parameter for write
       real*8 :: xjnx      ! Quadratic parameter for write
       real*8 :: xrpn      ! Quadratic parameter for write
       real*8 :: pansum    ! Sum pans for write
       real*8 :: hnosum    ! Sum hno2 hno4 for write

c ---------------------------------------------
      kk=1


c FIRST:  SOLVE FOR NOX.  USE ODDNSUM = CHEM PRODUCTION OF NOX
c  (excluding NO-NO2 conversions and NOx sinks).
c SOURCNX = NOX SOURCE WITH PRIOR.
c SINKNX = NOX SINKS, IGNORING NO-NO2 BACK-FORTH. (ALSO W/PRIOR)
c XNOXN = SOURCE/(SINK/XNOX)

c     do kk=1,c_kmax
        sourcnx(kk) = sourcnx(kk) +   c_xcin(kk,ic2)+  c_xcin(kk,ic3)
        sinknx(kk) =   xrm(kk,2) +  xrm(kk,3) - sinknx(kk)

        xnox(kk) = ( xrm(kk,  2)+ xrm(kk,  3))*sourcnx(kk)/sinknx(kk)

c OPTION:  INSERT THIS LINE FOR PRESET NOx CONCENTRATION
c       xnox(kk) =   c_xcin(kk,ic2)+  c_xcin(kk,ic3)
c      enddo

c NOXSOLVE DEBUG WRITE
       if(c_kkw.gt.0) write (c_out, 1851) 
     *      c_xcin(c_kkw,ic1),   c_xcin(c_kkw,ic2),   c_xcin(c_kkw,ic3),
     *    xrm(c_kkw,1),  xrm(c_kkw, 2), xrm(c_kkw,3),
     *    xnox(c_kkw), sourcnx(c_kkw), sinknx(c_kkw)
1851   format(' NOXSOLVE DEBUG: ',/,
     *       ' INITIAL XXO: O3, NO2,NO  = ',3(1pe10.3),/,
     *       ' PRIOR XRM:   O3, NO2,NO  = ',3(1pe10.3),/,
     *       ' XNOX (=) SOURCNX (/) SINKNX  = ',3(1pe10.3)    )  

c **** CHECK XRP AND XXO IN RPRO, RLOSS!!


c SENHCAT FOR NOX.  (PRIOR, dHx/dNOx dNOx/dOH product added to SENH.
c                    REPLACED, just include dNOx/dOH in HxDELTA sum.)
c THIS IS FROM THE NOX EQUATION:  NOXPRO=a+bOH
c dlnNOx/dlnOH = -[(OH+NO2) + sencatHO2*otherNOx losses]/ total loss;
c                                        total loss includes XRP(NOx).
c
c OPTIONS: USE REACTION R12 (OH+NO2->HNO3 EXPLICITLY.

c      do kk=1,c_kmax

c OPTION WITH R-12
c       senhcat(kk,12) = 0.- (  c_rr(kk,12) 
c    *  + senhcat(kk,10)
c    *       *(sinknx(kk)-c_rr(kk,12)- xrm(kk,  2)- xrm(kk,  3))
c    *                  )/sinknx(kk)
c OPTION WITHOUT EXPLICIT R-12
        senhcat(kk,12) = 0.- (            
     *  + senhcat(kk,10)*(sinknx(kk)        - xrm(kk,  2)- xrm(kk,  3))
     *                  )/sinknx(kk)
c END OPTION
        senhcat(kk,13) = senhcat(kk,12)
c       enddo

c STEADY-STATE.  THERE IS NO STEADY-STATE OPTION FOR NOX, ->INFINITE.
c BUT ISTS(NOX) CAN BE USED TO SET NOX EQUAL TO CONSTANT VALUE.
c  (FAILS 1999 - MUST USE HARD-WIRE OPTION, ABOVE.)
      if(c_lsts(ic3)) then
c       do kk=1,c_kmax
         xnox(kk) =  xrm(kk,  2)+ xrm(kk,  3)
         senhcat(kk,12) = 0.
         senhcat(kk,13) = 0.
c       enddo
      endif

c ODD OXYGEN (XOX):  SOLVE USING BACK-EULER: OX = RP/(RL/XRP).
c RP AND RL FOR ODD OXYGEN EQUAL THE SUM OF RP FOR O3 AND NO2
c WITH CROSS-CONVERSIONS (O3->NO2 AND NO2->O3) REMOVED.  
c IN CHEMSOLVE(O3,NO2,NO), CROSS-PRODUCTION IS ADDED TO CPRO, NOT RPRO,
c  BUT IT IS ALSO INCLUDED IN RLOSS.  SO RLOSS IS CORRECTED.

c PRELIMINARY:  ADD NO-to-NO2 conversions to RPM(NO2)
c  These are equal to CPM(NO->NO2) minus CPM(O3->NO2)
c  They count as Ox production
c  They also count in the solution for NO, 
c      which separates out O3+NO<->NO2 only.
c
c CHANGE:  WITH RPM INCLUDING CPM:
c PRELIMINARY:  REMOVE O3+NO<=>NO2 from RPM.
c     (Removal is CPM(1->2) and (2->1).  NOT (3->1). 
c       In original version, 3->1 is zero.  Why?? - error)
c   RPM includes all CPM. Other NO-to-NO2 count for Ox and NO solutions.

c     do kk=1,c_kmax         ! kk vector loop
c ORIGINAL
c       rpm(kk,2) = rpm(kk,2) + cpm(kk,3,2) - cpm(kk,1,2)

c OPTION
        rpm(kk,2) = rpm(kk,2) - cpm(kk,1,2)
        rpm(kk,1) = rpm(kk,1) - cpm(kk,2,1)
        rpm(kk,3) = rpm(kk,3) - cpm(kk,2,1)
c
c TEMPORARY BUG FIX:  O3+NO->NO2 COUNTED DOUBLE IN ORIGINAL ALGORITHM
C   CORRECT HERE.  NOXBUG - delete when CPM is correct
c       rpm(kk,2) = rpm(kk,2) - cpm(kk,1,2)

c     enddo                ! kk vector loop

c   NOXSOLVE DEBUG WRITE
      if(c_kkw.gt.0) write(c_out,1854) 
     *     rpm(c_kkw,1), rpm(c_kkw,2), rpm(c_kkw,3),
     *     rlm(c_kkw,1), rlm(c_kkw,2), rlm(c_kkw,3),
     *     cpm(c_kkw,1,2), cpm(c_kkw,2,1),
     *     cpm(c_kkw,3,2), cpm(c_kkw,2,3),
     *     cpm(c_kkw,1,3), cpm(c_kkw,3,1)
1854   format(' NOXSOLVE DEBUG:  (NO->NO2 added to rpNO2) ',/,
     *    ' RPM   O3, NO2, NO =  ', 3(1pe10.3), /,
     *    ' RLM   O3, NO2, NO =  ', 3(1pe10.3), /,
     *    'CPM O3->NO2, NO2->O3 = ',2(1pe10.3)   , /,
     *    'CPM NO->NO2, NO2->NO = ',2(1pe10.3)   , /,
     *    'CPM O3->NO , NO ->O3 = ',2(1pe10.3)   )

c     do kk=1,c_kmax         ! kk vector loop
        xox(kk) = ( xrm(kk,  1)+ xrm(kk,  2)) * 
     *                       ( rpm(kk,  1)+ rpm(kk,  2))
     *   /(  rlm(kk,  1)+  rlm(kk,  2)- cpm(kk,1,2)- cpm(kk,2,1) )

c OPTION:  INSERT THIS LINE FOR PRESET Ox CONCENTRATION
c  NOTE - ALSO CHANGE 'PRESET' OPTION IN presolve
c       xox(kk) =   c_xcin(kk,ic1)+  c_xcin(kk,ic1)

c     enddo                ! kk vector loop

c NOXSOLVE DEBUG WRITE
      if(c_kkw.gt.0) write(c_out,1855) xox(c_kkw)
1855   format(' NOXSOLVE XOX (= xrp*rpm/(rlm-cpm)) = ', 1pe10.3)

c --------------------------------------
c NO:  SOLVE BACKWARD EULER EQUATION FOR NO AS FUNCTION OF OX, NOx.
c
c THE EQUATION:  RP" + r1*NO2 = RL" + r2*NO*O3  BECOMES
c  RP" + r1*(NOx-NO) = RL"*NO + r2*NO*(Ox-NOx+NO).   
c  WHERE RP", RL" = production, loss of NO (including NO->NO2)
c      but without R1:  NO2->NO+O3 and R2: NO+O3->NO2.
c 
c  THIS PROVIDES A QUADRATIC FOR NO, SINCE Ox, NOx VARY SLOWLY.

c  SUBSTITUTE CPRO(O3->NO2) = r1*NO2prior; 
c             CPRO(NO2->O3)=r2*O3p*NOp, w/ aq sums.
c
c  -> RP" + CP21(NOx-NO)/NO2p = RL"*NO + CP12*[(Ox-NOx+NO)/O3p]*[NO/NOp]
c
c  -> RP" + CP21(NOx/NO2p) = NO* [RL" + CP21/NO2p + CP12*(Ox-NOx)/(O3p*NOp)]
c                          +NO^2 *[CP12/(O3p*NOp)]
c
c  JANUARY 2005: SOLVE THIS QUADRATIC WITH THE FOLLOWING SUBSTITUTIONS:  
c
c   RP" = rpm3  = p(NO), excluding NO2+hv.  (NO2+hv goes to cpro, not rpro)
c   RL" = (rlm3 - cpm(1,2).  rlm3 includes all losses of NO.
c          subtract cpm12 (not cpm32) because cpm12 is O3+NO->NO2 exactly.
c                   (cpm32 includes NO+HO2=>NO2)
c  CP21, CP12 = cpm(1,2), cpm(2,1)
c 
c   NOTE:  DIVIDES ADD COMPUTER TIME BUT PREVENT OVERFLOWS
c --------------------------------------

c     do kk=1,c_kmax
       alpha(kk) =  cpm(kk,1,2)/( xrm(kk,3)* xrm(kk,1))
       gamma(kk) =    rpm(kk,3)           
     *           + ( cpm(kk,2,1)/ xrm(kk,2)) * xnox(kk)
       beta(kk) = (  rlm(kk,3) -  cpm(kk,1,2))/ xrm(kk,3)
     *          +  cpm(kk,2,1)/ xrm(kk,2)
     *      + ( cpm(kk,1,2)/( xrm(kk,1)* xrm(kk,3))) 
     *           * (xox(kk)-xnox(kk))

        xrrm(kk, 3) = 0.5* (
     *           sqrt(beta(kk )**2 + 4.*alpha(kk )*gamma(kk ) ) 
     *           - beta(kk )       
     *                    )/alpha(kk )

        xrrm(kk, 2) = xnox(kk) - xrrm(kk, 3)
        xrrm(kk, 1) = xox(kk) - xrrm(kk, 2)

c ATTEMPTED NOXBUG FIX: IF O3<0.
C  ASSUME THAT O3+NO2->NO3 IS CAUSE,  it represents 2*Ox loss, 1*NOx loss.
c  USE BACK-EULER TO SET O3
c  FUTURE - ADD BETTER CRITERIA, TAKE WHICHEVER-IS-LOWER ESTIMATE?

        if(xrrm(kk,1).lt.0.001*xrm(kk,1)) then
          xrrm(kk,1) = xrm(kk,1) 
     *           *( rpm(kk,1)/rlm(kk,1))
        endif
c       enddo

c NOXSOLVE DEBUG WRITE
       if(c_kkw.gt.0) write (c_out,1858) 
     *      alpha(c_kkw), beta(c_kkw), gamma(c_kkw)
1858   format(' NOXSOLVE DEBUG: ',/,
     *  '     ALPHA = (cp:O3->NO2)/(O3*NO) =    ',1pe10.3,/,
     *  '     BETA  =                           ',1pe10.3,/,
     *  '     GAMMA = rpNO+(cpNO2->O3)*NOx/NO2 =', 1pe10.3  )

c NOXTEST:  IF KMAX=1 ONLY; SAVE TEST RATIO FOR NO2 vs PRIOR NO2
        c_notest = abs(1.-xrrm( 1, 2)/ xrm( 1,  2))

c  WRITE OPTION - WRITE SUMMARY FOR NOX-OX HERE
c   (STANDARD OUTPUT)

      if(c_kkw.gt.0) then

       write(c_out,1901)  ic1,ic2,ic3
1901  format(//, '   NOXSOLVE IC=',3i3)
      write(c_out,1902)
1902  format('NOX ANALYSIS:',/,'   O3        NO2       NO     ',
     * '   NO3       HNO3      PANs      HONO-HNO4  '        )

      pansum = 0.
      hnosum = 0.
      do ic = 1,c_nchem2
       if(c_icat(ic).eq.5) pansum=pansum+xc(c_kkw,ic)
       if(c_icat(ic).eq.6) hnosum = hnosum+xc(c_kkw,ic)
      enddo

      write(c_out,1911) xrrm(c_kkw,1),xrrm(c_kkw,2),xrrm(c_kkw,3),
     * xc(c_kkw,c_nno3     ),xc(c_kkw,c_nhno3    ),pansum,hnosum
1911  format(8(1pe10.3))

      write(c_out,1904)
1904  format(/,'PRIOR O3     NO2        NO')   
      write(c_out,1911)  xrm(c_kkw,1), xrm(c_kkw,2), xrm(c_kkw,3)

      write(c_out,1905)
1905  format(/,'RPRO  O3     NO2        NO')
      write(c_out,1911)  rpm(c_kkw,1), rpm(c_kkw,2), rpm(c_kkw,3)

      write(c_out,1906)
1906  format(/,'RLOSS (including NO+O3<->NO2)')
      write(c_out,1911)   rlm(c_kkw,1),  rlm(c_kkw,2),  rlm(c_kkw,3)

      write(c_out,1907)  cpm(c_kkw,2,1), cpm(c_kkw,1,2),
     *      sourcnx(c_kkw),sinknx(c_kkw)
1907  format(/,'CPRO:  NO2+hv ->',(1pe10.3),/,
     *         '       NO+O3  ->',(1pe10.3),/,
     *         'NOXPRO        = ',(1pe10.3),/,
     *         'NOXLOSS       = ',(1pe10.3))

      write(c_out,1908)
1908  format(/,'ANALYSIS:',/,
     * '   XNOX  =  SOURCNX / (SINKNX / XNOXP----------)')
      write(c_out,1911) xnox(c_kkw),sourcnx(c_kkw),sinknx(c_kkw),
     *     xrm(c_kkw,ic2),  xrm(c_kkw,ic3)

        oxox   =  xrm(c_kkw,ic1)+ xrm(c_kkw,ic2)
        sourcox =   rpm(c_kkw,  1)+ rpm(c_kkw,  2)
        sinkox =    rlm(c_kkw,  1)+  rlm(c_kkw,  2)
     *              - cpm(c_kkw,  2,1)- cpm(c_kkw,  1,2) 
      write(c_out,1909)
1909  format(/,'   XOX   =   OXPRO / (OXLOSS / OXP)')
      write(c_out,1911) xox(c_kkw),sourcox, sinkox,oxox


c QUADRATIC ANALYSIS:  xk2 + (xk2n+xrln) = xjnx + xrpn
c                     (~NO**2)  (~NO)        (~1)
c                   alpha*NO**2 + beta*NO = gamma  with new NO.

      if(xrrm(c_kkw,3).eq.0) xrrm(c_kkw,3)=1.
      xk2 = (xrrm(c_kkw,3)**2) * alpha(c_kkw)
      xk2n = xrrm(c_kkw,3) 
     *   *( cpm(c_kkw,1,2)/( xrm(c_kkw,1)* xrm(c_kkw,3))) 
     *                          * (xox(c_kkw)-xnox(c_kkw))
      xrln = xrrm(c_kkw,3)
     *          * (  rlm(c_kkw,3) -  cpm(c_kkw,1,2))/ xrm(c_kkw,3)
      xjn = xrrm(c_kkw,3)   *   cpm(c_kkw,2,1)/ xrm(c_kkw,2)
      xjnx =       ( cpm(c_kkw,2,1)/ xrm(c_kkw,2)) * xnox(c_kkw)
c     xrpn =  rpm(c_kkw,3)+ rpm(c_kkw,2)
      xrpn =  rpm(c_kkw,3)

      write(c_out,1912)
1912  format(/, 'NO ANALYSIS:  QUADRATIC EQUATION w/ k: NO+O3<->NO2',/,
     * ' alpha*NO**2 +  beta........beta.....beta*NO  =  gamma...gamma'
     *,/,
     * ' k2*NO**2  k2*NO*(Ox-NOx) otherL*NO    j1*NO   j1*NOx   rpNO')
      write(c_out,1911) xk2, xk2n, xrln, xjn,xjnx,xrpn
      write(c_out,1913)
1913  format(/,' alpha       beta     gamma                  ')
      write(c_out,1911) alpha(c_kkw), beta(c_kkw), gamma(c_kkw)


      endif

c ZERO PROTECT: THIS ALGORITHM SHOULD NEVER GENERATE NEGATIVE
c UNLESS THERE IS A MATH ERROR OR TINY NUMERICAL NEGATIVE.
c        do kk=1,c_kmax
          if(xrrm(kk,1).le.0) then
           xrrm(kk,1)=0.001
          endif
c        enddo

c        do kk=1,c_kmax
          if(xrrm(kk,2).le.0) then
           xrrm(kk,2)=0.001
          endif
c        enddo

c        do kk=1,c_kmax
          if(xrrm(kk,3).le.0) then
           xrrm(kk,3)=0.001
          endif
c        enddo

c NOTE:  SAVE PRIOR AS XNO, XNO2, XO3?

c DIFFICULT CONVERGENCE AID:  
c AVERAGE O3, NO2, NO WITH PRIOR TIME STEP HERE IF DESIRED.
c  - note 2009 ALTERNATIVE below.

         if(c_iter.gt.5) then
c      if(c_iter.eq.-1) then     
c        do kk=1,c_kmax
          xrrm(kk ,3) = 0.5*( xrm(kk,3 )+xrrm(kk,3) )
          xrrm(kk ,1) = 0.5*( xrm(kk,1 ) +xrrm(kk,1) )
          xrrm(kk ,2) = 0.5*( xrm(kk,2 )+xrrm(kk,2) )
c        enddo
        endif


c  ---- ADDITION: 2-21-13 - setgeom -------
c
c 2009 NOX CONVERGENCE OPTION/ALTERNATIVE:  SETGEOM
c  adjust based on history of NOx.  (or Ox).
c
c  Note, alt:  geom adjust xnox, xox above? No.
c        cannot do separate geom. avg for NOx; O3-NO-NO2 must be all together.

c       if(c_iter.ge.4) then
      if(c_iter.eq.-1) then
         call setgeom(ic2,geomavg,history,c_iter)
c      do kk=1, c_kmax          ! kk vector loop
          xrrm(kk ,3) =(xrrm(kk ,3)  **geomavg(kk,ic2) )
     *                * (xrm(kk ,3)    **(1.-geomavg(kk,ic2)) )
          xrrm(kk ,1) =(xrrm(kk ,1)  **geomavg(kk,ic2) )
     *                * (xrm(kk ,1)    **(1.-geomavg(kk,ic2)) )
          xrrm(kk ,2) =(xrrm(kk ,2)  **geomavg(kk,ic2) )
     *                * (xrm(kk ,2)    **(1.-geomavg(kk,ic2)) )
         history(kk,ic2,c_iter) = xrrm(kk,2)+xrrm(kk,3)   ! NOx
       endif               !if(c_iter.ge.4) then
c      enddo                  ! kk vector loop

       if(c_kkw.gt.0) write(c_out,*) 'NOx geomavg =',
     &      geomavg(c_kkw,ic2)



c END NOXSOLVE
2000  return
      end
c ---------------------------------------------
c

      subroutine ohsolve(ic1,ic2,ic3, 
c                       (OH, HO2, H2O2)
     &    c_xcin, c_xcout, c_xcav, c_xcemit,
     &    c_rp, c_rl, c_rr ,
     &    xc,xcfinr,  xclastq, 
     &    ratek, rateh, rateq, rateqq, rhdif, egasaq, 
     &     xrp, xrr, rpro, rloss, rpro1, rloss1, 
     &       rrp, rrl, cpro,
     &       xrm, xrrm, rpm, rlm,cpm, rpmulti, rlmulti, AX, BX, XX,
     &       rppair, rlpair, xrppair, xrrpair, 
     &       history, geomavg  

     &        , senshx, senhcat, c_ohtest
     &        , oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2, xfohtest
     &        ,sourcnx, sinknx, c_notest
     &     ,  c_h2oliq ,   c_rainfr,  c_lexpo, c_iter

     &        )

c 
c 
c
c Radical balance solution for odd-hydrogen radicals:
c   OH and HO2, also with adjustment for H2O2 and CO3 (aq)
c
c This uses the algorithm from Sillman, 1991 (J. Geophys. Res.)
c
c Critical parameters are:  
c   oddhsum:  prior sum of radical net production-loss
c   oddhdel:  prior sum weighted by d(lnHx)/d(lnOH)
c             = sensitivity of radical sum to OH.
c
c These form a linear sum (A+B*OH) for net radical production
c   which is used to solve for OH
c
c (Note:  change in Hx concentration is also added to oddhsum
c   for a non-steady-state solution).
c
c Solution is also adjusted by geometric mean with prior iteration
c   with geometric mean parameter set by iteration history (setgeom)
c
c The following solutions are generated: 
c
c   OH is solved from the radical balance equation.
c
c   HO2 is solved from OH/HO2 in the production/loss equation for OH.
c 
c   Both are partitioned between gas and aqueous species.
c 
c   CO3 (aq) is adjusted based on the radical balance.
c 
c   H2O2 is solved here by a normal call to CHEMSOLVE
c     (needed only to insure H2O2 is solved in the proper order)
c
c 2005 CHANGE:  USE RRP, RRL, where RP, RL IS SET TO ZERO.
c
c Inputs:    Species numbers (ic) for OH, HO2, H2O2
c            Also uses:  
c             oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2
c
c Outputs:  xc:    concentrations for OH and HO2
c
c
c Called by:    quadchem
c
c Calls to:     chemsolve (to solve H2O2)
c               setgeom
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c
c -------------------------------------------------------------------
c
c ---------------------------------------------------
c
      use h_rbchemmech
      use h_rbchemvars, only : c_time
      use h_rbchemlocal, only:  avogadrl
c      use h_rbchemlocal, only: kk,ic, is, ics,i, nr, neq, avogadrl,
c     & alpha, foh, beta, gamma,icc
c      use h_rbchemlocarr
c      use h_rbchemarrays
      IMPLICIT NONE


c#     include "rbchemmech.h"
c#     include "rbchemvars.h"
c#     include "rbchemlocal.h"
c#     include "rbchemlocarr.h"
c#     include "rbchemarrays.h"
c
       integer :: ic1, ic2, ic3
       real*8 ::  c_xcin( c_kvec,c_cdim) ! concentr molec/cm3
       real*8 ::  c_xcout( c_kvec,c_cdim) ! concentr, mol/cm3
       real*8 ::  c_xcav( c_kvec,c_cdim)  ! concentr, mol/cm3
       real*8 ::  c_xcemit( c_kvec,c_cdim) ! emis, molec/cm3
      real*8 ::  c_rp(c_kvec,c_cdim)  ! Production, molec/cm3
      real*8 ::  c_rl(c_kvec,c_cdim)  ! Loss, molec/cm3
      real*8 ::  c_rr(c_kvec,c_rdim)  ! Reaction rate m/cm3
        real*8 :: xc( c_kvec,c_cdim) ! concentration molec/cm3
       real*8 :: xcfinr( c_kvec,c_cdim) ! ratio FINAL/AVG cncn
       real*8 :: xclastq( c_kvec,c_cdim) ! conc molec/cm3
          real*8 :: ratek(c_kvec,c_rdim)  ! Rate constants
      real*8 :: rateh(c_kvec,c_hdim)  ! Henry's law constants
      real*8 :: rateq(c_kvec,c_aqdim)  ! Aqueous eq. constants
      real*8 :: rateqq(c_kvec,c_aqqdim) ! Special eq. constants
      real*8 :: rhdif(c_kvec,c_hdim) ! H. mod by diffusion
      real*8 :: egasaq(c_kvec,c_hdim) ! Gas-to-aq transfer s-1
      real*8 :: xrp(c_kvec,c_cdim)     !Solver prior concentration
      real*8 :: xrr(c_kvec,c_cdim)     ! Solver concentration
      real*8 :: rpro(c_kvec,c_cdim)    ! Solver production rate
      real*8 :: rloss(c_kvec,c_cdim)   ! Solver loss rate
      real*8 :: rpro1(c_kvec,c_cdim)   ! Interim rpro w/out cpro 
      real*8 :: rloss1(c_kvec,c_cdim)  ! Interim rloss w/out cpro
      real*8 :: rrp(c_kvec,c_cdim)     ! Interim production rate 
      real*8 :: rrl(c_kvec,c_cdim)     ! Interim loss rate 
      real*8 :: cpro(c_kvec,c_cdim, c_cdim)   ! Cross-production 
      real*8 :: xrm(c_kvec,c_cdim)     !  Initial conc. for multi
      real*8 :: xrrm(c_kvec,c_cdim)    !Initial solution for multi
      real*8 :: rpm(c_kvec,c_cdim)     ! Production rate for multi
       real*8 :: rlm(c_kvec,c_cdim)     ! Loss for multi-spec 
      real*8 :: cpm(c_kvec, c_cdim, c_cdim) !Cross-prod for multi
      real*8 :: rpmulti(c_kvec       ) !Product'n multi group sum
      real*8 :: rlmulti(c_kvec       ) ! Loss group sum for multi
      real*8 :: BX(100), XX(100), AX(100,100) ! For back_Eulersolution AX=B
      real*8 :: rppair(c_kvec,c_cdim)  ! Production for pair grp
      real*8 :: rlpair(c_kvec,c_cdim)  ! Loss rate for pair grp
      real*8 :: xrppair(c_kvec)        ! Solver prior pair group cn.
      real*8 :: xrrpair(c_kvec)        ! Solution for pair group sum
      real*8 :: history(c_kvec,c_cdim,400)  ! Cncn for each iter
      real*8 :: geomavg(c_kvec,c_cdim)   ! Geom. avg factor
      real*8 ::        senshx(c_kvec,2)   ! Hx sens: d ln(pHx)/d
c       ln([OH])
      real*8 ::        senhcat( c_kvec,99) ! Hx sens. by species categ.
      real*8 ::  c_ohtest      ! test: dOH/OH or dOH/HO2
      real*8 ::        oddhsum(c_kvec,2)   ! Summed net P/L Hx, mol/cm3
      real*8 ::        oddhdel(c_kvec,2)   ! Summed Hx sens to Oh,
      real*8 ::        oddhsrc(c_kvec,2)   ! Summed P Hx, mol/cm3
      real*8 ::        oddhloh(c_kvec,2)   ! Hx loss from OH,  mol/cm3
      real*8 ::        oddhlho2(c_kvec,2)   ! Hx loss from HO2,  mol/cm3
      real*8 :: xfohtest          ! OH/HO2 convergence test 
      real*8 :: sourcnx(c_kvec)   ! Summed NOx source, mol/cm3
      real*8 :: sinknx(c_kvec)    ! Summed NOx sink, mol/cm3
      real*8 ::  c_notest      ! test: dNO2/NO2
      real*8 ::  c_h2oliq(c_kvec)   ! LWC grams cm-3
      real*8 ::  c_rainfr(c_kvec)   ! rainout fraction
      logical :: c_lexpo(c_kvec)        ! Flag for expo decay solution
       integer :: c_iter                  ! chem. number of iterations
c----below are local varibles ---Guangxing Lin 
c 

       real*8 ::  c_droplet(c_kvec)        ! droplet radius, cm
c      below are from rbchemlcoal
c   mol/cm3


           real*8 :: stoicx                         ! stoichiometry sum
      integer :: ic,icc,ics,ics2,icc1,icc2,icc3 !Chem index
      integer :: is,iss,iscs,isc2                    !Chem local index
          integer :: ich,icq,icx,icx1,icx2,icy1,icy2,icp,icp1,icp2 !Chem
      integer :: icr1,icr2,icr,isr1,isr2,icpair  !chem index - pair and
      integer :: ionsum                                  ! aquasolve ion
      integer :: ica1,ica2,icb1,icb2,nra1,nra2,nrb1,nrb2 !Aquasolve chem
      integer :: neq                                     !Aqueous
      integer :: nc,nc1,nc2,ncc,ncf,nn,nne               !Chem species
      integer :: nr,nr1,nr2,nrh,nrq,nrqq,nrx,np          ! Reaction
      integer :: icat1,icat2,icatp,icatp2  ! indices used for species
      integer :: kk, kw                            ! Vectorization
      integer :: i, j, k, ii, ij, iii, n           ! General counters
       integer :: nsol           ! Number of pair groups being solved
      integer :: nsolv          ! Number of species being solved for
          real*8 :: alpha(c_kvec)   ! General vector variable
      real*8 ::  beta(c_kvec)   ! General vector variable
      real*8 :: gamma(c_kvec)   ! General vector variable
      real*8 :: prior(c_kvec)   ! Prior species conc (molec/cm3)
      integer :: ncsolv(c_cdim) ! Species number (ic) for species in
      integer :: nssolv(c_cdim) ! Pair group number (is) for spec. in
       real*8 :: foh(c_kvec)       ! OH/HO2.  not common

c      integer :: ic1,ic2,ic3

c LOCAL VARIABLES:
c
c LOCAL VARIABLES USED IN SOLUTION

c foh(kk)           OH/HO2 ratio (declared in 'chemlocal.EXT')
c foh1(kk)          OH/HO2 ratio from prior iteration
c foh1a(kk)         Production of HO2 from OH->HO2 conv, molec/cm3
c foh1b(kk)         Production of HO2 from other sources, molec/cm3
c
c ncsol(kk)              Species list passed to solver subroutine
c
c oddhfac(kk)       Calculated factor for updating OH:
c                        (OH = OHp*oddhfac)
c
      real*8 :: foh1(c_kvec)    ! OH/HO2 from prior iteration
      real*8 :: foh1a(c_kvec)   ! Prod HO2 from OH, mol/cm3
      real*8 :: foh1b(c_kvec)   ! Prod HO2 from other, mol/cm3
      real*8 :: oddhfac(c_kvec) ! Factor for updating OH

      real*8 ::        oddhfac1(c_kvec)  ! Factor for updating OH - w/RO2
      real*8 ::        oddhfac2(c_kvec)  ! Factor for updating OH - w/o RO2
      real*8 ::        oddhro2f(c_kvec)  ! Weighing Factor for oddhfac 1 vs 2

c
      integer :: ncsol(c_cdim)       ! Species list passed to chemsolve
      integer :: namechem            ! Function to return chem index ic

c DOUBLE PRECISION AX, BX, XX - to pass to QUADCHEM

c LOCAL VARIABLES USED  FOR WRITTEN OUTPUT ONLY
      real*8 :: pansum     ! Sum for written output
      real*8 :: hnosum     ! Sum for written output
      real*8 :: xrooh      ! Sum for written output
      real*8 :: xhno4      ! Sum for written output
      real*8 :: xhno3      ! Sum for written output
      real*8 :: xco3       ! Sum for written output
c
c ---------------------------------------------------
      kk=1

c PRELIMINARY OPTION:  SOLVE H2O2 BY A NORMAL CALL TO CHEMSOLVE.
c  NOTE:  H2O2 HAS A SPECIAL TREATMENT WITHIN CHEMSOLVE
c  THIS INSURES THAT NET PRODUCTION OF H2O2 IS NO GREATER THAN HX SOURCE.

      ncsol(1) = ic3
      do ic=2,nsdim
        ncsol(ic) = 0
      enddo
      call chemsolve(ncsol, 
     &    c_xcin, c_xcout, c_xcav, c_xcemit,
     &    c_rp, c_rl, c_rr ,
     &    xc,xcfinr,  xclastq, 
     &    ratek, rateh, rateq, rateqq, rhdif, egasaq, 
     &     xrp, xrr, rpro, rloss, rpro1, rloss1, 
     &       rrp, rrl, cpro,
     &       xrm, xrrm, rpm, rlm,cpm, rpmulti, rlmulti, AX, BX, XX,
     &       rppair, rlpair, xrppair, xrrpair, 
     &       history, geomavg 
     &        , senshx, senhcat
     &        , oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2, xfohtest
     &        ,sourcnx, sinknx, c_notest
     &     ,  c_h2oliq ,   c_rainfr,  c_lexpo, c_iter
     &        )
c
c 
c  
c PRELIMINARY:  CALCULATE REMAINING ODD-H REACTION RATES AND HX SUMS.
c INVOKE CHEMSOLVE FOR (OH,-3,HO2) AND (OH,-1,HO2).
c THE RESULTING XRP, RPRO, RLOSS ARE USED BELOW.

c (NOTE SWITCHED ORDER FOR MAY 1995.  CALC RATES, RP, RL BEFORE RLOSS.)


c 2005:  PRELIMINARY CALL TO CHEMSOLV REPLACED - ADD IN HERE
c  Add for do ic=1,3,2  => oh, ho2 
c  Generate rrp, rrl (include products)
c  Generate rpro, rloss, xrp
c  Zero rrp, rrl.

c -----------
c PRELIMINARY CALCULATION: REMAINING REACTIONS AND XR, RP, RL SUM
c -----------
c  SUM INTO RLOSS, RPRO, XRP (w/ ST ST ADJUSTMENT)
c  AND ZERO RP, RL
c  ADOPTED FROM CHEMSOLVE

       do is = 1, 3, 2
         ics = ic1
         if(is.eq.3) ics = ic2

c REACTIONS
         if(c_nnrchem(ics).gt.0) then
           do i=1,c_nnrchem(ics)
             nr =  c_nrchem(ics,i)
             call brpro(nr, ratek,
     &         c_rr, xc, rrl, rrp, rppair, rlpair, c_h2oliq
     &        , senshx, senhcat
     &        , oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2, xfohtest
     &        ,sourcnx, sinknx
     &        )


           enddo        !do i=1,c_nnrchem(ics)
         endif        !if(c_nnrchem(ics).gt.0)

         if(c_nnrchp(ics).gt.0) then
           do     i=1,c_nnrchp(ics)
             nr = c_nrchmp(ics,i)

             call brpro(nr, ratek,
     &         c_rr, xc, rrl, rrp, rppair, rlpair, c_h2oliq
     &        , senshx, senhcat
     &        , oddhsum, oddhdel, oddhsrc, oddhloh, oddhlho2, xfohtest
     &        ,sourcnx, sinknx
     &        )

           enddo
         endif


c ZERO RUNNING SUMS
c           do  kk=1,c_kmax           ! kk vector loop
             rpro(kk,is) =  0.
             rloss(kk,is) = 0.
             xrp(kk,is) = 0.
c           enddo                   ! kk vector loop

c ESTABLISH RPRO, RLOSS AND XRP FOR THE 'BASE' SPECIES
c        do kk=1,c_kmax      ! kk vector loop
          rpro(kk,is) = rpro(kk,is)+  rrp(kk, ics) +   c_xcin(kk,ics)
          rloss(kk,is) =rloss(kk,is)+  rrl(kk, ics)
          xrp(kk,is) =  xrp(kk,is) +  xc(kk,ics)
c        enddo               ! kk vector loop

c ADD ALL AQUEOUS-EQUILIBRIUM SPECIES INTO SUMMED RPRO, RLOSS AND XRP.
c CONVERTING AQUEOUS INTO GAS UNITS (AVOGADRL)
         if(c_nequil(ics).gt.0)       then
           do  neq=1,c_nequil(ics)
             ic = c_ncequil(ics,neq)
c            do kk=1,c_kmax    ! kk vector loop
               xrp(kk,is) =   xrp(kk,is) 
     *                       + xc(kk,ic)*c_h2oliq(kk)*avogadrl
               rpro(kk,is) = rpro(kk,is) + rrp(kk,ic)
               rloss(kk,is) = rloss(kk,is)+rrl(kk,ic)
c            enddo           ! kk vector loop
           enddo         ! do neq=1,c_nequil(ics)
         endif

c NONSTEADY STATE ADJ:  IF NONSTEADY STATE OR ZERO RLOSS, ADD XRP to RPRO.
c        do kk=1,c_kmax      ! kk vector loop
          if(.not.c_lsts(ics).or.rloss(kk,is).eq.0)
     *          rloss(kk,is) = rloss(kk,is) + xrp(kk,is)
          if(rloss(kk,is).le.0.) rloss(kk,is) = 1.0e-08
c        enddo                       ! kk vector loop

c RECORD RP, RL AND ZERO RRP, RRL
c  NOTE:  RRP, RRL represent running sum through cascade.
c         RP, RL represent final sum through cascade.
c  SUBSEQUENT OH CALCULATION uses final sum (RP, RL), not RRP, RRL.

         do  neq=1,(c_nequil(ics)+1)
           ic=ics
           if(neq.gt.1) ic=c_ncequil(ics,(neq-1))
           c_rp(kk, ic) = rrp(kk,ic)
           c_rl(kk,ic) = rrl(kk,ic)
           rrp(kk,ic) = 0.
           rrl(kk,ic) = 0.
         enddo              !do  neq=1,(c_nequil(ics)+1)

       enddo        !do is=1, 3, 2
c -----------
c END LOOP - PRELIMINARY CALCULATION OF RP, RL
c -----------

c ------------
c  PRELIMINARY:  ADD PRIOR OH, HO2, RO2 INTO ODDHSUM, ODDHDEL
c ------------
c FIRST:   ADD PRIOR OH, HO2, RO2 INTO ODDHSUM, ODDHDEL
c   (XR DOES NOT INCLUDE AQ SUM.  JUST RPRO, XRP, ETC.)
c  CHANGE 1195:  DO NOT INCLUDE PRIOR SUM IF STEADY STATE
c
c  CHANGE 1195:  PRIOR ODDHDEL.  If RP, RL is small, then there
c    PRIOR ODDH shows little sensitivity to OH.
c  This was solved by changing PRIOR ODDHDEL here and SENCAT below.
c  JULY 1996:  PRIOR ODDHDEL = XR or (RP+RL), whichever is smaller.
c  JULY 1996:  INCLUDE AQUEOUS IN PRIOR ODDHSUM AND ODDHDEL.

c  (NOTE:  COPY THIS SECTION TO 'PRIOR' IN OHWRITE.)

c 2005 CHANGE:  This is now done AFTER OH, HO2 reactions processed into RP
c  NOTE:  This uses FINAL RP, RL rather than running sum RRP, RRL
c         since RRP, RRL have been set to zero.
c
c 2009 CHANGE: Do not count lumped species accumulators in PRIOR Hx sum.
c  note icat .eq.8 also added along with 9 or 10 = CO3-?)

c 2009 modifications...

      do ic=1,c_nchem2 
        if((c_icat(ic).eq.3.or.c_icat(ic).eq.9.or.c_icat(ic).eq.10
     &             .or.c_icat(ic).eq.8 ).and.
     *          .not.c_lsts(ic).and. .not.c_llump(ic))  then

         if(ic.eq.c_npequil(ic)) then 
c kvec     do kk=1,c_kmax
            oddhsum(kk,1) = oddhsum(kk,1)
     &                   +   c_xcin(kk,ic) - xc(kk,ic)

            alpha(kk)=0.
c kvec     enddo                      ! kk vector sum
c kvec     do       kk=1,c_kmax       ! kk vector sum
            if(c_rl(kk,ic)+xc(kk,ic).gt.0) then
             alpha(kk) =  (c_rp(kk,ic)+c_rl(kk,ic))*xc(kk,ic)
     *           /(c_rp(kk,ic)+c_rl(kk,ic)+xc(kk,ic))

c   TEMPORARY 2005 BUG CHANGE!!!! 2005 -
c     SHOULDN'T oddhdel include PRIOR MINUS ANY SENSITIVITY????
c            alpha(kk) =  (xc(kk,ic)          )*xc(kk,ic)
c    *           /(c_rp(kk,ic)+c_rl(kk,ic)+xc(kk,ic))

             oddhdel(kk,1) = oddhdel(kk,1)
     &             - senhcat(kk,c_icat(ic)) * alpha(kk)
c TEMPORARY DEBUG WRITE
c            if(c_kkw.gt.0) write(c_out, 11019)
c    *        c_tchem(ic),c_icat(ic), xc(c_kkw,ic)
c    *       ,   c_xcin(c_kkw,ic), alpha(c_kkw)
c    *       , senhcat(c_kkw,c_icat(ic)), oddhsum(c_kkw,1), 
c    &          oddhdel(c_kkw,1)
c    &         ,oddhsum(c_kkw,2), oddhdel(c_kkw,2)
c 11019        format('OHSOLVE: CHEM ICAT  XR XXO DelFAC SENHCAT',
c    *                           /,a8,i4,4(1Pe10.3),/,
c    *        ' ODDHSUM1 ODDHDEL1, ODDHSUM2, ODDHDEL2=',7(1Pe10.3))

           endif
c          enddo
         else

           if(c_h2oliq(kk).gt.0) then
c kvec       do kk=1,c_kmax           ! kk vector sum
               oddhsum(kk,1) = oddhsum(kk,1)
     &       + (  c_xcin(kk,ic) - xc(kk,ic)) *c_h2oliq(kk)*avogadrl
               alpha(kk)=0.
c kvec       enddo                      ! kk vector sum
c kvec       do kk=1,c_kmax           ! kk vector sum
               if(c_rl(kk,ic)+xc(kk,ic).gt.0) then
                 alpha(kk) = ( c_rp(kk,ic)+c_rl(kk,ic))
     *                            *xc(kk,ic)*c_h2oliq(kk)*avogadrl
     *              /(c_rp(kk,ic)+c_rl(kk,ic)
     *                            +xc(kk,ic)*c_h2oliq(kk)*avogadrl)
                  oddhdel(kk,1) = oddhdel(kk,1)
     &             - senhcat(kk,c_icat(ic)) * alpha(kk)
               endif
c kvec       enddo                      ! kk vector sum

           endif             ! if(c_h2oliq(kk).gt.0) then

         endif                   ! if(ic.eq.c_npequil(ic)) then


c TEST WRITE
c          if(c_kkw.gt.0) write(c_out,11025) c_tchem(ic),xc(c_kkw,ic),
c    *       alpha(c_kkw), senhcat(c_kkw,c_icat(ic)),
c    *          oddhsum(c_kkw,1), oddhdel(c_kkw,1)
c 11025      format(/' PRIOR HX: XR  (RL*XR) SENHCAT ODDHSUM ODDHDEL =',
c    *  /,a8,2x,8(1pe10.3))


        endif     !  if((c_icat(ic).eq.3...) 
      enddo         !do ic=1,c_nchem2
c
c END : ADD PRIOR OH, HO2 TO ODDHSUM INCLUDING RO2
c
c REPEAT ADD PRIOR OH, HO2 WITH ODDHSUM-2, WITHOUT RO2


      do ic=1,c_nchem2

        if((c_icat(ic).eq.8.or.c_icat(ic).eq.9.or.c_icat(ic).eq.10 )
     *         .and.  .not.c_lsts(ic).and. .not.c_llump(ic))  then


         if(ic.eq.c_npequil(ic)) then
c kvec     do       kk=1,c_kmax       ! kk vector sum
            oddhsum(kk,2) = oddhsum(kk,2)
     &                   +   c_xcin(kk,ic) - xc(kk,ic)


            alpha(kk)=0.
c kvec     enddo                      ! kk vector sum
c kvec     do       kk=1,c_kmax       ! kk vector sum
            if(c_rl(kk,ic)+xc(kk,ic).gt.0) then
             alpha(kk) =  (c_rp(kk,ic)+c_rl(kk,ic))*xc(kk,ic)
     *           /(c_rp(kk,ic)+c_rl(kk,ic)+xc(kk,ic))

c   TEMPORARY 2005 BUG CHANGE!!!! 2005 -
c     SHOULDN'T oddhdel include PRIOR MINUS ANY SENSITIVITY????
c            alpha(kk) =  (xc(kk,ic)          )*xc(kk,ic)
c    *           /(c_rp(kk,ic)+c_rl(kk,ic)+xc(kk,ic))

             oddhdel(kk,2) = oddhdel(kk,2)
     &             - senhcat(kk,c_icat(ic)) * alpha(kk)

c TEMPORARY DEBUG WRITE
c            if(c_kkw.gt.0) write(c_out, 11019)
c    *        c_tchem(ic),c_icat(ic), xc(c_kkw,ic)
c    *       ,   c_xcin(c_kkw,ic), alpha(c_kkw)
c    *       , senhcat(c_kkw,c_icat(ic)), oddhsum(c_kkw,2), 
c    &          oddhdel(c_kkw,2)

            endif
c kvec     enddo                      ! kk vector sum
         else                    ! if(ic.eq.c_npequil(ic)) then


           if(c_h2oliq(kk).gt.0) then
c kvec       do kk=1,c_kmax           ! kk vector sum
               oddhsum(kk,2) = oddhsum(kk,2)
     &       + (  c_xcin(kk,ic) - xc(kk,ic)) *c_h2oliq(kk)*avogadrl
               alpha(kk)=0.
c kvec       enddo                      ! kk vector sum
c kvec       do kk=1,c_kmax           ! kk vector sum
               if(c_rl(kk,ic)+xc(kk,ic).gt.0) then
                 alpha(kk) = ( c_rp(kk,ic)+c_rl(kk,ic))
     *                            *xc(kk,ic)*c_h2oliq(kk)*avogadrl
     *              /(c_rp(kk,ic)+c_rl(kk,ic)
     *                            +xc(kk,ic)*c_h2oliq(kk)*avogadrl)
                  oddhdel(kk,2) = oddhdel(kk,2)
     &             - senhcat(kk,c_icat(ic)) * alpha(kk)
               endif
c kvec       enddo                      ! kk vector sum
           endif             ! if(c_h2oliq(kk).gt.0) then

         endif                   ! if(ic.eq.c_npequil(ic)) then


c TEST WRITE
c          if(c_kkw.gt.0) write(c_out,11025) c_tchem(ic),xc(c_kkw,ic),
c    *       alpha(c_kkw), senhcat(c_kkw,c_icat(ic)),
c    *          oddhsum(c_kkw,2), oddhdel(c_kkw,2)

        endif     !  if((c_icat(ic).eq.3...) 
      enddo         !do ic=1,c_nchem2

c END REPEAT: ADD PRIOR OH, HO2 TO ODDHSUM


c (dHxdNOx*dNOxdOH IS CUT.  IT IS COUNTED THROUGH SENHCAT(NOX).)

c Add PRIOR OH, HO2 to oddhloh, oddhlho2
c 2009 CORRECTION: steady state control

          if(.not.c_lsts(ic1))  then
            do i=1,2

      do neq=1,(c_nequil(ic1)+1)
        ic=ic1
        if(neq.gt.1)ic=c_ncequil(ic1,(neq-1))
c       do  kk=1,c_kmax        ! kk vector loop
          alpha(kk) = 1.
          if(neq.gt.1) alpha(kk)= c_h2oliq(kk)*avogadrl
           oddhloh(kk,i) = oddhloh(kk,i) + xc(kk,ic)*alpha(kk)
c       enddo                ! kk vector loop
      enddo            ! neq-1,(c_nequil(ic1)+1)

      do neq=1,(c_nequil(ic2)+1)
        ic=ic2
        if(neq.gt.1)ic=c_ncequil(ic2,(neq-1))
c       do  kk=1,c_kmax        ! kk vector loop
          alpha(kk) = 1.
          if(neq.gt.1) alpha(kk)= c_h2oliq(kk)*avogadrl
           oddhlho2(kk,i) = oddhlho2(kk,i) + xc(kk,ic)*alpha(kk)
c       enddo                ! kk vector loop
      enddo            ! neq-1,(c_nequil(ic2)+1)

           enddo         !do i=1,2
          endif                    !if(.not.c_lsts(ic1))  then



c ------------
c  END PRELIMINARY:  ADD PRIOR OH, HO2, RO2 INTO ODDHSUM, ODDHDEL
c ------------

c ------------
c CONVERGENCE TEST INDICES - SET TO PRIOR FOH, XOH.
c ------------
       xfohtest = foh(1)
       c_ohtest = xc(1,ic1)

c ------------
c  FOH = OH/HO2 RATIO
c ------------
c
c FOH=OH/HO2 RATIO.  THIS IS SPLIT INTO THREE COMPONENTS:
c FOH1A = OH SOURCE FROM HO2.  FOH1B=OTHER OH SOURCES
c FOH2 = OH SINKS.
c WHEN OH-HO2 EXCHANGE DOMINATES, OH/HO2 = FOH1A/FOH2.
c WHEN OTHER SOURCES DOMINATE, IT WORKS TO WEIGH FOH1B BY HO2 SINK/SRC.
c
c FROM OH BACK-EULER EQUATION: k2*HO2+Soh=kl*OH; FOH=k2/k1 + Soh/k2*HO2;
c -> FOH=FOHp*(FOH1a/FOH2 + FOH1b/(FOH2*xho2src/sink).

c NOTE:  USE XRP(KK,1) = OH PRIOR.  XRP(KK,3)=HO2 PRIOR.
c THESE WILL INCLUDE AQUEOUS EQUIVALENT SPECIES SUMS.
c
c  CHANGE 1195 - FOH1A, THE OH SOURCE FROM HO2, IS SUMMED 
c  FROM A SPECIAL REACTION INDEX (NRFOH).
c
c CHANGE 1996 
c  OH/HO2 IS ADJUSTED FROM PRIOR BASED ON RP/RL FOR OH, HO2
c  ALSO: DIFFICULT CONVERGENCE AID TO PROTECT AGAINST CRAZY HO2-H2O2
c  IN FIRST TIME STEP.
c    *** CHECK THAT THIS WORKS IN REMOTE TROP, OTHER ENVIRONMENTS.***
c         RP, RL SHOULD INCLUDE PRIOR OH, HO2 ALSO.

c 1996 ALTERNATIVE:   (GOES WITH 1996 WRITE, BELOW)
c      do kk=1,c_kmax             ! kk vector loop
         foh1(kk) = (xrp(kk,1)  / xrp(kk,3)) 
         foh(kk) = foh1(kk)
     *        * (rpro(kk,1)/rloss(kk,1) )  
     *        * (rloss(kk,3)/rpro(kk,3) )  

c 1997 - ADD PROTECT AGAINST WILD SWINGS IN FOH.  (10x, then 50/50 avg.)

        if(foh(kk).gt.100.*foh1(kk)) foh(kk)=100.*foh1(kk)
        if(foh(kk).lt.0.01*foh1(kk)) foh(kk)=0.01*foh1(kk)
c      enddo         ! kk vector loop


c  AUTOMATED DIFFICULT CONVERGENCE OPTION   (FEBRUARY 2005) :
c   SET GEOMETRIC MEAN vs PRIOR FOR CASES WITH DIFFICULT CONVERGENCE.
c   Set here for foh.   NOTE:  use OH to store FOH history.
c
       if(c_iter.ge.4) then
         call setgeom(ic1,  geomavg, history, c_iter)
       endif               !if(c_iter.ge.4) then
c      do kk=1, c_kmax          ! kk vector loop
         foh(kk)     =(foh(kk)    **geomavg(kk,ic1) )
     *                * (foh1(kk)    **(1.-geomavg(kk,ic1)) )
         history(kk,ic1,c_iter) = foh(kk)
c      enddo                  ! kk vector loop



c FOHTEST = CONVERGENCE TEST FOR FOH. (ABOVE,PRIOR FOHTEST=PRIOR FOH.)
      xfohtest = abs(1.-foh( 1  ) /(xfohtest    +1.0e-08))

c -------------------
c ODD HYDROGEN BALANCE.  
c -------------------
c     (A + B*OHp = ODDHSUM; A+B*OH=0; B*OHPp=ODDHDEL<0 ) 
c WITH ZERO PROTECT.  WARNING!  THIS HIDES A MULTITUDE OF (NIGHT) SINS.
c
c 1996 ADDITION:  OH, HO2 IS ADJUSTED 
c SO THAT % CHANGE IN OH*HO2 IS CONSTANT.  (OH=OH*sqrt(foh/fohp))
c
c  2005 CHANGE:  OH, HO2 adjusted based on Hx losses attributed to OH vs HO2
c  So that species responsible for losses changes closest to change in Hx.
c  Equations:   
c   Lho2*Fho2 + Loh*Foh = (Lho2+Loh )*Fh
c   OH/HO2 = (OHp/HO2p) * Fdroh      (Fdroh = FOH*HO2p/OHp)
c   => Foh*(Lho2/Fdroh + Loh) = (Lho2+Loh)*Fh
c    => Foh = (Lho2+Loh)*Fh/(Lho2/Fdroh+Loh)

c 2009 CHANGE:  ODDHFAC = weighted sum of two alternative oddh factors,
c   one with RO2 included and one WITHOUT RO2.
c   WEIGHTING: RATEK(no+ho2)*NO versus TIME  (r*NO/(r*NO + 1/time) sec-1/sec-1
c    (= include RO2 when rapid conversion RO2 to HO2)
c
c 2009 CHANGE: This only works for NON-AQUEOUS.
c
c   ODDH WEIGHTING FACTOR  (2009)

c kvec do kk=1,c_kmax         ! kk vector loop
        oddhro2f(kk) = 1.
c kvec enddo                  ! kk vector loop
c      if(c_iter.gt.1) then
c kvec   do kk=1,c_kmax         ! kk vector loop
c         if(c_h2oliq(kk).eq.0.) then
c          oddhro2f(kk) = (ratek(kk,c_nrho2no)*xc(kk,c_nno))

c-Guangxing Lin-----------02232013----no value for c_nrho2no----
           c_nrho2no=22
c-Guangxing Lin-----------02232013----no value for c_nrho2no----
           oddhro2f(kk) =2.* (ratek(kk,c_nrho2no)*xc(kk,c_nno))
           oddhro2f(kk) = oddhro2f(kk)/(oddhro2f(kk)+ 1.0/c_time)
c         endif
c kvec   enddo                  ! kk vector loop
c      endif          !if(c_iter.gt.1) then



c kvec do kk=1,c_kmax          ! kk vector loop

        oddhfac(kk) = 1.
        if(oddhdel(kk,1).ne.0) then
           oddhfac1(kk) = 1. - oddhsum(kk,1)/oddhdel(kk,1)
           if(oddhfac1(kk).le. 0.) oddhfac1(kk)= 0.2
        endif

        oddhfac2(kk) = 1.
        if(oddhdel(kk,2).ne.0) then
           oddhfac2(kk) = 1. - oddhsum(kk,2)/oddhdel(kk,2)
           if(oddhfac2(kk).le. 0.) oddhfac2(kk)= 0.2
        endif

        oddhfac(kk) = oddhro2f(kk)*oddhfac1(kk)
     &      + (1.-oddhro2f(kk)) *oddhfac2(kk)
c       oddhfac(kk) = oddhfac1(kk)

c          xc(kk,ic1) = xrp(kk,1)*oddhfac(kk)   ! moved just below

c
c kvec   enddo                  ! kk vector loop
c
c OHSOLVE DEBUG WRITE
c       if(c_kkw.gt.0) write(c_out,*) 'ODDHFAC1 = 1-ODDHSUM/ODDHDEL:',
c    &    oddhfac1(c_kkw), oddhsum(c_kkw,1), oddhdel(c_kkw,1)
c       if(c_kkw.gt.0) write(c_out,*) 'ODDHFAC2 = 1-ODDHSUM/ODDHDEL:',
c    &    oddhfac2(c_kkw), oddhsum(c_kkw,2), oddhdel(c_kkw,2)
c       if(c_kkw.gt.0) write(c_out,*) 'ODDHFAC, ODDHRO2F = ',
c    &    oddhfac(c_kkw), oddhro2f(c_kkw)

c ---------------
c 2009 OPTION to adjust dHx/dOH based on iter history was deleted from here
c ---------------

c --------------------------
c APPLY HX FACTOR TO OH, HO2
c --------------------------

c kvec do kk=1,c_kvec            ! kk vector loop
        xc(kk,ic1) = xrp(kk,1)*oddhfac(kk)

        if(foh(kk).ne.0) then

c 2005 OPTION
c  (2009 NOTE: what about steady state? Included in foh, added for oddhlho2)
c     OK:  oddhloh represents loss of Hx linked to OH, not source (=OHinit).
c     Prior OH counts as loss in nonsteady state case only.)
c
        alpha(kk) = oddhfac1(kk)*oddhloh(kk,1)
     &          +   oddhfac2(kk)*oddhloh(kk,2)
         beta(kk) = oddhfac1(kk)*oddhlho2(kk,1)
     &          +   oddhfac2(kk)*oddhlho2(kk,2)
c       if(oddhloh(kk,2).gt.0.and.oddhlho2(kk,2).gt.0.) then
        if(alpha(kk)    .gt.0.and.beta(kk)      .gt.0.) then

         xc(kk,ic1) = xc(kk,ic1)*
     *       (oddhloh(kk,2) + oddhlho2(kk,2)) /
     *         (oddhloh(kk,2)  + oddhlho2(kk,2)
     *                         /(foh(kk)*xrp(kk,3)/xrp(kk,1)) )
        else          !if(oddhloh(kk).gt.0.and.oddhlho2(kk).gt.0.) then
         xc(kk,ic1) = xc(kk,ic1)*sqrt(foh(kk)*xrp(kk,3)/xrp(kk,1))
        endif         !if(oddhloh(kk).gt.0.and.oddhlho2(kk).gt.0.) then

c HO2
         xc(kk,ic2) = xc(kk,ic1)/foh(kk)
        endif
c      enddo

c XOHTEST = CONVERGENCE TEST FOR OH.  
c  Based on prior GAS+AQ SUM (XRP) vs NEWLY CALCULATED
      c_ohtest = xrp(1,1)
      if(c_ohtest.eq.0.) c_ohtest = 1.0e-08
      c_ohtest = abs(1.-xc(1,ic1)  /(c_ohtest           ))

c 2009 CHANGE:  CONVERGENCE TEST = change in OH, HO2 vs RPRO-RLOSS
c      a1 =  rloss - rpro  (includes xc-xcin if non-steady-state)
c      a2 = max:  xc, (rpro+rloss)/2. 1e-8
c     test = abs(a1/a2)
c     test for OH, HO2, maximum

c CONVERGENCE TEST FOR OH
      alpha(1) = rloss(1,1) - rpro(1,1)    !rloss, rpro already include xc-xcin
c     if(.not.c_lsts(ic1))
c    &      alpha(1) = alpha(1) + xc(1,ic1) - c_xcin(1,ic1)
      beta(1) = 0.5*(rloss(1,1)+rpro(1,1))
      if(xc(1,ic1).gt.beta(1)) beta(1)=xc(1,ic1)
      if(beta(1).le.0.) beta(1) = 1.0D-8
      c_ohtest = abs(alpha(1)/beta(1) )

c CONVERGENCE TEST FOR HO2 (note, rpro(1,3) for HO2)
      alpha(1) = rloss(1,3) - rpro(1,3)  ! rloss, rpro include xc-xcin
c     if(.not.c_lsts(ic2))
c    &      alpha(1) = alpha(1) + xc(1,ic2) - c_xcin(1,ic2)
      beta(1) = 0.5*(rloss(1,3)+rpro(1,3))
      if(xc(1,ic2).gt.beta(1)) beta(1)=xc(1,ic2)
      if(beta(1).le.0.) beta(1) = 1.0D-8
      gamma(1) = abs(alpha(1)/beta(1) )
      if(gamma(1).gt.c_ohtest) c_ohtest = gamma(1)

c ADDED CONVERGENCE EXIT AT NIGHT:  IF XOH IS LOW AND NOX CONVERGES.
c  CUT  

c DIFFICULT CONVERGENCE OPTION:  GEOMETRIC  AVERAGE WITH PRIOR OH, HO2.

c  AUTOMATED CONVERGENCE OPTION (FEBRUARY 2005):
c   SET GEOMETRIC MEAN BASED ON HISTORY (vs HARD-WIRED OPTION above)
c  HARD-WIRED OPTION:  cut call to setgeom
c
c   Set here for Hx.   Use HO2 to store Hx history.
c   For Hx, adjust based on Hx parameter 1.-oddhsum/oddhdel
c           and history is running product of adjustments
c
c   Note OPTION WITHIN AUTOMATED CONV (SETGEO):  delta vs ratio.

           if(c_iter.ge.4) then
             call setgeom(ic2,  geomavg, history, c_iter)
           endif               !if(c_iter.ge.4) then
c          do kk=1, c_kmax          ! kk vector loop
             xc(kk,ic1) = (xc(kk,ic1)**geomavg(kk,ic2) )
     *                      *(xrp(kk,1)**(1.-geomavg(kk,ic2)) )
             xc(kk,ic2) = (xc(kk,ic2)**geomavg(kk,ic2)
     *                      )*(xrp(kk,3)**(1.-geomavg(kk,ic2)) )
             history(kk,ic2,c_iter) = oddhfac(kk)
c            oddhfacp(kk) = oddhfac(kk)**geomavg(kk,ic2)
c          enddo                  ! kk vector loop
           if(c_iter.gt.1) then
c 2009 CORRECTION: ics is error
c            do kk=1, c_kmax          ! kk vector loop
c              history(kk,ic2,c_iter) = history(kk,ics,c_iter)
c    *           * history(kk,ics,(c_iter-1))
               history(kk,ic2,c_iter) = history(kk,ic2,c_iter)
     *           * history(kk,ic2,(c_iter-1))
c            enddo                  ! kk vector loop
           endif              !if(c_iter.gt.1) then

c FEBRUARY 2005 CO3 HX ADJUSTMENT
c  ADJUST CO3 BY SAME ADJUSTMENT AS ODD-H.
c  WITH TEST TO MAKE SURE P(CO3) IS SIGNIFICANT

        ics = namechem('     CO3')
        if(ics.gt.0) then
c        do kk=1,c_kmax             ! kk vector loop
          alpha(kk) = 0.
c        enddo                    ! kk vector loop
         do neq=1,(c_nequil(ics)+1)
          icc=ics
          if(neq.gt.1)icc=c_ncequil(ics,(neq-1))
c         do kk=1,c_kmax            ! kk vector loop
            alpha(kk) = alpha(kk) + c_rp(kk,icc)
c         enddo                   ! kk vector loop
         enddo            ! neq-1,(c_nequil(ics)+1)
         do neq=1,(c_nequil(ics)+1)
          icc=ics
          if(neq.gt.1)icc=c_ncequil(ics,(neq-1))
c         do kk=1,c_kmax            ! kk vector loop
           if(alpha(kk).ge.0.1*oddhsrc(kk,1)) then
c            xc(kk,icc) = xc(kk,icc) * (oddhfac(kk)**0.5)
             xc(kk,icc) = xc(kk,icc) * (oddhfac(kk)**geomavg(kk,ic2) )
           endif                !if(alpha(kk).ge.0.1*oddhsrc(kk)) then
c         enddo                   ! kk vector loop
         enddo            ! neq-1,(c_nequil(ics)+1)
        endif        !if(ics.gt.0) then


C -----------------------------------------------
c TEST WRITE FOR OH, HO2 AND FOH FOR THIS ITERATION. 
c INCLUDING CALL TO OHWRITE FOR ODD HYDROGEN BALANCE

      if(c_kkw.gt.0) then

        write(c_out,1851) c_iter
1851    format(//,' ITER =',i3)

        xrooh = 0.
        pansum = 0.
        xhno4 = 0.
        xco3 = 0.
        do ic=1,c_nchem2
         if(c_icat(ic).eq. 4) xrooh = xrooh + xc(c_kkw,ic)
         if(c_icat(ic).eq. 5) pansum=pansum + xc(c_kkw,ic)
         if(c_icat(ic).eq. 6) xhno4 = xhno4 + xc(c_kkw,ic)
         if(c_tchem(c_npequil(ic)).eq.'     CO3') then
           if(c_tchem(ic).eq.'     CO3') then
              xco3 = xco3+ xc(c_kkw,ic)
           else
              xco3 = xco3+ xc(c_kkw,ic)*c_h2oliq(kk)*avogadrl
           endif
         endif 
        enddo

        write(c_out,1852)  
1852    format('ODD HYDROGEN:',//,
     *  '   OH       HO2      H2O2    H+ROOH     PAN     HNO3   ',
     *  '    CO3   FOH')

        if(oddhdel(c_kkw,1).eq.0.or.oddhdel(c_kkw,2).eq.0) then
         write(c_out, 1898)
1898     format(/,' WARNING!  ODDHDEL=0.  OH UNCHANGED.')
        else
c 2009 modification
         if(oddhsum(c_kkw,1)/oddhdel(c_kkw,1).gt.1) write(c_out,1849)
         if(oddhsum(c_kkw,2)/oddhdel(c_kkw,2).gt.1) write(c_out,1849)
1849      format(/,' WARNING!  OH<0, CORRECTED.')
        endif

        write(c_out,1899) xc(c_kkw,ic1),xc(c_kkw,ic2), xc(c_kkw,ic3),
     * xrooh, pansum, xc(c_kkw,6), xco3,foh(c_kkw)
c    * xrooh, pansum, xc(c_kkw,6),xhno4,foh(c_kkw)
1899    format(8(1pe10.3))

      write(c_out,1902)
1902  format(/,'   O3        NO2       NO     ',
     * '   NO3       HNO3      PANs      HONO    HNO4  '        )


      pansum = 0.
      hnosum = 0.
      do ic=1,c_nchem2
       if(c_icat(ic).eq.5) pansum=pansum+xc(c_kkw,ic)
       if(c_icat(ic).eq.6) hnosum = hnosum+xc(c_kkw,ic)
      enddo

      write(c_out,1911) xc(c_kkw,c_no3      ),xc(c_kkw,c_nno2     )
     * ,xc(c_kkw,c_nno      ), xc(c_kkw,c_nno3     )
     *     ,xc(c_kkw,c_nhno3    )
     * ,pansum,hnosum
1911  format(8(1pe10.3))

c PRIOR OH:  NOTE, THIS GIVES OH FROM PREVIOUS TIME STEP
c   AFTER DIFFICULT CONVERGENCE CORRECTION.
        write(c_out,1853)
1853    format(/'PRIOR OH     HO2    ODDHFAC   ',
     &     '(ODDHFAC1  ODDHFAC2  ODDHRO2F)')
        write(c_out,1899) xrp(c_kkw,1),xrp(c_kkw,3), oddhfac(c_kkw)
     &              ,oddhfac1(c_kkw), oddhfac2(c_kkw),oddhro2f(c_kkw)

c       write(c_out,1853)
c 1853    format(/'PRIOR OH     HO2     ODDHSUM1   ODDHDEL1'
c    &            '  ODDHSUM2  ODDHDEL2  ODDHRO2F'  )
c c  *    '  (OH=prior*(1-sum/del)')
c       write(c_out,1899) xrp(c_kkw,1),xrp(c_kkw,3), oddhsum(c_kkw,1),
c    *    oddhdel(c_kkw,1), oddhsum(c_kkw,2), oddhdel(c_kkw,2)
c    &    ,oddhro2f(c_kkw)


c FOH WRITE FOR 1195 METHOD:  
c       write(c_out,1854)
c c -> FOH=FOHp*(FOH1a/FOH2 + FOH1b/(FOH2*xho2src/sink).
c 1854    format(/,'FOH ANALYSIS:   (FOH1B=non-ho2 sources of OH)',/,
c    * '  _FOH   =  _FOHp  *(  (_RP9 /_RL9 )  + _FOH1b/rl9 *  (_RL10  ',
c    * '/ _RP10  -1) )') 
c      fohp = xrp(c_kkw,1)/xrp(c_kkw,3)
c      write(c_out,1899) foh(c_kkw), fohp      
c    *   , rpro(c_kkw,1),rloss(c_kkw,1),
c    *     foh1b(c_kkw),  rloss(c_kkw,3), rpro(c_kkw,3)

c FOH WRITE FOR 1996 METHOD:  
       write(c_out, 1856)
1856   format(/,'FOH ANALYSIS: ',/,
     * ' FOH   =   FOHp  *(RPRO(oh)/ RLOSS(oh))/(RPRO(ho2)/RLOSS(ho2))')
       write(c_out,1899) foh(c_kkw), foh1(c_kkw) 
     *  , rpro(c_kkw,1),rloss(c_kkw,1),
     *    rpro(c_kkw,3), rloss(c_kkw,3)

       write(c_out,1861) oddhloh(c_kkw,1), oddhlho2(c_kkw,1)
       write(c_out,1861) oddhloh(c_kkw,2), oddhlho2(c_kkw,2)
1861   format(/,'  ODDHLOH, ODDHLHO2 = ',(2(1pe10.3)))

       write(c_out,1862) geomavg(c_kkw,ic1), geomavg(c_kkw, ic2)
1862   format(/,'  GEOM AVG FACTOR FOR FOH, Hx = ', 2f10.4)

        if(c_kkw.gt.0) call ohwrite(c_kkw,
     &            c_xcin, xc, c_rp, c_rl, c_rr 
     &            , c_h2oliq, senshx, senhcat, oddhsum, oddhdel)

      endif

c  END WRITE FOR ODD-HYDROGEN.  BACK TO THE SOLUTION.
c  ----------------------------------------------------


c PARTITION OH AND HO2 BETWEEN GAS AND AQUEOUS-EQUILIBRIUM SPECIES
c AS IN CHEMSOLVE.  WATCH INDICES! FOR OH, XR(IC2) AND XRP(3).

        do 235 is=1,3
          icc = ic1
          if(is.eq.2) go to 235
          if(is.eq.3) icc=ic2

          if(icc.gt.0) then
            if(c_nequil(icc).gt.0) then

c AQUEOUS CONCENTRATIONS ARE UPDATED BY THE RATIO XR/XRP.  
            do neq=1,c_nequil(icc)
             ic = c_ncequil(icc,neq)
             if(ic.gt.0) then
c              do kk=1,c_kmax
                xc(kk,ic) = xc(kk,ic) 
     *                    * xc(kk,icc)/xrp(kk,is)
c              enddo
             endif
            enddo ! neq

c GAS-MASTER XR IS REDUCED BY AQUEOUS CONCENTRATIONS
c  WITH UNIT CONVERSION (AVOGADRL)
            do neq=1,c_nequil(icc)
             ic = c_ncequil(icc,neq)
             if(ic.gt.0) then
c              do kk=1,c_kmax
                xc(kk,icc) = xc(kk,icc) 
     *            - xc(kk,ic)*c_h2oliq(kk)*avogadrl
c              enddo
             endif
            enddo ! neq
            endif
          endif
235     continue

c ESTABLISH SENHCAT (=d lnXR/d lnOH) FOR ODD HYDROGEN SPECIES.
c  (SENHCAT IS USED TO RESOLVE HX = A+B*OH.  IN THE SUM, 
c   ODDHSUM = + dHx; ODDHDEL = + dHx*SENHCAT =>B*OH.)

c FOR HO2:  dHO2/dOH from HO2 EQ:  
c  kOH+otherSho2=k1HO2+2k2HO2**2 = 2RPh2o2 + other RLho2.
c   (NOTE:  THIS USES IC INDEX FOR HO2, H2O2.)

c (NOTE:  TECHNICALLY, d/dOH = d/dOH + d/dHO2 dHO2/dOH
c  + d/dNOx dNOx/dOH.  NORMALLY, d/dNOx IS USED ONLY FOR NOX SPECIES.
c  OLD PROBLEM:  PAN OSCILLATION.  Maybe solve this by adding
c   dRO2/dOH = dHO2/dOH + dRO2/dNOx dNOx/dOH = dHO2/dOH (1-dNOx/dOH)
c   where dNOx/dOH<0.  )
c
c  Possible error (bmaingtest).  Originally both HO2 and RO2 senhcat
c  were multiplied by RP/RP+XXO.  This term was first dropped for RO2.
c  Then after 'bmaingtest' it worked best to drop the term for HO2 and RO2.
c
c 1996 - AQUEOUS CHANGE.  RL, RP IS SCREWED UP BY EXCORR WITH AQUEOUS.
c TO FIX - REPLACE RL, RP WITH RLOSS, (RPRO-XRP) = GAS+AQUEOUS SUM.
c  ALGORITHM:  SENHCAT = RLOSS(HO2)/(RLOSS(HO2) + 2 RP(H2O2))

c 2009 CORRECTION: rp(H2O2) includes other sources (ALK+O3), leads to error.
c Trying an alternative:  oddhlho2/2. = rp(H2O2) from Hx...
c     but it also includes rp (ROOH).
c
c note - is not the maximum dln HO2/dln OH = 0.5, not 0? from HO2+HO2,
c   but less if other sources
c  -> it should be rp(HO2 from OH)/(*+rp(HO2 from other sources))
c    = rp(HO2)
c
c OPTION TO TRY: rp(HO2)*(1+RO2/HO2)/(+oddhsrc)?
c    rpHO2(1+oddhsrc2/oddhsrc)/(*+oddhsrc)  ?
c ALT: loh/(+(oddhsrc-2O3+hv)


c SUM GAS AND AQUEOUS:  RLOSS(HO2), RPRO(H2O2)
c     do kk=1,c_kmax
       alpha(kk) = c_rl(kk,ic2)
       beta(kk) = c_rp(kk,ic3)
c     enddo

      if(c_nequil(ic2).gt.0)       then
        do neq=1,c_nequil(ic2)
         ic = c_ncequil(ic2,neq)
c          do kk=1,c_kmax
              alpha(kk) = alpha(kk) + c_rl(kk,ic)
c          enddo
        enddo
      endif

      if(c_nequil(ic3).gt.0)       then
        do neq=1,c_nequil(ic3)
         ic = c_ncequil(ic3,neq)
c          do kk=1,c_kmax
              beta(kk) = beta(kk) + c_rp(kk,ic)
c          enddo
        enddo
      endif

c 2009 correction: beta (ph2o2) < alpha (lho2)
c    protects against pH2O2 from source other than HO2 (TERP+O3->0.02 H2O2)
c kvec do kk=1,c_kmax   ! kvec
          if(beta(kk).gt.alpha(kk)) beta(kk) = alpha(kk)
c         if(beta(kk).gt.0.5*alpha(kk)) beta(kk) = 0.5*alpha(kk)
c kvec enddo            ! kvec
c

c MAIN SENHCAT CALCULATION.
c     do kk=1,c_kmax

c 1996 VERSION
c 2005:  GEOM MEAN for DIFFICULT CONVERGENCE (HO2-MCO3-PAN oscillation)
        senhcat(kk,10) =  alpha(kk)  
     *                /  (alpha(kk) +2.*beta(kk)  ) 
c OPTION w/o  GEOM MEAN
c       senhcat(kk,3) = senhcat(kk,10)
c OPTION with GEOM MEAN
        senhcat(kk,3) = (senhcat(kk,10)**0.5)*(senhcat(kk,3)**0.5)
        senhcat(kk,10) = senhcat(kk,3)

c DEBUG OPTION
c       senhcat(kk,3) = 1.
c       senhcat (kk,10) = 1.


c OLDER HO2-RO2 SENHCAT ALGORITHMS
c PRIOR 1994 VERSION
c       senhcat(kk,10) =  c_rl(kk,ic2)
c    *                /  (c_rl(kk,ic2)+2.*c_rp(kk,ic3)) 
c       senhcat(kk,3) = senhcat(kk,10)

c OLDER HO2-RO2 SENHCAT ALGORITHMS
c       senhcat(kk,10) = (c_rp(kk,ic2)-  c_xcin(kk,ic2)) * c_rl(kk,ic2)
c    *          /(      c_rp(kk,ic2) * (c_rl(kk,ic2)+2.*c_rp(kk,ic3)) )
c       senhcat(kk,10) = (c_rp(kk,ic2)            ) * c_rl(kk,ic2)
c    *                /(     ( c_rp(kk,ic2)+  c_xcin(kk,ic2))
c    *                        * (c_rl(kk,ic2)+2.*c_rp(kk,ic3)) )
c       senhcat(kk, 3) =   
c    *    c_rl(kk,ic2)    /(   (c_rl(kk,ic2)+2.*c_rp(kk,ic3)) )

c dRO2/dNOx OPTION
c    *     *(1.-senhcat(kk,12))
c     enddo

c SENHCAT FOR RO2, RCO3:  FOR NOW, SET EQUAL TO HO2.
c THE TRUE VALUE IS 1-fhp*(1-fhr)
c WHERE fhp=k(RO2+HO2)/c_rl(RO2); fhr=k(ROOH+OH)/c_rl(ROOH).
c AND FOR RCO3:  1 - (1-fpp)(fhp)(1-fhr)
c WHERE fpp=k(RCO3+HO2)/c_rl(RCO3); fhp, fhr as above.

c IF ROOH IS STEADY STATE, ITS SENHCAT IS EQUAL TO THAT FOR RO2.
c THIS REQUIRES THAT ROOH STEADY STATE IS THE SAME AS H2O2.
c   (PROBABLY DOESN'T WORK ANYWAY.... FOR ROOH STST REMOVE ROOH.)
      if(c_lsts(ic3)) then
c       do kk=1,c_kmax
         senhcat(kk,4) = senhcat(kk,3)
c       enddo
      endif

c TEMPORARY DEBUG WRITE
c      if(c_kkw.gt.0) write(c_out,1009) 
c    *       senhcat(c_kkw,3),alpha(c_kkw), beta(c_kkw)
c 1009   format('SENHCAT(ho2), RLho2, RPh2o2=',5(1pe10.3))


c ----------------------------------------------------------
c END OHSOLVE
2000  return
      end
c ----------------------------------------------------------

      subroutine setgeom(ic  ,  geomavg, history, c_iter)
c
c This calculates the geometric average factor (geomavg(kk,kc))
c    which is used to adjust concentrations as a  geom. avg. with prior.
c      ( XR = (XR**F)*(XRP**(1-F))
c
c The factor is set based on the HISTORY of the past 3 iterations.
c When XR oscillates, GEOMAVG gets lower towards minimum (0.5 or 0.1)
c When XR keeps increasing or decreasing, GEOMAVG moves towards 1.
c
c OPTION:  How strongly does GEOMAVG react to history?
c OPTION:  Set minimum value for GEOMAVG (geomin)
c           Currently set to 0.1 for slow, sure convergence.
c            (previously 0.5)
c
c Inputs:    Species number (ic) 
c            history(kk,ic,c_iter)  - past iterative solutions
c            geomavg(kk,ic) - previous geometric avg factor.
c            
c
c Outputs:  geomavg(kk,ic) = new geometric average factor for species.
c
c
c Called by:    quadchem
c
c Calls to:     chemsolve (to solve H2O2)
c               setgeom
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c
c -------------------------------------------------------------------
c
c ----------------------------------------
      use h_rbchemmech
c      use h_rbchemvars
c      use h_rbchemlocal, only: kk, alpha, beta, gamma 
       IMPLICIT NONE

c#     include "rbchemmech.h"
c#     include "rbchemvars.h"
c#     include "rbchemlocal.h"

c VARIABLES FROM rbchemarrays.h, rbchemlocarr.h
      integer :: ic
      real*8 :: geomavg(c_kvec,c_cdim)   ! Geom. avg factor
      real*8 :: history(c_kvec,c_cdim,400)  ! Cncn for each iter
c      integer :: ic
      integer :: c_iter                  ! chem. number of iterations

!======below are local variables ===Guangxing LIN
          real*8 :: stoicx                         ! stoichiometry sum
      integer :: ic1,ic2,ic3,icc,ics,ics2,icc1,icc2,icc3 !Chem index
      integer :: is,iss,iscs,isc2                    !Chem local index
          integer :: ich,icq,icx,icx1,icx2,icy1,icy2,icp,icp1,icp2 !Chem
      integer :: icr1,icr2,icr,isr1,isr2,icpair  !chem index - pair and
      integer :: ionsum                                  ! aquasolve ion
      integer :: ica1,ica2,icb1,icb2,nra1,nra2,nrb1,nrb2 !Aquasolve chem
      integer :: neq                                     !Aqueous
      integer :: nc,nc1,nc2,ncc,ncf,nn,nne               !Chem species
      integer :: nr,nr1,nr2,nrh,nrq,nrqq,nrx,np          ! Reaction
      integer :: icat1,icat2,icatp,icatp2  ! indices used for species
      integer :: kk, kw                            ! Vectorization
      integer :: i, j, k, ii, ij, iii, n           ! General counters
       integer :: nsol           ! Number of pair groups being solved
      integer :: nsolv          ! Number of species being solved for
          real*8 :: alpha(c_kvec)   ! General vector variable
      real*8 ::  beta(c_kvec)   ! General vector variable
      real*8 :: gamma(c_kvec)   ! General vector variable
      real*8 :: prior(c_kvec)   ! Prior species conc (molec/cm3)
      integer :: ncsolv(c_cdim) ! Species number (ic) for species in
      integer :: nssolv(c_cdim) ! Pair group number (is) for spec. in

c LOCAL VARIABLES
      real*8 :: geomin  ! Minimum value for geo. avg. parameter

c ----------------------------------------
      kk=1

c LINE TO COMMENT OUT AUTOMATIC SETTING
c     if(c_iter.gt.0) return

c RETURN IF ITER < 4
      if(c_iter.lt.4) return
c
c OPTION:  SET MINIMUM VALUE FOR GEOMETRIC AVERAGE PARAMETER.
c   Originally set at 0.5, then 0.3.
c   Currently set at 0.1.  Low value insures slow, sure convergence.

      geomin = 0.1

c ESTABLISH TREND OVER PAST 3 ITERATIONS.
c  A = delta(last iter) / delta (previous iter)
c  OPTION:  A= [new/old ratio(last iter)] / [new/old ratio(previous iter)]
c     A<0:  oscillates.
c     A<-1:  oscillation getting worse
c     A>0:  monotone increase/decrease
c     A=> 0:  approaching convergence

c     do kk=1,c_kmax                ! kk vector loop
        alpha(kk) = 0.
        if(history(kk,ic,(c_iter-2)).ne.0.and.
     *      history(kk,ic,(c_iter-3)).ne.0)  then
c DELTA OPTION
c         beta(kk) = history(kk,ic,(c_iter -1 ))
c    *              - history(kk,ic,(c_iter-2))
c         gamma(kk) = history(kk,ic,(c_iter-2))
c    *              - history(kk,ic,(c_iter-3)) 
c RATIO OPTION
          beta(kk) = history(kk,ic,(c_iter -1 ))
     *               /history(kk,ic,(c_iter-2))  - 1.
          gamma(kk) = history(kk,ic,(c_iter-2))
     *               /history(kk,ic,(c_iter-3))  - 1.

           alpha(kk) = beta(kk)/gamma(kk)
        endif
        if(alpha(kk).gt.1.) alpha(kk)= 1.
        if(alpha(kk).lt.-1.) alpha(kk)= -1.
c     enddo                       ! kk vector loop

c DIFFICULT CONVERGENCE  OPTION:  MULTIPLY SETGEOM BY DAMPENING FACTOR.
c    Multiply by 1:  moves factor towards 1 or 0.5 instantly.
c    Multiply by 0.5 (standard):  moves factor more slowly.
c
c     do kk=1,c_kmax                ! kk vector loop
        alpha(kk) = alpha(kk) * 0.5
c     enddo                       ! kk vector loop
c
c ADJUST GEOMETRIC AVERAGING FACTOR:
c   If A<0 (oscillating), move towards minimum (0.5)(0.1)(geomin)
c   If A>0 (steady trend),move towards maximum (1.)
c
c     do kk=1,c_kmax                ! kk vector loop
        if(alpha(kk).gt.0.2) then
          geomavg(kk,ic) = geomavg(kk,ic)
     *       + alpha(kk)*(1.-geomavg(kk,ic))
        endif
        if(alpha(kk).lt.-0.2) then
          geomavg(kk,ic) = geomavg(kk,ic)
     *       + alpha(kk)*(geomavg(kk,ic) - geomin)
        endif
c     enddo                       ! kk vector loop

c TEST WRITE SETGEO
c     if(c_kkw.gt.0) then
c         write(c_out,101) ic, c_tchem(ic)
c 101       format (/,'GEOMAVG:  TEST IC = ', i4,2x,a8)
c         write(c_out,102) history(kk,ic,(c_iter-3)),
c    *       history(kk,ic,(c_iter-2)), history(kk,ic,(c_iter-1))
c 102       format(' HISTORY:  ', 3(1pe10.3))
c         write(c_out,103) alpha(c_kkw), geomavg(c_kkw,ic)
c 103       format (' ALPHA, GEOMAVG = ', 2(1pe10.3))
c     endif

c END GEOMAVG

2000  return
      end
c ----------------------------------------

       subroutine presolve
     &    (c_xcin, xc, ratek)

c  This sets the initial value for six key species 
c                        (OH, HO2,O3,NO,NO2,NO3, also H+)
c   based on NO-NO2-O3 equilibrium for NOx-Ox
c   and simple OH/HO2 ratio and jO3 for odd-h radicals.

c  The subroutine uses the following reactions:  
c
c       #1:  NO2+hv->NO+O3          #2: NO+O3->NO2
c       #9:  O3+hv->2OH             #12: NO2+OH->HNO3
c       #16: OH+CO->HO2             #17: OH+O3->HO2
c       #18: HO2+NO->OH+NO2         #22: HO2+HO2->H2O2
c       #3:  NO2+O3->NO3
c  Also:  
c       #17 OH+O3
c       #46 OH+CH4->HO2.
c
c The reactions are identified from "family" array:
c   family(1,i) = OH, HO2, H2O2
c   family(2,i) = O3, NO2, NO
c   family(3,i) = NO3, N2O5, HNO3
c These are used to identify reactions based on reactants.
c
c Initial  H+ is set to 1e-5.
c 
c Inputs:    Initial species concentrations (c_xcin)
c            Chemical reactions
c            family array to ID reactions
c
c Outputs:   Estimated xc for OH, HO2, O3, NO, NO2, NO3, H+
c
c
c Called by:    quadchem
c
c Calls to:     None.
c
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c
c -------------------------------------------------------------------
c
      use h_rbchemmech
      use h_rbchemvars, only: c_time
c      use h_rbchemlocal
      IMPLICIT NONE

c#     include "rbchemmech.h"
c#     include "rbchemvars.h"
c#     include "rbchemlocal.h"

c VARIABLES FROM rbchemarrays.h, rbchemlocarr.h
       real*8 ::  c_xcin(c_kvec,c_cdim) ! concentr molec/cm3
       real*8 ::  xc    (c_kvec,c_cdim) ! concentration molec/cm3
       real*8 ::  ratek (c_kvec,c_rdim) ! Rate constants
!=========Guangxing Lin below are local variables==       
       real*8 :: foh(c_kvec)       ! OH/HO2.  not common
       integer :: c_iter                  ! chem. number of
          real*8 :: stoicx                         ! stoichiometry sum
      integer :: ic,ic1,ic2,ic3,icc,ics,ics2,icc1,icc2,icc3 !Chem index
      integer :: is,iss,iscs,isc2                    !Chem local index
          integer :: ich,icq,icx,icx1,icx2,icy1,icy2,icp,icp1,icp2 !Chem
      integer :: icr1,icr2,icr,isr1,isr2,icpair  !chem index - pair and
      integer :: ionsum                                  ! aquasolve ion
      integer :: ica1,ica2,icb1,icb2,nra1,nra2,nrb1,nrb2 !Aquasolve chem
      integer :: neq                                     !Aqueous
      integer :: nc,nc1,nc2,ncc,ncf,nn,nne               !Chem species
      integer :: nr,nr1,nr2,nrh,nrq,nrqq,nrx,np          ! Reaction
      integer :: icat1,icat2,icatp,icatp2  ! indices used for species
      integer :: kk, kw                            ! Vectorization
      integer :: i, j, k, ii, ij, iii, n           ! General counters
       integer :: nsol           ! Number of pair groups being solved
      integer :: nsolv          ! Number of species being solved for
          real*8 :: alpha(c_kvec)   ! General vector variable
      real*8 ::  beta(c_kvec)   ! General vector variable
      real*8 :: gamma(c_kvec)   ! General vector variable
      real*8 :: prior(c_kvec)   ! Prior species conc (molec/cm3)
      integer :: ncsolv(c_cdim) ! Species number (ic) for species in
      integer :: nssolv(c_cdim) ! Pair group number (is) for spec. in

c LOCAL VARIABLES
       integer :: nspecial(40)
c                         (ic1,ic2,ic3,ic9,ic10 ic4)
c                         (O3  NO2 NO  OH  HO2  NO3)

       kk=1

c REACTION SUM:  IDENTIFY SPECIAL REACTIONS FOR HX, NOX
c AND SUM 'CRATE' (ALPHA) = OH+C REACTIONS.
c AND SUM GAMMA = NO3 LOSS REACTIONS.
       
       do i = 1, 40
        nspecial(i) = 0
       enddo
c      do kk=1,c_kmax
          alpha(kk) = 0.
          gamma(kk) = 0.
c      enddo

      do nr = 1, c_nreac
c IDENTIFY SPECIAL REACTIONS BY REACTANT NUMBER
c       #1:  NO2+hv->NO+O3          #2: NO+O3->NO2
c       #9:  O3+hv->2OH             #12: NO2+OH->HNO3
c       #16: OH+CO->HO2             #17: OH+O3->HO2
c       #18: HO2+NO->OH+NO2         #22: HO2+HO2->H2O2
c       #3:  NO2+O3->NO3
       if(c_reactant(nr,1).eq.c_nno2.and.c_reactant(nr,2).eq.-1)
     *  nspecial(1)=nr
       if( (c_reactant(nr,1).eq.c_no3.and.c_reactant(nr,2).eq.c_nno)
     *  .or.(c_reactant(nr,1).eq.c_nno.and.c_reactant(nr,2).eq.c_no3) )
     *  nspecial(2)=nr
       if(c_reactant(nr,1).eq.c_no3.and.c_reactant(nr,2).eq.-1
     *     .and.c_product(nr,1).eq.c_noh)
     *  nspecial(9)=nr
       if( (c_reactant(nr,1).eq.c_nno2.and.c_reactant(nr,2).eq.c_noh)
     *  .or.(c_reactant(nr,1).eq.c_noh.and.c_reactant(nr,2).eq.c_nno2) )
     *  nspecial(12)=nr
       if( (c_reactant(nr,1).eq.c_nno.and.c_reactant(nr,2).eq.c_nho2)
     *  .or.(c_reactant(nr,1).eq.c_nho2.and.c_reactant(nr,2).eq.c_nno) )
     *  nspecial(18)=nr
       if(c_reactant(nr,1).eq.c_nho2.and.c_reactant(nr,2).eq.c_nho2)
     *  nspecial(22)=nr
       if( (c_reactant(nr,1).eq.c_nno2.and.c_reactant(nr,2).eq.c_no3)
     *  .or.(c_reactant(nr,1).eq.c_no3.and.c_reactant(nr,2).eq.c_nno2) )
     *  nspecial(3)=nr
      
c IDENTIFY OH+CO,HC OR O3 REACTIONS BY CATEGORY. SUM AS 'CRATE'(alpha).
       icat1 = 0
       icat2 = 0
       icr = 0
       if(c_reactant(nr,1).gt.0) then
         icat1 = c_icat(c_reactant(nr,1))
       endif
       if(c_reactant(nr,2).gt.0) then
         icat2 = c_icat(c_reactant(nr,2))
       endif
       if(icat1.eq.9.and.(icat2.eq.11.or.icat2.le.3))
     *   icr = c_reactant(nr,2)
       if(icat2.eq.9.and.(icat1.eq.11.or.icat1.le.3))
     *   icr = c_reactant(nr,1)

       if(icr.gt.0) then
c        do kk=1,c_kmax
            alpha(kk) =   alpha(kk)+ratek(kk,nr)*xc(kk,icr)
c        enddo
       endif

c IDENTIFY NO3 LOSS REACTIONS (gamma)
       icr = -1 
       if(c_reactant(nr,1).eq.c_nno3) icr = c_reactant(nr,2)
       if(c_reactant(nr,2).eq.c_nno3) icr = c_reactant(nr,1)

       if(icr.gt.0) then
c        do kk=1,c_kmax
            gamma(kk) =   gamma(kk)+ratek(kk,nr)*xc(kk,icr)
c         enddo
       endif

       if(icr.eq.0) then
c        do kk=1,c_kmax
            gamma(kk) =   gamma(kk)+ratek(kk,nr)
c         enddo
       endif
      enddo ! nr
c
c NOX:  A PRECISE INITIALIZATION WOULD SET EQUILIBRIUM BTWEEN NO+O3=NO2.
c EQUATION:  (NO)**2 + NO (Ox-NOx+j1/k2) - NOx j1/k2 = 0 
c WHERE j1:  NO2+hv; k2: NO+O3.

       if(nspecial(1).gt.0.and.nspecial(2).gt.0) then
c       do kk=1,c_kmax
          beta(kk) = ratek(kk,nspecial(1))/ratek(kk,nspecial(2))
         xc(kk,c_nno)= 0.5*
     *             ( sqrt(  (  c_xcin(kk,c_no3) -   c_xcin(kk,c_nno) 
     *                                            +  beta(kk) )**2
     *               + 4.* beta(kk)
     *               *(  c_xcin(kk,c_nno2)+  c_xcin(kk,c_nno))  )
     *               -( c_xcin(kk,c_no3)-  c_xcin(kk,c_nno)+ beta(kk)) )
         xc(kk,c_nno2) =   
     *         c_xcin(kk,c_nno2)+  c_xcin(kk,c_nno)-xc(kk,c_nno)
         xc(kk,c_no3)=  
     *          c_xcin(kk,c_no3)+  c_xcin(kk,c_nno2)-xc(kk,c_nno2)

c ZERO-PROTECT SHOULD APPLY ONLY IF ERROR IN FORMULA.
         if(xc(kk,c_nno2).le.0) xc(kk,c_nno2)=0.1
         if(xc(kk,c_nno).le.0) xc(kk,c_nno)=0.1
         if(xc(kk,c_no3).le.0) xc(kk,c_no3)=0.1
c       enddo

c INITIAL NOX REDUCED BASED ON 5-HOUR LIFETIME
c  COMMENT OUT IF PRESET NOx OPTION)
        if(.not.c_lsts(c_nno2).and..not.c_lsts(c_nno)) then
c         do kk=1,c_kmax
           xc(kk,c_nno)= xc(kk,c_nno)/(1.+c_time/18000.)
           xc(kk,c_nno2) = xc(kk,c_nno2)/(1.+c_time/18000.)
c         enddo
        endif

       endif

c NO3:  PRELIMINARY VALUE SET FROM BACK-EULER FORMULA:
c  NO2+O3 SOURCE, ALL NO3 SINKS. (Added to deal with large-isop. crash.)

c       do kk=1,c_kmax
          xc(kk,c_nno3) =   c_xcin(kk,c_nno3) 
c       enddo

        if(nspecial(3).gt.0) then
c         do kk=1,c_kmax
            xc(kk,c_nno3) = xc(kk,c_nno3)
     *         +   ratek(kk,nspecial(3)) *xc(kk,c_no3)*xc(kk,c_nno2)   
c         enddo
        endif
c       do kk=1,c_kmax
            xc(kk,c_nno3) = xc(kk,c_nno3) / (1. + gamma(kk) )
c       enddo

c xno, xno2, xo3, xoh, xho2 - ALL CUT.
c oxno, oxno2, oxo3, rno, rno1, rno2, xnox - ALL CUT

c OH, HO2 INIITIALIZE:REQUIRES PRE-SET REACTION NUMBERS (see above) 
c  PLUS AUTOMATED CRATE = #16, OH+CO, #17 OH+O3,#46 OH+CH4->HO2.
c  DECEMBER 1994:  CHANGE TO FULL QUADRATIC WITH PRIOR OH, HO2
c  AND PROTECT AGAINST NIGHTTIME STEADY STATE ZERO.  HO2 prior>1E6.

c     do kk=1,c_kmax
        foh(kk ) = 0.01
        if (ratek(kk ,nspecial(1)).ge.1.0E-03) 
     *    foh(kk )  = ratek(kk ,nspecial(18))
     *                     *xc(kk ,c_nno) /  alpha(kk ) 

c FULL QUADRATIC SOLVE WITH PRIOR OH, HO2
        gamma(kk ) = 2.*c_time*ratek(kk ,nspecial(9))*xc(kk ,c_no3)
     *           +  c_xcin(kk,c_noh)+  c_xcin(kk,c_nho2)
        if(gamma(kk ).lt.1.01E+04) then
            c_xcin(kk,c_nho2) =   c_xcin(kk,c_nho2)+1.0E+04
            c_xcin(kk,c_noh) =   c_xcin(kk,c_noh) + 1.0E+02
          gamma(kk ) = gamma(kk ) + 1.01E+04
        endif

        beta(kk ) = 1. + 1./foh(kk ) + 
     *                c_time*ratek(kk ,nspecial(12)) *xc(kk ,c_nno2) 
        alpha(kk ) =  c_time* ratek(kk  ,nspecial(22))/(foh(kk )**2)

        if(alpha(kk )*gamma(kk ).lt.beta(kk)**2) then
          xc(kk,c_noh) = gamma(kk)/beta(kk)
        else
          xc(kk,c_noh) = (sqrt(beta(kk )**2 + 4.*alpha(kk )*gamma(kk ) )
     *    - beta(kk )  )/(2.*alpha(kk ))
        endif

         xc(kk,c_nho2) = xc(kk,c_noh)/foh(kk)
c       enddo

c WRITE PRELIMINARY CONCENTRATIONS.  (PERMANENT)
c    (Add parameters if needed. CRATE is new and suspect.)

      if(c_kkw.gt.0) then
         write(c_out,1011) xc(c_kkw,c_noh),xc(c_kkw,c_nho2)
     *     ,  xc(c_kkw,c_nno), xc(c_kkw,c_nno2) ,xc(c_kkw,c_no3)
     *     , xc(c_kkw,c_nno3)
1011     format(/' PRELIM oh,ho2,no,no2,o3,no3=',6(1pe10.3))

c TEST WRITE
c        xx1 = 2.*c_time*ratek(c_kkw,nspecial(9))*xc(c_kkw,c_no3)
c        write(c_out,1012) xx1,   c_xcin(c_kkw,c_noh),  c_xcin(c_kkw,c_nho2)
c 1012     format(/,' HX SOURCES:  O3+hv   PRIOR OH   PRIOR HO2', /,
c    *       10x,5(1pe10.3))

c        xx2 = c_time*ratek(c_kkw,nspecial(22))
c        xx3 = c_time*ratek(c_kkw,nspecial(12))*xc(c_kkw,c_nno2)
c        write(c_out,1013) xx2,xx3
c 1013     format(/,'SINK FACTORS:',/,
c    *    ' OH=QUADRATIC(sources,1+1/foh+rHNO3,rH2O2)'
c    *    ,/,'rH2O2=',(1pe10.3),'  rHNO3=',(1pe10.3))
c        write(c_out,1021)  ratek(c_kkw ,nspecial(1)) , 
c    *                    ratek(c_kkw,nspecial(18)), foh(c_kkw)
c 1021     format(/,'FOH=RK18*NO/(RK*CO,HC) (if RK1>0.001).  '
c    *     /,'RK1, RK18, FOH=',3(1pe10.3))
c END TEST WRITE 

      endif

c INITIAL CONCENTRATION FOR H+
c --------------------------
      if(c_aqueous(1,2).gt.0) then
c       do kk=1,c_kmax
          xc(kk,c_aqueous(1,2)) = 1.0E-05
          xc(kk,c_aqueous(1,3)) = 1.0E-09
c       enddo
      endif

c --------------------------
c END INITIALIZATION ROUTINE
c END PRESOLVE
2000  return
      end
c ---------------------------------------------------------------

      subroutine prelump    
c               (prechem)
     &   (c_xcin, c_xcout, c_xcav, c_xcemit,
     &    c_rp, c_rl, c_rr ,
     &    xc, xcfinr,  xclastq ,
     &    rrp, rrl, rppair, rlpair, geomavg, c_rainfr) 
c ----------------------------------------------------------------
c This sets initial concentrations for 'lumped' species.
c And sets other initial values  for the start of the chemistry solver
c  ('prechem').
c
c For lumped species:  It converts lumped sum and partition fractions 
c    into concentrations for individual species.
c    and sets the input concentration (c_xcin) for lumped species.
c
c For all species:  it enters the input concentration (c_xcin) as xc.
c
c It  sets aqueous species to zero.
c   (Assumes that the saved values and initial concentrations
c     represent gas+aqueous sum for the gas-master species; 
c     output aqueous concentrations are for information only.)
c
c It  sets zero initial values for running sums
c   (production, loss - RP, RL)
c
c It sets initial value for geometric average.
c it sets initial value for xcfinr (FINAL/AVG RATIO)
c
c OPTION:  SET XR=0.1 FOR ALL STEADY-STATE SPECIES?
c HOWEVER, IF A SPECIES IS SET IN STEADY STATE VS EMISSIONS,
c (e.g. ISOPRENE) THEN THIS SHOULD BE OVERRIDDEN.

c OPTION:  SET XR=0 FOR ALL UNPARTITIONED LUMPED SPECIES (RCO3-RPAN)
c SO THAT RP() IN MIDLUMP SETS PARTITIONING CORRECTLY
c
c Inputs:    Initial species concentrations (c_xcin)
c
c Outputs:   Working species concentrations (xc)
c            Initial species concentrations for lumped sp. (as c_xcin)
c            Initial zero values  for production, loss (rp, rl)
c            Initial geomavg
c
c Called by:    quadchem
c
c Calls to:     None.
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c
c -------------------------------------------------------------------
      use h_rbchemmech
c      use h_rbchemvars
c      use h_rbchemarrays
c      use h_rbchemlocal, only: kk, ic, i, ics, ic1,ic2, alpha, ic, icc,
c     & neq
      IMPLICIT NONE

c#     include "rbchemmech.h"
c#     include "rbchemvars.h"
c#     include "rbchemarrays.h"
c#     include "rbchemlocal.h"
c#     include "rbchemlocarr.h"
c
c VARIABLES FROM rbchemlocarr.h
       real*8 ::  c_xcin( c_kvec,c_cdim) ! concentr molec/cm3
       real*8 ::  c_xcout( c_kvec,c_cdim) ! concentr, mol/cm3
       real*8 ::  c_xcav( c_kvec,c_cdim)  ! concentr, mol/cm3
       real*8 ::  c_xcemit( c_kvec,c_cdim) ! emis, molec/cm3
      real*8 ::  c_rp(c_kvec,c_cdim)  ! Production, molec/cm3
      real*8 ::  c_rl(c_kvec,c_cdim)  ! Loss, molec/cm3
      real*8 ::  c_rr(c_kvec,c_rdim)  ! Reaction rate m/cm3

      real*8 :: xc      (c_kvec, c_cdim) ! concentration molec/cm3
      real*8 :: xcfinr  (c_kvec, c_cdim) ! ratio FINAL/AVG cncn
      real*8 :: xclastq (c_kvec, c_cdim) ! conc molec/cm3
      real*8 :: rrp     (c_kvec, c_cdim) ! Interim production rate
      real*8 :: rrl     (c_kvec, c_cdim) ! Interim loss rate
      real*8 :: rppair  (c_kvec, c_cdim) ! Production for pair grp
      real*8 :: rlpair  (c_kvec, c_cdim) ! Loss rate for pair grp
      real*8 :: geomavg (c_kvec, c_cdim) ! Geom. avg factor
        real*8 ::  c_rainfr(c_kvec)   ! rainout fraction
      
!=========below are local varibles ==Guangxign Lin===
       real*8 :: foh(c_kvec)       ! OH/HO2.  not common
       integer :: c_iter                  ! chem. number of
          real*8 :: stoicx                         ! stoichiometry sum
      integer :: ic,ic1,ic2,ic3,icc,ics,ics2,icc1,icc2,icc3 !Chem index
      integer :: is,iss,iscs,isc2                    !Chem local index
          integer :: ich,icq,icx,icx1,icx2,icy1,icy2,icp,icp1,icp2 !Chem
      integer :: icr1,icr2,icr,isr1,isr2,icpair  !chem index - pair and
      integer :: ionsum                                  ! aquasolve ion
      integer :: ica1,ica2,icb1,icb2,nra1,nra2,nrb1,nrb2 !Aquasolve chem
      integer :: neq                                     !Aqueous
      integer :: nc,nc1,nc2,ncc,ncf,nn,nne               !Chem species
      integer :: nr,nr1,nr2,nrh,nrq,nrqq,nrx,np          ! Reaction
      integer :: icat1,icat2,icatp,icatp2  ! indices used for species
      integer :: kk, kw                            ! Vectorization
      integer :: i, j, k, ii, ij, iii, n           ! General counters
       integer :: nsol           ! Number of pair groups being solved
      integer :: nsolv          ! Number of species being solved for
          real*8 :: alpha(c_kvec)   ! General vector variable
      real*8 ::  beta(c_kvec)   ! General vector variable
      real*8 :: gamma(c_kvec)   ! General vector variable
      real*8 :: prior(c_kvec)   ! Prior species conc (molec/cm3)
      integer :: ncsolv(c_cdim) ! Species number (ic) for species in
      integer :: nssolv(c_cdim) ! Pair group number (is) for spec. in

c ----------------------------------------------------
      kk=1

c SET H2O CONCENTRATION IN CHEMISTRY ARRAY FROM INPUT VARIABLE
c   - note, move below prelump if used.
c     if(c_nh2o.gt.0) then
c vec                      do kk=1,c_kvec
c        xc( kk ,c_nh2o) = c_h2ogas(kk)
c vec                      enddo ! do kk=1,c_kvec
c     endif                  !if(c_nh2o.gt.0) then
c


c  SET XC=XCIN, ZERO PROTECT,  AND MAYBE XC=0.1 FOR STEADY STATE.
c  FOR ALL TRANSPORTED SPECIES.
c
      do ic = 1, c_nchem1

c vec                    do kk=1,c_kmax

c possible error with this and other ZERO PROTECT?
c 2004 fix - insures no descent to zero->NaN.

        if(c_xcin(kk,ic).le.1.0e-15) c_xcin(kk,ic) = 1.0e-15 !JIALEI for Ester
        !!if(c_xcin(kk,ic).le.0.000001) c_xcin(kk,ic) = 0.000001
        xc(kk,ic) = c_xcin(kk,ic) 
c vec                     enddo ! do kk=1,c_kmax

c SET XR=0.1 AND XXO=0. FOR STEADY-STATE.  
c  XXO MUST BE ZERO, ELSE TROUBLE WHEN RP, RL=0.
c  THIS IS THE LINE TO COMMENT OUT, MAYBE.
        if(c_lsts(ic)) then
c          do kk=1,c_kmax
             xc( kk, ic) = 0.1
             c_xcin(kk,ic) = 0.
             c_xcemit(kk,ic) = 0.
c          enddo
        endif
      enddo ! ic

c ----------------------
c LUMPED SPECIES ARE IDENTIFIED FROM ARRAY LUMP(I,J) 
c READ FROM REACTION.DAT

      do i = 1,c_cdim
        ics = c_lump(i,1)
        if(ics.eq.0) go to 101

        ic1 = c_lump(i,2)
        ic2 = c_lump(i,3)

c SUM PARTITION FRACTIONS AND CORRECT IF ZERO.
c IF PRIOR PARTITION FRACTIONS ARE NOT SAVED: INITIAL XR SHOULD BE ZERO.
c ORIGINAL OPTION:  SET PARTITION = 100% FIRST SPECIES (IC1)
c AND CORRECT AT MIDLUMP BASED ON RP.  PROBLEM:  RCO3-RPAN.

c ALTERNATIVE OPTION: LEAVE PARTITIONED XR=0 AND CORRECT AT MIDLUMP.
c TO IMPLEMENT, SEE 'LUMPED SPECIES' BELOW.

c         do kk=1,c_kmax
           alpha(kk) = 0.
c         enddo
        do ic = ic1, ic2
c         do kk=1,c_kmax
           alpha(kk) = alpha(kk) + xc(kk,ic)
c         enddo
        enddo  !ic
c        do kk=1,c_kmax
           if(alpha(kk).eq.0) then
            xc(kk,ic1) = 1.
            alpha(kk) = 1.
           endif
c         enddo

c CONVERT PARTITION FRACTIONS INTO CONCENTRATIONS. ALSO ENTER XXO.
c ALSO ZERO-PROTECT AND ENTER XXO.
        do ic = ic1, ic2
          c_lsts(ic) = c_lsts(ics)
c          do kk=1,c_kmax

c ORIGINAL VERSION - SET LUMPED SPECIES HERE.
c            xc(kk,ic) = xc(kk,ic)*xc(kk,ics)/alpha(kk)
c            if(xc(kk,ic).le.0.1) xc(kk,ic) = 0.1
c ALTERNATIVE OPTION:  SET LUMPED SPECIES = 0. FOR FIRST ITERATION; 
c          THEN RESET BASED ON RP IN MIDLUMP.
             xc(kk,ic) = 0.1

c PUT LUMPED VALUES IN xcin
               c_xcin(kk,ic) = xc(kk,ic)
c          enddo
        enddo ! ic

c SET EMISSIONS TO BE LESS THAN INPUT CONCENTRATION
c   (Input concentration includes emissions; 
c      emissions are only used for timing in expo decay solution)

        do ic = 1, c_nchem2

c v         do kk=1,c_kmax
               if (c_xcemit(kk,ic).gt.0.99*c_xcin(kk,ic))
     *                    c_xcemit(kk,ic)=0.99*c_xcin(kk,ic)
c v         enddo ! do kk=1,c_kmax
        enddo                  !do ic=1,nchem2

c WRITE PRELUMP (PERMANENT)
        if(c_kkw.gt.0) write(c_out,131) c_tchem(ic1),c_tchem(ic2),
     *    (xc(c_kkw,ic),ic=ic1,ic2)
131     format(/,' TEST PRELUMP XR:  IC1-2= ',a8,2x,a8,/,(8(1pe10.3)))

      enddo ! i
101   continue

c SET AQUEOUS CONCENTRATIONS EQUAL TO ZERO
      do i = 1, c_nreach
        icc = c_henry(i,1)
        if(icc.gt.0) then
          if(c_nequil(icc).gt.0) then
            do neq = 1,c_nequil(icc)
             ic = c_ncequil(icc,neq)
c            do kk=1,c_kmax
             xc(kk,ic) = 0.
c            enddo 
            enddo
          endif
        endif
      enddo ! i

c ZERO REACTION RATES (RR) TO PREVENT CARRY-OVER FROM PREVIOUS RUN.
      do ic = 1, c_nreac
c       do kk=1,c_kmax
         c_rr(kk,ic) = 0.
c       enddo
      enddo   ! ic
c
c ZERO RP, RL, RRP, RRL TO PREVENT CARRY-OVER FROM PREVIOUS RUN
c         (does not zero RRP, RRL before each iteration, only at start)
c
      do ic = 1, c_nchem2
c       do kk=1,c_kmax      ! kk vector loop
         c_rp( kk ,ic) = 0.
         c_rl( kk ,ic) = 0.
         rrp( kk ,ic) = 0.
         rrl( kk ,ic) = 0.
         rppair(kk,ic) = 0.
         rlpair(kk,ic) = 0.
c       enddo             ! kk vector loop
      enddo           !do ic=1,c_nchem2

c TEST AND CORRECT RAINOUT PARAMETER (0<=rainfr<1)

c       do kk=1,c_kmax      ! kk vector loop
          if(c_rainfr(kk).lt.0)  c_rainfr(kk) = 0.
          if(c_rainfr(kk).gt.0.9999) c_rainfr(kk) = 0.9999
c       enddo               ! kk vector loop
c
c SET DIFFICULT CONVERGENCE PARAMETER - INITIAL VALUES
c   Normally set to 1 for all except odd hydrogen.
c   Odd hydrogen:  HO2 (=Hxsum) factor set to 0.7
c                  OH  (=FOH)   factor set to 1.

      do ic = 1, c_nchem2
c       do kk=1, c_kmax          ! kk vector loop
          geomavg(kk,ic) = 1.
c       enddo                    ! kk vector loop
      enddo

c HO2  (ic = namechem('     HO2') or c_nho2     )
      ic = c_nho2     
c     do kk=1, c_kmax            ! kk vector loop
          geomavg(kk,ic) = 0.7
c     enddo                      ! kk vector loop

c OH   (ic = namechem('      OH') or c_noh      )
      ic = c_noh      
c     do kk=1, c_kmax            ! kk vector loop
          geomavg(kk,ic) = 1.0
c     enddo                      ! kk vector loop
c
c Set ratio of FINAL to AVERAGE species concentrations (xcfinr)
c   This ratio is always ONE for back-Euler solution.
c   It is set to a different value when the EXPO DECAY solution is used.

      do ic = 1, c_nchem2
c       do kk=1, c_kmax          ! kk vector loop
          xcfinr(kk,ic) = 1.0
c       enddo                    ! kk vector loop
      enddo
c
c  END PRELUMP
2000  return
      end
c -------------------------------------
c
      subroutine midlump
     &    (c_xcin, xc, c_rp, c_rl, c_iter)


!Guangxing Lin added c_iter
c This re-sets the initial concentrations (c_xcin) for lumped species
c  based on calculated chemical production and loss.
c
c Initial input for lumped species consists of the lumped sum only. 
c The partitioning of the sum into individual species 
c  is assumed to be in proportion to chemical production.
c
c As production rates are iteratively calculated, 
c   the assumed initial concentration of lumped species is adjusted.
c
c This is called at the start of each iteration (except #1).
c It adjusts the concentration at the start of the time step (c_xcin) 
c   ,not the solution for the end of the time step (xc).
c
c 2006 MODIFICATION - SUM LUMPED GAS-PHASE SPECIES INTO XC
c   FOR USE IN REACTION RATES.
c
c 2008 LUMP OPTION: partition initial concentrations based on rp/(1+rl)
c      rather than rp (to avoid major error if lumped species are
c      chemically different - THEY SHOULD NOT BE DIFFERENT.
c
c Inputs:    Initial species concentrations (xc)
c
c Outputs:   Modified initial concentrations for lumped species (c_xcin)
c            Sums for lumped species (xc)
c
c
c Called by:    quadchem
c
c Calls to:     None.
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c
c -------------------------------------------------------------------
c
c ----------------------

      use h_rbchemmech
c      use h_rbchemvars
c      use h_rbchemlocal
      IMPLICIT NONE

c#     include "rbchemmech.h"
c#     include "rbchemvars.h"
c#     include "rbchemlocal.h"
c
c VARIABLES FROM rbchemarrays.h, rbchemlocarr.h
       real*8 ::  c_xcin( c_kvec,c_cdim) ! concentr molec/cm3
       real*8 ::  xc( c_kvec,c_cdim) ! concentration molec/cm3
       real*8 ::  c_rp(c_kvec,c_cdim)  ! Production, molec/cm3
       real*8 ::  c_rl(c_kvec,c_cdim)  ! Loss,       molec/cm3
       integer :: c_iter                  ! chem. number of
!========below are local variables  =====Guangxing Lin

          real*8 :: foh(c_kvec)       ! OH/HO2.  not common
          real*8 :: stoicx                         ! stoichiometry sum
      integer :: ic,ic1,ic2,ic3,icc,ics,ics2,icc1,icc2,icc3 !Chem index
      integer :: is,iss,iscs,isc2                    !Chem local index
          integer :: ich,icq,icx,icx1,icx2,icy1,icy2,icp,icp1,icp2 !Chem
      integer :: icr1,icr2,icr,isr1,isr2,icpair  !chem index - pair and
      integer :: ionsum                                  ! aquasolve ion
      integer :: ica1,ica2,icb1,icb2,nra1,nra2,nrb1,nrb2 !Aquasolve chem
      integer :: neq                                     !Aqueous
      integer :: nc,nc1,nc2,ncc,ncf,nn,nne               !Chem species
      integer :: nr,nr1,nr2,nrh,nrq,nrqq,nrx,np          ! Reaction
      integer :: icat1,icat2,icatp,icatp2  ! indices used for species
      integer :: kk, kw                            ! Vectorization
      integer :: i, j, k, ii, ij, iii, n           ! General counters
       integer :: nsol           ! Number of pair groups being solved
      integer :: nsolv          ! Number of species being solved for
          real*8 :: alpha(c_kvec)   ! General vector variable
      real*8 ::  beta(c_kvec)   ! General vector variable
      real*8 :: gamma(c_kvec)   ! General vector variable
      real*8 :: prior(c_kvec)   ! Prior species conc (molec/cm3)
      integer :: ncsolv(c_cdim) ! Species number (ic) for species in
      integer :: nssolv(c_cdim) ! Pair group number (is) for spec. in

       kk=1

c LUMPED SPECIES ARE IDENTIFIED FROM ARRAY LUMP(I,J) 
c ICS = ACCUMULATOR; IC1-IC2 ARE INDIVIDUAL SPECIES.

       do 100 i=1,c_cdim
        ics = c_lump(i,1)
        if(ics.eq.0) go to 101

        ic1 = c_lump(i,2)
        ic2 = c_lump(i,3)

c SUM LUMPED SPECIES (IC1-IC2) INTO ACCUMULATOR (ICS)
c   (for use in rate calculations)
c         do  kk=1,c_kmax
          xc(kk,ics) = 0.
c         enddo
        do  ic=ic1,ic2
c         do kk=1,c_kmax
           xc(kk,ics) = xc(kk,ics) + xc(kk,ic)
c         enddo
        enddo

c RESET INITIAL CONCENTRATIONS OF INDIVIDUAL LUMPED SPECIES (xcin) 
c   based on chemistry production and loss in previous time step 
c   (the  input initial concentration is partitioned among the lumped species) 

c ( OPTIONS: (i) partition by rp (ii) partition by rp/(1+rl) (fails)
c            (iii) partition by xc*(rp/(xc+rl)) or by rp if xc=0 (current) )

c Initial zero normalization factor for partitioning
c         do kk=1,c_kmax
          alpha(kk) = 0.
c         enddo

c Enter partitioning factors for individual lumped species 
c
        do ic=ic1,ic2
c        do kk=1,c_kmax

           beta(kk) = c_xcin(kk,ic)    ! prior calculated initial concentration

           if(xc(kk,ic).gt.0.) then
              c_xcin(kk,ic) =          ! partition factor
     &       xc(kk,ic) * (c_rp(kk,ic)/(xc(kk,ic)+c_rl(kk,ic)) )  
           else
              c_xcin(kk,ic) =      c_rp(kk,ic)  
           endif

c DIFFICULT CONVERGENCE OPTION - adjust by prior partitioning
           if(c_iter.gt.4) c_xcin(kk,ic) =
     &       (c_xcin(kk,ic)**0.5)*(beta(kk)**0.5)
c END OPTION

           if(  c_xcin(kk,ic).le.0)   c_xcin(kk,ic) = 0.000001

           alpha(kk) = alpha(kk) + c_xcin(kk,ic)  ! normalization factor

c        enddo           ! kk=1,c_kmax
        enddo

c Normalize xcin (partition factors) so that lumped species sum
c   equals the input sum for initial concentrations

        do ic=ic1,ic2
c        do kk=1,c_kmax

           if(alpha(kk).gt.0.) then
             c_xcin(kk,ic) =c_xcin(kk,ics) 
     &            * ( c_xcin(kk,ic)/alpha(kk) )
           endif

c        enddo           ! kk=1,c_kmax
        enddo

c WRITE MIDLUMP (PERMANENT)
        if(c_kkw.gt.0) then
          write(c_out,131) c_tchem(ic1),c_tchem(ic2),
     *    (  c_xcin(c_kkw,ic),ic=ic1,ic2)
131    format(/,' TEST MIDLUMP XXO:  IC1-2= ',a8,2x,a8,/,(8(1pe10.3)))
          if(c_kkw.gt.0) write(c_out,132) c_tchem(ic1),c_tchem(ic2),
     *    (xc(c_kkw,ic),ic=ic1,ic2)
132    format(/,' TEST MIDLUMP XR:  IC1-2= ',a8,2x,a8,/,(8(1pe10.3)))
        endif

c DIAGNOTIC WRITE:  MIDLUMP
       if(c_kkw.gt.0) then
         write(c_out,141) c_xcin(c_kkw,ics),alpha(c_kkw)
141      format(/'MIDLUMP DIAGNOSTICS: xcin, alpha (norm f.) ='
     &       ,5(1pe10.3))
         write(c_out,*) "species new-xcin xc  rp  rl  rp/(1+rl)"
         do ic=ic1,ic2
           beta(c_kkw) = c_rp(c_kkw,ic)
           if(xc(c_kkw,ic).gt.0.)
     &     beta(c_kkw) = xc(c_kkw,ic)
     &         *( c_rp(c_kkw,ic)/(xc(c_kkw,ic)+c_rl(c_kkw,ic)) )

           write(c_out,142) c_tchem(ic), c_xcin(c_kkw,ic)
     &      ,xc(c_kkw,ic)
     &      ,c_rp(c_kkw,ic), c_rl(c_kkw,ic), beta(c_kkw)
142      format(' midlump',a8,7(1pe10.3))
         enddo
       endif
c

c
c END LUMPED-SPECIES LOOP
100   continue
101   continue
c
c
c  END MIDLUMP
2000  return
      end
c -------------------------------------

      subroutine postlump
     &   (c_xcin, c_xcout, c_xcav, c_xcemit,
c    &                                        c_xcwdep,
     &    c_rp, c_rl, c_rr ,
     &    xc,xcfinr,  c_h2oliq, c_rainfr )
c
c This calculates the final output species concentration (c_xcout)
c  from the working concentration (xc) (average over time step). 
c  It also preserves the average species concentrations (c_xcav) 
c  and the FINAL/AVERAGE ratio (xcfinr).
c
c It also  calculates sums for all 'lumped' species 
c  at the end of the chemistry solution, and saves them in xc.
c
c This also sums aqueous equilibrium species 
c  into a combined gas+aqueous value (in gas-phase units, molec/cm3)
c  which is saved as the gas-master species concentration.
c
c Aqueous concentrations are saved as output (in aqueous M/liter), 
c   but it is assumed that these are not transported from step to step.
c   The gas-aqueous partitioning would also change with LWC.
c
c Partition fractions for individual lumped species
c   (as fraction of the lumped sum) are also saved for output.
c
c RAINOUT:  The sum of aqueous species into the gas-master
c   includes removal through rainout, based on the parameter RAINFR.
c
c WET DEPOSITION is calculated and saved for each aqueous species,
c   but it can be calculated more easily by using the output 
c   concentrations of aqueous species (in M/L) and the rainfall rate.

c    (in gas-phase units).
c
c  (c_xcwdep)
c Here, this is omitted. Wet deposition can be derived instead
c  from the average concentrations of aqueous species (xcav, M/L)
c  multiplied by the  rain volume (liters) equivalent to rainfr
c  (=c_rainfr * c_h2oliq (kg/L) * alt. thickness (m) * 10 (dm/m))
c
c
c Inputs:    Average species concentrations (xc), 
c              FINAL/AVERAGE ratio (xcfinr)
c              final spec ratio (xcf)
c
c Outputs:   Modified species concentrations (xc)
c            Final species concentrations (c_xcout)
c            Wet deposition:  c_xcwdep
c
c Called by:    quadchem
c
c Calls to:     None.
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c
c -------------------------------------------------------------------
      use h_rbchemmech
c      use h_rbchemvars
c      use h_rbchemarrays
      use h_rbchemlocal, only:  avogadrl
c      use h_rbchemlocal, only: kk,ic, ic1, alpha, avogadrl, i, ics, ic2
      IMPLICIT NONE


c#     include "rbchemmech.h"
c#     include "rbchemvars.h"
c#     include "rbchemarrays.h"
c#     include "rbchemlocal.h"

c VARIABLES FROM rbchemlocarr.h
        real*8 ::  c_xcin( c_kvec,c_cdim) ! concentr molec/cm3
       real*8 ::  c_xcout( c_kvec,c_cdim) ! concentr, mol/cm3
       real*8 ::  c_xcav( c_kvec,c_cdim)  ! concentr, mol/cm3
       real*8 ::  c_xcemit( c_kvec,c_cdim) ! emis, molec/cm3
      real*8 ::  c_rp(c_kvec,c_cdim)  ! Production, molec/cm3
      real*8 ::  c_rl(c_kvec,c_cdim)  ! Loss, molec/cm3
      real*8 ::  c_rr(c_kvec,c_rdim)  ! Reaction rate m/cm3

       real*8 :: xc( c_kvec,c_cdim) ! concentration molec/cm3
       real*8 :: xcfinr( c_kvec,c_cdim) ! ratio FINAL/AVG cncn
        real*8 ::  c_h2oliq(c_kvec)   ! LWC grams cm-3
      real*8 ::  c_rainfr(c_kvec)   ! rainout fraction
!=========below are local variables =========
           real*8 :: foh(c_kvec)       ! OH/HO2.  not common
       integer :: c_iter                  ! chem. number of
          real*8 :: stoicx                         ! stoichiometry sum
      integer :: ic,ic1,ic2,ic3,icc,ics,ics2,icc1,icc2,icc3 !Chem index
      integer :: is,iss,iscs,isc2                    !Chem local index
          integer :: ich,icq,icx,icx1,icx2,icy1,icy2,icp,icp1,icp2 !Chem
      integer :: icr1,icr2,icr,isr1,isr2,icpair  !chem index - pair and
      integer :: ionsum                                  ! aquasolve ion
      integer :: ica1,ica2,icb1,icb2,nra1,nra2,nrb1,nrb2 !Aquasolve chem
      integer :: neq                                     !Aqueous
      integer :: nc,nc1,nc2,ncc,ncf,nn,nne               !Chem species
      integer :: nr,nr1,nr2,nrh,nrq,nrqq,nrx,np          ! Reaction
      integer :: icat1,icat2,icatp,icatp2  ! indices used for species
      integer :: kk, kw                            ! Vectorization
      integer :: i, j, k, ii, ij, iii, n           ! General counters
       integer :: nsol           ! Number of pair groups being solved
      integer :: nsolv          ! Number of species being solved for
          real*8 :: alpha(c_kvec)   ! General vector variable
      real*8 ::  beta(c_kvec)   ! General vector variable
      real*8 :: gamma(c_kvec)   ! General vector variable
      real*8 :: prior(c_kvec)   ! Prior species conc (molec/cm3)
      integer :: ncsolv(c_cdim) ! Species number (ic) for species in
      integer :: nssolv(c_cdim) ! Pair group number (is) for spec. in
 
c 
       kk=1
c
c   WRITE FINAL SPECIES CONCENTRATIONS TO OUTPUT ARRAY:
c    final conc. (xcout) = avg concentration (xc) * ratio (xcfinr)
c     where ratio is always for gas-master species
c    average concentrations also entered into output array (xcav)
c
      do ic=1,c_nchem2
        ic1 = c_npequil(ic)
c v                                do kk=1,c_kmax
        c_xcav (kk,ic) = xc(kk,ic)
        c_xcout(kk,ic) = xc(kk,ic) * xcfinr(kk, ic1)
c v                                enddo !do kk=1,c_kmax
      enddo              !do ic=1,nchem2
c
c  AQUEOUS SPECIES:  
c        SUM AQUEOUS SPECIES INTO GAS-MASTER SUM (in GAS UNITS)
c       (Individual aqueous species, aqueous units, are preserved,
c          but the gas-aq sum is the main output)

       do ic=1,c_nchem2
c 
c    AQUEOUS SPECIES identified by npequil (pointer, aq-to-gas)
        ic1 = c_npequil(ic)
        if(ic1.ne.ic) then

c v                                do kk=1,c_kmax
           alpha(kk) = c_xcav(kk,ic)*c_h2oliq(kk)*avogadrl
           c_xcav (kk,ic1) = c_xcav(kk,ic1) + alpha(kk)
           c_xcout(kk,ic1) = c_xcout(kk,ic1)
     *                     + alpha(kk)*xcfinr(kk,ic)
c
c v                                enddo !do kk=1,c_kmax
         endif

       enddo         !do ic=1,c_nchem2
c

c ------------------------------------
c LUMPED SPECIES ARE IDENTIFIED FROM ARRAY LUMP(I,J)
c READ FROM REACTION.DAT

       do 100 i=1,c_cdim
        ics = c_lump(i,1)
        if(ics.eq.0) go to 101

        ic1 = c_lump(i,2)
        ic2 = c_lump(i,3)

c SUM LUMPED SPECIES (IC1-IC2) INTO ACCUMULATOR (ICS)
c OPTION - INCLUDE RP, RL, PRODUCTION AND LOSS
c         do kk=1,c_kmax
          c_xcout(kk,ics) = 0.
          c_rp(kk,ics) = 0.
          c_rl(kk,ics) = 0.
c         endo

        do ic=ic1,ic2
c         do kk=1,c_kmax
           c_xcout(kk,ics) = c_xcout(kk,ics) + c_xcout(kk,ic)
           c_rp(kk,ics) = c_rp(kk,ics) + c_rp(kk,ic)
           c_rl(kk,ics) = c_rl(kk,ics) + c_rl(kk,ic)
c         endo
        enddo

c OPTION:  CONVERT LUMPED SPECIES VALUES (IC1-IC2) INTO PARTITION FR.
        do ic=ic1,ic2
c        do kk=1,c_kmax
         kk=1
         if(c_kkw.gt.0) kk=c_kkw
           if(c_xcout(kk,ics).gt.0) then
c           c_xcout(kk,ic) = c_xcout(kk,ic)/c_xcout(kk,ics)
           endif
c        enddo
        enddo

100   continue
101    continue

c
c  END POSTLUMP
2000  return
      end
c -------------------------------------

       subroutine ohwrite(kw,
     &            c_xcin, xc, c_rp, c_rl, c_rr 
     &            ,c_h2oliq, senshx, senhcat, oddhsum, oddhdel)

c This prints a summary of odd-h radical sources by category,
c  and also sensitivity to OH (d(lnHx)/d(lnOH)) by category.
c
c (Representing the components of oddhsum and oddhdel
c   used in the odd-h radical solution.) 

c
c Inputs:    Species concentrations (xc), reaction rates (rr),
c              odd-h sums 
c
c Outputs:   None
c
c
c Called by:    boxmain,  ohsolve
c
c Calls to:     None.
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c
c -------------------------------------------------------------------

c -------------------------------------------------------
      use h_rbchemmech
c      use h_rbchemvars
      use h_rbchemlocal, only:  avogadrl
c      use h_rbchemlocal, only: kk,ii,ic, alpha, avogadrl, nr, icat1,
c     & icat2, icatp,i, icatp2
      IMPLICIT NONE

c#     include "rbchemmech.h"
c#     include "rbchemvars.h"
c#     include "rbchemlocal.h"


c VARIABLES FROM rbchemarrays.h,  rbchemlocarr.h
      integer :: kw
       real*8 ::  c_xcin( c_kvec,c_cdim) ! concentr molec/cm3
      real*8 ::  xc( c_kvec,c_cdim)     ! concentration molec/cm3
      real*8 ::  c_rp(c_kvec,c_cdim)    ! Production, molec/cm3
      real*8 ::  c_rl(c_kvec,c_cdim)    ! Loss, molec/cm3
      real*8 ::  c_rr(c_kvec,c_rdim)    ! Reaction rate m/cm3
         real*8 ::  c_h2oliq(c_kvec)   ! LWC grams cm-3
      real*8 ::        senshx(c_kvec,2)   ! Hx sens: d ln(pHx)/d
c       ln([OH])
      real*8 ::        senhcat( c_kvec,99) ! Hx sens. by species categ.
      real*8 ::        oddhsum(c_kvec,2)   ! Summed net P/L Hx, mol/cm3
      real*8 ::        oddhdel(c_kvec,2)   ! Summed Hx sens to
     
!=========below are local variables=====Guangxing Lin 
      real*8 ::        oddhloh(c_kvec,2)   ! Hx loss from OH,  mol/cm3
      real*8 ::        oddhlho2(c_kvec,2)   ! Hx loss from HO2,  mol/cm3
      real*8 ::        oddhsrc(c_kvec,2)   ! Summed P Hx, mol/cm3

            real*8 :: foh(c_kvec)       ! OH/HO2.  not common
       integer :: c_iter                  ! chem. number of
          real*8 :: stoicx                         ! stoichiometry sum
      integer :: ic,ic1,ic2,ic3,icc,ics,ics2,icc1,icc2,icc3 !Chem index
      integer :: is,iss,iscs,isc2                    !Chem local index
          integer :: ich,icq,icx,icx1,icx2,icy1,icy2,icp,icp1,icp2 !Chem
      integer :: icr1,icr2,icr,isr1,isr2,icpair  !chem index - pair and
      integer :: ionsum                                  ! aquasolve ion
      integer :: ica1,ica2,icb1,icb2,nra1,nra2,nrb1,nrb2 !Aquasolve chem
      integer :: neq                                     !Aqueous
      integer :: nc,nc1,nc2,ncc,ncf,nn,nne               !Chem species
      integer :: nr,nr1,nr2,nrh,nrq,nrqq,nrx,np          ! Reaction
      integer :: icat1,icat2,icatp,icatp2  ! indices used for species
      integer :: kk                            ! Vectorization
      integer :: i, j, k, ii, ij, iii, n           ! General counters
       integer :: nsol           ! Number of pair groups being solved
      integer :: nsolv          ! Number of species being solved for
          real*8 :: alpha(c_kvec)   ! General vector variable
      real*8 ::  beta(c_kvec)   ! General vector variable
      real*8 :: gamma(c_kvec)   ! General vector variable
      real*8 :: prior(c_kvec)   ! Prior species conc (molec/cm3)
      integer :: ncsolv(c_cdim) ! Species number (ic) for species in
      integer :: nssolv(c_cdim) ! Pair group number (is) for spec. in

c LOCAL VARIABLES: 

c hxsum(20,2):  Summed (1) net Hx source and 
c                (2) Hx sensitivity to OH (as dHx/dOH*[OH])
c                for different source categories

      real*8 :: hxsum(20,2)  ! Summed Hx source and dH/dOH*[OH]

c -------------------------------------------------------

       kk=1

       if(kw.le.0) return

        write(c_out,1858) senhcat(kw,10), senhcat(kw,3)
1858    format(/,' OH-SENSITIVITY COEFF:  HO2, RO2=',2f10.4)  

c HX WRITE MATRIX FOR HX SOURCES.
        write(c_out,1859)
1859    format(/,'ODD-H RADICAL SOURCES AND SINKS:')

c INITIAL ZERO
        do ii = 1, 20
         hxsum(ii,1) = 0.
         hxsum(ii,2) = 0.
        enddo

c PRIOR ODD HYDROGEN (16).  (COPY FROM PRIOR IN 'OHSOLVE').
      do ic=1,c_nchem2
        if((c_icat(ic).eq.3.or.c_icat(ic).eq.9.or.c_icat(ic).eq.10).and.
     *          .not.c_lsts(ic))  then
         if(ic.eq.c_npequil(ic)) then 
            hxsum(16,1) = hxsum(16,1) +   c_xcin(kk,ic) - xc(kk,ic)
            alpha(kk)=0.
            if(c_rl(kk,ic)+xc(kk,ic).gt.0) then
             alpha(kk) =  (c_rp(kk,ic)+c_rl(kk,ic))*xc(kk,ic)
     *           /(c_rp(kk,ic)+c_rl(kk,ic)+xc(kk,ic))
             hxsum(16,2) = hxsum(16,2) - senhcat(kk,c_icat(ic))
     *         * alpha(kk)
            endif
         else

           if(c_h2oliq(kk).gt.0) then
               hxsum(16,1) = hxsum(16,1) + (  c_xcin(kk,ic) - xc(kk,ic))
     *            *c_h2oliq(kk)*avogadrl 

               alpha(kk)=0.
               if(c_rl(kk,ic)+xc(kk,ic).gt.0) then
                 alpha(kk) = ( c_rp(kk,ic)+c_rl(kk,ic))
     *                            *xc(kk,ic)*c_h2oliq(kk)*avogadrl
     *              /(c_rp(kk,ic)+c_rl(kk,ic)
     *                       +xc(kk,ic)*c_h2oliq(kk)*avogadrl)
                  hxsum(16,1) = hxsum(16,1) - senhcat(kk,c_icat(ic))
     *              * alpha(kk)
               endif
           endif

         endif

       endif
      enddo

c LOOP THROUGH REACTIONS, SET SENHCAT FOR EACH.  
c THEN SUM ODDHSUM ODDHDEL BY SPECIES CATEGORIES
        do 1870 nr=1,c_nreac
         icat1 = 0
         icat2 = 0
         icatp = 0
         icatp2 = 0
         if(c_reactant(nr,1).gt.0) then
             icat1=c_icat(c_reactant(nr,1))
         endif
         if(c_reactant(nr,2).gt.0) then
            icat2=c_icat(c_reactant(nr,2))
         endif
         if(c_product(nr,1).gt.0) then
            icatp=c_icat(c_product(nr,1))
         endif
         if(c_product(nr,2).gt.0) then
            icatp2=c_icat(c_product(nr,2))
         endif
c 2009 modifications
         senshx( kw,1) = 0.
         if(icat1.gt.0) senshx( kw,1) = senshx( kw,1)
     *     + senhcat( kw,icat1)
         if(icat2.gt.0) senshx( kw,1) = senshx( kw,1)
     *     + senhcat( kw,icat2)

c           if(nr.ge.370.and.nr.le.372) write(c_out,*)  nr, icat1,
c    *        icat2,icatp,icatp2,c_oddhx(nr,1) 

c SUM BY SPECIES CATEGORIES:  
c (1) HO2+HO2->H2O2.
         if(icat1.eq.10.and.icat2.eq.10) then
           hxsum(1,1) = hxsum(1,1)+ c_rr( kw,nr)*c_oddhx(nr,1)
           hxsum(1,2) = hxsum(1,2)+ c_rr( kw,nr)*c_oddhx(nr,1)
     *   *senshx( kw,1)
         endif
c (2) HO2+RO2->ROOH
c      excluding CO3 (HCO3, icatp=20)
         if( (  (icat1.eq.10.and.icat2.eq.3)
     *    .or.(icat2.eq.10.and.icat1.eq.3)  )
     *    .and.(icatp.ne.20.and.icatp2.ne.20)
     *                                      )then
           hxsum(2,1) = hxsum(2,1)+ c_rr( kw,nr)*c_oddhx(nr,1)
           hxsum(2,2) = hxsum(2,2)+ c_rr( kw,nr)*c_oddhx(nr,1)
     *   *senshx( kw,1)
         endif
c (3) RO2+RO2->products
         if (icat1.eq. 3.and.icat2.eq.3) then
           hxsum(3,1) = hxsum(3,1)+ c_rr( kw,nr)*c_oddhx(nr,1)
           hxsum(3,2) = hxsum(3,2)+ c_rr( kw,nr)*c_oddhx(nr,1)
     *   *senshx( kw,1)
c TEMPORARY TEST
c          write(c_out,12001) nr, (c_treac(ir,nr),ir=1,5),
c    *      c_rr( kw,nr),c_oddhx(nr), hxsum(3,1)
c 12001      format(i5,2x,a8,'+',a8,'=>',a8,'+',a8,'+',a8,/,3(1pe10.3))
         endif
c (4) H2O2, ROOH+hv
         if(icat1.eq.4.or.icat2.eq.4) then
           hxsum(4,1) = hxsum(4,1)+ c_rr( kw,nr)*c_oddhx(nr,1)
           hxsum(4,2) = hxsum(4,2)+ c_rr( kw,nr)*c_oddhx(nr,1)
     *   *senshx( kw,1)
         endif
c (5) OH+HO2
         if( (icat1.eq.9.and.icat2.eq.10)
     *   .or.(icat2.eq.9.and.icat1.eq.10) ) then
           hxsum(5,1) = hxsum(5,1)+ c_rr( kw,nr)*c_oddhx(nr,1)
           hxsum(5,2) = hxsum(5,2)+ c_rr( kw,nr)*c_oddhx(nr,1)
     *   *senshx( kw,1)
         endif

c (6)  HNO3 production;  including aqueous NO3L+HO2-.
         if((      (icat1.eq.9.and.icat2.eq.12)
     *       .or.  (icat2.eq.9.and.icat1.eq.12)
     *       .or. (icat1.eq.14.and.icat2.eq.10)
     *       .or. (icat1.eq.10.and.icat2.eq.14)
     * ) .and. icatp.ne.6)  then
           hxsum(6,1) = hxsum(6,1)+ c_rr( kw,nr)*c_oddhx(nr,1)
           hxsum(6,2) = hxsum(6,2)+ c_rr( kw,nr)*c_oddhx(nr,1)
     *   *senshx( kw,1)
         endif

c (7,8)  HNOx:  SEPARATE INTO SINKS AND SOURCES
c JUST SO THAT 'EXCHANGE' CAN BE REMOVED.
         if (icat1.eq.6.or.icat2.eq.6              ) then
           hxsum(7,1) = hxsum(7,1)+ c_rr( kw,nr)*c_oddhx(nr,1)
           hxsum(7,2) = hxsum(7,2)+ c_rr( kw,nr)*c_oddhx(nr,1)
     *   *senshx( kw,1)
         endif

         if (                            icatp.eq.6) then
           hxsum(8,1) = hxsum(8,1)+ c_rr( kw,nr)*c_oddhx(nr,1)
           hxsum(8,2) = hxsum(8,2)+ c_rr( kw,nr)*c_oddhx(nr,1)
     *   *senshx( kw,1)
         endif

c (9,10) PANs SINKS, SOURCES.  (Note: ODDHDEL for PAN is re-set below.)
         if (icat1.eq.5.or.icat2.eq.5              ) then
           hxsum(9,1) = hxsum(9,1)+ c_rr( kw,nr)*c_oddhx(nr,1)
           hxsum(9,2) = hxsum(9,2)+ c_rr( kw,nr)*c_oddhx(nr,1)
     *   *senshx( kw,1)
         endif

         if (                            icatp.eq.5) then
           hxsum(10,1) = hxsum(10,1)+ c_rr( kw,nr)*c_oddhx(nr,1)
           hxsum(10,2) = hxsum(10,2)+ c_rr( kw,nr)*c_oddhx(nr,1)
     *   *senshx( kw,1)
         endif

c (11) RNO3
         if  (icatp.ne.5.and.
     *    (   (icat1.eq.3.and.icat2.eq.13)  
     *   .or. (icat2.eq.3.and.icat1.eq.13)  )    )then
           hxsum(11,1) = hxsum(11,1)+ c_rr( kw,nr)*c_oddhx(nr,1)
           hxsum(11,2) = hxsum(11,2)+ c_rr( kw,nr)*c_oddhx(nr,1)
         endif

c (12) O3+hv
         if  (icat1.eq.11.and.icat2.eq.0)   then
           hxsum(12,1) = hxsum(12,1)+ c_rr( kw,nr)*c_oddhx(nr,1)
           hxsum(12,2) = hxsum(12,2)+ c_rr( kw,nr)*c_oddhx(nr,1)
     *   *senshx( kw,1)
         endif
c (13)  ALD+hv
         if  (icat1.le. 2.and.icat2.eq.0
     *                        .and.c_reactant(nr,2).eq.-1) then
           hxsum(13,1) = hxsum(13,1)+ c_rr( kw,nr)*c_oddhx(nr,1)
           hxsum(13,2) = hxsum(13,2)+ c_rr( kw,nr)*c_oddhx(nr,1)
     *   *senshx( kw,1)
         endif
c (14)  O3+OLEFIN
         if( (icat1.eq.11.and.(icat2.eq.1.or.icat2.eq.2))  
     *   .or.(icat2.eq.11.and.(icat1.eq.1.or.icat1.eq.2))  )then
           hxsum(14,1) = hxsum(14,1)+ c_rr( kw,nr)*c_oddhx(nr,1)
           hxsum(14,2) = hxsum(14,2)+ c_rr( kw,nr)*c_oddhx(nr,1)
     *   *senshx( kw,1)
         endif

c (15)  AQUEOUS O2- + HCO3-.   (expand to include H+, OH- = cat 17,18?)
         if( (icat1.eq.20.or.icat2.eq.20.or.
     *                           icatp.eq.20.or.icatp2.eq.20) .or.
     *       (icat1.eq.0.or.(icat2.eq.0.and.c_reactant(nr,2).gt.0.))
     *       ) then
           hxsum(15,1) = hxsum(15,1)+ c_rr( kw,nr)*c_oddhx(nr,1)
           hxsum(15,2) = hxsum(15,2)+ c_rr( kw,nr)*c_oddhx(nr,1)
     *   *senshx( kw,1)
c           if(nr.ge.370.and.nr.le.372) write(c_out,*)  nr, icat1,
c    *        icat2,icatp,icatp2,c_oddhx(nr) 
         endif
1870   continue

c CORRECTION FOR 'EXCHANGE' REACTIONS:  PAN, HNOx EXCHANGE
c  IS REMOVED (APPROXIMATELY) FROM ODDHDEL
c   (NOTE:  THIS CAUSES ERROR WITH HONO)
       if((0.-hxsum(8,1)).gt.hxsum(7,1).and.hxsum(8,1).ne.0) then
         hxsum(8,2) = hxsum(8,2)*(1.+hxsum(7,1)/hxsum(8,1))
        else
         hxsum(8,2) = 0.
        endif

c APPROXIMATE RE-SET FOR PAN ODDHDEL.
       if((0.-hxsum(10,1)).gt.hxsum(9,1).and.hxsum(10,1).ne.0) then
         hxsum(10,2) = hxsum(10,2)*(1.+hxsum(10,1)/hxsum(10,1) )
        else
         hxsum(10,2) = 0.
        endif

c MISCELLANEOUS(17) AND TOTAL SUM (18)
c   (NOTE MISCELLANEOUS OFTEN CORRECTS FOR APPROXIMATION IN PAN ODDHDEL)
c 2009 modifications
       hxsum(17,1) = oddhsum( kw,1)
       hxsum(17,2) = oddhdel( kw,1)
       hxsum(18,1) = oddhsum( kw,1)
       hxsum(18,2) = oddhdel( kw,1)
       hxsum(19,1) = oddhsum( kw,2)
       hxsum(19,2) = oddhdel( kw,2)
       do ii=1,16
        hxsum(17,1) = hxsum(17,1)-hxsum(ii,1)
        hxsum(17,2) = hxsum(17,2)-hxsum(ii,2)
       enddo

c WRITE ODD-H SOURCES AND SINKS.  NOTE DISCREPANCY IN PRIOR HX.
c THE TRUE ODDHSUM USES (XXO-XRP, prior) IN THE CALC FOR NEW OH.
c HXSUM RECORDS (XXO-XRP) for RO2 + (XXO-XR,new) for OH+HO2.
c THIS CAUSES A SMALL DISCREPANCY BETWEEN HXSUM AND ODDHSUM
c THE DISCREPANCY IS ADDED INTO HXSUM-MISCELLANEOUS.

      write(c_out,1876) ((hxsum(ii,i),i=1,2),ii=1,18)
1876  format(/,' ---ODD-H SOURCES---     ODDHSUM   ODDHDEL',/,
     *         ' HO2+HO2->H2O2        ',2(1pe10.3),/,
     *         ' HO2+RO2->ROOH        ',2(1pe10.3),/,
     *         ' RO2+RO2-> prod       ',2(1pe10.3),/,
     *         ' H2O2,ROOH+hv         ',2(1pe10.3),/,
     *         ' OH+HO2 -> SINK       ',2(1pe10.3),/,
     *         ' OH+NO2->HNO3         ',2(1pe10.3),/,
     *         ' HONO, HNO4 SINK      ',2(1pe10.3),/,
     *         ' HONO, HNO4 SOURCE    ',2(1pe10.3),/,
     *         ' PANS SINK            ',2(1pe10.3),/,
     *         ' PANS SOURCE          ',2(1pe10.3),/,
     *         ' RO2+NO->RNO3         ',2(1pe10.3),/,
     *         ' O3+hv->2OH           ',2(1pe10.3),/,
     *         ' ALD+hv               ',2(1pe10.3),/,
     *         ' O3+OLEFIN            ',2(1pe10.3),/,
     *         ' AQUEOUS HCO3-        ',2(1pe10.3),/,
     *         ' PRIOR OH, HO2,RO2    ',2(1pe10.3),/,
     *         ' MISCELLANEOUS        ',2(1pe10.3),/,
     *         ' FINAL SUM            ',2(1pe10.3)        )

c SPECIAL AQUEOUS DEBUG WRITE - TEMPORARY. aqueous reactions 370-376
c         do nr=384,386
c          write(c_out,102) nr,(c_treac(i,nr),i=1,5),
c    *      c_rr( kw,nr) , ratek( kw,nr)
c         enddo

102    format(i4,2x,a8,'+',a8,'=>',a8,'+',a8,'+',a8,2x,3((1pe10.3),2x))

c END OHWRITE
2000  return
      end
c
c --------------------
c END  OF chemsolve/quadchem. (aquasolve may be attached below.)
c --------------------
c
c ==============================================================
c
      subroutine aquasolve
     &   (c_xcin, c_xcout, c_xcav, c_xcemit,
     &    c_rp, c_rl, c_rr ,
     &    xc,xcfinr,  xclastq, 
     &    ratek, rateh, rateq, rateqq, rhdif, egasaq, 
     &    xrp, xrr, rpro, rloss, rpro1, rloss1, 
     &    rrp, rrl, cpro,
     &    xrm, xrrm, rpm, rlm,cpm, rpmulti, rlmulti,
     &    rppair, rlpair, xrppair, xrrpair, 
     &    history, geomavg 
     &    , c_temp,  c_h2oliq ,  c_droplet, c_iter)
c 

c THIS  CALCULATES (H+) AND AQUEOUS SPECIES CONCENTRATIONS
c   (MOLES PER LITER)  BASED ON PRIOR (H+).    
c
c THE SOLUTION FOR (H+)  MUST BE RUN ITERATIVELY.  
c EACH AQUASOLVE CALL REPRESENTS A SINGLE ITERATION. 
c
c CHEMISTRY IS ASSUMED TO BE REPRESENTED BY DISSOCIATING SPECIES
c WITH CHAINS OF UP TO THREE DISSOCIATIONS 
c (GAS->AQUEOUS, AQUEOUS->SINGLE ION, ION->DOUBLE ION)
c EACH RELEASING H+ AND OH-.

c THIS RESULTS IN A SINGLE FOURTH-ORDER EQUATION FOR H+
c  (0 = THE SUM OF H+, OH-, CATIONS AND ANIONS, ALL FUNCTIONS OF H+) 
c WHICH IS SOLVED ITERATIVELY USING NEWTON-RAFSON. 
c TYPICALLY IT IS CALLED FROM WITHIN THE GAS-PHASE ITERATION.
c
c INCLUDES LELIEVELD 1991,etc. FOR WATER DROPLET DIFFUSION
c (SEE 'NEWTON-RAFSON' AND 'Lelieveld' FOR DETAILS BELOW.)
c
c INCLUDES GAS-PHASE DIFFUSION MODIFICATION FOR PARTITIONING  
c  WHERE GAS->AQ TRANSFER IS SLOWER THAN AQUEOUS LOSS.
c  (with ACCOMODATION coefficients - see ACCOMODATION)
c
c NEW ADD:  GAS/AQUEOUS PARTITIONING BASED ON GAS/AQUEOUS SPEED.
c
c NOTE:  DROPLET RADIUS, GAS DIFFUSION CONSTANT, HARD-WIRED. PI also.
c
c ----------------
c A NOTE ON UNITS:  WITHIN THIS LOOP, THE GAS-MASTER SPECIES
c IS CONVERTED INTO LIQUID UNITS (MOLES GAS PER LIQUID WATER).
c   (=MOLECULES/CM3 /(AVOGADRL*AQUA(KK)).
c THE HENRY'S LAW COEFFICIENTS (RATEH) WERE ALSO CONVERTED
c   (MULTIPLIED BY AVOGADRL*AQUA(KK).)
c THIS ALLOWS FOR EASY CONSERVATION OF THE GAS+AQUEOUS SPECIES SUM.
c GAS-MASTER SPECIES IS CONVERTED BACK TO MOLECULES/CM3 AT THE END.
c
c NOTE:  0.1-1e6 gram/cm3 is typical.
c ----------------
c 
c Inputs:    Species concentrations (xc), chemistry, LWC
c
c Outputs:   Gas and aqueous concentrations (xc), 
c               in mole/cm3 (gas) and mole/liter (aq)
c
c
c Called by:    quadchem
c
c Calls to:     None.
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c
c -------------------------------------------------------------------

      use h_rbchemmech
      use h_rbchemvars, only: c_time
c      use h_rbchemarrays
c      use h_rbchemlocal, only: kk, nrh,ic, ich, prior, i, neq, icq,
c     & avogadrl, alpha, gamma, dropdif,beta, RUMOLEC, PII, DIFGAS, ica1,
c     & ica2, icb1, icb2, nra1,nra2, nrb1, nrb2, nrq,ionsum, j
      use h_rbchemlocal, only: 
     & avogadrl,dropdif,RUMOLEC, PII, DIFGAS
c      use h_rbchemlocarr
      IMPLICIT NONE


c#     include "rbchemmech.h"
c#     include "rbchemvars.h"
c#     include "rbchemarrays.h"
c#     include "rbchemlocal.h"
c#     include "rbchemlocarr.h"

c       c --below are originally from rbchemarrys---
       real*8 ::  c_xcin( c_kvec,c_cdim) ! concentr molec/cm3
       real*8 ::  c_xcout( c_kvec,c_cdim) ! concentr, mol/cm3
       real*8 ::  c_xcav( c_kvec,c_cdim)  ! concentr, mol/cm3
       real*8 ::  c_xcemit( c_kvec,c_cdim) ! emis, molec/cm3
      real*8 ::  c_rp(c_kvec,c_cdim)  ! Production, molec/cm3
      real*8 ::  c_rl(c_kvec,c_cdim)  ! Loss, molec/cm3
      real*8 ::  c_rr(c_kvec,c_rdim)  ! Reaction rate m/cm3
        real*8 :: xc( c_kvec,c_cdim) ! concentration molec/cm3
       real*8 :: xcfinr( c_kvec,c_cdim) ! ratio FINAL/AVG cncn
       real*8 :: xclastq( c_kvec,c_cdim) ! conc molec/cm3
          real*8 :: ratek(c_kvec,c_rdim)  ! Rate constants
      real*8 :: rateh(c_kvec,c_hdim)  ! Henry's law constants
      real*8 :: rateq(c_kvec,c_aqdim)  ! Aqueous eq. constants
      real*8 :: rateqq(c_kvec,c_aqqdim) ! Special eq. constants
      real*8 :: rhdif(c_kvec,c_hdim) ! H. mod by diffusion
      real*8 :: egasaq(c_kvec,c_hdim) ! Gas-to-aq transfer s-1
      real*8 :: xrp(c_kvec,c_cdim)     !Solver prior concentration
      real*8 :: xrr(c_kvec,c_cdim)     ! Solver concentration
      real*8 :: rpro(c_kvec,c_cdim)    ! Solver production rate
      real*8 :: rloss(c_kvec,c_cdim)   ! Solver loss rate
      real*8 :: rpro1(c_kvec,c_cdim)   ! Interim rpro w/out cpro 
      real*8 :: rloss1(c_kvec,c_cdim)  ! Interim rloss w/out cpro
      real*8 :: rrp(c_kvec,c_cdim)     ! Interim production rate 
      real*8 :: rrl(c_kvec,c_cdim)     ! Interim loss rate 
      real*8 :: cpro(c_kvec,c_cdim, c_cdim)   ! Cross-production 
      real*8 :: xrm(c_kvec,c_cdim)     !  Initial conc. for multi
      real*8 :: xrrm(c_kvec,c_cdim)    !Initial solution for multi
      real*8 :: rpm(c_kvec,c_cdim)     ! Production rate for multi
      real*8 :: rlm(c_kvec,c_cdim)     ! Loss for multi-spec 
      real*8 :: cpm(c_kvec, c_cdim, c_cdim) !Cross-prod for multi
      real*8 :: rpmulti(c_kvec       ) !Product'n multi group sum
      real*8 :: rlmulti(c_kvec       ) ! Loss group sum for multi
      real*8 :: rppair(c_kvec,c_cdim)  ! Production for pair grp
      real*8 :: rlpair(c_kvec,c_cdim)  ! Loss rate for pair grp
      real*8 :: xrppair(c_kvec)        ! Solver prior pair group cn.
      real*8 :: xrrpair(c_kvec)        ! Solution for pair group sum
      real*8 :: history(c_kvec,c_cdim,400)  ! Cncn for each iter
      real*8 :: geomavg(c_kvec,c_cdim)   ! Geom. avg factor
       real*8 ::  c_temp(c_kvec)    ! temperature K
      real*8 ::  c_h2oliq(c_kvec)   ! LWC grams cm-3
       real*8 ::  c_droplet(c_kvec)        ! droplet radius, cm
       integer :: c_iter                  ! chem. number of iterations
c----below are local variables ===Guangxing LIn 
c 


      real*8 ::  c_rainfr(c_kvec)   ! rainout fraction
      logical :: c_lexpo(c_kvec)        ! Flag for expo decay solution

             real*8 :: foh(c_kvec)       ! OH/HO2.  not common
          real*8 :: stoicx                         ! stoichiometry sum
      integer :: ic,ic1,ic2,ic3,icc,ics,ics2,icc1,icc2,icc3 !Chem index
      integer :: is,iss,iscs,isc2                    !Chem local index
          integer :: ich,icq,icx,icx1,icx2,icy1,icy2,icp,icp1,icp2 !Chem
      integer :: icr1,icr2,icr,isr1,isr2,icpair  !chem index - pair and
      integer :: ionsum                                  ! aquasolve ion
      integer :: ica1,ica2,icb1,icb2,nra1,nra2,nrb1,nrb2 !Aquasolve chem
      integer :: neq                                     !Aqueous
      integer :: nc,nc1,nc2,ncc,ncf,nn,nne               !Chem species
      integer :: nr,nr1,nr2,nrh,nrq,nrqq,nrx,np          ! Reaction
      integer :: icat1,icat2,icatp,icatp2  ! indices used for species
      integer :: kk, kw                            ! Vectorization
      integer :: i, j, k, ii, ij, iii, n           ! General counters
       integer :: nsol           ! Number of pair groups being solved
      integer :: nsolv          ! Number of species being solved for
          real*8 :: alpha(c_kvec)   ! General vector variable
      real*8 ::  beta(c_kvec)   ! General vector variable
      real*8 :: gamma(c_kvec)   ! General vector variable
      real*8 :: prior(c_kvec)   ! Prior species conc (molec/cm3)
      integer :: ncsolv(c_cdim) ! Species number (ic) for species in
      integer :: nssolv(c_cdim) ! Pair group number (is) for spec. in

c LOCAL VARIABLES

      character*8 :: tsum
      real*8 :: sum

      real*8 :: xhinv(c_kvec)     ! 1/[H+]
      real*8 :: xohinv(c_kvec)    ! 1/[OH]
      real*8 :: xkwinv(c_kvec)    ! 1/Kw =1/[H+][OH-}

      real*8 :: xtest             ! Test for LWC>0 in loop
      real*8 :: xxx1              ! Variable for write statement
      real*8 :: xxx2              ! Variable for write statement
      real*8 :: acquacon          ! Aq conversion fac for write
c
      kk=1

c IF ACQUA=0, RETURN.  ONLY FOR NONVECTORIZED VERSION.
      xtest=0.
c     do kk=1,c_kmax
       if(c_h2oliq(kk).gt.xtest) xtest=c_h2oliq(kk)
c     enddo

c     if(xtest.eq.0.) return
      if(xtest.le.1.000E-25) return

c AVOGADRL converts MOLES/LITER to MOLECULES/CM3.-> entered in chemvars
c RTCON
c     avogadrl = 6.02E+20

c pi (if not built-in special)-> entered in chemvars

c H+ species number  -ih, ioh replaced with c_nhplus, c_nohmin
c      c_nhplus=c_aqueous(1,2)
c      c_nohmin = c_aqueous(1,3)

c DROPLET RADIUS (.001 cm) AND DIFFUSION COEFFICENT (2E-5 cm2/sec)
c AND GAS-PHASE DIFFUSION COEFFICIENT (0.1 cm2/sec) 
c   (all with values from Lelieveld and Crutzen, 1991)
c AND "RU" PARAMETER FOR  molecular speed (Barth, personal comm)
c   (8.314e7 g-cm2/s2-mol-K)   (note: 8.314e0 kg-m2/s2-mol-K)

c v      do kk=1,c_kmax  
c      c_DROPLET(kk) = .001   => now INPUT VALUE
c        enddo              ! do kk=1,c_kmax

c      DROPDIF = 2.0E-05  - entered as parameter in chemvar1
c      DIFGAS  = 0.1  - entered as parameter in chemvar1
c       RUMOLEC = 8.314E7 - entered as parameter in chemvar1

c  (   VMOLEC = MOLECULAR SPEED, CALCULATED BELOW)
 

c NEW PRELIMINARY:  CALCULATE ADJUSTMENT TO HENRY'S LAW COEFFICIENTS
c  TO REPRESENT DROPLET DIFFUSION LIMITATION 
c  (Lelieveld, J. At. Chem 12, 229, 1991 - see p. 241).
c
c  Q = Cavg/Csurf = 3 (coth q /q - 1/q**2)**-1;  q=r*(ka/Da)**0.5
c
c   where r= droplet radius (.001cm)  Da=droplet diffusion (2e-5 cm)
c   ka = pseudo-1st-order aqueous loss rate.
c  This Q applies only to the species component that originated in the 
c   gas phase and was transported to aqueous (Sg) as opposed to being 
c   produced chemically in the aqueous phase (Pa).
c
c  Total Q' = (Pa + QSg)/(Pa+Sg);  Sg=source to aqueous from gas.
c
c  (Pa, Sg should be from prior iteration.  Here,
c   Sg = (rpgas+c_xcin) * prior aq/gas ratio (= rlaq/(rlaq+rlgas).  Slight overest.
c   other options:  (i)= (Pgas+c_xcin)*(xraq/xrtot).  (ii)=rlaq-rpaq  )
c

c ------------
c BEGIN LOOP FOR DIFFUSION-HENRY'S LAW MODIFICATION.
c ------------
        do  nrh=1,c_nreach
         ic = c_henry(nrh,1)
         ich=c_ncequil(ic,1)

c DIFFUSION-MODIFIED HENRY'S LAW COEFFICIENT: FIRST SET EQUAL TO ONE
c ALSO, SAVE PRIOR RHDIF
c        do kk=1,c_kmax
           prior(kk) = rhdif(kk,nrh)
           rhdif(kk,nrh) = 1.
c        enddo  

c FOR ITER>1, USE PRIOR RP, RL TO SET DIFFUSION-MODIFICATION.

         if(c_iter.gt.1.and.c_nequil(ic).gt.0)  then
   
c PRELIMINARY ZERO
           do i=1,3
c            do kk=1,c_kmax
              rpro(kk,i) = 0.
              rloss(kk,i) = 0.
              xrp(kk,i) = 0.
c             rpro(kk,i) = 0.00001
c             rloss(kk,i) = 0.00001
c             xrp(kk,i) = 0.1
c            enddo
           enddo

c SUM AQUEOUS CONCENTRATIONS IN GAS-EQUIVALENT UNITS (xrp1)
c ALSO SUM AQUEOUS PRODUCTION (rpro1) AND LOSS (rloss1)
           do    neq=1,c_nequil(ic)
            icq = c_ncequil(ic,neq)
            if(icq.gt.0) then

c v           do  kk=1,c_kmax
              if(c_h2oliq(kk).gt.0) then
                if(c_iter.le.2) then
                 xrp(kk,1) =xrp(kk,1) + xc(kk,icq)*c_h2oliq(kk)*avogadrl
                else
c xclastq = from prior iteration (=xc end of last aquasolve)
c
c OPTION and possible ERROR:  
c    (1)  Why use xclastq and not current xr?
c    (2)  Were rp preserved from prior iteration?
c
                 xrp(kk,1) =xrp(kk,1) 
c    *             + xclastq(kk,icq)*c_h2oliq(kk)*avogadrl
     *             + xc     (kk,icq)*c_h2oliq(kk)*avogadrl
                endif
                rpro(kk,1) = rpro(kk,1) + c_rp(kk,icq)
                rloss(kk,1) = rloss(kk,1) + c_rl(kk,icq)
                if(neq.eq.1)  xrp(kk,2) =  xrp(kk,1)
              endif
c v           enddo  

             endif
           enddo     

c PSEUDO-FIRST-ORDER AQUEOUS LOSS CONSTANT (alpha)
c  (NOTE:  if RL and XR=0, initial values above make lifetime long.)

c          do kk=1,c_kmax
           if(c_h2oliq(kk).gt.0) then
             alpha(kk) = 0.00001
             if(xrp(kk,1).gt.0.and.rloss(kk,1).gt.0)
     *        alpha(kk) = rloss(kk,1)/(xrp(kk,1)*c_time)
           endif
c          enddo

c Lelieveld Q-FACTOR FOR AQUEOUS DIFFUSION  (beta)
c
c v        do kk=1,c_kmax
           if(c_h2oliq(kk).gt.0) then
             gamma(kk) = c_DROPLET(kk) * sqrt(alpha(kk)/DROPDIF)

c PROTECT AGAINST EXTREME q (=droplet ratio)
             if(gamma(kk).lt..01) then 
              beta(kk)=1.
             else
              if(gamma(kk).gt.100.) then
               beta(kk)=0.001
              else

               beta(kk) = (exp(gamma(kk)) + exp(0.-gamma(kk)) ) 
     *                  / (exp(gamma(kk)) - exp(0.-gamma(kk)) )
                if(c_kkw.gt.0) then  
                  xxx1 = beta(c_kkw)  
                endif  
               beta(kk) = 3.* (beta(kk)/gamma(kk) - 1./(gamma(kk)**2) )
                if(c_kkw.gt.0) then  
                 xxx2 = beta(c_kkw)  
                endif  
               if(beta(kk).gt.1.) beta(kk)=1.
               if(beta(kk).lt.0.) beta(kk)=0.
              endif
             endif

           endif
c v        enddo

c TEST WRITE
c c          if(c_kkw.gt.0) write(c_out,19104) xxx1, xxx2
c c 19104      format(' TEST Q (0-1). coth q, formula Q=',2e10.3)
c          if(c_kkw.gt.0) write(c_out, 19101) 
c    *      nrh, beta(c_kkw), gamma(c_kkw), alpha(c_kkw)
c 19101      format(/,' TEST AQUEOUS DIFF: NRHENRY, Q, q, La(sec-1)='
c    *        ,i5,3e10.3)

c Lelieveld Q-FACTOR, ADJUSTED FOR AQUEOUS PRODUCTION VS DIFFUSION FROM GAS
c Q APPLIED TO GAS DIFFUSION ONLY:  Q' = (Sgas*Q + Saq)/(Sgas+Saq)
c
c (Final option for S=source of aqueous from gas:
c  S = (rpgas+c_xcin) * aq/gas ratio (= rlaq/(rlaq+rlgas).  Slight overest.
c
c  alternatives: (i)  aq/gas=1; aq/gas=Ca/Ct where H=Ca/Cg
c               (ii) S = aqueous rl-rp if >0.                  
c )

c v          do kk=1,c_kmax
             if(c_h2oliq(kk).gt.0) then

              gamma(kk) = (  c_xcin(kk,ic) + c_rp(kk,ic))
              if(rloss(kk,1).gt.0)
     *         gamma(kk) = (  c_xcin(kk,ic) + c_rp(kk,ic)) 
     *                  *rloss(kk,1)/(rloss(kk,1)+c_rl(kk,ic))

c Options
c             gamma(kk) = (  c_xcin(kk,ic) + c_rp(kk,ic)) 

c             gamma(kk) = (  c_xcin(kk,ic) + c_rp(kk,ic)) *rateh(kk,nrh)
c    *                             /(rateh(kk,nrh)+1.)

c             gamma(kk) = rloss(kk,1) - rpro(kk,1)
c             if(gamma(kk).lt.0) gamma(kk).eq.0.)


c Q adjustment:  Q'=rhdif;  Q=beta; Sgas=gamma; Saq=rpro
c  Apply only if aqueous concentration is not zero.

              if(xrp(kk,1).gt.0.and.(gamma(kk)+rpro(kk,1).gt.0))
     *        rhdif(kk,nrh)  = (beta(kk)*gamma(kk) + rpro(kk,1))
     *                       / (gamma(kk) +          rpro(kk,1))

             endif
c v          enddo

c  TEST WRITE
c              if(c_kkw.gt.0) write(c_out, 19107) rateh(c_kkw,nrh),
c    *         rpro(c_kkw,1),   c_xcin(c_kkw,ic), c_rp(c_kkw,ic)
c 19107        format(' TEST rateh rproq c_xcin rpgas=',4e10.3)
c            if(c_kkw.gt.0) write(c_out,19108) c_rl(c_kkw,ic)
c    *         ,rloss(c_kkw,1) ,alpha(c_kkw)
c 19108        format(' TEST rlgas rlaq, Laq(s-1)=',3e10.3)
c            if(c_kkw.gt.0) write(c_out,19103) gamma(c_kkw)
c    *         , rhdif(c_kkw,nrh)
c 19103        format(' TEST S (=RPGAS*(aq/gas)); MODIFIED Q (0-1): '
c    *             ,2e10.3)

         endif
c END IF FOR ITER>1, AQUEOUS>0.

c ENTER COMBINED HENRY-S LAW-w AQ DIFFUSION (relative units).  

c        do kk=1,c_kmax
         if(c_h2oliq(kk).gt.0) then
          rhdif(kk,nrh) = rhdif(kk,nrh)*rateh(kk,nrh)
c----------Guangxing Lin--08142012------test for no Q adjustment---
c       rhdif(kk,nrh) = rateh(kk,nrh)
c----------Guangxing Lin--08142012------test for no Q adjustment---
         endif
c        enddo
c
c DIAGNOSTIC WRITE
           if(c_kkw.gt.0) then  
             write(c_out,19105) (c_treach(i,nrh),i=1,2)  
     *        ,rateh(c_kkw,nrh), rhdif(c_kkw,nrh)
19105         format(a8,'=',a8,'  HENRYs LAW COEFF.=',1pe10.3,
     *       '  H+DROPLET DIFF COEF=', 1pe10.3)
           endif

c -----------------------------------------------------------
c GAS-PHASE DIFFUSION MODIFICATION:  
c ADJUST GAS/AQUEOUS RATIO TO ACCOUNT FOR CASE WHERE AQUEOUS LOSS RATE
c  (FROM AQUEOUS AND IONIC EQUILIBRIA SPECIES)
c  IS FASTER THEN THE NEEDED GAS-AQUEOUS TRANSFER.
c -----------------------------------------------------------
c
c
c USES ACCOMODATION COEFFICIENTS, FORMULAS IN Lelieveld AND INFO 
c  FROM MARY BARTH.  

c PARTITION COEFFICIENT IS BASED ON STEADY STATE BETWEEN Pg, Pa,
c Lg, La, Eg, Ea=Eg/H.  Eg calculated from Lelieveld, Barth.
c Lg, La, Eg in s-1;  Pg, Pa in equivalent units.
c
c (note rpro(kk,1) = aqueous production; rloss(kk,1)=aqloss,
c  c_rp(kk,ic) = gas pro, c_rl(kk,ic) = aqueous pro; xc(kk,ic) = gas cncn.
c  xrp(kk,2) = aqueous concentr (w/o ion sum), gas units.
c
c  (Pg-(Lg+Eg)Cg+Ea"Ca' = 0.;  Pa'+EgCg-(Ea'+La')Ca' = 0.
c    where Ca', Pa', La',Ea' are for sum of aq-equil species;
c    See hand notes in Lelieveld 1991)
c
c AQUEOUS/GAS = (H*(Pa+Pg) + H(Lg/Eg)*Pa)/(Pa+Pg+H(La/Eg)*Pg)
c HENRY ADJUSTMENT = (Pa + Pg + (Lg/Eg)Pa)/(Pa+Pg+H(La/Eg)Pg)

c *** ONLY IF c_iter>1. AND WITH SOFTENING.
c ----------------------------------------------------

c ESTABLISH GAS=>AQUEOUS EXCHANGE RATE (Eg).
c  MOLEC SPEED (VMOLEC, gamma) = sqrt(8*ru*temp/(pi*c_molwt)) 
c    = 3e4 cm/s, speed of sound.  (Barth).
c
c   Eg (s-1) = [(DROPLET**2/3DIFGAS) + 4DROPLET/(3*VMOLEC*ACCOM)]**-1
c   (Lelieveld).
c   THEN MULTIPLY BY LIQUID WATER CONTENT (acqua)
c
c (STILL WITHIN HENRY'S LAW LOOP) 
c ----------------------------------------------------
c
c OPTION:  INCLUDE OR OMIT, WITH ITER CONTROL HERE.

         if(c_iter.gt.1) then
c        if(c_iter.lt.0) then

c v       do kk=1,c_kmax
           gamma(kk) = sqrt( 8.*RUMOLEC*c_temp(kk)/(pii*c_molwt(nrh)) )
           egasaq(kk,nrh) = c_h2oliq(kk)/ 
     *             ( (c_DROPLET(kk)**2/(3.*DIFGAS))
     *             + (4.*c_DROPLET(kk)/(3.*gamma(kk)*c_accom(nrh)))  )
c---Guangxing Lin-------0822-2012--for test only-------------------
c           egasaq(kk,nrh) = 1.0/ 
c     *             ( (c_DROPLET(kk)**2/(3.*DIFGAS))
c     *             + (4.*c_DROPLET(kk)/(3.*gamma(kk)*c_accom(nrh)))  )
c---Guangxing Lin-------0822-2012--for test only-------------------
           beta(kk) = rpro(kk,1) + c_rp(kk,ic)

c TEST WRITE
c          if(c_kkw.gt.0)  
c    *       write(c_out,18099) gamma(kk), egasaq(kk,nrh), beta(kk)
c 18099      format(8(1pe10.3))

c 2009 CORRECTION
c THESE NEXT LINES WERE COMMENTED OUT IN quadchv7.f
c   WHICH GENERATES A SUCCESSFUL SOLUTION
c   (w/ these lines commented in GAS DIFF ADJUST alpha = 1.000)
c-------Guangxing Lin------08-12-2012-----------------------------

          if(beta(kk).lt.0.) beta(kk) = 0.
          gamma(kk) = beta(kk)
          if( c_rl(kk,ic).gt.0.and.(xc(kk,ic)-xrp(kk,1)).gt.0
     *         .and.rpro(kk,1).gt.0.and.egasaq(kk,nrh).gt.0)  
     *     beta(kk) = beta(kk) + (
     *                 ( c_rl(kk,ic)/(xc(kk,ic)-xrp(kk,1)) )
     *                 /(c_time*egasaq(kk,nrh) ))     *rpro(kk,1)
c---Guangxing Lin-------0822-2012--for test only-------------------
c     *     beta(kk) = beta(kk) + (
c     *                 ( c_rl(kk,ic)/(xc(kk,ic)-xrp(kk,1)) )
c     *                 /(c_time*egasaq(kk,nrh)*c_h2oliq(kk) )) *rpro(kk,1)
c---Guangxing Lin-------0822-2012--for test only-------------------

c-------Guangxing Lin------08-12-2012-----------------------------
c TEST WRITE
c          if(c_kkw.gt.0)  
c    *       write(c_out,18099) c_rl(kk,ic), xc(kk,ic), xrp(kk,1),
c    *      rpro(kk,1), beta(kk)

c OLD  CRASH HERE 
           if( rloss(kk,1).gt.0.and.(xrp(kk,2)).gt.0
     *         .and.c_rp(kk,ic).gt.0.and.egasaq(kk,nrh).gt.0
     *         .and.rhdif(kk,nrh).ne.0)  
     *     gamma(kk) = gamma(kk) + (
     *                ( (rloss(kk,1)/(xrp(kk,2)))
c     *                 /(c_time*egasaq(kk,nrh) )*rhdif(kk,nrh)) )
     *              /(c_time*egasaq(kk,nrh) )) *rhdif(kk,nrh) )  ! corrected     
     *                  *c_rp(kk,ic)

c TEST WRITE
c          if(c_kkw.gt.0)  
c    *       write(c_out,18099) rloss(kk,1), xrp(kk,2), c_rp(kk,ic),
c    *       gamma(kk)

           alpha(kk) = 1.
           if(beta(kk).gt.0.and.gamma(kk).gt.0)
     *       alpha(kk) = beta(kk)/gamma(kk)
c-------------Guangxing Lin---08142012----------test for no gas diff adjustment
c           alpha(kk) = 1.
c-------------Guangxing Lin---08142012----------test for no gas diff adjustment

           rhdif(kk,nrh) = rhdif(kk,nrh)*alpha(kk)

c v       enddo

c TEST WRITE
c           if(c_kkw.gt.0) write(c_out,19097)
c    *      egasaq(c_kkw,nrh),c_rl(c_kkw,ic), rloss(c_kkw,1)
c    *      ,xc(c_kkw,ic), xrp(c_kkw,2), xrp(c_kkw,1)
c    *      ,c_rp(c_kkw,ic), rpro(c_kkw,1)
c 19097     format('TEST: EGAS  RLgas  RLaq  XRgas XRaq XRaqall  RPgas'
c    *      , '  RPaq',/,8(1pe10.3))

c DIAGNOSTIC WRITE
           if(c_kkw.gt.0) then  
             write(c_out,19095) (c_treach(i,nrh),i=1,2)  
     *        ,alpha(c_kkw), rhdif(c_kkw,nrh)
19095         format(a8,'=',a8,'  GAS DIFF. ADJUST.=',1pe10.3,
     *       '  HENRY+ DIFFUS COEFS=', 1pe10.3)

c TEST WRITE
            if(c_kkw.gt.0) write(c_out,19097) (c_treach(i,nrh),i=1,2),
     *      egasaq(c_kkw,nrh),c_rl(c_kkw,ic), rloss(c_kkw,1)
     *      ,xc(c_kkw,ic), xrp(c_kkw,2), xrp(c_kkw,1)
     *      ,c_rp(c_kkw,ic), rpro(c_kkw,1)
     &      , alpha(c_kkw), beta(c_kkw), gamma(c_kkw)
19097     format('TEST: EGAS  RLgas  RLaq  XRgas XRaq XRaqall  RPgas'
     *       '  RPaq',/,
     &       a8, '=', a8, /,
     &         8(1pe10.3)/, 'alpha, beta, gamma =', 3(1pe10.3))
   
           endif  

         endif
c END GAS-PHASE DIFFUSION MODIFICATION: 
c
c MODIFY GAS/AQ RATIO FOR TROUBLESOME CONVERGENCE
         if(c_iter.gt.2) then
c v       do kk=1,c_kmax
           rhdif(kk,nrh) = (rhdif(kk,nrh)**0.5)*(prior(kk)**0.5)
c v       enddo
         endif

c DIAGNOSTIC WRITE
           if(c_kkw.gt.0) then  
              write(c_out,19096) (c_treach(i,nrh),i=1,2)  
     *        ,rateh(c_kkw,nrh), rhdif(c_kkw,nrh)
19096         format(a8,'=',a8,'  HENRYs LAW COEFF.=',1pe10.3,
     *       '  H+GAS+DROP DIF COEF=', 1pe10.3)
           endif

c -----------------------------------------------------------
        enddo
c -----------------------------------------------------------
c END LOOP FOR GAS-MASTER SPECIES FOR AQUEOUS DIFFUSION MODIFICATION
c -----------------------------------------------------------

c PRELIMINARY:  ZERO XR PRIOR (USED IN AQUEOUS SUMS). INITIALIZE H+,OH-.
c SUM PRIOR GAS AND AQUEOUS SPECIES INTO THE GAS-MASTER
c ALSO SUM PRODUCTION (RP in gas units)
c        RPRO1 =  SUMMED NET RP FOR THE INDIV. SPECIES
c        RPRO2 =  RUNNING SUM OF NET RP FOR IONS.             
c        RPRO3 =  RUNNING SUM OF PRIOR ION CONCENTRATION
c         RPRO2/3, EFFECT OF PRIOR CHEM PRODUCTION OF IONS
c         WILL BE FACTORED INTO BETA (d/dH slope).

c     do kk=1,c_kmax
      if(c_h2oliq(kk).gt.0) then
       rpro(kk,1) = 0.
       rpro(kk,2) = 0.
       rpro(kk,3) = 0.
       if(xc(kk,c_nhplus).le.0) then
         xc(kk,c_nhplus) = 1.0E-05
         xc(kk,c_nohmin) = rateq(kk,1)/xc(kk,c_nhplus)
       endif
       if(xc(kk,c_nohmin).le.0) then
         xc(kk,c_nohmin) = rateq(kk,1)/xc(kk,c_nhplus)
       endif
      endif
c     enddo

      do 50 nrh=1,c_nreach
       ic = c_henry(nrh,1)

       if(c_nequil(ic).gt.0)  then

c        do kk=1,c_kmax
         if(c_h2oliq(kk).gt.0) then
          rpro(kk,1) = c_rp(kk,ic)-c_rl(kk,ic)
         endif
c        enddo

c SUM AQUEOUS SPECIES INTO THE GAS-MASTER, SUM GAS+AQUEOUS RPRO1.
         do neq=1,c_nequil(ic)
          icq = c_ncequil(ic,neq)
          if(icq.gt.0) then

c    TEST WRITE - TEMPORARY
c               if(c_kkw.gt.0) write(c_out, 19004) c_tchem(ic)
c    *                , xc(c_kkw,ic)
c 19004           format(' AQUEOUS CALCULATION:   ', a8,2x,1pe10.3)

c           do kk=1,c_kmax
            if(c_h2oliq(kk).gt.0) then
             xc(kk,ic) = xc(kk,ic) + xc(kk,icq)*c_h2oliq(kk)*avogadrl
             rpro(kk,1) = rpro(kk,1) + c_rp(kk,icq)-c_rl(kk,icq)

c            if(c_kkw.gt.0) write(c_out, 19004) c_tchem(ic),xc(c_kkw,ic)

            endif
c           enddo
          endif
         enddo

c SUM PRIOR NET CHEM ION PRODUCTION (RPRO2) AND ION SUM (RPRO3)
c ION CHEM. INFERRED FROM PRIOR GAS-AQ-ION PARTITIONING AND RPRO1.
c ALSO ZERO AQUEOUS CONCENTRATIONS

         do neq=1,c_nequil(ic)
          icq = c_ncequil(ic,neq)
          if(icq.gt.0) then

            if(c_iter.gt.1) then
c            do kk=1,c_kmax     ! kk vector loop
              if(c_h2oliq(kk).gt.0) then
               if(xc(kk,ic).gt.0) then        
                 rpro(kk,3) = rpro(kk,3) + (rpro(kk,1)/xc(kk,ic))
     *            * abs(c_ion(icq))*c_h2oliq(kk)*avogadrl*xc(kk,icq)
               endif                 !if(xc(kk,ic).gt.0) then        
               rpro(kk,2) = rpro(kk,2) + 
     *            abs(c_ion(icq)) * c_h2oliq(kk)*avogadrl*xc(kk,icq)
              endif              !if(c_h2oliq(kk).gt.0) then
c            enddo           !kk vector loop
            endif          !if(c_iter.gt.1) then

c INTERIM DIAGNOSTIC WRITE
c         if(c_kkw.gt.0) then
c           write(c_out,19104) c_tchem(ic),c_tchem(icq),xc(c_kkw,ic),
c    *        xc(c_kkw,icq)
c 19104       format(/'TEST AQ. ADJUSTMENT FOR PRIOR ION PRODUCTION:',
c    *      /,' TCHEM ION XR XRQ=   ',a8,2x,a8,2x,2(1pe10.3))
c           write(c_out,19105) rpro(c_kkw,1),rpro(c_kkw,2),rpro(c_kkw,3)
c 19105       format(/,' GAS-EQUIV RP FOR GAS+AQ    =',1pe10.3,/,
c    *               ' GAS-EQUIVALENT ION SUM     =',1pe10.3,/,
c    *               ' GAS-EQUIVALENT ION RP SUM  =',1pe10.3)
c         endif

c ZERO AQUEOUS CONCENTRATIONS
c           do kk=1,c_kmax
             xc(kk,icq)=0.
c           enddo
          endif
         enddo

       endif
50    continue

c WRITE FOR RPRO AQUEOUS SUM -  DIAGNOSTIC WRITE
          if(c_kkw.gt.0) then
            write(c_out,19106)  rpro(c_kkw,2),rpro(c_kkw,3)
19106       format(/,' AQUEOUS ION PRODUCTION SUMMARY:'   ,/,
     *               ' GAS-EQUIVALENT ION SUM     =',1pe10.3,/,
     *               ' GAS-EQUIVALENT ION RP SUM  =',1pe10.3)
          endif

c --------------------------------------------------
c CALCULATE AQUEOUS CONCENTRATIONS BASED ON PRIOR H+. 
C ALSO CALCULATE ACIDSUM, NET SUM OF AQUEOUS IONS (=ALPHA) 
c  AND ACIDDEL, d(ACIDSUM)/dH+  (=BETA).
c THESE ARE THE NEWTON-RAFSON PARAMETERS.

c -------------------------------------------------------------
c FOR A GAS+AQUEOUS GROUP Xg, Xa, X1, X2 w/ constants Kh,K1, K2
c where Xa = Kh*Xg  X1*H = Xa*K1  X2*H=X1*K2  and Kw=H*OH: 
c
c UNITS:  Kh was converted from (MOLES/LITER)/ATMOSPHERE
c to (MOLES/LITER)/(MOLEC/CM3) to (RELATIVE UNITS) in BRATES.
c K1, K2 are in MOLES/LITER.  Xg is converted to MOLES/LITER here.
c The MOLES/LITER conversion is based on LIQUID WATER (acqua). 
c
c DOUBLE CATION:    (Xt=Xg+Xa+X1+X2)
c  Xg = Xt/ (1 + Kh*(1 + K1/H*(1 + K2/H) ) )
c  Xa = Xg*Kh   X1=Xa*K1/H  X2=X1*K2/H
c           ( X1 = Xt*Rh*R1*H / (H**2*(1+Kh) + H*Kh*K1 + Kh*K1*K2)   )
c  dX1/dH= X1/H - (X1**2/(Xt*Kh*K1*H) (2H(1+Kh)+Kh*K1)           
c           ( X2 = XtKhK1K2/(H**2(1+Kh)+ H*Kh*K1 + Kh*K1*K2)         )
c  dX2/dH = (X2**2/(Xt*Kh*K1*K2)(2H(1+Kh)+Kh*K1)                 
c SINGLE CATION:    (K2=0)
c  dX1/dt = -(X1**2/(Xt*Kh*K1*H))*(1+Kh)
c
c DOUBLE ANION:  
c Xg = Xt/ (1 + Kh*(1 + R1/OH*(1 + R2/OH) ) )
c Xa = Xg*Kh    X1=Xa*K1/OH   X2=X1*K2/OH
c           ( X1=XtKhK1OH/( OH**2(1+Kh) + OHKhK1 + KhK1K2 )
c           ( dX1/dH = -(OH/H)*dX1/dOH                                )
c dX1/dt= -X1/H + (X1**2/Xt*Kh*K1*H)*(2OH*(1+Kh)+Kh*K1) )
c           ( X2=XtKhK1K2/(OH**2(1+Kh)+OH*K1*Kh + K2*K1*Kh )
c dX2/dH =  (OH/H) * (X2**2/Xt*Kh*K1*K2)*(2OH(1+Kh)*Kh*K1)

c SINGLE ANION:  
c dX1/dH =  (OH/H)* (X1**2)/(Xt*Kh*K1)*(1+Kh)  
c        (   dX1/dH = (X1*Xg/Xt*H)*(1+Kh)                             )
c
c -------------------------------------------------------------
c
c SET INITIAL ALPHA (=ion sum) AND BETA (=d/dH)
c TO INCLUDE THE IMPACT OF (H+) and (OH-).  (Note:  H+>0, above.)
c
c SPECIAL FUNCTIONS:  1/H=xhinv(kk)   1/OH = xohinv(kk)  
c                     1/Kw = xkwinv(kk)  1/Xt = gamma(kk)
c
c     do kk=1,c_kmax
      if(c_h2oliq(kk).gt.0) then
           xkwinv(kk) = 1./rateq(kk,1)
           xhinv(kk) = xkwinv(kk)*xc(kk,c_nohmin)
           xohinv(kk) = xkwinv(kk)*xc(kk,c_nhplus)
           alpha(kk) = xc(kk,c_nohmin)  - xc(kk,c_nhplus)
           beta(kk) =  1.+ xc(kk,c_nohmin)*xhinv(kk)  
      endif
c     enddo
c
c DIAGNOSTIC TEST WRITE FOR H+ SOLVER
          if(c_kkw.gt.0) then
           write(c_out,1201) c_tchem(c_nhplus), alpha(c_kkw)
     *              , beta (c_kkw)
1201       format(' AQ SPECIES, IONSUM, ION d/dH=',2x,a8,2(1pe12.3))
          endif

c         TEST WRITE
c         write(c_out,*) c_aqueous(1,2), c_aqueous(1,3)
c           write(c_out,*) rateq(1,1),xc(1,c_nhplus)
c         write(c_out,*) alpha(1), beta(1), xc(1,c_nhplus)

c BEGIN XR CALCULATION.
c LOOP THROUGH HENRY'S LAW TO IDENTIFY GAS-MASTER
c AND SOLVE FOR THE ASSOCIATED AQUEOUS GROUP.
      do 100 nrh=1,c_nreach
       ic = c_henry(nrh,1)
       ich=c_henry(nrh,2)
c               write(c_out,*) nrh,ic,ich
       if(c_nequil(ic).le.0) go to 100

c ZERO COUNTER FOR ION CHARGE 
c AND INDICES FOR FIRST, SECOND ACID  OR BASE REACTIONS
      ionsum=0
      ica1 = 0
      ica2 = 0
      icb1 = 0
      icb2 = 0
      nra1 = 0
      nra2 = 0
      nrb1 = 0
      nrb2 = 0

c  CONVERT GAS-MASTER SUM TO LIQUID-EQUIVALENT UNITS (MOLES/LITER)
c  WITH ZERO-PROTECT
c        do kk=1,c_kmax
          if(c_h2oliq(kk).gt.0) then

c   TEST WRITE - TEMPORARY
c            if(c_kkw.gt.0) write(c_out, 19004) c_tchem(ic),xc(c_kkw,ic)

             xc(kk,ic)=xc(kk,ic)/(c_h2oliq(kk)*avogadrl)

c  TEST WRITE - TEMPORARY
c            if(c_kkw.gt.0) write(c_out, 19004) c_tchem(ic),xc(c_kkw,ic)

          endif
c        enddo

c HENRY'S LAW CALCULATION FOR SPECIES WITH NO AQUEOUS EQUILIBRIA
c  (dimensionless H=Ca/Cg, Cg=Ct/(1+H), Ca=Ct*H/(1+H) )

       if(c_nequil(ic).eq.1)  then
c        do kk=1,c_kmax
         if(c_h2oliq(kk).gt.0) then
         gamma(kk) = 1.
     *        /(rhdif(kk,nrh) + 1.)
         xc(kk, ich) = (xc(kk,ic)*rhdif(kk,nrh)) * gamma(kk)
         xc(kk, ic)  = xc(kk,ic) * 
     *        (gamma(kk)*(c_h2oliq(kk)*avogadrl) )
         endif
c        enddo

c LOOP FOR AQUEOUS EQUILIBRIA.  (c_nequil(ic).gt.1)
       else

c PRELIMINARY: IDENTIFICATION OF FIRST AND SECOND ACID-FORMING
c OR BASE-FORMING REACTIONS FOR THE AQUEOUS GROUP (ICA,NRA,ICB,NRB).
c THIS ESTABLISHES SOLUTION PROCEDURE FOR THE GROUP, TO BE USED BELOW.

         do 90 neq=2,c_nequil(ic)
          icq = c_ncequil(ic,neq)
          nrq = c_nrequil(ic,neq)
c               write(c_out,*) neq,nrq,icq,ic
          if(icq.gt.0.and.nrq.gt.0) then
            if(c_aqueous(nrq,3).eq. c_nhplus) then
              if(ica1.eq.0) then
                ica1=icq
                nra1=nrq
              else
                ica2=icq
                nra2=nrq
              endif
            endif
            if(c_aqueous(nrq,3).eq. c_aqueous(1,3)) then
              if(icb1.eq.0) then
                icb1=icq
                nrb1=nrq
              else
                icb2=icq
                nrb2=nrq
              endif
            endif
          endif
90       continue

c AQUEOUS IC, NR RECORDED FOR FIRST AQUEOUS REACTION
c  (USED IF IT IS NOT ACID OR BASE REACTION).  
        icq = c_ncequil(ic,2)
        nrq = c_nrequil(ic,2)

        if(icq.le.0.and.nrq.le.0) 
     *     write(c_out,91) ic, c_tchem(ic), icq,nrq
91      format(/,' CHEMISTRY INDEX ERROR IN AQUASOLVE:',/,
     *   ' IC, TCHEM =  ', i5, 2x,a8,/,
     *   ' 2ND AQUEOUS IC, NR <=0 IN AQUEOUS LOOP; = ',2i5)

c               write(c_out,*) neq,nrq,icq
c               write(c_out,*) ica1, ica2, icb1, icb2
c               write(c_out,*) nra1, nra2, nrb1, nrb2



c MAIN SOLVER FOR AQUEOUS IONS. 
c OPTIONS FOR: NEUTRAL ION, SINGLE OR DOUBLE CATION, ANION.

c NEUTRAL AQUEOUS EQUILIBRIA:  GAS <-> AQUEOUS <-> NEUTRAL AQUEOUS.
c   THIS WORKS ONLY FOR A SINGLE CHAIN:  ONE NEUTRAL ION.

c Xa = Xg*Kh    X1=Xa*K1

            if(ica1.eq.0.and.icb1.eq.0) then

c                   TEST WRITE
c               if(c_kkw.gt.0) write(c_out,19003)
c 19003           format('NEUTRAL SPEC.')
c               if(c_kkw.gt.0) write(c_out,19004)c_tchem(icq), xc(1,icq)

c      do kk=1,c_kmax
       if(c_h2oliq(kk).gt.0) then
         gamma(kk) = 1.
     *        /((rateq(kk,nrq)+1.)*rhdif(kk,nrh) + 1.)
         xc(kk, icq) = xc(kk,ic)*(rateq(kk,nrq)*rhdif(kk,nrh)) 
     *                 * gamma(kk)
         xc(kk, ich) = (xc(kk,ic)*rhdif(kk,nrh)) * gamma(kk)
         xc(kk, ic)  = xc(kk,ic) * 
     *        (gamma(kk)*(c_h2oliq(kk)*avogadrl) )
       endif
c      enddo

c              TEST WRITE
c              if(c_kkw.gt.0) write(c_out,19004)c_tchem(icq), xc(1,icq)

            endif

c SINGLE CATION  - NOTE, SOLVE X1 FIRST, IN CASE Xgas, Xaq => 0.
c  Xg = Xt/ (1 + Kh*(1 + K1/H            ) )
c  Xa = Xg*Kh   X1=Xa*K1/H 
c  dX1/dt = -(X1**2/(Xt*Kh*K1*H))*(1+Kh)
c NOTE:  SOLVE X1 FIRST, INCASE Xgas, Xaq =>0.
c ALSO:  Initially use XR(IC) = TOTAL GAS+AQUEOUS, in aq. units.
c Then solve for GAS at the end.

            if(ica1.gt.0.and.ica2.eq.0) then

c               TEST WRITE
c               if(c_kkw.gt.0) write(c_out,19005)
c 19005           format('SINGLE CATION')
c               if(c_kkw.gt.0) write(c_out,19004)c_tchem(icq), xc(1,icq)

c     do kk=1,c_kmax
       if(c_h2oliq(kk).gt.0.and.xc(kk,ic).gt.1.0D-40) then

         gamma(kk) = 1.
     *        /((rateq(kk,nra1)*xhinv(kk) + 1.)*rhdif(kk,nrh) + 1.)

         xc(kk, ica1)  = 
     *         (xc(kk,ic)*xhinv(kk)*rhdif(kk,nrh)*rateq(kk,nra1))
     *        *gamma(kk)

         xc(kk, ich) = (xc(kk,ic)*rhdif(kk,nrh)) * gamma(kk)

c ION SUM (alpha), d/dH ION SUM (beta): 
         alpha(kk) = alpha(kk) - xc(kk,ica1)*c_ion(ica1)

         beta(kk) = beta(kk) - c_ion(ica1)*(
     *   (1. + rhdif(kk,nrh))
     *   *(xc(kk,ica1)/xc(kk,ic))  
     *   *(xc(kk,ica1)/(rhdif(kk,nrh)*rateq(kk,nra1)) )
     *    )

c GAS SUM, converted to GAS units.
         xc(kk, ic)  = xc(kk,ic) * 
     *        (gamma(kk)*(c_h2oliq(kk)*avogadrl) )

       endif
c     enddo
c              TEST WRITE
c              if(c_kkw.gt.0) write(c_out,19004)c_tchem(icq), xc(1,icq)

c DETAILED TEST WRITE
c 19011   format(8(1pe10.3))
c       if (c_kkw.gt.0) write(c_out,19011) xc(c_kkw,ica1)
c       if (c_kkw.gt.0) write(c_out,19011) 
c    *    xc(c_kkw,ic), xhinv(c_kkw), rhdif(c_kkw,nrh)
c    *   , rateq(c_kkw,nra1), gamma(c_kkw)

c        if(c_kkw.gt.0) write(c_out,19011) xc(c_kkw,ich)
c        if(c_kkw.gt.0) write(c_out,19011) rhdif(c_kkw,nrh)
c    *      , gamma(c_kkw)

c        if(c_kkw.gt.0) write(c_out,19011) xc(c_kkw,ic)
c        if(c_kkw.gt.0) write(c_out,19011)  gamma(c_kkw)
c    *      , c_h2oliq(c_kkw),    avogadrl
c END TEST WRITES

            endif

c DOUBLE CATION
c  Xg = Xt/ (1 + Kh*(1 + K1/H*(1 + K2/H) ) )
c  Xa = Xg*Kh   X1=Xa*K1/H  X2=X1*K2/H
c  dX1/dH= X1/H - (X1**2/(Xt*Kh*K1*H) (2H(1+Kh)+Kh*K1)           
c  dX2/dH = (X2**2/(Xt*Kh*K1*K2)(2H(1+Kh)+Kh*K1)                 

            if(ica1.gt.0.and.ica2.gt.0) then

c               TEST WRITE
c               if(c_kkw.gt.0) write(c_out,19006)
c 19006           format('DOUBLE CATION')
c               if(c_kkw.gt.0) write(c_out,19004)c_tchem(icq), xc(1,icq)

c     do kk=1,c_kmax
       if(c_h2oliq(kk).gt.0.and.xc(kk,ic).gt.1.0D-40) then

         gamma(kk) = 1.
     *        /(((rateq(kk,nra2)*xhinv(kk) + 1.) 
     *           *rateq(kk,nra1)*xhinv(kk) + 1.)*rhdif(kk,nrh) + 1.)

         xc(kk, ica1)  = 
     *         (xc(kk,ic)*xhinv(kk)*rhdif(kk,nrh)*rateq(kk,nra1))
     *        *gamma(kk)

         xc(kk, ica2) = xc(kk,ica1)*rateq(kk,nra2)*xhinv(kk)

         xc(kk, ich) = (xc(kk,ic)*rhdif(kk,nrh)) * gamma(kk)

c ION SUM (alpha), d/dH ION SUM (beta): 
         alpha(kk) = alpha(kk) - xc(kk,ica1)*c_ion(ica1)
     *                         - xc(kk,ica2)*c_ion(ica2) 

         beta(kk) = beta(kk) 
     *    + c_ion(ica1) *xc(kk,ica1)*xhinv(kk) 

     *    - (2.*xc(kk,c_nhplus)*(1.+rhdif(kk,nrh)) 
     *               + rhdif(kk,nrh)*rateq(kk,nra1)  )   
     *    * (

     *    c_ion(ica1)*(xc(kk,ica1)/xc(kk,ic))  
     *    *(xc(kk,ica1)/(rhdif(kk,nrh)*rateq(kk,nra1)*xc(kk,c_nhplus)) )

     *    + c_ion(ica2)*(xc(kk,ica2)/xc(kk,ic))
     *   *(xc(kk,ica2)/(rhdif(kk,nrh)*rateq(kk,nra1)*rateq(kk,nra2)) )
     *    )

c GAS SUM, converted to GAS units.
         xc(kk, ic)  = xc(kk,ic) * 
     *        (gamma(kk)*(c_h2oliq(kk)*avogadrl) )

       endif
c     enddo

c     TEST WRITE
c              if(c_kkw.gt.0) write(c_out,19004)c_tchem(icq), xc(1,icq)

c DETAILED TEST WRITE
c       if (c_kkw.gt.0) write(c_out,19011) xc(c_kkw,ica1)
c       if (c_kkw.gt.0) write(c_out,19011) 
c    *    xc(c_kkw,ic), xhinv(c_kkw), rhdif(c_kkw,nrh)
c    *   , rateq(c_kkw,nra1), gamma(c_kkw)

c       if (c_kkw.gt.0) write(c_out,19011) xc(c_kkw,ica2)
c       if (c_kkw.gt.0) write(c_out,19011) rateq(c_kkw,nra2)
c    *    , xhinv(c_kkw)

c        if(c_kkw.gt.0) write(c_out,19011) xc(c_kkw,ich)
c        if(c_kkw.gt.0) write(c_out,19011) rhdif(c_kkw,nrh)
c    *    , gamma(c_kkw)

c        if(c_kkw.gt.0) write(c_out,19011) xc(c_kkw,ic)
c        if(c_kkw.gt.0) write(c_out,19011)  gamma(c_kkw)
c    *    , c_h2oliq(c_kkw),   avogadrl
c END TEST WRITE

            endif

c SINGLE ANION 
c Xg = Xt/ (1 + Kh*(1 + R1/OH             ) )
c Xa = Xg*Kh    X1=Xa*K1/OH   
c dX1/dH =  (OH/H)* (X1**2)/(Xt*Kh*K1)*(1+Kh)  

            if(icb1.gt.0.and.icb2.eq.0) then

c               TEST WRITE
c               if(c_kkw.gt.0) write(c_out,19007)
c 19007           format('SINGLE ANION ')
c               if(c_kkw.gt.0) write(c_out,19004)c_tchem(icq), xc(1,icq)

c     do kk=1,c_kmax
       if(c_h2oliq(kk).gt.0.and.xc(kk,ic).gt.1.0D-40) then


         gamma(kk) = 1.
     *        /((rateq(kk,nrb1)*xohinv(kk) + 1.)*rhdif(kk,nrh) + 1.)

         xc(kk, icb1)  = 
     *         (xc(kk,ic)*xohinv(kk)*rhdif(kk,nrh)*rateq(kk,nrb1))
     *        *gamma(kk)

         xc(kk, ich) = (xc(kk,ic)*rhdif(kk,nrh)) * gamma(kk)

c ION SUM (alpha), d/dH ION SUM (beta): 
         alpha(kk) = alpha(kk) - xc(kk,icb1)*c_ion(icb1)

         beta(kk) = beta(kk) + c_ion(icb1)*(
     *   (xc(kk,c_nohmin)*xhinv(kk))   *  (1. + rhdif(kk,nrh))  
     *   *(xc(kk,icb1)/xc(kk,ic))  
     *   *(xc(kk,icb1)/(rhdif(kk,nrh)*rateq(kk,nrb1)) )
     *    )

c GAS SUM, converted to GAS units.
         xc(kk, ic)  = xc(kk,ic) * 
     *        (gamma(kk)*(c_h2oliq(kk)*avogadrl) )

       endif
c     enddo

c               TEST WRITE
c               if(c_kkw.gt.0) write(c_out,19004)c_tchem(icq), xc(1,icq)

            endif

c DOUBLE ANION
c Xg = Xt/ (1 + Kh*(1 + R1/OH*(1 + R2/OH) ) )
c Xa = Xg*Kh    X1=Xa*K1/OH   X2=X1*K2/OH
c dX1/dt= -X1/H + (X1**2/Xt*Kh*K1*H)*(2OH*(1+Kh)+Kh*K1) )
c dX2/dH =  (OH/H) * (X2**2/Xt*Kh*K1*K2)*(2OH(1+Kh)*Kh*K1)

            if(icb1.gt.0.and.icb2.gt.0) then

c TEST WRITE
c               if(c_kkw.gt.0) write(c_out,19008)
c 19008           format('DOUBLE ANION ')
c               if(c_kkw.gt.0) write(c_out,19004)c_tchem(icq), xc(1,icq)

c     do kk=1,c_kmax
       if(c_h2oliq(kk).gt.0.and.xc(kk,ic).gt.1.0D-40) then

         gamma(kk) = 1.
     *        /(((rateq(kk,nrb2)*xohinv(kk) + 1.) 
     *           *rateq(kk,nrb1)*xohinv(kk) + 1.)*rhdif(kk,nrh) + 1.)

         xc(kk, icb1)  = xc(kk,ic) * (
     *         (          xohinv(kk)*rhdif(kk,nrh)*rateq(kk,nrb1))
     *        *gamma(kk)  )


         xc(kk, icb2) = xc(kk,icb1)*(rateq(kk,nrb2)*xohinv(kk))

         xc(kk, ich) = xc(kk,ic)*(rhdif(kk,nrh) * gamma(kk) )

c ION SUM (alpha), d/dH ION SUM (beta): 
         alpha(kk) = alpha(kk) - xc(kk,icb1)*c_ion(icb1)
     *                         - xc(kk,icb2)*c_ion(icb2) 

         beta(kk) = beta(kk) 
     *    - c_ion(icb1) *xc(kk,icb1)*xhinv(kk) 

         beta(kk) = beta(kk) 
     *    + (2.*xc(kk,c_nohmin)*(1.+rhdif(kk,nrh)) 
     *               + rhdif(kk,nrh)*rateq(kk,nrb1)  )   
     *    * (

     *    c_ion(icb1)*(xc(kk,icb1)/xc(kk,ic))  
     *    *(xc(kk,icb1)/(rhdif(kk,nrh)*rateq(kk,nrb1)*xc(kk,c_nhplus)) )

     *    + c_ion(icb2)*(xc(kk,icb2)/xc(kk,ic))  
     *                    *(xc(kk,c_nohmin)*xhinv(kk) )
     *   *(xc(kk,icb2)/(rhdif(kk,nrh)*rateq(kk,nrb1)*rateq(kk,nrb2)) )
     *    )

c GAS SUM, converted to GAS units.
         xc(kk, ic)  = xc(kk,ic) * 
     *        (gamma(kk)*(c_h2oliq(kk)*avogadrl) )


       endif
c     enddo

c              TEST WRITE
c              if(c_kkw.gt.0) write(c_out,19004)c_tchem(icq), xc(1,icq)

c DETAILED TEST WRITE
c       if (c_kkw.gt.0) write(c_out,19011) xc(c_kkw,icb1)
c       if (c_kkw.gt.0) write(c_out,19011) 
c    *    xc(c_kkw,ic), xohinv(c_kkw), rhdif(c_kkw,nrh)
c    *  , rateq(c_kkw,nrb1), gamma(c_kkw)

c       if (c_kkw.gt.0) write(c_out,19011) xc(c_kkw,icb2)
c       if (c_kkw.gt.0) write(c_out,19011) rateq(c_kkw,nrb2)
c    *    , xohinv(c_kkw)

c        if(c_kkw.gt.0) write(c_out,19011) xc(c_kkw,ich)
c        if(c_kkw.gt.0) write(c_out,19011) rhdif(c_kkw,nrh)
c    *    , gamma(c_kkw)

c        if(c_kkw.gt.0) write(c_out,19011) xc(c_kkw,ic)
c        if(c_kkw.gt.0) write(c_out,19011)  gamma(c_kkw)
c    *    , c_h2oliq(c_kkw),    avogadrl
c END TEST WRITE

            endif

c 2006 ERROR CORRECTION - PROTECT AGAINST ZERO AQUEOUS - CUT
c     if(xc(1,ich).eq.0.) then
c       if(rhdif(1,nrh).gt.1.0e-10) xc(1,ich) = 1.0e-34
c     endif                        !if(xc(kk,ich).eq.0.) then

c DIAGNOSTIC TEST WRITE FOR H+ SOLVER
          if(c_kkw.gt.0) then
           write(c_out,1201) c_tchem(ic), alpha(c_kkw), beta (c_kkw)
          endif


c               TEST WRITE
c               write(c_out,*) xc( 1,icq),c_ion(icq),xc( 1,ic)
c               write(c_out,*) xc(1,ich), alpha(1),beta(1)

c END LOOP FOR AQUEOUS EQUILIBRIUM SPECIES
       endif


100   continue

c ----------------------------------------------
c END OF LOOP TO CALCULATE AQUEOUS CONCENTRATIONS
c ----------------------------------------------
c
c
c -------------------------------
c CALCULATE H+ (AND OH-) FROM NEWTON-RAPHSON
c -------------------------------
c
c MODIFICATION FOR DIFFICULT CONVERGENCE: H+=geometric mean w/ prior H+
c   (To prevent oscillation with H+/SO2->HSO3-->SO4=.)

c ALSO:  INCREASE BETA BY RATIO:  ION RPRO/PRIOR ION SUM
c TO ACCOUNT FOR CHEM PRODUCTION ->H+ FEEDBACK.

c      do kk=1,c_kmax
       if(c_h2oliq(kk).gt.0) then
       if(rpro(kk,2).gt.rpro(kk,3).and.rpro(kk,3).gt.0) 
     *   beta(kk) = beta(kk) 
     *       *(1.+rpro(kk,3)/(rpro(kk,2)-rpro(kk,3))  )
c
       if(beta(kk).ne.0) alpha(kk)  = alpha(kk)/beta(kk)
c      if(alpha(kk).lt.-0.9*xc(kk,c_nhplus)) 
c    *          alpha(kk)=-0.9*xc(kk,c_nhplus)
c      if(alpha(kk).gt.10*xc(kk,c_nhplus)) alpha(kk)=10.*xc(kk,c_nhplus)
c      xc(kk,c_nhplus) = xc(kk,c_nhplus) +  alpha(kk)
       if(alpha(kk).lt.-0.99*xc(kk,c_nhplus)) 
     *        alpha(kk)=-0.9*xc(kk,c_nhplus)
       if(alpha(kk).gt.100*xc(kk,c_nhplus)) 
     *        alpha(kk)=10.*xc(kk,c_nhplus)
       xc(kk,c_nhplus) = 
     *        sqrt( xc(kk,c_nhplus) * (xc(kk,c_nhplus)+alpha(kk)) )
       if(xc(kk,c_nhplus).gt.0) then
           xc(kk,c_nohmin) = rateq(kk,1)/xc(kk,c_nhplus)
       endif

       endif
c      enddo
c ----------------------------------------------
c SUMMARY WRITE:   WRITE RESULTS OF ITERATION IF LPRT AND ACQUA>0.
c ----------------------------------------------

       if(c_kkw.gt.0) then
         if(c_h2oliq(c_kkw).gt.0) then
           write(c_out,1301)   c_h2oliq(c_kkw)
           acquacon = c_h2oliq( c_kkw)*avogadrl
           write(c_out,1303) acquacon
c 1303       format(/,'AQUATIC CONVERSION FACTOR:',
c    *          ' MOLE/LITER per MOLEC/CM3 =', 1pe10.3)
1303        format( ' GAS SPECIES is gas only, molec/cm3. ',
     *      ' AQUEOUS moles/liter.',/,
     *       '                             CONVERSION= ', 1pe10.3)
           if(c_aqueous(1,2).gt.0) then
             write(c_out,1302) xc(c_kkw,c_aqueous(1,2))
           endif
1301        format(/,'AQUEOUS CHEMISTRY ',/,'WATER (grams/cm3) =',
     *          1pe10.3)
1302        format( ' [H+] (moles per liter) =',1pe10.3)

c NOTE:  GAS species given in molec/cm3; AQUEOUS in moles/liter
c        GAS is NOT YET GAS-AQ SUM.
           tsum = '     SUM'
           do ic=1,c_nchem2
            if(c_nequil(ic).gt.0)  then
              sum = xc(c_kkw,ic)
              do i=1,c_nequil(ic)
               sum = sum + xc(c_kkw,c_ncequil(ic,i))*acquacon
              enddo
              write(c_out,1304) c_tchem(ic), xc(c_kkw,ic),
     *        ( c_tchem(c_ncequil(ic,i)),xc(c_kkw,c_ncequil(ic,i))
     *                                      ,i=1,c_nequil(ic) )
     *        ,tsum, sum
1304           format(4(a8,1pe10.3),/,
     *        '         0.000E+00',3(a8,1pe10.3))
            endif
           enddo
c
          write(c_out,1306)
1306       format(/,'HENRYS LAW RATES(+DIFFUSION ADJUSTMENT)',
     *       ' AND EQUILIBRIUM CONSTANTS:')
          if(c_nreach.gt.0) then
c           write(c_out,1307) ((c_treach(j,i),j=1,2),
c    *         rateh(c_kkw,i),rhdif(c_kkw,i),i=1,c_nreach)
            do i=1,c_nreach
             write(c_out,1307) (c_treach(j,i),j=1,2),
     *         rateh(c_kkw,i),  rhdif(c_kkw,i)
            enddo
1307       format(a8,'=',a8,2x,2(1pe10.3))
          endif

          if(c_nreacq.gt.0) then
            write(c_out,1308) (c_treacq(j,1),j=1,3),
     *       rateq(c_kkw,1), c_ion(c_aqueous(1,2))
1308       format(a8,'=',a8,'+',a8,2x,e10.3,'    ION2=',i2)
          endif

          if(c_nreacq.gt.1) then
c           write(c_out,1308) ((c_treacq(j,i),j=1,3),
c    *       rateq(c_kkw,i), c_ion(c_aqueous(i,2))  ,i=2,c_nreacq)
            do i=2,c_nreacq
             write(c_out,1308) (c_treacq(j,i),j=1,3),
     *       rateq(c_kkw,i), c_ion(c_aqueous(i,2))  
            enddo
          endif

         endif
       endif
c ----------------------------------------------
c END LOOP:  SUMMARY WRITE    
c ----------------------------------------------


c END AQUASOLVE
2000   return
       end
c --------------------------------------------------------------
c
c --------------------
c END  OF chemsolve/quadchem/aquasolve
c --------------------
c
c
c ==============================================================
